@@ -1,4 +1,4 @@
-/*  $Id: blast_args.hpp 350372 2012-01-19 22:17:04Z rafanovi $
+/*  $Id: blast_args.hpp 366366 2012-06-13 20:38:02Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -933,8 +933,6 @@ private:
     /// comma-separated value (populated if applicable)
     string m_CustomOutputFormatSpec;
 
-    /// Issues warnings that can only be caught at runtime
-    void x_IssueWarningsOnIncompatibleOptions(const CArgs& args) const;
 };
 
 /// Argument class to collect multi-threaded arguments
