@@ -1,4 +1,4 @@
-/*  $Id: cmdline_flags.hpp 350250 2012-01-18 21:22:09Z rafanovi $
+/*  $Id: cmdline_flags.hpp 368006 2012-06-29 19:48:05Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -311,6 +311,10 @@ NCBI_BLASTINPUT_EXPORT extern const string kArgOutputSearchStrategy;
 NCBI_BLASTINPUT_EXPORT extern const string kArgUseIndex;
 /// Megablast database index name.
 NCBI_BLASTINPUT_EXPORT extern const string kArgIndexName;
+/// Use old style megablast index.
+NCBI_BLASTINPUT_EXPORT extern const string kArgOldStyleIndex;
+/// Default value for use old style megablast index.
+NCBI_BLASTINPUT_EXPORT extern const string kDfltArgOldStyleIndex;
 
 /// Entrez query.
 NCBI_BLASTINPUT_EXPORT extern const string kArgEntrezQuery;
