@@ -1,4 +1,4 @@
-/* $Id: blast_hits.h 346479 2011-12-07 14:30:16Z fongah2 $
+/* $Id: blast_hits.h 358495 2012-04-03 13:19:52Z merezhuk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -808,6 +808,15 @@ Blast_HSPResultsFromHSPStreamWithLimit(struct BlastHSPStream* hsp_stream,
                                    Uint4 max_num_hsps,
                                    Boolean* removed_hsps);
 
+BlastHSPResults*
+/** As Blast_HSPResultsFromHSPStreamWithLimit, except accept and return 
+ * array of Boolen flags specifying which query exceeded HSP limits.
+ */
+Blast_HSPResultsFromHSPStreamWithLimitEx(struct BlastHSPStream* hsp_stream, 
+                                   Uint4 num_queries, 
+                                   SBlastHitsParameters* hit_param,
+                                   Uint4 max_num_hsps,
+                                   Boolean* removed_hsps);
 /** Splits the BlastHSPResults structure for a PHI BLAST search into an array of
  * BlastHSPResults structures, corresponding to different pattern occurrences in
  * query. All HSPs are copied, so it is safe to free the returned 
