@@ -1,4 +1,4 @@
-/* $Id: blast_options.h 347205 2011-12-14 20:08:44Z boratyng $
+/* $Id: blast_options.h 364124 2012-05-22 21:03:27Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -171,7 +171,9 @@ typedef enum {
     ePhiLookupTable,  /**< protein lookup table specialized for phi-blast */
     ePhiNaLookupTable,  /**< nucleotide lookup table for phi-blast */
     eRPSLookupTable, /**< RPS lookup table (rpsblast and rpstblastn) */
-    eIndexedMBLookupTable /**< use database index as a lookup structure */
+    eIndexedMBLookupTable, /**< use database index as a lookup structure */
+    eMixedMBLookupTable /**< use when some volumes are searched with index and 
+                             some are not */
 } ELookupTableType;
 
 /** Options needed to construct a lookup table 
