@@ -1,4 +1,4 @@
-/* $Id: blast_format.hpp 347205 2011-12-14 20:08:44Z boratyng $
+/* $Id: blast_format.hpp 352627 2012-02-08 15:06:54Z camacho $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -66,16 +66,19 @@ public:
     /// @param opts BLAST options used in the search [in]
     /// @param db_adapter Adapter object representing a BLAST database or
     /// subject sequences [in]
-    /// @param format_type Integer indication the type of output [in]
-    /// @param believe_query true if sequence ID's of query sequences
+    /// @param format_type enumeration that specifies the requested type of output [in]
+    /// @param believe_query set to true if sequence IDs of query sequences
     ///                are to be parsed. If multiple queries are provieded,
-    ///                their sequence ID's must be distinct [in]
+    ///                their sequence ID's must be distinct. If you don't know
+    ///                what this means, set to false [in]
     /// @param outfile Stream that will receive formatted output
     /// @param num_summary The number of 1-line summaries at the top of
     ///                   the blast report (for output types that have
-    ///                   1-line summaries) [in]
+    ///                   1-line summaries). Default value for the BLAST
+    ///                   programs is align_format::kDfltArgNumDescriptions [in]
     /// @param num_alignments The number of alignments to display in the BLAST
-    ///                 report [in] 
+    ///                 report. Default value for the BLAST programs is
+    ///                 align_format::kDfltArgNumAlignments [in] 
     /// @param scope The scope to use for retrieving sequence data
     ///              (must contain query and database sequences) [in]
     /// @param matrix_name Name of protein score matrix (BLOSUM62 if
@@ -124,15 +127,18 @@ public:
     /// @param opts BLAST options used in the search [in]
     /// @param dbinfo_list Database info list, provided directly [in]
     /// @param format_type Integer indication the type of output [in]
-    /// @param believe_query true if sequence ID's of query sequences
+    /// @param believe_query set to true if sequence IDs of query sequences
     ///                are to be parsed. If multiple queries are provieded,
-    ///                their sequence ID's must be distinct [in]
+    ///                their sequence ID's must be distinct. If you don't know
+    ///                what this means, set to false [in]
     /// @param outfile Stream that will receive formatted output
     /// @param num_summary The number of 1-line summaries at the top of
     ///                   the blast report (for output types that have
-    ///                   1-line summaries) [in]
+    ///                   1-line summaries). Default value for the BLAST
+    ///                   programs is align_format::kDfltArgNumDescriptions [in]
     /// @param num_alignments The number of alignments to display in the BLAST
-    ///                 report [in] 
+    ///                 report. Default value for the BLAST programs is
+    ///                 align_format::kDfltArgNumAlignments [in] 
     /// @param scope The scope to use for retrieving sequence data
     ///              (must contain query and database sequences) [in]
     /// @param show_gi When printing database sequence identifiers, 
@@ -306,13 +312,11 @@ private:
     /// numeric_limits<unsigned int>::max() [in]
     /// @param prev_seqids list of previously found Seq-ids, if applicable,
     /// otherwise it should be an empty list [in]
-    /// @param skip_from skipping display from [in]
-    /// @param skip_to skipping display to [in]
+    /// @param additional show additional aligment summary [in]
     void x_DisplayDeflines(CConstRef<CSeq_align_set> aln_set,
                    unsigned int itr_num,
                    blast::CPsiBlastIterationState::TSeqIds& prev_seqids,
-                   int skip_from = -1,
-                   int skip_to = -1);
+                   int additional = 0);
 
     /// Split the full alignment into two sets of alignments: one for those
     /// seen in the previous iteration and used to build the PSSM and the other
