@@ -1,4 +1,4 @@
-/*  $Id: igblast.hpp 347537 2011-12-19 16:45:43Z maning $
+/*  $Id: igblast.hpp 359118 2012-04-10 19:00:13Z maning $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -30,8 +30,8 @@
 /// @file igblast.hpp
 /// Declares CIgBlast, the C++ API for the IG-BLAST engine.
 
-#ifndef ALGO_BLAST_API___IGBLAST__HPP
-#define ALGO_BLAST_API___IGBLAST__HPP
+#ifndef ALGO_BLAST_IGBLAST___IGBLAST__HPP
+#define ALGO_BLAST_IGBLAST___IGBLAST__HPP
 
 #include <algo/blast/api/setup_factory.hpp>
 #include <algo/blast/api/uniform_search.hpp>
@@ -47,6 +47,16 @@ BEGIN_NCBI_SCOPE
 
 BEGIN_SCOPE(blast)
 
+/// Keeps track of the version of IgBLAST in the NCBI C++ toolkit.
+/// Used to perform run-time version checks
+///
+/// For reference, please refer to http://apr.apache.org/versioning.html
+class CIgBlastVersion : public CVersionInfo {
+public:
+    CIgBlastVersion()
+        : CVersionInfo(1, 0, 0) {}
+};
+
 class IQueryFactory;
 
 class CIgBlastOptions : public CObject
@@ -57,6 +67,7 @@ public:
     string m_Origin;                 // the origin of species
     string m_DomainSystem;           // domain system for annotation
     string m_AuxFilename;            // auxulary file name
+    string m_IgDataPath;             // internal data path
     CRef<CLocalDbAdapter> m_Db[4];   // user specified germline database
                                      // 0-2: - user specified V, D, J
                                      // 3:   - the default V gl db
@@ -292,4 +303,4 @@ END_NCBI_SCOPE
 
 /* @} */
 
-#endif  /* ALGO_BLAST_API___IGBLAST__HPP */
+#endif  /* ALGO_BLAST_IGBLAST___IGBLAST__HPP */
