@@ -1,7 +1,7 @@
 #ifndef NCBICGI__HPP
 #define NCBICGI__HPP
 
-/*  $Id: ncbicgi.hpp 257501 2011-03-14 17:21:47Z grichenk $
+/*  $Id: ncbicgi.hpp 357979 2012-03-28 14:45:54Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -38,6 +38,7 @@
 #include <corelib/rwstream.hpp>
 #include <corelib/stream_utils.hpp>
 #include <cgi/cgi_util.hpp>
+#include <cgi/user_agent.hpp>
 #include <set>
 
 
@@ -1059,7 +1060,7 @@ void CCgiEntry::x_ForceComplete() const
     _ASSERT(m_Data->m_Value.empty());
     SData& data = const_cast<SData&>(*m_Data);
     auto_ptr<IReader> reader(data.m_Reader.release());
-    ExtractReaderContents(*reader, data.m_Value);
+    g_ExtractReaderContents(*reader, data.m_Value);
 }
 
 
