@@ -1,7 +1,7 @@
 #ifndef CGI___NCBICGIR__HPP
 #define CGI___NCBICGIR__HPP
 
-/*  $Id: ncbicgir.hpp 344530 2011-11-16 16:27:54Z ucko $
+/*  $Id: ncbicgir.hpp 354606 2012-02-28 17:00:34Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -116,7 +116,9 @@ public:
     const CCgiCookies& Cookies(void) const;
     CCgiCookies&       Cookies(void);
 
-    /// Set output stream (NULL here means "no output stream")
+    /// Set output stream (NULL here means "no output stream").
+    /// This does not flush the previous stream (if any), so some
+    /// data may be lost.
     void SetOutput(CNcbiOstream* os, int fd = -1);
 
     /// Get output stream (NULL here means "no output stream").
@@ -153,6 +155,9 @@ public:
     /// @sa GetOutput()
     void SetThrowOnBadOutput(bool throw_on_bad_output);
 
+    /// Check if 'Accept-Ranges' header is set to 'bytes'.
+    bool AcceptRangesBytes(void) const;
+
 public:
     void x_SetSession(const CCgiSession& session);
 
@@ -169,6 +174,8 @@ protected:
     static const string sm_HTTPStatusDefault;   // Default HTTP status:  200 OK
     static const string sm_BoundaryPrefix;      // Start of multipart boundary
     static const string sm_CacheControl;        // Cache-Control
+    static const string sm_AcceptRanges;        // Accept-Ranges
+    static const string sm_AcceptRangesBytes;   // bytes
 
     typedef map<string, string, PNocase> TMap;
 
