@@ -1,7 +1,7 @@
 #ifndef COMMON___NCBI_EXPORT__H
 #define COMMON___NCBI_EXPORT__H
 
-/*  $Id: ncbi_export.h 343757 2011-11-09 15:58:16Z vasilche $
+/*  $Id: ncbi_export.h 368190 2012-07-03 20:42:50Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -126,6 +126,7 @@
 #  define NCBI_SUBMIT_EXPORTS
 #  define NCBI_TAXON1_EXPORTS
 #  define NCBI_TAXON3_EXPORTS
+#  define NCBI_VARIATION_EXPORTS
 #endif
 
 
@@ -830,6 +831,14 @@
 #  define NCBI_TAXON3_EXPORT NCBI_DLL_IMPORT
 #endif
 
+/* Export specifier for library variation
+ */
+#ifdef NCBI_VARIATION_EXPORTS
+#  define NCBI_VARIATION_EXPORT NCBI_DLL_EXPORT
+#else
+#  define NCBI_VARIATION_EXPORT NCBI_DLL_IMPORT
+#endif
+
 /* Export specifier for library threader
  */
 #ifdef NCBI_THREADER_EXPORTS
