@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONN_STREAM__HPP
 #define CONNECT___NCBI_CONN_STREAM__HPP
 
-/* $Id: ncbi_conn_stream.hpp 350569 2012-01-21 00:27:04Z rafanovi $
+/* $Id: ncbi_conn_stream.hpp 362481 2012-05-09 17:23:43Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -68,7 +68,6 @@
  *      and file lists from remote FTP servers, and upload files as well.
  */
 
-#include <corelib/ncbistd.hpp>
 #include <connect/ncbi_ftp_connector.h>
 #include <connect/ncbi_namedpipe_connector.hpp>
 #include <connect/ncbi_memory_connector.h>
@@ -91,7 +90,7 @@ class CConn_Streambuf;  // Forward declaration
 class CSocket;          // Forward declaration
 
 
-const streamsize kConn_DefaultBufSize = 4096;
+const size_t kConn_DefaultBufSize = 4096;
 
 
 
@@ -138,7 +137,7 @@ public:
     (CONN            conn,
      bool            close    = false,
      const STimeout* timeout  = kDefaultTimeout,
-     streamsize      buf_size = kConn_DefaultBufSize,
+     size_t          buf_size = kConn_DefaultBufSize,
      bool            tie      = true,
      CT_CHAR_TYPE*   ptr      = 0,
      size_t          size     = 0);
@@ -163,7 +162,7 @@ protected:
     CConn_IOStream
     (CONNECTOR       connector,
      const STimeout* timeout  = kDefaultTimeout,
-     streamsize      buf_size = kConn_DefaultBufSize,
+     size_t          buf_size = kConn_DefaultBufSize,
      bool            tie      = true,
      CT_CHAR_TYPE*   ptr      = 0,
      size_t          size     = 0);
@@ -184,10 +183,13 @@ public:
     string          GetDescription(void) const;
 
     /// Set connection timeout for "direction"
+    /// @param
+    ///   Can accept a pointer to a finite timeout, or either of the special
+    ///   values: kInfiniteTimeout, kDefaultTimeout.
     /// @sa
     ///   CONN_SetTimeout, SetReadTimeout, SetWriteTimeout
     EIO_Status      SetTimeout(EIO_Event       direction,
-                               const STimeout* timeout= kDefaultTimeout) const;
+                               const STimeout* timeout) const;
 
     /// @return
     ///   Connection timeout for "direction"
@@ -196,16 +198,16 @@ public:
     const STimeout* GetTimeout(EIO_Event direction) const;
 
     /// @return
-    ///   Status of last performed I/O from the underlying CONN in
-    ///   the specified "direction" (either eIO_Read or eIO_Write);
-    ///   if "direction" is not specified (eIO_Open), return status
+    ///   Status of the last I/O performed by the underlying CONN in
+    ///   the specified "direction" (either eIO_Open, IO_Read or eIO_Write);
+    ///   if "direction" is not specified (eIO_Close), return status
     ///   of the last CONN I/O performed by the stream.
     /// @sa
     ///   CONN_Status
-    EIO_Status      Status(EIO_Event direction = eIO_Open) const;
+    EIO_Status      Status(EIO_Event direction = eIO_Close) const;
 
     /// Close CONNection, free all internal buffers and underlying structures,
-    /// and render stream unusable for further I/O.
+    /// and render the stream unusable for further I/O.
     /// Can be used at places where reaching end-of-scope for the stream
     /// would be impractical.
     /// @sa
@@ -214,7 +216,7 @@ public:
 
     /// Cancellation support
     /// @note ICanceled implementation must be derived from CObject as its
-    /// first subclass.
+    /// first superclass.
     /// @sa
     ///   ICanceled
     EIO_Status      SetCanceledCallback(const ICanceled* canceled);
@@ -259,7 +261,7 @@ private:
 class CConn_IOStreamSetReadTimeout : protected CConn_IOStreamSetTimeout
 {
 public:
-    CConn_IOStreamSetTimeout::GetTimeout;
+    using CConn_IOStreamSetTimeout::GetTimeout;
 
 protected:
     CConn_IOStreamSetReadTimeout(const STimeout* timeout)
@@ -289,7 +291,7 @@ inline CConn_IOStream& operator>> (CConn_IOStream& is,
 class CConn_IOStreamSetWriteTimeout : protected CConn_IOStreamSetTimeout
 {
 public:
-    CConn_IOStreamSetTimeout::GetTimeout;
+    using CConn_IOStreamSetTimeout::GetTimeout;
 
 protected:
     CConn_IOStreamSetWriteTimeout(const STimeout* timeout)
@@ -320,7 +322,7 @@ inline CConn_IOStream& operator<< (CConn_IOStream& os,
 /////////////////////////////////////////////////////////////////////////////
 ///
 /// This stream exchanges data in a TCP channel, using socket interface.
-/// The endpoint is specified as "host:port" pair.  The maximal
+/// The endpoint is specified as a "host:port" pair.  The maximal
 /// number of connection attempts is given via "max_try".
 /// More details on that: <connect/ncbi_socket_connector.h>.
 ///
@@ -346,11 +348,11 @@ public:
     /// @sa
     ///  CConn_IOStream
     CConn_SocketStream
-    (const string&   host,         /* host to connect to  */
-     unsigned short  port,         /* ... and port number */
-     unsigned short  max_try,      /* number of attempts  */
+    (const string&   host,                        ///< host to connect to
+     unsigned short  port,                        ///< ... and port number
+     unsigned short  max_try,                     ///< number of attempts
      const STimeout* timeout  = kDefaultTimeout,
-     streamsize      buf_size = kConn_DefaultBufSize);
+     size_t          buf_size = kConn_DefaultBufSize);
 
     /// Create a direct connection to "host:port" and pass an initial "data"
     /// block of the specified "size".
@@ -377,9 +379,9 @@ public:
      const void*     data     = 0,                ///< initial data block
      size_t          size     = 0,                ///< size of the data block
      TSOCK_Flags     flags    = fSOCK_LogDefault, ///< see ncbi_socket.h
-     unsigned short  max_try  = 3,                ///< number of attempts
+     unsigned short  max_try  = DEF_CONN_MAX_TRY, ///< number of attempts
      const STimeout* timeout  = kDefaultTimeout,
-     streamsize      buf_size = kConn_DefaultBufSize);
+     size_t          buf_size = kConn_DefaultBufSize);
 
     /// Create a tunneled socket stream connection.
     ///
@@ -389,6 +391,7 @@ public:
     /// host:port                       -- target server
     /// http_proxy_host:http_proxy_port -- proxy server to tunnel via
     /// http_proxy_user:http_proxy_pass -- credentials for the proxy, if needed
+    /// http_proxy_leak                 -- ignore bad proxy and connect direct
     /// timeout                         -- I/O timeout (NB: no kDefaultTimeout)
     /// firewall                        -- if true then look at proxy_server
     /// proxy_server                    -- use as "host" if non-empty and FW
@@ -409,7 +412,7 @@ public:
      const void*         data     = 0,
      size_t              size     = 0,
      TSOCK_Flags         flags    = fSOCK_LogDefault,
-     streamsize          buf_size = kConn_DefaultBufSize);
+     size_t              buf_size = kConn_DefaultBufSize);
 
     /// This variant uses an existing socket "sock" to build a stream upon it.
     /// The caller may retain the ownership of "sock" by passing "if_to_own" as
@@ -429,7 +432,7 @@ public:
     (SOCK            sock,         ///< socket
      EOwnership      if_to_own,    ///< whether stream to own "sock" param
      const STimeout* timeout  = kDefaultTimeout,
-     streamsize      buf_size = kConn_DefaultBufSize);
+     size_t          buf_size = kConn_DefaultBufSize);
 
     /// This variant uses existing CSocket to build a stream upon it.
     /// NOTE:  it revokes all ownership of the "socket"'s internals
@@ -444,12 +447,7 @@ public:
     CConn_SocketStream
     (CSocket&        socket,       ///< socket, underlying SOCK always grabbed
      const STimeout* timeout  = kDefaultTimeout,
-     streamsize      buf_size = kConn_DefaultBufSize);
-
-    /// Return the underlying SOCK object (which is currently in use!)
-    /// Don't call this method unless you absolutely need it and know what
-    /// you are doing;  otherwise, the stream can easily become corrupted.
-    SOCK GetSOCK(void) const { return m_SockPtr ? *m_SockPtr : 0; }
+     size_t          buf_size = kConn_DefaultBufSize);
 
 protected:
     SOCK* m_SockPtr;
@@ -502,14 +500,14 @@ public:
      unsigned short      port         = 0, ///< 0 means default (80 for HTTP)
      THTTP_Flags         flags        = fHTTP_AutoReconnect,
      const STimeout*     timeout      = kDefaultTimeout,
-     streamsize          buf_size     = kConn_DefaultBufSize
+     size_t              buf_size     = kConn_DefaultBufSize
      );
 
     CConn_HttpStream
     (const string&       url,
      THTTP_Flags         flags        = fHTTP_AutoReconnect,
      const STimeout*     timeout      = kDefaultTimeout,
-     streamsize          buf_size     = kConn_DefaultBufSize
+     size_t              buf_size     = kConn_DefaultBufSize
      );
 
     CConn_HttpStream
@@ -518,7 +516,7 @@ public:
      const string&       user_header  = kEmptyStr,
      THTTP_Flags         flags        = fHTTP_AutoReconnect,
      const STimeout*     timeout      = kDefaultTimeout,
-     streamsize          buf_size     = kConn_DefaultBufSize
+     size_t              buf_size     = kConn_DefaultBufSize
      );
 
     CConn_HttpStream
@@ -530,7 +528,7 @@ public:
      FHTTP_Cleanup       cleanup      = 0,
      THTTP_Flags         flags        = fHTTP_AutoReconnect,
      const STimeout*     timeout      = kDefaultTimeout,
-     streamsize          buf_size     = kConn_DefaultBufSize
+     size_t              buf_size     = kConn_DefaultBufSize
      );
 
 private:
@@ -548,7 +546,7 @@ private:
 /// (details: <connect/ncbi_server_info.h>).
 ///
 /// Additional specifications can be passed in the SConnNetInfo structure,
-/// otherwise created by using service name as a registry section
+/// otherwise created by using the service name as a registry section
 /// to obtain the information from (details: <connect/ncbi_connutil.h>).
 ///
 /// Provided "timeout" is set at connection level, and if different from
@@ -565,7 +563,7 @@ public:
      const SConnNetInfo*   net_info    = 0,
      const SSERVICE_Extra* params      = 0,
      const STimeout*       timeout     = kDefaultTimeout,
-     streamsize            buf_size    = kConn_DefaultBufSize);
+     size_t                buf_size    = kConn_DefaultBufSize);
 
     CConn_ServiceStream
     (const string&         service,
@@ -573,7 +571,7 @@ public:
      TSERV_Type            types       = fSERV_Any,
      const SSERVICE_Extra* params      = 0,
      const STimeout*       timeout     = kDefaultTimeout,
-     streamsize            buf_size    = kConn_DefaultBufSize);
+     size_t                buf_size    = kConn_DefaultBufSize);
 
 private:
     // Disable copy constructor and assignment.
@@ -591,14 +589,14 @@ private:
 class NCBI_XCONNECT_EXPORT CConn_MemoryStream : public CConn_IOStream
 {
 public:
-    CConn_MemoryStream(streamsize  buf_size = kConn_DefaultBufSize);
+    CConn_MemoryStream(size_t      buf_size = kConn_DefaultBufSize);
 
     /// Build a stream on top of an NCBI buffer (which in turn
     /// could have been built over a memory area of a specified size).
     /// BUF's ownership is assumed by the stream as specified in "owner".
     CConn_MemoryStream(BUF         buf,
                        EOwnership  owner    = eTakeOwnership,
-                       streamsize  buf_size = kConn_DefaultBufSize);
+                       size_t      buf_size = kConn_DefaultBufSize);
 
     /// Build a stream on top of an existing data area of a specified size.
     /// The contents of the area is what will be read first from the stream.
@@ -615,21 +613,18 @@ public:
     /// while it is still holding the data yet to be read from the stream.
     CConn_MemoryStream(const void* ptr,
                        size_t      size,
-                       EOwnership  owner/**no default for satefy*/,
-                       streamsize  buf_size = kConn_DefaultBufSize);
+                       EOwnership  owner/**no default for safety*/,
+                       size_t      buf_size = kConn_DefaultBufSize);
 
     virtual ~CConn_MemoryStream();
 
     /// The CConnMemoryStream::To* methods allow to obtain unread portion of
-    /// the stream in a single container (a string, a vector, or a character
-    /// array) so that all data is kept in sequential memory locations.
+    /// the stream in a single container (as a string or a vector) so that all
+    /// data is kept in sequential memory locations.
     /// Note that the operation is considered an extraction, so it empties
     /// the stream.
-
-    void    ToString(string*); ///< fill in the data, NULL is not accepted
+    void    ToString(string*);      ///< fill in the data, NULL is not accepted
     void    ToVector(vector<char>*);///< fill in the data, NULL is not accepted
-    NCBI_DEPRECATED
-    char*   ToCStr(void);      ///< '\0'-terminated; free() when done using it 
 
 protected:
     const void* m_Ptr;         ///< pointer to read memory area (if owned)
@@ -658,7 +653,7 @@ public:
      const vector<string>& args,
      CPipe::TCreateFlags   create_flags = 0,
      const STimeout*       timeout      = kDefaultTimeout,
-     streamsize            buf_size     = kConn_DefaultBufSize
+     size_t                buf_size     = kConn_DefaultBufSize
      );
     virtual ~CConn_PipeStream();
 
@@ -690,7 +685,7 @@ public:
     (const string&   pipename,
      size_t          pipebufsize = 0/*default*/,
      const STimeout* timeout     = kDefaultTimeout,
-     streamsize      buf_size    = kConn_DefaultBufSize
+     size_t          buf_size    = kConn_DefaultBufSize
      );
 
 private:
