@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONN_TEST__HPP
 #define CONNECT___NCBI_CONN_TEST__HPP
 
-/* $Id: ncbi_conn_test.hpp 347447 2011-12-16 20:32:53Z lavr $
+/* $Id: ncbi_conn_test.hpp 366508 2012-06-14 18:04:58Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -75,6 +75,9 @@ public:
     CConnTest(const STimeout* timeout = kDefaultTimeout,
               CNcbiOstream* output = 0, SIZE_TYPE width = 72);
 
+    void     SetEmail(const string& email)
+    { m_Email = email; }
+
     void     SetWidth(SIZE_TYPE width = 72)
     { m_Width = width; }
 
@@ -211,6 +214,7 @@ protected:
     const STimeout*       m_Timeout;
     CNcbiOstream*         m_Output;
     SIZE_TYPE             m_Width;
+    string                m_Email;
 
     /// Certain properties of communication as determined by configuration
     bool                  m_HttpProxy;
