@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONNUTIL__H
 #define CONNECT___NCBI_CONNUTIL__H
 
-/* $Id: ncbi_connutil.h 348471 2011-12-29 16:38:20Z vakatov $
+/* $Id: ncbi_connutil.h 366365 2012-06-13 20:36:46Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -123,10 +123,10 @@ typedef unsigned EBURLScheme;
 
 
 typedef enum {
-    eFWMode_Legacy   = 0,  /** Relay, no firewall                           */
-    eFWMode_Adaptive = 1,  /** Regular firewall ports first, then fallback  */
-    eFWMode_Firewall = 2,  /** Regular firewall ports only, no fallback     */
-    eFWMode_Fallback = 3  /** Fallback ports only (w/o trying any regular) */
+    eFWMode_Legacy   = 0,  /**< Relay, no firewall                           */
+    eFWMode_Adaptive = 1,  /**< Regular firewall ports first, then fallback  */
+    eFWMode_Firewall = 2,  /**< Regular firewall ports only, no fallback     */
+    eFWMode_Fallback = 3   /**< Fallback ports only (w/o trying any regular) */
 } EFWMode;
 
 typedef unsigned EBFWMode;
@@ -299,7 +299,7 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ ConnNetInfo_Boolean
  );
 
 
-/* This function to fill out the "*info" structure using
+/* This function fills out the "*info" structure using
  * registry entries named (see above) in macros REG_CONN_<NAME>:
  *
  *  -- INFO FIELD --  ----- NAME -----  ---------- REMARKS/EXAMPLES ---------
