@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_SOCKET_CONNECTOR__H
 #define CONNECT___NCBI_SOCKET_CONNECTOR__H
 
-/* $Id: ncbi_socket_connector.h 217171 2010-12-14 19:38:35Z lavr $
+/* $Id: ncbi_socket_connector.h 351841 2012-02-01 18:30:40Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -89,8 +89,8 @@ extern NCBI_XCONNECT_EXPORT CONNECTOR SOCK_CreateConnector
  * sock, and that the delayed connection open will always result in eIO_Closed.
  */
 extern NCBI_XCONNECT_EXPORT CONNECTOR SOCK_CreateConnectorOnTopEx
-(SOCK                   sock,    /* existing socket handle (may not be NULL) */
- unsigned short/*bool*/ own_sock,/* non-zero if connector will own "sock"    */
+(SOCK                   sock,    /* existing socket handle (NULL is allowed) */
+ unsigned short/*bool*/ own_sock,/* non-zero if connector is to own "sock"   */
  const char*            hostport /* connection point name taken verbatim     */
  );
 
@@ -98,8 +98,8 @@ extern NCBI_XCONNECT_EXPORT CONNECTOR SOCK_CreateConnectorOnTopEx
 /* Equivalent to SOCK_CreateConnectorOnTopEx(sock, own_sock, 0)
  */
 extern NCBI_XCONNECT_EXPORT CONNECTOR SOCK_CreateConnectorOnTop
-(SOCK                   sock,    /* existing socket handle (may not be NULL) */
- unsigned short/*bool*/ own_sock /* non-zero if connector will own "sock"    */
+(SOCK                   sock,    /* existing socket handle (NULL is allowed) */
+ unsigned short/*bool*/ own_sock /* non-zero if connector is to own "sock"   */
  );
 
 
