@@ -1,7 +1,7 @@
 #ifndef CONNECT___SERVER__HPP
 #define CONNECT___SERVER__HPP
 
-/* $Id: server.hpp 341365 2011-10-19 14:10:03Z lavr $
+/* $Id: server.hpp 364729 2012-05-30 15:10:17Z ivanovp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,7 +34,7 @@
 /// Framework to create multithreaded network servers with thread-per-request
 /// scheduling.
 
-#include <util/thread_pool.hpp>
+#include <connect/impl/thread_pool_for_server.hpp>
 #include <connect/ncbi_conn_stream.hpp>
 #include <connect/ncbi_conn_exception.hpp>
 #include <connect/ncbi_socket.hpp>
@@ -54,7 +54,6 @@ class  IServer_ConnectionBase;
 struct SServer_Parameters;
 class  CServer_ConnectionPool;
 class  CServer_Connection;
-class  CStdPoolOfThreads;
 
 
 /// Extended copy of the type EIO_Event allowing to distinguish between
@@ -66,7 +65,8 @@ enum EServIO_Event {
     eServIO_ReadWrite   = 0x03, /**< eIO_Read | eIO_Write */
     eServIO_ClientClose = 0x04,
     eServIO_OurClose    = 0x08,
-    eServIO_Inactivity  = 0x10
+    eServIO_Inactivity  = 0x10,
+    eServIO_Delete      = 0x20
 };
 
 
@@ -146,6 +146,11 @@ public:
     /// Should be called if IsReadyToProcess() for some connection handler
     /// became true.
     void WakeUpPollCycle(void);
+    /// Set custom suffix to use on all threads in the server's pool.
+    /// Value can be set only before call to Run(), any change of the value
+    /// after call to Run() will be ignored.
+    void SetCustomThreadSuffix(const string& suffix)
+    { m_ThreadSuffix = suffix; }
 
 protected:
     /// Initialize the server
@@ -172,17 +177,16 @@ protected:
     virtual bool ShutdownRequested(void) { return false; }
 
 private:
-    void CreateRequest(IServer_ConnectionBase* conn_base,
-                       EServIO_Event event,
-                       const STimeout* timeout);
+    void x_AddRequests(const vector<CRef<CStdRequest> >& reqs);
     void x_DoRun(void);
 
     friend class CNetCacheServer;
-    CStdPoolOfThreads* GetThreadPool(void) { return m_ThreadPool.get(); }
+    CPoolOfThreads_ForServer* GetThreadPool(void) { return m_ThreadPool.get(); }
 
-    auto_ptr<CStdPoolOfThreads> m_ThreadPool;
+    auto_ptr<CPoolOfThreads_ForServer> m_ThreadPool;
     SServer_Parameters*         m_Parameters;
     CServer_ConnectionPool*     m_ConnectionPool;
+    string m_ThreadSuffix;
 };
 
 
