@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBIARGS__HPP
 #define CORELIB___NCBIARGS__HPP
 
-/*  $Id: ncbiargs.hpp 348257 2011-12-27 16:34:56Z gouriano $
+/*  $Id: ncbiargs.hpp 360817 2012-04-26 15:09:00Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -376,6 +376,22 @@ public:
     /// Remove all arguments
     void Reset(void);
 
+    /// Get current command
+    /// @sa CCommandArgDescriptions
+    string GetCommand(void) const
+    {
+        return m_Command;
+    }
+
+protected:
+    /// Set current command
+    /// @sa CCommandArgDescriptions
+    CArgs* SetCommand(const string& command)
+    {
+        m_Command = command;
+        return this;
+    }
+
 private:
     typedef set< CRef<CArgValue> >  TArgs;   ///< Type for arguments
     typedef TArgs::iterator         TArgsI;  ///< Type for iterator
@@ -383,10 +399,12 @@ private:
 
     TArgs  m_Args;    ///< Assoc. map of arguments' name/value
     size_t m_nExtra;  ///< Cached # of unnamed positional arguments 
+    string m_Command;
 
     /// Find argument value with name "name".
     TArgsCI x_Find(const string& name) const;
     TArgsI  x_Find(const string& name);
+    friend class CCommandArgDescriptions;
 };
 
 
@@ -541,7 +559,10 @@ public:
         fOptionalSeparatorAllowConflict = (1 << 9),
         
         /// Require '=' separator
-        fMandatorySeparator = (1 << 7)
+        fMandatorySeparator = (1 << 7),
+
+        /// Hide it in Usage
+        fHidden = (1 << 10)
     };
     typedef unsigned int TFlags;  ///< Binary OR of "EFlags"
 
@@ -631,6 +652,30 @@ public:
                  bool          set_value = true ///< Is value set or not?
                 );
 
+
+    /// Add description of mandatory opening positional argument.
+    ///
+    /// Mandatory opening argument has the following syntax:
+    ///   arg_pos := <value>
+    ///
+    /// NOTE:
+    ///   In command line, mandatory opening arguments must go first,
+    ///   before any other arguments; their order is defined by the order
+    ///   in which they were described and added into CArgDescriptions.
+    ///
+    /// Will throw exception CArgException if:
+    ///  - description with name "name" already exists
+    ///  - "name" contains symbols other than {alnum, '-', '_'}
+    ///  - "name" starts with more than one '-'
+    ///  - "flags" are inconsistent with "type"
+    ///
+    /// Any argument can be later referenced using its unique name "name".
+    void AddOpening(const string& name,     ///< Name of argument
+                    const string& comment,  ///< Argument description
+                    EType         type,     ///< Argument type
+                    TFlags        flags = 0 ///< Optional file flags
+                    );
+
     /// Add description for mandatory postional argument.
     ///
     /// Mandatory positional argument has the following syntax:
@@ -834,7 +879,7 @@ public:
     ///
     /// @param out
     ///   Print into this output stream
-    void PrintUsageXml(CNcbiOstream& out) const;
+    virtual void PrintUsageXml(CNcbiOstream& out) const;
 
     /// Verify if argument "name" is spelled correctly.
     ///
@@ -872,7 +917,8 @@ private:
 private:
     EArgSetType  m_ArgsType;     ///< Type of arguments
     TArgs        m_Args;         ///< Assoc.map of arguments' name/descr
-    TPosArgs     m_PosArgs;    ///< Pos. args, ordered by position in cmd.-line
+    TPosArgs     m_PosArgs;      ///< Pos. args, ordered by position in cmd.-line
+    TPosArgs     m_OpeningArgs;  ///< Opening args, ordered by position in cmd.-line
     TKeyFlagArgs m_KeyFlagArgs;  ///< Key/flag args, in order of insertion
     string       m_NoSeparator;  ///< Arguments allowed to use no separator
     unsigned     m_nExtra;       ///> # of mandatory extra args
@@ -883,12 +929,14 @@ private:
     TDependencies      m_Dependencies;   ///< Arguments' dependencies
 
     // Extra USAGE info
+protected:
     string    m_UsageName;         ///< Program name
     string    m_UsageDescription;  ///< Program description
     bool      m_UsageSortArgs;     ///< Sort alphabetically on usage printout
     SIZE_TYPE m_UsageWidth;        ///< Maximum length of a usage line
     bool      m_AutoHelp;          ///< Special flag "-h" activated
     bool      m_UsageIfNoArgs;     ///< Print usage and exit if no args passed
+private:
 
     CRef<CArgErrorHandler> m_ErrorHandler; ///< Global error handler or NULL
 
@@ -965,6 +1013,32 @@ private:
     /// Returns TRUE if parameter supports multiple arguments
     bool x_IsMultiArg(const string& name) const;
 
+protected:
+// PrintUsage helpers
+    class CPrintUsage
+    {
+    public:
+        CPrintUsage(const CArgDescriptions& desc);
+        ~CPrintUsage();
+        void AddSynopsis(list<string>& arr, const string& intro, const string& prefix) const;
+        void AddDescription(list<string>& arr) const;
+        void AddCommandDescription(list<string>& arr, const string& cmd, 
+            const map<string,string>* aliases, size_t max_cmd_len, bool detailed) const;
+        void AddDetails(list<string>& arr) const;
+    private:
+        const CArgDescriptions& m_desc;
+        list<const CArgDesc*> m_args;
+    };
+    class CPrintUsageXml
+    {
+    public:
+        CPrintUsageXml(const CArgDescriptions& desc, CNcbiOstream& out);
+        ~CPrintUsageXml();
+        void PrintArguments(const CArgDescriptions& desc) const;
+    private:
+        CNcbiOstream& m_out;
+    };
+
 public:
     /// Create parsed arguments in CArgs object.
     ///
@@ -1021,7 +1095,7 @@ public:
     }
 
     /// Parse command-line arguments 'argv'
-    CArgs* CreateArgs(const CNcbiArguments& argv) const;
+    virtual CArgs* CreateArgs(const CNcbiArguments& argv) const;
 
     /// Convert argument map (key-value pairs) into arguments in accordance
     /// with the argument descriptions
@@ -1068,6 +1142,92 @@ public:
     }
 };
 
+/////////////////////////////////////////////////////////////////////////////
+///
+/// CCommandArgDescriptions --
+///
+/// Container for several CArgDescriptions objects.
+///
+/// Sometimes, it is convenient to use a command line tool as follows:
+///    tool <command> [options] [args]
+/// Here, <command> is an alphanumeric string,
+/// and options and arguments are different for each command.
+/// With this mechanism, it is possible to describe arguments for
+/// each command separately. At run time, argument parser will choose
+/// proper CArgDescriptions object based on the value of the first argument.
+
+class NCBI_XNCBI_EXPORT CCommandArgDescriptions : public CArgDescriptions
+{
+public:
+
+    /// Command presence requirement
+    enum ECommandPresence {
+        eCommandOptional,    ///< Command is not necessary
+        eCommandMandatory    ///< Nonempty command is required
+    };
+
+    /// Constructor.
+    ///
+    /// If "auto_help" is passed TRUE, then a special flag "-h" will be added
+    /// to the list of accepted arguments. Passing "-h" in the command line
+    /// will printout USAGE and ignore all other passed arguments.
+    /// Error handler is used to process errors when parsing arguments.
+    /// If not set the default handler is used.
+    CCommandArgDescriptions(bool auto_help = true,
+                          CArgErrorHandler* err_handler = 0,
+                          ECommandPresence cmd_req = eCommandMandatory);
+    /// Destructor.
+    virtual ~CCommandArgDescriptions(void);
+
+    /// Set current command group name. When printing help,
+    /// commands will be arranged by group name.
+    void SetCurrentCommandGroup(const string& group);
+
+    /// Add command argument descriptions
+    ///
+    /// @param cmd
+    ///    Command string
+    /// @param description
+    ///    Argument description
+    /// @param alias
+    ///    Alternative name of the command
+    void AddCommand(const string& cmd, CArgDescriptions* description,
+                    const string& alias = kEmptyStr);
+
+    /// Parse command-line arguments 'argv'
+    virtual CArgs* CreateArgs(const CNcbiArguments& argv) const;
+
+    /// Print usage message to end of specified string.
+    ///
+    /// Printout USAGE and append to the string "str" using  provided
+    /// argument descriptions and usage context.
+    /// @return
+    ///   Appended "str"
+    virtual string& PrintUsage(string& str, bool detailed = false) const;
+
+    /// Print argument description in XML format
+    ///
+    /// @param out
+    ///   Print into this output stream
+    virtual void PrintUsageXml(CNcbiOstream& out) const;
+
+private:
+    // not allowed
+    void SetArgsType(EArgSetType args_type) {}
+
+    size_t x_GetCommandGroupIndex(const string& group);
+    string x_IdentifyCommand(const string& command) const;
+
+    typedef map<string, AutoPtr<CArgDescriptions> > TDescriptions;
+    ECommandPresence m_Cmd_req;
+    TDescriptions m_Description;    ///< command to ArgDescriptions
+    map<string, size_t > m_Groups;  ///< command to group #
+    map<string,string> m_Aliases;   ///< command to alias; one alias only
+    vector<string> m_CmdGroups;     ///< group names, and order
+    size_t m_CurrentGroup;          ///< current group #
+    mutable string m_Command;       ///< current command
+};
+
 
 
 /////////////////////////////////////////////////////////////////////////////
