@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBISTRE__HPP
 #define CORELIB___NCBISTRE__HPP
 
-/*  $Id: ncbistre.hpp 301000 2011-06-06 16:55:36Z gouriano $
+/*  $Id: ncbistre.hpp 372997 2012-08-23 20:51:17Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,8 +36,8 @@
 /// "old" C++ stream libraries.
 
 
-#include <corelib/ncbistl.hpp>
 #include <corelib/ncbitype.h>
+#include <corelib/ncbistl.hpp>
 
 
 /// Determine which iostream library to use, include appropriate
@@ -420,24 +420,27 @@ extern const char* Endl(void);
 
 /// Read from "is" to "str" up to the delimiter symbol "delim" (or EOF)
 NCBI_XNCBI_EXPORT
-extern CNcbiIstream& NcbiGetline(CNcbiIstream& is, string& str, char delim);
+extern CNcbiIstream& NcbiGetline(CNcbiIstream& is, string& str, char delim,
+                                 string::size_type* count = NULL);
 
 /// Read from "is" to "str" up to any symbol contained within "delims" (or EOF)
 NCBI_XNCBI_EXPORT
 extern CNcbiIstream& NcbiGetline(CNcbiIstream& is, string& str,
-                                 const string& delims);
+                                 const string& delims,
+                                 string::size_type* count = NULL);
 
 /// Read from "is" to "str" the next line 
 /// (taking into account platform specifics of End-of-Line)
 NCBI_XNCBI_EXPORT
-extern CNcbiIstream& NcbiGetlineEOL(CNcbiIstream& is, string& str);
+extern CNcbiIstream& NcbiGetlineEOL(CNcbiIstream& is, string& str,
+                                    string::size_type* count = NULL);
 
 
 /// Copy entire contents of stream "is" into "os".
 /// @return
 /// "true" if the operation was successful, "is" was read entirely,
 /// and all of its contents had been written to "os";
-/// "false" if either extraction from "is" or insertion into "os" had failed.
+/// "false" if either extraction from "is" or insertion into "os" have failed.
 ///
 /// Note that upon successful completion, is.eof() may not always be true.
 /// The call may throw exceptions only if they are enabled on the
@@ -449,7 +452,7 @@ extern CNcbiIstream& NcbiGetlineEOL(CNcbiIstream& is, string& str);
 ///
 /// NOTE that the call (as well as the mentioned STL counterpart) provides
 /// only a mechanism of delivering data to the destination "os" stream(buf);
-/// and the successful result code does not, generally, guarantee that the
+/// and the successful return result does not generally guarantee that the
 /// data have yet reached the physical destination.  Other "os"-specific API
 /// must be performed to assure the data integrity at the receiving device;
 /// such as checking for errors after doing a "close()" on an ofstream "os".
