@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBITHR__HPP
 #define CORELIB___NCBITHR__HPP
 
-/*  $Id: ncbithr.hpp 337464 2011-09-12 19:56:33Z lavr $
+/*  $Id: ncbithr.hpp 355547 2012-03-06 20:39:08Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -43,12 +43,10 @@
 ///
 
 
-#include <corelib/ncbiobj.hpp>
 #include <corelib/ncbimtx.hpp>
 #include <corelib/ncbi_process.hpp>
 #include <corelib/ncbi_safe_static.hpp>
 #include <list>
-#include <set>
 
 
 BEGIN_NCBI_SCOPE
