@@ -1,7 +1,7 @@
 #ifndef CORELIB__NCBITIME__HPP
 #define CORELIB__NCBITIME__HPP
 
-/*  $Id: ncbitime.hpp 342257 2011-10-26 17:06:46Z lavr $
+/*  $Id: ncbitime.hpp 366015 2012-06-11 14:34:08Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -467,6 +467,15 @@ public:
     ///   Time in time_t format.
     time_t GetTimeT(void) const;
 
+    /// Get current GMT time in time_t format (with nanoseconds).
+    ///
+    /// @param sec
+    ///   The function return the number of seconds elapsed since
+    ///   midnight (00:00:00), January 1, 1970.
+    /// @param nanosec
+    ///   Number of nanoseconds (0, if not possible to get).
+    static void GetCurrentTimeT(time_t *sec, long *nanosec = 0);
+
     /// Set time using "struct tm" time value.
     ///
     /// @param t
@@ -1688,6 +1697,7 @@ public:
     NCBI_DEPRECATED_CTOR(CStopWatch(bool start));
 
     /// Start the timer.
+    /// Do nothing if already started.
     void Start(void);
 
     /// Return time elapsed since first Start() or last Restart() call
@@ -2477,6 +2487,9 @@ CStopWatch::CStopWatch(EStart state)
 inline
 void CStopWatch::Start(void)
 {
+    if ( m_State == eStart ) {
+        return;
+    }
     m_Start = GetTimeMark();
     m_State = eStart;
 }
