@@ -1,7 +1,7 @@
 #ifndef CORELIB___REQUEST_CTX__HPP
 #define CORELIB___REQUEST_CTX__HPP
 
-/*  $Id: request_ctx.hpp 344942 2011-11-18 22:58:55Z grichenk $
+/*  $Id: request_ctx.hpp 372362 2012-08-16 18:59:00Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -161,6 +161,32 @@ public:
     /// Get default auto-increment flag.
     static bool GetDefaultAutoIncRequestIDOnPost(void);
 
+    /// Session ID format
+    enum ESessionIDFormat {
+        eSID_Ncbi,     ///< Strict NCBI format: (UID:16)_(RqID:4)SID
+        eSID_Standard, ///< Alpanum, underscore, -.:@, (default)
+        eSID_Other     ///< Any other format
+    };
+    /// Session ID error actions
+    enum EOnBadSessionID {
+        eOnBadSID_Allow,           ///< Don't validate session id.
+        eOnBadSID_AllowAndReport,  ///< Accept but show warning (default).
+        eOnBadSID_Ignore,          ///< Ignore bad session id.
+        eOnBadSID_IgnoreAndReport, ///< Ignore and show warning.
+        eOnBadSID_Throw            ///< Throw on bad session id.
+    };
+
+    /// Check if session id fits the allowed format.
+    static bool IsValidSessionID(const string& session_id);
+
+    /// Get/set session id error action.
+    static EOnBadSessionID GetBadSessionIDAction(void);
+    static void SetBadSessionIDAction(EOnBadSessionID action);
+
+    /// Get/set allowed session id format.
+    static ESessionIDFormat GetAllowedSessionIDFormat(void);
+    static void SetAllowedSessionIDFormat(ESessionIDFormat fmt);
+
 private:
     // Prohibit copying
     CRequestContext(const CRequestContext&);
@@ -213,6 +239,25 @@ private:
 };
 
 
+class NCBI_XNCBI_EXPORT CRequestContextException :
+    EXCEPTION_VIRTUAL_BASE public CException
+{
+public:
+    /// Error types that  CRequestContext can generate.
+    ///
+    /// These generic error conditions can occur for corelib applications.
+    enum EErrCode {
+        eBadSession    ///< Invalid session id
+    };
+
+    /// Translate from the error code value to its string representation.
+    virtual const char* GetErrCodeString(void) const;
+
+    // Standard exception boilerplate code.
+    NCBI_EXCEPTION_DEFAULT(CRequestContextException, CException);
+};
+
+
 inline
 CRequestContext::TCount CRequestContext::GetRequestID(void) const
 {
@@ -283,13 +328,6 @@ const string& CRequestContext::GetEncodedSessionID(void) const
         : GetDiagContext().GetEncodedSessionID();
 }
 
-inline
-void CRequestContext::SetSessionID(const string& session)
-{
-    x_SetProp(eProp_SessionID);
-    m_SessionID.SetString(session);
-}
-
 inline
 bool CRequestContext::IsSetSessionID(void) const
 {
