@@ -1,7 +1,7 @@
 #ifndef CORELIB___RWSTREAM__HPP
 #define CORELIB___RWSTREAM__HPP
 
-/*  $Id: rwstream.hpp 347186 2011-12-14 19:07:56Z lavr $
+/*  $Id: rwstream.hpp 354815 2012-02-29 20:10:49Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -82,7 +82,8 @@ BEGIN_NCBI_SCOPE
 /// respective IReader/IWriter API explicitly.
 
 
-/// Reader-based stream; @sa IReader
+/// Reader-based stream.
+/// @sa IReader
 ///
 /// @param buf_size
 ///     specifies the number of bytes for internal I/O buffer, entirely used
@@ -118,7 +119,8 @@ private:
 };
 
 
-/// Writer-based stream; @sa IWriter
+/// Writer-based stream.
+/// @sa IWriter
 ///
 /// @param buf_size
 ///     specifies the number of bytes for internal I/O buffer, entirely used
@@ -154,7 +156,8 @@ private:
 };
 
 
-/// Reader-writer based stream; @sa IReaderWriter
+/// Reader-writer based stream.
+/// @sa IReaderWriter, IReader, IWriter
 ///
 /// @param buf_size
 ///     specifies the number of bytes for internal I/O buffer,
@@ -186,6 +189,16 @@ public:
         init(&m_Sb);
     }
 
+    CRWStream(IReader*             r,
+              IWriter*             w,
+              streamsize           buf_size = 0,
+              CT_CHAR_TYPE*        buf      = 0,
+              CRWStreambuf::TFlags flags    = 0)
+        : CNcbiIostream(0), m_Sb(r, w, buf_size, buf, flags)
+    {
+        init(&m_Sb);
+    }
+
 private:
     CRWStreambuf m_Sb;
 };
