@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER_IMPL___DBAPI_IMPL_CONTEXT__HPP
 #define DBAPI_DRIVER_IMPL___DBAPI_IMPL_CONTEXT__HPP
 
-/* $Id: dbapi_impl_context.hpp 333164 2011-09-02 16:04:31Z ivanovp $
+/* $Id: dbapi_impl_context.hpp 369361 2012-07-18 17:39:07Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -152,6 +152,8 @@ public:
     virtual unsigned int NofConnections(const string& srv_name  = "",
                                         const string& pool_name = "")
         const;
+    virtual unsigned int NofConnections(const TSvrRef& svr_ref,
+                                        const string& pool_name = "") const;
 
     /// Add message handler "h" to process 'context-wide' (not bound
     /// to any particular connection) error messages.
