@@ -1,7 +1,7 @@
 #ifndef OBJMGR_IMPL_SCOPE_IMPL__HPP
 #define OBJMGR_IMPL_SCOPE_IMPL__HPP
 
-/*  $Id: scope_impl.hpp 352026 2012-02-02 18:21:55Z vasilche $
+/*  $Id: scope_impl.hpp 366487 2012-06-14 16:53:31Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -361,6 +361,19 @@ public:
    
     bool IsTransactionActive() const;
 
+    TSeqPos GetSequenceLength(const CSeq_id_Handle& id, bool force_load);
+    CSeq_inst::TMol GetSequenceType(const CSeq_id_Handle& id, bool force_load);
+
+    // Get a set of bioseq lengths
+    typedef vector<TSeqPos> TSequenceLengths;
+    void GetSequenceLengths(TSequenceLengths& ret,
+                            const TIds& idhs, bool force_load);
+    // Get a set of bioseq types
+    typedef vector<CSeq_inst::TMol> TSequenceTypes;
+    void GetSequenceTypes(TSequenceTypes& ret,
+                          const TIds& idhs, bool force_load);
+
+
 private:
     // constructor/destructor visible from CScope
     CScope_Impl(CObjectManager& objmgr);
