@@ -1,7 +1,7 @@
 #ifndef OBJECTS_OBJMGR_IMPL___TSE_INFO__HPP
 #define OBJECTS_OBJMGR_IMPL___TSE_INFO__HPP
 
-/*  $Id: tse_info.hpp 323253 2011-07-26 16:52:15Z vasilche $
+/*  $Id: tse_info.hpp 369613 2012-07-20 18:37:00Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -80,6 +80,7 @@ class CTSE_Default_Assigner;
 class IEditSaver;
 
 class CSeq_annot_Finder;
+class CMasterSeqSegments;
 
 ////////////////////////////////////////////////////////////////////
 //
@@ -323,6 +324,7 @@ public:
     SSeqMatch_TSE GetSeqMatch(const CSeq_id_Handle& id) const;
 
     CConstRef<CBioseq_Info> GetSegSetMaster(void) const;
+    CConstRef<CMasterSeqSegments> GetMasterSeqSegments(void) const;
 
     // fill ids with all Bioseqs Seq-ids from this TSE
     // the result will be sorted and contain no duplicates
@@ -693,6 +695,9 @@ private:
     typedef map<CBioObjectId, CTSE_Info_Object*> TBioObjects;
     TBioObjects m_BioObjects;
 
+    mutable bool m_MasterSeqSegmentsLoaded;
+    mutable CConstRef<CMasterSeqSegments> m_MasterSeqSegments;
+
 private:
     // Hide copy methods
     CTSE_Info(const CTSE_Info&);
