@@ -1,7 +1,7 @@
 #ifndef OBJMGR_SCOPE__HPP
 #define OBJMGR_SCOPE__HPP
 
-/*  $Id: scope.hpp 352026 2012-02-02 18:21:55Z vasilche $
+/*  $Id: scope.hpp 366487 2012-06-14 16:53:31Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -380,6 +380,18 @@ public:
     int GetTaxId(const CSeq_id_Handle& idh,
                  EForceLoad force_load = eNoForceLoad);
 
+    // returns kInvalidSeqPos if sequence is not known
+    TSeqPos GetSequenceLength(const CSeq_id& id,
+                              EForceLoad force_load = eNoForceLoad);
+    TSeqPos GetSequenceLength(const CSeq_id_Handle& id,
+                              EForceLoad force_load = eNoForceLoad);
+
+    // returns CSeq_inst::eMol_not_set if sequence is not known
+    CSeq_inst::TMol GetSequenceType(const CSeq_id& id,
+                                    EForceLoad force_load = eNoForceLoad);
+    CSeq_inst::TMol GetSequenceType(const CSeq_id_Handle& id,
+                                    EForceLoad force_load = eNoForceLoad);
+
     /// Bulk retrieval methods
     typedef vector<CSeq_id_Handle> TSeq_id_Handles;
     TSeq_id_Handles GetAccVers(const TSeq_id_Handles& idhs,
@@ -406,6 +418,19 @@ public:
                    const TSeq_id_Handles& idhs,
                    EForceLoad force_load = eNoForceLoad);
 
+    typedef vector<TSeqPos> TSequenceLengths;
+    TSequenceLengths GetSequenceLengths(const TSeq_id_Handles& idhs,
+                                        EForceLoad force_load = eNoForceLoad);
+    void GetSequenceLengths(TSequenceLengths* results,
+                            const TSeq_id_Handles& idhs,
+                            EForceLoad force_load = eNoForceLoad);
+    typedef vector<CSeq_inst::TMol> TSequenceTypes;
+    TSequenceTypes GetSequenceTypes(const TSeq_id_Handles& idhs,
+                                    EForceLoad force_load = eNoForceLoad);
+    void GetSequenceTypes(TSequenceTypes* results,
+                          const TSeq_id_Handles& idhs,
+                          EForceLoad force_load = eNoForceLoad);
+
     /// Get bioseq synonyms, resolving to the bioseq in this scope.
     CConstRef<CSynonymsSet> GetSynonyms(const CSeq_id&        id);
 
