@@ -1,7 +1,7 @@
 #ifndef OBJECTS_OBJMGR___SEQ_MAP__HPP
 #define OBJECTS_OBJMGR___SEQ_MAP__HPP
 
-/*  $Id: seq_map.hpp 210309 2010-11-03 12:14:55Z vasilche $
+/*  $Id: seq_map.hpp 370666 2012-07-31 20:32:41Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -259,9 +259,6 @@ protected:
         bool                 m_RefMinusStrand;
         TSeqPos              m_RefPosition;
         CConstRef<CObject>   m_RefObject; // CSeq_data, CSeqMap, CSeq_id
-
-        typedef list<TSeqPos>::iterator TList0_I;
-        TList0_I m_Iterator;
     };
 
     class SPosLessSegment
@@ -342,7 +339,7 @@ protected:
     bool x_UpdateSeq_inst(CSeq_inst& inst);
     virtual bool x_DoUpdateSeq_inst(CSeq_inst& inst);
 
-    CBioseq_Handle x_GetBioseqHandle(const CSegment& seg, CScope* scope) const;
+    const CBioseq_Info& x_GetBioseqInfo(const CSegment& seg, CScope* scope) const;
 
     CConstRef<CSeqMap> x_GetSubSeqMap(const CSegment& seg, CScope* scope,
                                       bool resolveExternal = false) const;
