@@ -1,7 +1,7 @@
 #ifndef SEQUENCE__HPP
 #define SEQUENCE__HPP
 
-/*  $Id: sequence.hpp 282526 2011-05-12 15:04:18Z kornbluh $
+/*  $Id: sequence.hpp 370666 2012-07-31 20:32:41Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -43,7 +43,7 @@
 #include <objects/seqloc/Na_strand.hpp>
 #include <objects/seqloc/Seq_interval.hpp>
 #include <objects/seqloc/Seq_loc.hpp>
-#include <objects/seq/seq_id_handle.hpp>
+#include <objects/seq/seq_id_mapper.hpp>
 #include <util/strsearch.hpp>
 #include <objmgr/feat_ci.hpp>
 #include <objmgr/scope.hpp>
@@ -266,9 +266,9 @@ enum EGetTitleFlags {
 };
 typedef int TGetTitleFlags;
 
-NCBI_XOBJUTIL_EXPORT
+NCBI_DEPRECATED NCBI_XOBJUTIL_EXPORT
 string GetTitle(const CBioseq_Handle& hnd, TGetTitleFlags flags = 0);
-NCBI_XOBJUTIL_EXPORT
+NCBI_DEPRECATED NCBI_XOBJUTIL_EXPORT
 bool GetTitle(const CBioseq& seq, string* title_ptr, TGetTitleFlags flags = 0);
 
 /* @} */
@@ -722,20 +722,31 @@ END_SCOPE(sequence)
 class NCBI_XOBJUTIL_EXPORT CFastaOstream {
 public:
     enum EFlags {
-        fAssembleParts   = 0x01, ///< assemble FAR delta sequences
-        fInstantiateGaps = 0x02, ///< with Ns or Xs as appropriate, vs. one -
-        fSuppressRange   = 0x04, ///< never include location details in defline
-        fReverseStrand   = 0x08, ///< flip the (implicit) location
-        fKeepGTSigns     = 0x10, ///< don't convert '>' to '_' in title
-        fMapMasksUp      = 0x20, ///< honor masks specified at a lower level
-        fMapMasksDown    = 0x40, ///< honor masks specified at a higher level
-        fNoExpensiveOps  = 0x80, ///< don't try too hard to find titles
+        fAssembleParts   = 0x001, ///< assemble FAR delta sequences; on by dflt
+        fInstantiateGaps = 0x002, ///< honor specifed gap mode; on by default
+        fSuppressRange   = 0x004, ///< never include location details in defline
+        fReverseStrand   = 0x008, ///< flip the (implicit) location
+        fKeepGTSigns     = 0x010, ///< don't convert '>' to '_' in title
+        fMapMasksUp      = 0x020, ///< honor masks specified at a lower level
+        fMapMasksDown    = 0x040, ///< honor masks specified at a higher level
+        fNoExpensiveOps  = 0x080, ///< don't try too hard to find titles
+        fShowModifiers   = 0x100, ///< show key-value pair modifiers (e.g. "[organism=Homo sapiens]")
         // historically misnamed as eFlagName
         eAssembleParts   = fAssembleParts,
         eInstantiateGaps = fInstantiateGaps
     };
     typedef int TFlags; ///< binary OR of EFlags
 
+    /// How to represent gaps with fInstantiateGaps enabled, as it is
+    /// by default.  (Disabling fInstantiateGaps is equivalent to
+    /// requesting eGM_one_dash.)
+    enum EGapMode {
+        eGM_one_dash, ///< A single dash, followed by a line break.
+        eGM_dashes,   ///< Multiple inline dashes.
+        eGM_letters,  ///< Multiple inline Ns or Xs as appropriate (default).
+        eGM_count     ///< >?N or >?unk100, as appropriate.
+    };
+
     CFastaOstream(CNcbiOstream& out);
     virtual ~CFastaOstream();
 
@@ -785,6 +796,8 @@ public:
     void    SetAllFlags(TFlags flags)  { m_Flags = flags;  }
     void    SetFlag    (EFlags flag)   { m_Flags |=  flag; }
     void    ResetFlag  (EFlags flag)   { m_Flags &= ~flag; }
+    void    SetGapMode (EGapMode mode) { m_GapMode = mode; }
+    EGapMode GetGapMode(void) const    { return m_GapMode; }
 
 private:
     CNcbiOstream&       m_Out;
@@ -793,12 +806,20 @@ private:
     CConstRef<CSeq_loc> m_HardMask;
     TSeqPos             m_Width;
     TFlags              m_Flags;
+    EGapMode            m_GapMode;
+    TSeq_id_HandleSet   m_PreviousWholeIds;
 
-    void x_WriteSeqIds  (const CBioseq& bioseq,
-                         const CSeq_loc* location);
-    void x_WriteSeqTitle(const CBioseq& bioseq,
-                         CScope* scope,
-                         const string& custom_title);
+    void x_WriteSeqIds    ( const CBioseq& bioseq,
+                            const CSeq_loc* location);
+    void x_WriteModifiers ( const CBioseq_Handle & handle );
+    void x_WriteSeqTitle  ( const CBioseq& bioseq,
+                            CScope* scope,
+                            const string& custom_title);
+
+    void x_PrintStringModIfNotDup(
+        bool *seen, const CTempString & key, const CTempString & value );
+    void x_PrintIntModIfNotDup(
+        bool *seen, const CTempString & key, const int value );
 
     CConstRef<CSeq_loc> x_MapMask(CSeq_loc_Mapper& mapper, const CSeq_loc& mask,
                                   const CSeq_id* base_seq_id, CScope* scope);
