@@ -1,4 +1,4 @@
-/*  $Id: showalign.hpp 347179 2011-12-14 18:51:54Z jianye $
+/*  $Id: showalign.hpp 372466 2012-08-17 17:40:12Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,9 +50,11 @@
 
 #include <objmgr/object_manager.hpp>
 
-#include <cgi/cgictx.hpp>
 
 BEGIN_NCBI_SCOPE
+
+class CCgiContext;
+
 BEGIN_SCOPE(align_format)
 
 /// Auxiliary type to embed a CConstRef<objects::CSeq_id> in STL containers that require
@@ -238,7 +240,7 @@ class NCBI_ALIGN_FORMAT_EXPORT CDisplaySeqalign {
                      const char* matrix_name = BLAST_DEFAULT_MATRIX);
     
     /// Destructor
-    ~CDisplaySeqalign();
+    virtual ~CDisplaySeqalign();
     
     ///call this to display seqalign
     ///@param out: stream for display
@@ -403,6 +405,7 @@ class NCBI_ALIGN_FORMAT_EXPORT CDisplaySeqalign {
     void SetPreComputedResID(string preComputedResID) {
         m_PreComputedResID = preComputedResID;
     }
+
     /// static functions
     ///Need to call this if the seqalign is stdseg or dendiag for ungapped
     ///blast alignment display as each stdseg ro dendiag is a distinct
@@ -413,6 +416,14 @@ class NCBI_ALIGN_FORMAT_EXPORT CDisplaySeqalign {
     static CRef < objects::CSeq_align_set >
     PrepareBlastUngappedSeqalign(const objects::CSeq_align_set & alnset);
 
+    /// static functions
+    /// same as PrepareBlastUngappedSeqalign, but process seg scores uniformly even if there is only a single seg.
+    ///@param alnset: input alnset
+    ///@return processed alnset
+    ///
+    static CRef < objects::CSeq_align_set >
+    PrepareBlastUngappedSeqalignEx(const objects::CSeq_align_set & alnset);
+
     void SetAlignTemplates(SAlignTemplates *alignTemplates) {m_AlignTemplates = alignTemplates;}    
     
     
@@ -431,7 +442,9 @@ private:
     CDisplaySeqalign(const CDisplaySeqalign& other);
     /// Prohibit assignment operator
     CDisplaySeqalign& operator=(const CDisplaySeqalign& rhs);
- 
+
+protected: 
+
     ///internal insert information
     ///aln_start. insert right after this position
     struct SInsertInformation : public CObject {
@@ -840,7 +853,7 @@ private:
 
     ///Display pairwise alignment
     ///
-    string x_DisplayRowData(SAlnRowInfo *alnRoInfo);
+    virtual string x_DisplayRowData(SAlnRowInfo *alnRoInfo);
 
     ///Display identities,positives,frames etc
     ///
@@ -904,6 +917,8 @@ private:
 
     void x_AddTranslationForLocalSeq(vector<TSAlnFeatureInfoList>& retval,
                                      vector<string>& sequence) const; 
+
+    static int x_GetGiForSeqIdList(const list< CRef<objects::CSeq_id> >& ids);
 };
 
 
