@@ -1,4 +1,4 @@
-/*  $Id: showdefline.hpp 311249 2011-07-11 14:12:16Z camacho $
+/*  $Id: showdefline.hpp 354621 2012-02-28 18:29:04Z zaretska $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -331,6 +331,8 @@ protected:
         int match;                     //number of matches for the top hsp with the hit
         int master_covered_length;     //total length covered by alignment
         int align_length;              //length of alignment
+        int percent_coverage;          //query coverage 
+        int percent_identity;          //percent identity   
         CConstRef<objects::CSeq_id> id;
         int blast_rank;                // "Rank" of defline.
         int hspNum;                    //hsp number
