@@ -1,4 +1,4 @@
-/* $Id: tabular.hpp 347537 2011-12-19 16:45:43Z maning $ 
+/* $Id: tabular.hpp 367113 2012-06-20 19:26:08Z rafanovi $ 
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -159,6 +159,13 @@ public:
     /// @param field Which field to test [in]
     void SetParseLocalIds(bool val) { m_ParseLocalIds = val; }
 
+    /// Avoid fetching sequence (if possible)
+    /// If the sequence is needed (e.g., will be formatted, it will be fetched)
+    /// @param nofetch Do not fetch if true [in]
+    void SetNoFetch(bool nofetch);
+    /// Avoid fetch of sequence if true returned
+    bool GetNoFetch();
+
 protected:
     bool x_IsFieldRequested(ETabularField field);
     /// Add a field to the list of fields to show, if it is not yet present in
@@ -266,6 +273,7 @@ protected:
     int m_SubjectStart;      ///< Starting offset in subject
     int m_SubjectEnd;        ///< Ending offset in subject 
     int m_SubjectFrame;      ///< subject frame
+    bool m_NoFetch;	     ///< program as a string
 
 private:
 
@@ -417,6 +425,16 @@ inline void CBlastTabularInfo::x_PrintSubjectLength(void)
     m_Ostream << m_SubjectLength;
 }
 
+inline void CBlastTabularInfo::SetNoFetch(bool nofetch)
+{
+    m_NoFetch = nofetch;
+}
+
+inline bool CBlastTabularInfo::GetNoFetch(void)
+{
+    return m_NoFetch;
+}
+
 /// Class containing information needed for tabular formatting of BLAST 
 /// results.
 class NCBI_ALIGN_FORMAT_EXPORT CIgBlastTabularInfo : public CBlastTabularInfo
@@ -475,16 +493,16 @@ public:
         x_ResetIgFields();
     };
 
-    virtual void PrintHeader(const string& program, 
+    void PrintHeader(const string& program, 
                      const objects::CBioseq& bioseq, 
                      const string& dbname, 
+                     const string& domain_sys,
                      const string& rid = kEmptyStr,
                      unsigned int iteration = 
                                 numeric_limits<unsigned int>::max(),
                      const objects::CSeq_align_set* align_set=0,
                      CConstRef<objects::CBioseq> subj_bioseq
-                                = CConstRef<objects::CBioseq>(),
-                     const string& domain_sys = "Kabat"); 
+                                = CConstRef<objects::CBioseq>());
 
     /// Set fields for master alignment
     int SetMasterFields(const objects::CSeq_align& align, 
@@ -493,7 +511,7 @@ public:
                         CNcbiMatrix<int>*          matrix=0);
 
     /// Set fields for all other alignments
-    virtual int SetFields(const objects::CSeq_align& align,
+    int SetFields(const objects::CSeq_align& align,
                   objects::CScope&           scope,
                   const string&              chain_type,
                   CNcbiMatrix<int>*          matrix=0);
