@@ -1,7 +1,7 @@
 #ifndef CLEANUP___CLEANUP_CHANGE__HPP
 #define CLEANUP___CLEANUP_CHANGE__HPP
 
-/*  $Id: cleanup_change.hpp 342514 2011-10-28 13:19:52Z kornbluh $
+/*  $Id: cleanup_change.hpp 366493 2012-06-14 16:55:00Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -155,6 +155,8 @@ public:
         eTrimFlankingQuotes,
         eCleanBioseqTitle,
         eDecodeXML,
+        eRemoveDupBioSource, // 90
+        eCleanOrgref,
 
         // set when any other change is made.
         eChangeOther,
