@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_DATA_LOADERS_BLASTDB___BDBLOADER__HPP
 #define OBJTOOLS_DATA_LOADERS_BLASTDB___BDBLOADER__HPP
 
-/*  $Id: bdbloader.hpp 219674 2011-01-12 20:09:13Z vasilche $
+/*  $Id: bdbloader.hpp 368105 2012-07-02 20:06:14Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -106,6 +106,15 @@ public:
     /// Load a description or data chunk.
     virtual void GetChunk(TChunk chunk);
 
+    virtual int GetTaxId(const CSeq_id_Handle& idh);
+    virtual void GetTaxIds(const TIds& ids, TLoaded& loaded, TTaxIds& ret);
+    virtual TSeqPos GetSequenceLength(const CSeq_id_Handle& idh);
+    virtual void GetSequenceLengths(const TIds& ids, TLoaded& loaded,
+                                    TSequenceLengths& ret);
+    virtual CSeq_inst::TMol GetSequenceType(const CSeq_id_Handle& idh);
+    virtual void GetSequenceTypes(const TIds& ids, TLoaded& loaded,
+                                  TSequenceTypes& ret);
+
     /// Gets the blob id for a given sequence.
     ///
     /// Given a Seq_id_Handle, this method finds the corresponding top
@@ -142,7 +151,7 @@ public:
     virtual TTSE_Lock GetBlobById(const TBlobId& blob_id);
     
     /// A mapping from sequence identifier to blob ids.
-    typedef map< CSeq_id_Handle, int > TIds;
+    typedef map< CSeq_id_Handle, int > TIdMap;
 
     /// @note this is added to temporarily comply with the toolkit's stable
     /// components rule of having backwards compatible APIs
@@ -209,7 +218,7 @@ protected:
     EDbType         m_DBType;      ///< Is this database protein or nucleotide?
     CRef<IBlastDbAdapter> m_BlastDb;       ///< The sequence database
 
-    TIds            m_Ids;         ///< ID to OID translation
+    TIdMap          m_Ids;         ///< ID to OID translation
 
     /// Configuration value specified to the CCachedSequence
     bool            m_UseFixedSizeSlices;
