@@ -1,7 +1,7 @@
 #ifndef READER_ID2_BASE__HPP_INCLUDED
 #define READER_ID2_BASE__HPP_INCLUDED
 
-/*  $Id: reader_id2_base.hpp 347885 2011-12-21 18:20:59Z vasilche $
+/*  $Id: reader_id2_base.hpp 370295 2012-07-27 16:38:16Z vasilche $
 * ===========================================================================
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
@@ -198,6 +198,8 @@ protected:
     TErrorFlags x_GetError(CReaderRequestResult& result,
                            const CID2_Reply& reply);
     TErrorFlags x_GetMessageError(const CID2_Reply& reply);
+    TBlobState x_GetBlobState(const CID2_Reply& reply,
+                              TErrorFlags* errors_ptr = 0);
 
     void x_ProcessReply(CReaderRequestResult& result,
                         SId2LoadedSet& loaded_set,
