@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___CONTEXT__HPP
 #define OBJTOOLS_FORMAT___CONTEXT__HPP
 
-/*  $Id: context.hpp 346027 2011-12-02 13:43:47Z kornbluh $
+/*  $Id: context.hpp 372931 2012-08-23 16:50:20Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -143,6 +143,12 @@ public:
     const string& GetWGSMasterAccn(void) const { return m_WGSMasterAccn; }
     const string& GetWGSMasterName(void) const { return m_WGSMasterName; }
 
+    // Transcriptome Shotgun Assembly 
+    bool IsTSA      (void) const { return m_IsTSA;       }
+    bool IsTSAMaster(void) const { return m_IsTSAMaster; }
+    const string& GetTSAMasterAccn(void) const { return m_TSAMasterAccn; }
+    const string& GetTSAMasterName(void) const { return m_TSAMasterName; }
+
     TReferences& SetReferences(void) { return m_References; }
     const TReferences& GetReferences(void) const { return m_References; }
 
@@ -169,6 +175,7 @@ public:
     bool IsPatent         (void) const { return m_IsPatent; }  // patent
     bool IsGbGenomeProject(void) const { return m_IsGbGenomeProject; } // AE
     bool IsNcbiCONDiv     (void) const { return m_IsNcbiCONDiv; }      // CH
+    bool IsNcbiGenomes    (void) const { return m_IsNcbiGenomes; } // db NCBI_GENOMES
     
     // RefSeq ID queries
     bool IsRefSeq(void) const { return m_IsRefSeq; }
@@ -192,13 +199,17 @@ public:
     bool IsGenomeAssembly(void) const;
 
     NCBI_DEPRECATED bool IsUnverified(void) const;
-    enum EUnverified {
-        eUnverified_None = 1,
-        eUnverified_SequenceOrAnnotation,
-        eUnverified_Organism
-    };
-    EUnverified GetUnverifiedType(void) const;
+    enum FUnverified {
+        fUnverified_None = 0, // for convenience
 
+        fUnverified_Organism              = 1 << 0,
+        fUnverified_SequenceOrAnnotation  = 1 << 1
+        // if you add more here, make sure 
+        // to update CFlatGatherer::x_UnverifiedComment
+    };
+    typedef Int8 TUnverified;
+    TUnverified GetUnverifiedType(void) const;
+    bool ShowAnnotCommentAsCOMMENT(void) const { return m_ShowAnnotCommentAsCOMMENT; }
 
     bool IsHup(void) const { return m_IsHup; }  // !!! should move to global?
 
@@ -234,6 +245,7 @@ private:
     void x_SetMapper(const CSeq_loc& loc);
     void x_SetHasMultiIntervalGenes(void);
     void x_SetDataFromUserObjects(void);
+    void x_SetDataFromAnnot(void);
     void x_SetTaxname(void);
 
     CSeq_inst::TRepr x_GetRepr(void) const;
@@ -250,6 +262,8 @@ private:
     string                m_Accession;
     string                m_WGSMasterAccn;
     string                m_WGSMasterName;
+    string                m_TSAMasterAccn;
+    string                m_TSAMasterName;
     string                m_FinishingStatus;
     string                m_Taxname;
 
@@ -280,10 +294,13 @@ private:
     unsigned int m_RefseqInfo;
     bool m_IsGbGenomeProject;  // GenBank Genome project data
     bool m_IsNcbiCONDiv;       // NCBI CON division
+    bool m_IsNcbiGenomes;      // NCBI_GENOMES
     bool m_IsPatent;
     bool m_IsGI;
     bool m_IsWGS;
     bool m_IsWGSMaster;
+    bool m_IsTSA;
+    bool m_IsTSAMaster;
     bool m_IsHup;
     int  m_Gi;
     bool m_ShowGBBSource;
@@ -291,7 +308,8 @@ private:
     bool m_HasOperon;
     bool m_HasMultiIntervalGenes;
     bool m_IsGenomeAssembly;
-    EUnverified m_eUnverified;
+    TUnverified m_fUnverified;
+    bool m_ShowAnnotCommentAsCOMMENT;
 
     CConstRef<CUser_object> m_Encode;
     
@@ -401,9 +419,11 @@ public:
     CTopLevelSeqEntryContext( const CSeq_entry_Handle &entry_handle );
 
     bool GetCanSourcePubsBeFused(void) const { return m_CanSourcePubsBeFused; }
+    bool GetHasSmallGenomeSet(void) const { return m_HasSmallGenomeSet; }
 
 private:
     bool m_CanSourcePubsBeFused;
+    bool m_HasSmallGenomeSet;
 };
 
 
@@ -529,13 +549,13 @@ bool CBioseqContext::IsGenomeAssembly(void) const
 inline
 bool CBioseqContext::IsUnverified(void) const
 {
-    return ( m_eUnverified != eUnverified_None );
+    return ( m_fUnverified != fUnverified_None );
 }
 
 inline
-CBioseqContext::EUnverified CBioseqContext::GetUnverifiedType(void) const
+CBioseqContext::TUnverified CBioseqContext::GetUnverifiedType(void) const
 {
-    return m_eUnverified;
+    return m_fUnverified;
 }
 
 inline
