@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___GATHER_ITEMS__HPP
 #define OBJTOOLS_FORMAT___GATHER_ITEMS__HPP
 
-/*  $Id: gather_items.hpp 294826 2011-05-27 11:19:20Z kornbluh $
+/*  $Id: gather_items.hpp 372931 2012-08-23 16:50:20Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -100,6 +100,8 @@ protected:
     void x_GetFeatsOnCdsProduct(const CSeq_feat& feat, const CSeq_loc& mapped_loc, CBioseqContext& ctx,
         CRef<CSeq_loc_Mapper> slice_mapper,
         CConstRef<CFeatureItem> cdsFeatureItem = CConstRef<CFeatureItem>() ) const;
+    static void x_GiveOneResidueIntervalsBogusFuzz(CSeq_loc & loc);
+    static void x_RemoveBogusFuzzFromIntervals(CSeq_loc & loc);
     void x_CopyCDSFromCDNA(const CSeq_feat& feat, CBioseqContext& ctx) const;
     bool x_SkipFeature(const CSeq_feat& feat, const CBioseqContext& ctx) const;
     virtual void x_GatherFeaturesOnLocation(const CSeq_loc& loc, SAnnotSelector& sel,
@@ -125,21 +127,32 @@ protected:
     void x_GatherAlignments(void) const;
 
     // comments
+    enum EGenomeAnnotComment {
+        eGenomeAnnotComment_No = 0,
+        eGenomeAnnotComment_Yes
+    };
     void x_GatherComments  (void) const;
     void x_AddComment(CCommentItem* comment) const;
     void x_AddGSDBComment(const CDbtag& dbtag, CBioseqContext& ctx) const;
+    void x_RemoveDupComments(void) const;
     void x_FlushComments(void) const;
     void x_UnverifiedComment(CBioseqContext& ctx) const;
-    void x_IdComments(CBioseqContext& ctx) const;
-    void x_RefSeqComments(CBioseqContext& ctx) const;
+    void x_IdComments(CBioseqContext& ctx, 
+        EGenomeAnnotComment eGenomeAnnotComment) const;
+    void x_RefSeqComments(CBioseqContext& ctx, 
+        EGenomeAnnotComment eGenomeAnnotComment) const;
     void x_HistoryComments(CBioseqContext& ctx) const;
     void x_WGSComment(CBioseqContext& ctx) const;
+    void x_TSAComment(CBioseqContext& ctx) const;
     void x_GBBSourceComment(CBioseqContext& ctx) const;
     void x_BarcodeComment(CBioseqContext& ctx) const;
     void x_DescComments(CBioseqContext& ctx) const;
     void x_MaplocComments(CBioseqContext& ctx) const;
     void x_RegionComments(CBioseqContext& ctx) const;
     void x_HTGSComments(CBioseqContext& ctx) const;
+    void x_AnnotComments(CBioseqContext& ctx) const;
+    CConstRef<CUser_object> x_PrepareAnnotDescStrucComment(CBioseqContext& ctx) const;
+    CConstRef<CUser_object> x_GetAnnotDescStrucCommentFromBioseqHandle( CBioseq_Handle bsh ) const;
     void x_FeatComments(CBioseqContext& ctx) const;
     void x_NameComments(CBioseqContext& ctx) const;
     void x_StructuredComments(CBioseqContext& ctx) const;
