@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT_ITEMS___COMMENT_ITEM__HPP
 #define OBJTOOLS_FORMAT_ITEMS___COMMENT_ITEM__HPP
 
-/*  $Id: comment_item.hpp 225521 2011-01-27 18:36:14Z kornbluh $
+/*  $Id: comment_item.hpp 370808 2012-08-01 20:00:57Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -81,7 +81,8 @@ public:
         eRefTrackStatus_Validated,
         eRefTrackStatus_Reviewed,
         eRefTrackStatus_Model,
-        eRefTrackStatus_WGS
+        eRefTrackStatus_WGS,
+        eRefTrackStatus_TSA
     };
 
     // typedefs
@@ -93,6 +94,7 @@ public:
         const CSerialObject* obj = 0);
     CCommentItem(const CSeqdesc&  desc, CBioseqContext& ctx);
     CCommentItem(const CSeq_feat& feat, CBioseqContext& ctx);
+    CCommentItem(const CUser_object & userObject, CBioseqContext& ctx);
 
     void Format(IFormatter& formatter, IFlatTextOStream& text_os) const;
 
@@ -118,9 +120,16 @@ public:
     static const string& GetNsAreGapsStr(void);
     static string GetStringForTPA(const CUser_object& uo, CBioseqContext& ctx);
     static string GetStringForBankIt(const CUser_object& uo);
+    enum EGenomeBuildComment {
+        eGenomeBuildComment_No = 0,
+        eGenomeBuildComment_Yes
+    };
     static string GetStringForRefTrack(const CUser_object& uo,
-        const CBioseq_Handle& seq, ECommentFormat format = eFormat_Text);
+        const CBioseq_Handle& seq, ECommentFormat format = eFormat_Text,
+        EGenomeBuildComment eGenomeBuildComment = eGenomeBuildComment_Yes
+        );
     static string GetStringForWGS(CBioseqContext& ctx);
+    static string GetStringForTSA(CBioseqContext& ctx);
     static string GetStringForMolinfo(const CMolInfo& mi, CBioseqContext& ctx);
     static string GetStringForHTGS(CBioseqContext& ctx);
     static string GetStringForModelEvidance(const SModelEvidance& me,
@@ -143,6 +152,7 @@ protected:
     void x_GatherInfo(CBioseqContext& ctx);
     void x_GatherDescInfo(const CSeqdesc& desc);
     void x_GatherFeatInfo(const CSeq_feat& feat, CBioseqContext& ctx);
+    void x_GatherUserObjInfo(const CUser_object& userObject );
 
     void x_SetComment(const string& comment);
     void x_SetCommentWithURLlinks(const string& prefix, const string& str,
