@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT_ITEMS___FLAT_FEATURE__HPP
 #define OBJTOOLS_FORMAT_ITEMS___FLAT_FEATURE__HPP
 
-/*  $Id: feature_item.hpp 348666 2012-01-03 17:16:14Z ludwigf $
+/*  $Id: feature_item.hpp 372931 2012-08-23 16:50:20Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -215,7 +215,7 @@ protected:
     void x_AddQualNote( CConstRef<CSeq_feat> );
     void x_AddQualOldLocusTag( CConstRef<CSeq_feat> );
     void x_AddQualDb( const CGene_ref* );
-    void x_AddQualSeqfeatNote();
+    void x_AddQualSeqfeatNote( CBioseqContext & );
     void x_AddQualTranslation( CBioseq_Handle&, CBioseqContext&, bool );
     void x_AddQualTranslationTable( const CCdregion&, CBioseqContext& );
     void x_AddQualCodonStart( const CCdregion&, CBioseqContext& );
@@ -311,6 +311,7 @@ protected:
     bool x_GetGbValue(
         const string&,
         string& ) const;
+    bool x_HasMethodtRNAscanSE(void) const;
 
     // format
     void x_FormatQuals(CFlatFeature& ff) const;
@@ -442,8 +443,6 @@ private:
             CFlatFeature::TQuals& qvec, TQualFlags flags = 0) const {
         x_FormatQual(slot, name, qvec, flags | IFlatQVal::fIsNote);
     }
-    
-    
 
     bool           m_WasDesc;
     mutable TQuals m_Quals;
