@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS___ALN_READER__HPP
 #define OBJTOOLS_READERS___ALN_READER__HPP
 
-/*  $Id: aln_reader.hpp 347988 2011-12-22 12:21:43Z kornbluh $
+/*  $Id: aln_reader.hpp 358248 2012-03-30 13:43:31Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -142,7 +142,7 @@ public:
     /// Read the file
     /// This is the main function
     /// that would parse the alignment file and create the result data
-    void Read(bool guess=false);
+    void Read(bool guess=false, bool generate_local_ids=false);
 
 
 
