@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS___FASTA__HPP
 #define OBJTOOLS_READERS___FASTA__HPP
 
-/*  $Id: fasta.hpp 346991 2011-12-13 13:21:26Z kornbluh $
+/*  $Id: fasta.hpp 366496 2012-06-14 16:55:52Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -210,10 +210,12 @@ protected:
 
     void x_RecursiveApplyAllMods( CSeq_entry& entry );
 
+    std::string x_NucOrProt(void) const;
+
 private:
     struct SGap {
-        TSeqPos pos; // 0-based, and NOT counting previous gaps
-        TSeqPos len; // may be zero for gaps of unknown length
+        TSeqPos       pos; // 0-based, and NOT counting previous gaps
+        TSignedSeqPos len; // 0: unknown, negative: negated nominal length
     };
     typedef vector<SGap>        TGaps;
     typedef set<CSeq_id_Handle> TIDTracker;
