@@ -1,4 +1,4 @@
-/*  $Id: line_error.hpp 346668 2011-12-08 17:47:17Z kornbluh $
+/*  $Id: line_error.hpp 355595 2012-03-07 12:20:34Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -64,6 +64,7 @@ public:
         eProblem_QualifierBadValue,
         eProblem_BadScoreValue,
         eProblem_MissingContext,
+        eProblem_BadTrackLine,
 
         eProblem_GeneralParsingError
     };
@@ -161,6 +162,8 @@ public:
             return "Invalid score value";
         case eProblem_MissingContext:
             return "Value ignored due to missing context";
+        case eProblem_BadTrackLine:
+            return "Bad track line: Expected \"track key1=value1 key2=value2 ...\"";
         default:
             return "Unknown problem";
         }
@@ -263,7 +266,8 @@ public:
         const std::string & strQualifierName = string(""),
         const std::string & strQualifierValue = string("") )
     : CObjReaderParseException( DIAG_COMPILE_INFO, 0, eFormat, strMessage, uLine,
-        eDiag_Info ), m_strSeqId(strSeqId), m_uLineNumber(uLine), 
+        eDiag_Info ), 
+        m_eProblem(eProblem), m_strSeqId(strSeqId), m_uLineNumber(uLine), 
         m_strFeatureName(strFeatureName), m_strQualifierName(strQualifierName), 
         m_strQualifierValue(strQualifierValue)
     {
