@@ -1,7 +1,7 @@
 #ifndef OBJECTINFO__HPP
 #define OBJECTINFO__HPP
 
-/*  $Id: objectinfo.hpp 351983 2012-02-02 16:11:31Z ivanov $
+/*  $Id: objectinfo.hpp 363589 2012-05-17 17:08:07Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -268,9 +268,7 @@ public:
     /// Set global (for all streams) read hook
     /// @param hook
     ///   Pointer to hook object
-    /// @deprecated
-    ///   Use local hooks instead
-    NCBI_DEPRECATED void SetGlobalReadHook(CReadObjectHook* hook) const;
+    void SetGlobalReadHook(CReadObjectHook* hook) const;
 
     /// Reset local read hook
     /// @param stream
@@ -302,9 +300,7 @@ public:
     /// Set global (for all streams) write hook
     /// @param hook
     ///   Pointer to hook object
-    /// @deprecated
-    ///   Use local hooks instead
-    NCBI_DEPRECATED void SetGlobalWriteHook(CWriteObjectHook* hook) const;
+    void SetGlobalWriteHook(CWriteObjectHook* hook) const;
 
     /// Reset local write hook
     /// @param stream
@@ -333,21 +329,11 @@ public:
     void SetLocalSkipHook(CObjectIStream& stream,
                           CSkipObjectHook* hook) const;
 
-    /// Set global (for all streams) skip hook
-    /// @param hook
-    ///   Pointer to hook object
-    /// @deprecated
-    ///   Use local hooks instead
-    NCBI_DEPRECATED void SetGlobalSkipHook(CSkipObjectHook* hook) const;
-
     /// Reset local skip hook
     /// @param stream
     ///   Input data stream reader
     void ResetLocalSkipHook(CObjectIStream& stream) const;
 
-    /// Reset global skip hooks
-    void ResetGlobalSkipHook(void) const;
-
     /// Set local context-specific skip hook
     /// @param stream
     ///   Input data stream reader
@@ -370,9 +356,7 @@ public:
     /// Set global (for all streams) copy hook
     /// @param hook
     ///   Pointer to hook object
-    /// @deprecated
-    ///   Use local hooks instead
-    NCBI_DEPRECATED void SetGlobalCopyHook(CCopyObjectHook* hook) const;
+    void SetGlobalCopyHook(CCopyObjectHook* hook) const;
 
     /// Reset local copy hook
     /// @param stream
@@ -420,7 +404,8 @@ private:
 private:
     CTypeInfo* GetNCTypeInfo(void) const;
 
-    bool MatchPattern(vector<int>& pattern, size_t& pos, int depth) const;
+    bool MatchPattern(vector<int>& pattern, size_t& pos, int depth,
+                      const CItemInfo* item = NULL) const;
     friend class CObjectIStreamAsnBinary;
 };
 
