@@ -1,7 +1,7 @@
 #ifndef SERIAL___OBJISTR__HPP
 #define SERIAL___OBJISTR__HPP
 
-/*  $Id: objistr.hpp 348645 2012-01-03 15:54:46Z vasilche $
+/*  $Id: objistr.hpp 363042 2012-05-11 19:25:07Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -642,7 +642,7 @@ public:
     void ReadStd(unsigned& data);
     void SkipStd(const int& );
     void SkipStd(const unsigned& );
-#if SIZEOF_LONG == 4
+#ifndef NCBI_INT8_IS_LONG
     void ReadStd(long& data);
     void ReadStd(unsigned long& data);
     void SkipStd(const long& );
@@ -1072,10 +1072,12 @@ inline
 bool GoodVisibleChar(char c);
 
 NCBI_XSERIAL_EXPORT
-char ReplaceVisibleChar(char c, EFixNonPrint fix_method, size_t at_line);
+char ReplaceVisibleChar(char c, EFixNonPrint fix_method,
+    const CObjectStack* io, const string& str);
 
 inline
-void FixVisibleChar(char& c, EFixNonPrint fix_method, size_t at_line = 0);
+void FixVisibleChar(char& c, EFixNonPrint fix_method,
+    const CObjectStack* io, const string& str);
 
 
 /// Guard class for CObjectIStream::StartDelayBuffer/EndDelayBuffer
