@@ -1,7 +1,7 @@
 #ifndef OBJOSTR__HPP
 #define OBJOSTR__HPP
 
-/*  $Id: objostr.hpp 345009 2011-11-21 16:24:08Z vasilche $
+/*  $Id: objostr.hpp 363042 2012-05-11 19:25:07Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -118,11 +118,15 @@ public:
     /// @param deleteOutStream
     ///   When eTakeOwnership, the output stream will be deleted automatically
     ///   when the writer is deleted
+    /// @param formatFlags
+    ///   Formatting flags (see ESerial_xxx_Flags)
     /// @return
     ///   Writer (created on heap)
+    /// @sa ESerial_AsnText_Flags, ESerial_Xml_Flags, ESerial_Json_Flags
     static CObjectOStream* Open(ESerialDataFormat format,
                                 CNcbiOstream& outStream,
-                                EOwnership deleteOutStream = eNoOwnership);
+                                EOwnership deleteOutStream = eNoOwnership,
+                                TSerial_Format_Flags formatFlags = 0);
 
     /// Create serial object writer and attach it to a file stream.
     ///
@@ -132,12 +136,15 @@ public:
     ///   Output file name
     /// @param openFlags
     ///   File open flags
+    /// @param formatFlags
+    ///   Formatting flags (see ESerial_xxx_Flags)
     /// @return
     ///   Writer (created on heap)
-    /// @sa ESerialOpenFlags
+    /// @sa ESerialOpenFlags, ESerial_AsnText_Flags, ESerial_Xml_Flags, ESerial_Json_Flags
     static CObjectOStream* Open(ESerialDataFormat format,
                                 const string& fileName,
-                                TSerialOpenFlags openFlags = 0);
+                                TSerialOpenFlags openFlags = 0,
+                                TSerial_Format_Flags formatFlags = 0);
 
     /// Create serial object writer and attach it to a file stream.
     ///
@@ -145,10 +152,14 @@ public:
     ///   Output file name
     /// @param format
     ///   Format of the output data
+    /// @param formatFlags
+    ///   Formatting flags (see ESerial_xxx_Flags)
     /// @return
     ///   Writer (created on heap)
+    /// @sa ESerial_AsnText_Flags, ESerial_Xml_Flags, ESerial_Json_Flags
     static CObjectOStream* Open(const string& fileName,
-                                ESerialDataFormat format);
+                                ESerialDataFormat format,
+                                TSerial_Format_Flags formatFlags = 0);
 
     /// Get data format
     ///
@@ -209,6 +220,19 @@ public:
     /// @return
     ///   TRUE or FALSE
     bool GetUseIndentation(void) const;
+
+    /// Set up end-of-line symbol usage in text streams.
+    /// 
+    /// @param set
+    ///   When TRUE, the writer puts end-of-line symbol where needed,
+    ///   otherwise, the output is a single line.
+    void SetUseEol(bool set);
+
+    /// Get end-of-line symbol usage in text streams.
+    /// 
+    /// @return
+    ///   TRUE or FALSE
+    bool GetUseEol(void) const;
     
     /// Set up writing named integers (in ANS.1 sense) by value only.
     ///
@@ -259,6 +283,12 @@ public:
     ///   TRUE or FALSE
     void SetAutoSeparator(bool value);
 
+    /// Set output formatting flags
+    ///
+    /// @param flags
+    ///   Formatting flag
+    virtual void SetFormattingFlags(TSerial_Format_Flags flags);
+
 //---------------------------------------------------------------------------
 // Stream state
 
@@ -433,7 +463,7 @@ public:
     void WriteStd(const unsigned short& data);
     void WriteStd(const int& data);
     void WriteStd(const unsigned int& data);
-#if SIZEOF_LONG == 4
+#ifndef NCBI_INT8_IS_LONG
     void WriteStd(const long& data);
     void WriteStd(const unsigned long& data);
 #endif
@@ -733,6 +763,7 @@ protected:
     ESerialDataFormat   m_DataFormat;
     bool  m_WriteNamedIntegersByValue;
     EDelayBufferParsing  m_ParseDelayBuffers;
+    bool  m_FastWriteDouble;
 
 private:
     static CObjectOStream* OpenObjectOStreamAsn(CNcbiOstream& out,
