@@ -1,7 +1,7 @@
 #ifndef OBJOSTRJSON__HPP
 #define OBJOSTRJSON__HPP
 
-/*  $Id: objostrjson.hpp 336735 2011-09-07 16:16:59Z vasilche $
+/*  $Id: objostrjson.hpp 362573 2012-05-09 20:35:46Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -108,6 +108,19 @@ public:
     ///   string
     virtual string GetPosition(void) const;
 
+    /// Set JSONP mode
+    /// JSONP prefix will become "function_name("
+    /// JSONP suffix will become ");"
+    void SetJsonpMode(const string& function_name);
+
+    /// Get JSONP padding (prefix and suffix)
+    ///
+    /// @param prefix
+    ///   Receives JSONP prefix
+    /// @param suffix
+    ///   Receives JSONP suffix
+    void GetJsonpPadding(string* prefix, string* suffix) const;
+
     virtual void WriteFileHeader(TTypeInfo type);
     virtual void EndOfWrite(void);
 
@@ -120,6 +133,7 @@ protected:
     virtual void WriteUint8(Uint8 data);
     virtual void WriteFloat(float data);
     virtual void WriteDouble(double data);
+    void WriteDouble2(double data, size_t digits);
     virtual void WriteCString(const char* str);
     virtual void WriteString(const string& s,
                              EStringType type = eStringTypeVisible);
@@ -213,6 +227,8 @@ private:
     string m_SkippedMemberId;
     EEncoding m_StringEncoding;
     EBinaryDataFormat m_BinaryFormat;
+    string m_JsonpPrefix;
+    string m_JsonpSuffix;
 };
 
 
