@@ -1,7 +1,7 @@
 #ifndef OBJOSTRXML__HPP
 #define OBJOSTRXML__HPP
 
-/*  $Id: objostrxml.hpp 336735 2011-09-07 16:16:59Z vasilche $
+/*  $Id: objostrxml.hpp 359471 2012-04-13 12:48:55Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -251,6 +251,12 @@ public:
     /// @param fmt
     ///   Formatting flag
     void SetRealValueFormat(ERealValueFormat fmt);
+    
+    /// Set output formatting flags
+    ///
+    /// @param flags
+    ///   Formatting flag
+    virtual void SetFormattingFlags(TSerial_Format_Flags flags);
 
     virtual void WriteFileHeader(TTypeInfo type);
     virtual void EndOfWrite(void);
@@ -434,6 +440,7 @@ private:
     ERealValueFormat m_RealFmt;
     EEncoding m_Encoding;
     EEncoding m_StringEncoding;
+    bool m_UseXmlDecl;
     bool m_UseSchemaRef;
     bool m_UseSchemaLoc;
     bool m_UseDTDRef;
