@@ -1,7 +1,7 @@
 #ifndef UTIL_COMPRESS__LZO__HPP
 #define UTIL_COMPRESS__LZO__HPP
 
-/*  $Id: lzo.hpp 345063 2011-11-21 20:17:21Z ivanov $
+/*  $Id: lzo.hpp 367949 2012-06-29 15:16:21Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -544,7 +544,7 @@ protected:
                             /* out */            size_t* out_avail);
     virtual EStatus Finish (char*       out_buf, size_t  out_size,
                             /* out */            size_t* out_avail);
-    virtual EStatus End    (void);
+    virtual EStatus End    (int abandon = 0);
 
 protected:
     /// Compress block of data in the cache buffer.
@@ -587,7 +587,7 @@ protected:
                             /* out */            size_t* out_avail);
     virtual EStatus Finish (char*       out_buf, size_t  out_size,
                             /* out */            size_t* out_avail);
-    virtual EStatus End    (void);
+    virtual EStatus End    (int abandon = 0);
 
 protected:
     /// Decompress block of data in the cache buffer.
