@@ -1,7 +1,7 @@
 #ifndef UTIL_COMPRESS__STREAM_UTIL__HPP
 #define UTIL_COMPRESS__STREAM_UTIL__HPP
 
-/*  $Id: stream_util.hpp 210522 2010-11-04 16:08:29Z ivanov $
+/*  $Id: stream_util.hpp 366481 2012-06-14 16:51:25Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -74,8 +74,8 @@ public:
         eBZip2,               ///< bzip2
         eLZO,                 ///< LZO (LZO1X)
         eZip,                 ///< zlib (raw zip data / DEFLATE method)
-        eGZipFile,            ///< .gz file
-        eConcatenatedGZipFile ///< Concatenated .gz file (decompress only)
+        eGZipFile,            ///< .gz file (including concatenated)
+        eConcatenatedGZipFile ///< Synonym for eGZipFile (for backward compatibility)
     };
 
     /// Default algorithm-specific compression/decompression flags.
