@@ -1,7 +1,7 @@
 #ifndef UTIL___VALUE_CONV_POLICY__HPP
 #define UTIL___VALUE_CONV_POLICY__HPP
 
-/* $Id: value_convert_policy.hpp 145768 2008-11-17 15:38:49Z ssikorsk $
+/* $Id: value_convert_policy.hpp 363039 2012-05-11 19:24:09Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NTOICE
@@ -441,6 +441,30 @@ private:
     obj_type m_Value;
 };
 
+#ifndef NCBI_INT8_IS_LONG
+template <>
+class CConvPolicy<SRunTimeCP, long>
+{
+public:
+    typedef long obj_type;
+
+    CConvPolicy(obj_type value)
+    : m_Value(value)
+    {
+    }
+
+public:
+    template <typename TO>
+    operator TO(void) const
+    {
+        return ConvertUsingRunTimeCP<TO>(m_Value);
+    }
+
+private:
+    obj_type m_Value;
+};
+#endif
+
 template <>
 class CConvPolicy<SRunTimeCP, Int8>
 {
