@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# $Id: check_make_cfg.sh 345646 2011-11-29 15:49:55Z ivanovp $
+# $Id: check_make_cfg.sh 363590 2012-05-17 17:08:32Z rafanovi $
 # Author:  Vladimir Ivanov, NCBI 
 #
 ###########################################################################
@@ -40,8 +40,10 @@ x_cfg=$5
 x_build_dir=$6
 
 
+is_msvc=false
 case "$x_compiler" in
-   MSVC | XCODE ) ;;
+   MSVC  ) is_msvc=true ;;
+   XCODE ) ;;
    * ) echo "Unknown compiler name \"$x_compiler\"."; exit 1 ;;
 esac
 
@@ -140,6 +142,18 @@ if test -n "$NCBI_AUTOMATED_BUILD"; then
    fi
 fi
 
+
+# Check timeout multiplier (increase default check timeout in x times)
+x_check_mult=1
+if test $x_compiler == "MSVC" ; then
+    case "$x_cfg" in
+       Debug* )
+           x_check_mult=2
+           ;;        
+    esac
+fi
+
+
 # Clean up check directory
 (cd "$x_check_dir"  &&  find "$x_check_dir" -maxdepth 1 -mindepth 1 -type d -print | xargs rm -rf)
 
@@ -280,6 +294,14 @@ export CHECK_SIGNATURE="\$signature"
 # Enable silent abort for NCBI applications on fatal errors
 export DIAG_SILENT_ABORT="Y"
 
+# Use AppLog-style output format in the testsuite by default
+test -z "\$DIAG_OLD_POST_FORMAT"  &&  export DIAG_OLD_POST_FORMAT=false
+
+# Check timeout multiplier (increase default check timeout in x times)
+if test -z "\$NCBI_CHECK_TIMEOUT_MULT"; then
+    NCBI_CHECK_TIMEOUT_MULT=$x_check_mult
+fi
+
 # Path to test data, used by some scripts and applications
 if test -z "\$NCBI_TEST_DATA"; then
     case `uname -s` in
@@ -297,7 +319,7 @@ configurations="$x_confs"
 
 if \$is_db_load; then
     echo "--------------------------------------------------" >> "\$build_dir/test_stat_load.log" 2>&1
-    echo "Loading tests from: \$check_dir"                     >> "\$build_dir/test_stat_load.log" 2>&1
+    echo "Loading tests from: \$check_dir"                    >> "\$build_dir/test_stat_load.log" 2>&1
     echo "--------------------------------------------------" >> "\$build_dir/test_stat_load.log" 2>&1
 else
     rm -f "\$res_journal"
@@ -397,18 +419,27 @@ RunTest() {
    x_run_fix=\`echo "\$x_run" | sed -e 's/""/'"'&'/g" -e "s/''/\\\\'\\\\'/g"\`
 
    # Run check
-   export CHECK_TIMEOUT="\$x_timeout"
+   CHECK_TIMEOUT=\`expr \$x_timeout \* \$NCBI_CHECK_TIMEOUT_MULT\`
+   export CHECK_TIMEOUT
+   rm -f check_exec.pid > /dev/null 2>&1
    start_time="\`date +'$x_date_format'\`"
    check_exec="\$root_dir/scripts/common/check/check_exec.sh"
    \$check_exec $x_time \`eval echo \$x_run_fix\` > \$x_test_out.\$\$ 2>&1
    result=\$?
    stop_time="\`date +'$x_date_format'\`"
+   load_avg="\`uptime | sed -e 's/.*averages*: *\(.*\) *$/\1/' -e 's/[, ][, ]*/ /g'\`"
 
    sed -e '/ ["][$][@]["].*\$/ {
        s/^.*: //
        s/ ["][$][@]["].*$//
     }' \$x_test_out.\$\$ >> \$x_test_out
 
+   # RunID
+   runpid='?'
+   test -f check_exec.pid  &&  runpid="\`cat check_exec.pid\`"
+   runid="\`date -u +%y%m%d%H%M%S\`-\$runpid-\`uname -n\`"
+   rm -f check_exec.pid > /dev/null 2>&1
+                
    # Get application execution time
    exec_time=\`tail -3 \$x_test_out.\$\$\`
    exec_time=\`echo "\$exec_time" | tr '\n\r' '%%'\`
@@ -424,6 +455,7 @@ RunTest() {
    # Write result of the test into the his output file
    echo "Start time   : \$start_time"   >> \$x_test_out
    echo "Stop time    : \$stop_time"    >> \$x_test_out
+   echo "Load averages: \$load_avg"     >> \$x_test_out
    echo >> \$x_test_out
    echo "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@" >> \$x_test_out
    echo "@@@ EXIT CODE: \$result" >> \$x_test_out
@@ -472,6 +504,8 @@ RunTest() {
       echo "\$result"     >> "\$x_test_rep"
       echo "\$exec_time"  >> "\$x_test_rep"
       echo "\$x_authors"  >> "\$x_test_rep"
+      echo "\$load_avg"   >> "\$x_test_rep"
+      echo "\$runid"      >> "\$x_test_rep"
    fi
 }
 
@@ -535,7 +569,12 @@ for x_row in $x_tests; do
       for i in $x_files ; do
          x_copy="$x_src_dir/$i"
          if test -f "$x_copy"  -o  -d "$x_copy" ; then
-            cp -prf "$x_copy" "$x_path"
+            if $is_msvc; then
+               # Do not preserve permissions, it is not always possible for NTFS
+               cp -rf "$x_copy" "$x_path"
+            else
+               cp -prf "$x_copy" "$x_path"
+            fi
             find "$x_path/$i" -name .svn -print 2> /dev/null | xargs rm -rf
          else
             echo "Warning:  \"$x_copy\" must be file or directory!"
