@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# $Id: check_make_unix.sh 345646 2011-11-29 15:49:55Z ivanovp $
+# $Id: check_make_unix.sh 363590 2012-05-17 17:08:32Z rafanovi $
 # Author:  Vladimir Ivanov, NCBI 
 #
 ###########################################################################
@@ -319,6 +319,10 @@ export top_srcdir
 FEATURES="$x_features"
 export FEATURES
 
+# Redirect output for C++ diagnostic framework to stderr
+NCBI_CONFIG__LOG__FILE="-"
+export NCBI_CONFIG__LOG__FILE
+
 # Add current, build and scripts directories to PATH
 PATH=".:\${build_dir}:\${root_dir}/scripts/common/impl:\${PATH}"
 export PATH
@@ -337,10 +341,27 @@ export CHECK_EXEC
 export CHECK_EXEC_STDIN
 export CHECK_SIGNATURE
 
+# Use AppLog-style output format in the testsuite by default
+if test -z "\$DIAG_OLD_POST_FORMAT"; then
+    DIAG_OLD_POST_FORMAT=false
+    export DIAG_OLD_POST_FORMAT
+fi
+
 # Avoid possible hangs on Mac OS X.
 DYLD_BIND_AT_LAUNCH=1
 export DYLD_BIND_AT_LAUNCH
 
+case " \$FEATURES " in
+    *\ MaxDebug\ * )
+         case "\$signature" in
+	     *-linux* ) MALLOC_DEBUG_=2; export MALLOC_DEBUG_ ;;
+         esac
+         case "\$signature" in
+             GCC* | ICC* ) NCBI_CHECK_TIMEOUT_MULT=20 ;;
+         esac
+         ;;
+esac
+
 EOF
 
 if test -n "$x_conf_dir"  -a  -d "$x_conf_dir/lib";  then
@@ -466,6 +487,8 @@ RunTest()
                 regular  ) ;;
                 valgrind ) NCBI_CHECK_TOOL="\$NCBI_CHECK_TOOL \$VALGRIND_CMD" 
                            NCBI_CHECK_TIMEOUT_MULT=10
+                           NCBI_RUN_UNDER_VALGRIND="yes"
+                           export NCBI_RUN_UNDER_VALGRIND
                            ;;
                 esac
                 export NCBI_CHECK_TOOL
@@ -506,7 +529,8 @@ RunTest()
 
                 # Remove old core file if it exist (for clarity of the test)
                 corefile="\$x_work_dir/core"
-                rm -f "\$corefile"  > /dev/null 2>&1
+                rm -f "\$corefile" > /dev/null 2>&1
+                rm -f check_exec.pid > /dev/null 2>&1
 
                 # Run check
                 start_time="\`date +'$x_date_format'\`"
@@ -522,14 +546,21 @@ EOF_launch
                 chmod a+x \$launch_sh
                 \$launch_sh >\$x_log 2>&1
                 result=\$?
-                rm \$launch_sh
                 stop_time="\`date +'$x_date_format'\`"
+                load_avg="\`uptime | sed -e 's/.*averages*: *\(.*\) *$/\1/' -e 's/[, ][, ]*/ /g'\`"
+                rm \$launch_sh
 
                 sed -e '/ ["][$][@]["].*\$/ {
                         s/^.*: //
                         s/ ["][$][@]["].*$//
                 }' \$x_log >> \$x_test_out
 
+                # RunID
+                runpid='?'
+                test -f check_exec.pid  &&  runpid="\`cat check_exec.pid\`"
+                runid="\`date -u +%y%m%d%H%M%S\`-\$runpid-\`uname -n\`"
+                rm -f check_exec.pid > /dev/null 2>&1
+                
                 # Get application execution time
                 exec_time=\`\$build_dir/sysdep.sh tl 7 \$x_log | tr '\n\r' '%%' | tr -d '\000-\037' | tr  -d '\176-\377'\`
                 echo \$exec_time | egrep 'real [0-9]|Maximum execution .* is exceeded' > /dev/null 2>&1 
@@ -563,6 +594,7 @@ EOF_launch
                 # Write result of the test into the his output file
                 echo "Start time   : \$start_time"   >> \$x_test_out
                 echo "Stop time    : \$stop_time"    >> \$x_test_out
+                echo "Load averages: \$load_avg"     >> \$x_test_out
                 echo >> \$x_test_out
                 echo "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@" >> \$x_test_out
                 echo "@@@ EXIT CODE: \$result" >> \$x_test_out
@@ -618,6 +650,9 @@ EOF_launch
                     echo "\$start_time" >> "\$x_test_rep"
                     echo "\$result"     >> "\$x_test_rep"
                     echo "\$exec_time"  >> "\$x_test_rep"
+                    echo "\$x_authors"  >> "\$x_test_rep"
+                    echo "\$load_avg"   >> "\$x_test_rep"
+                    echo "\$runid"      >> "\$x_test_rep"
                 fi
 
             else  # Run test if it exist
@@ -627,8 +662,9 @@ EOF_launch
                     count_absent=\`expr \$count_absent + 1\`
 
                     if test -n "\$NCBI_AUTOMATED_BUILD"; then
-                        echo "ABS"      >> "\$x_test_rep"
+                        echo "ABS"         >> "\$x_test_rep"
                         echo "\`date +'$x_date_format'\`" >> "\$x_test_rep"
+                        echo "\$x_authors" >> "\$x_test_rep"
                     fi
                 fi
             fi
@@ -641,15 +677,12 @@ EOF_launch
                 count_absent=\`expr \$count_absent + 1\`
 
                 if test -n "\$NCBI_AUTOMATED_BUILD"; then
-                    echo "ABS"      >> "\$x_test_rep"
+                    echo "ABS"         >> "\$x_test_rep"
                     echo "\`date +'$x_date_format'\`" >> "\$x_test_rep"
+                    echo "\$x_authors" >> "\$x_test_rep"
                 fi
             fi
         fi
-
-        if \$no_report_err; then
-           test -n "\$NCBI_AUTOMATED_BUILD" && echo "\$x_authors" >> "\$x_test_rep"
-        fi
         
     done  # Run test under all specified check tools   
 }
