@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: blast_objmgr_tools.cpp 348089 2011-12-23 13:33:21Z fongah2 $";
+    "$Id: blast_objmgr_tools.cpp 350677 2012-01-23 19:46:06Z maning $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 /* ===========================================================================
@@ -420,12 +420,7 @@ public:
     virtual void GetStrandData(objects::ENa_strand strand, unsigned char* buf) {
         x_FixStrand(strand);
         for (CSeqVector_CI itr(m_SeqVector, strand); itr; ++itr) {
-            if (itr.IsInGap()) {
-                *buf++ = 0xf;
-            }
-            else {
-                *buf++ = *itr;
-            }
+            *buf++ = *itr;
         }
     }
 
