@@ -1,4 +1,4 @@
-/*  $Id: blast_seqalign.cpp 346479 2011-12-07 14:30:16Z fongah2 $
+/*  $Id: blast_seqalign.cpp 358152 2012-03-29 14:42:07Z fongah2 $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -869,12 +869,12 @@ s_OOFBlastHSP2SeqAlign(EBlastProgramType program, BlastHSP* hsp,
 /// @param i Integer value of the score. [in]
 /// @return Resulting CScore object.
 static CRef<CScore> 
-s_MakeScore(const string& ident_string, double d = 0.0, int i = 0)
+s_MakeScore(const string& ident_string, double d, int i, bool is_integer)
 {
     CRef<CScore> retval(new CScore());
     retval->SetId().SetStr(ident_string);
 
-    if (i)
+    if (is_integer)
         retval->SetValue().SetInt(i);
     else
         retval->SetValue().SetReal(d);
@@ -909,7 +909,7 @@ s_CalculateScoreVectorSize(const BlastHSP* hsp, const vector<int>  & gi_list)
         retval++;
     }
 
-    if (hsp->num_ident > 0) {
+    if (hsp->num_ident >= 0) {
         retval++;
     }
 
@@ -943,50 +943,50 @@ s_BuildScoreList(const BlastHSP     * hsp,
 
     if (hsp->score) {
         static const string kScore("score");
-        scores.push_back(s_MakeScore(kScore, 0.0, hsp->score));
+        scores.push_back(s_MakeScore(kScore, 0.0, hsp->score, true));
     }
 
     if (hsp->num > 1) {
         static const string kSumN("sum_n");
-        scores.push_back(s_MakeScore(kSumN, 0.0, hsp->num));
+        scores.push_back(s_MakeScore(kSumN, 0.0, hsp->num, true));
     }
 
     // Set the E-Value
     double evalue = (hsp->evalue < SMALLEST_EVALUE) ? 0.0 : hsp->evalue;
     if (evalue >= 0.0) {
         static string score_type = (hsp->num <= 1) ? "e_value" : "sum_e";
-        scores.push_back(s_MakeScore(score_type, evalue));
+        scores.push_back(s_MakeScore(score_type, evalue, 0, false));
     }
 
     // Calculate the bit score from the raw score
 
     if (hsp->bit_score >= 0.0) {
         static const string kBitScore("bit_score");
-        scores.push_back(s_MakeScore(kBitScore, hsp->bit_score));
+        scores.push_back(s_MakeScore(kBitScore, hsp->bit_score, 0, false));
     }
 
     // Set the identity score
-    if (hsp->num_ident > 0) {
+    if (hsp->num_ident >= 0) {
         static const string kNumIdent("num_ident");
-        scores.push_back(s_MakeScore(kNumIdent, 0.0, hsp->num_ident));
+        scores.push_back(s_MakeScore(kNumIdent, 0.0, hsp->num_ident, true));
     }
 
     if (hsp->comp_adjustment_method > 0) {
         static const string kCompAdj("comp_adjustment_method");
         scores.push_back(s_MakeScore(kCompAdj, 0.0,
-                                     hsp->comp_adjustment_method));
+                                     hsp->comp_adjustment_method, true));
     }
     
     if ( !gi_list.empty() ) {
         static const string kUseThisGi("use_this_gi");
         ITERATE(vector<int>, gi, gi_list) {
-            scores.push_back(s_MakeScore(kUseThisGi, 0.0, *gi));
+            scores.push_back(s_MakeScore(kUseThisGi, 0.0, *gi, true));
         }
     }
     
     if (hsp->num_positives > 0) {
             static const string kNumPositives("num_positives");
-            scores.push_back(s_MakeScore(kNumPositives, 0.0, hsp->num_positives));
+            scores.push_back(s_MakeScore(kNumPositives, 0.0, hsp->num_positives, true));
     }
     return;
 }
@@ -1231,6 +1231,11 @@ BLASTHspListToSeqAlign(EBlastProgramType program, BlastHSPList* hsp_list,
                                     query_length, subject_length);
         }
         
+        if(eBlastTypePsiBlast == program)
+        {
+        	if(hsp->num_ident == 0)
+        		hsp->num_ident = -1;
+        }
         s_AddScoresToSeqAlign(seqalign, hsp, gi_list);
         sa_vector.push_back(seqalign);
     }
@@ -1743,6 +1748,116 @@ LocalBlastResults2SeqAlign(BlastHSPResults   * hsp_results,
     return retval;
 }
 
+/// Creates a Std-seg object from HSP information and sequence identifiers
+/// for a non-translated ungapped search.
+/// @param hsp An HSP structure [in]
+/// @param query_id Query sequence identifier [in]
+/// @param subject_id Subject sequence identifier [in]
+/// @param query_length Length of the query [in]
+/// @param subject_length Length of the subject [in]
+/// @param gi_list List of GIs for the subject sequence.
+/// @return Resulting Std-seg object.
+CRef<CStd_seg>
+x_NonTranslatedHSPToStdSeg(BlastHSP* hsp, CRef<CSeq_id> query_id,
+                      	   CRef<CSeq_id> subject_id,
+                      	   Int4 query_length, Int4 subject_length,
+                      	   const vector<int> & gi_list)
+{
+    CRef<CStd_seg> retval(new CStd_seg());
+
+    retval->SetDim(kBlastAlignmentDim);
+    retval->SetLoc().reserve(kBlastAlignmentDim);
+
+    CRef<CSeq_loc> query_loc(new CSeq_loc());
+    CRef<CSeq_loc> subject_loc(new CSeq_loc());
+    query_loc->SetInt().SetId(*query_id);
+    subject_loc->SetInt().SetId(*subject_id);
+
+    // Set the sequence ids
+    CStd_seg::TIds& ids = retval->SetIds();
+    ids.reserve(kBlastAlignmentDim);
+    ids.push_back(query_id);
+    ids.push_back(subject_id);
+
+    query_loc->SetInt().SetStrand(s_Frame2Strand(hsp->query.frame));
+    subject_loc->SetInt().SetStrand(s_Frame2Strand(hsp->subject.frame));
+
+    if (hsp->query.frame >= 0) {
+           query_loc->SetInt().SetFrom(hsp->query.offset);
+           query_loc->SetInt().SetTo(hsp->query.end -1);
+   } else {
+           query_loc->SetInt().SetFrom(query_length - hsp->query.end);
+           query_loc->SetInt().SetTo(query_length - hsp->query.offset -1);
+   }
+
+   if (hsp->subject.frame >= 0) {
+       subject_loc->SetInt().SetFrom(hsp->subject.offset);
+       subject_loc->SetInt().SetTo(hsp->subject.end-1);
+   } else {
+	   subject_loc->SetInt().SetFrom(subject_length - hsp->subject.end);
+	   subject_loc->SetInt().SetTo(subject_length - hsp->subject.offset -1);
+   }
+
+    retval->SetLoc().push_back(query_loc);
+    retval->SetLoc().push_back(subject_loc);
+
+    CSeq_align::TScore& score_list = retval->SetScores();
+    s_BuildScoreList(hsp, score_list, gi_list);
+
+    return retval;
+}
+
+
+void
+BLASTPrelminSearchHitListToStdSeg(EBlastProgramType 	   program,
+                     	 	 	  BlastHitList*			   hit_list,
+                     	 	 	  const CSeq_loc & 		   query_loc,
+                     	 	 	  TSeqPos				   query_length,
+                     	 	 	  const IBlastSeqInfoSrc * subject_seqinfo,
+                     	 	 	  list<CRef<CStd_seg > > & seg_list)
+{
+	seg_list.clear();
+
+	CRef<CSeq_id> query_id(new CSeq_id);
+	SerialAssign(*query_id, CSeq_loc_CI(query_loc).GetSeq_id());
+	_ASSERT(query_id);
+
+	CRef<CStd_seg>
+	(*fun_ptr) (BlastHSP* , CRef<CSeq_id> , CRef<CSeq_id> , Int4 , Int4 ,
+	                      	   const vector<int> & ) = NULL;
+
+	if((TRANSLATED_QUERY_MASK | TRANSLATED_SUBJECT_MASK) & program )
+		fun_ptr = x_UngappedHSPToStdSeg;
+	else
+		fun_ptr = x_NonTranslatedHSPToStdSeg;
+
+    for (int i = 0; i < hit_list->hsplist_count; i++)
+    {
+    	BlastHSPList* hsp_list = hit_list->hsplist_array[i];
+	    if (!hsp_list)
+	    	continue;
+
+	    BlastHSP ** hsp_array = hsp_list->hsp_array;
+        for (int j = 0; j < hsp_list->hspcnt; j++)
+        {
+        	BlastHSP* hsp = hsp_array[j];
+
+        	if(!hsp)
+        		continue;
+
+        	const Uint4 oid = hsp_list->oid;
+        	TSeqPos subject_length = 0;
+        	CRef<CSeq_id> subject_id;
+        	vector<int> gi_list;
+        	GetFilteredRedundantGis(*subject_seqinfo, oid, gi_list);
+        	GetSequenceLengthAndId(subject_seqinfo, oid, subject_id, &subject_length);
+        	seg_list.push_back((*fun_ptr) (hsp, query_id, subject_id,
+        									query_length, subject_length,
+        									 gi_list));
+        }
+    }
+}
+
 END_SCOPE(blast)
 END_NCBI_SCOPE
 
