@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: prelim_stage.cpp 321879 2011-07-26 16:09:25Z camacho $";
+    "$Id: prelim_stage.cpp 369385 2012-07-18 19:41:03Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 /* ===========================================================================
  *
@@ -47,6 +47,7 @@ static char const rcsid[] =
 #include "blast_aux_priv.hpp"
 #include "psiblast_aux_priv.hpp"
 #include "split_query_aux_priv.hpp"
+#include "blast_seqalign.hpp"
 #include <sstream>
 
 #include <algo/blast/api/blast_dbindex.hpp>
@@ -60,11 +61,13 @@ BEGIN_NCBI_SCOPE
 USING_SCOPE(objects);
 BEGIN_SCOPE(blast)
 
+
+
 CBlastPrelimSearch::CBlastPrelimSearch(CRef<IQueryFactory> query_factory,
                                        CRef<CBlastOptions> options,
                                        const CSearchDatabase& dbinfo)
     : m_QueryFactory(query_factory), m_InternalData(new SInternalData),
-    m_Options(options)
+    m_Options(options), m_DbAdapter(NULL), m_DbInfo(&dbinfo)
 {
     BlastSeqSrc* seqsrc = CSetupFactory::CreateBlastSeqSrc(dbinfo);
     x_Init(query_factory, options, CRef<CPssmWithParameters>(), seqsrc);
@@ -76,7 +79,7 @@ CBlastPrelimSearch::CBlastPrelimSearch(CRef<IQueryFactory> query_factory,
                                        CRef<CBlastOptions> options,
                                        CRef<CLocalDbAdapter> db)
     : m_QueryFactory(query_factory), m_InternalData(new SInternalData),
-    m_Options(options)
+    m_Options(options), m_DbAdapter(db), m_DbInfo(NULL)
 {
     BlastSeqSrc* seqsrc = db->MakeSeqSrc();
     x_Init(query_factory, options, CRef<CPssmWithParameters>(), seqsrc);
@@ -88,7 +91,7 @@ CBlastPrelimSearch::CBlastPrelimSearch(CRef<IQueryFactory> query_factory,
                                BlastSeqSrc* seqsrc,
                                CConstRef<objects::CPssmWithParameters> pssm)
     : m_QueryFactory(query_factory), m_InternalData(new SInternalData),
-    m_Options(options)
+    m_Options(options),  m_DbAdapter(NULL), m_DbInfo(NULL)
 {
     x_Init(query_factory, options, pssm, seqsrc);
     m_InternalData->m_SeqSrc.Reset(new TBlastSeqSrc(seqsrc, 0));
@@ -157,6 +160,11 @@ CBlastPrelimSearch::x_LaunchMultiThreadedSearch(SInternalData& internal_data)
         }
     }
 
+    // Inform indexing library about the number of concurrent
+    // search threads.
+    //
+    GetDbIndexSetNumThreadsFn()( GetNumberOfThreads() );
+
     // ... launch the threads ...
     NON_CONST_ITERATE(TBlastThreads, thread, the_threads) {
         (*thread)->Run();
@@ -202,13 +210,10 @@ CBlastPrelimSearch::Run()
 
     auto_ptr<const CBlastOptionsMemento> opts_memento
         (m_Options->CreateSnapshot());
-    BlastSeqSrc * seqsrc = m_InternalData->m_SeqSrc->GetPointer();
     BLAST_SequenceBlk* queries = m_InternalData->m_Queries;
     LookupTableOptions * lut_options = opts_memento->m_LutOpts;
     BlastInitialWordOptions * word_options = opts_memento->m_InitWordOpts;
 
-    GetDbIndexSetNumThreadsFn()( seqsrc, GetNumberOfThreads() );
-
     // Query splitting data structure (used only if applicable)
     CRef<SBlastSetupData> setup_data(new SBlastSetupData(m_QueryFactory, m_Options));
     CRef<CQuerySplitter> query_splitter = setup_data->m_QuerySplitter;
@@ -231,8 +236,9 @@ CBlastPrelimSearch::Run()
                         chunk_qf->MakeLocalQueryData( &*m_Options ) );
                 BLAST_SequenceBlk * chunk_queries = 
                     query_data->GetSequenceBlk();
+                GetDbIndexSetUsingThreadsFn()( IsMultiThreaded() );
                 GetDbIndexRunSearchFn()( 
-                        seqsrc, chunk_queries, lut_options, word_options );
+                        chunk_queries, lut_options, word_options );
 
                 if (IsMultiThreaded()) {
                      x_LaunchMultiThreadedSearch(*chunk_data);
@@ -284,8 +290,8 @@ CBlastPrelimSearch::Run()
         }
     } else {
 
-        GetDbIndexRunSearchFn()( 
-                seqsrc, queries, lut_options, word_options );
+        GetDbIndexSetUsingThreadsFn()( IsMultiThreaded() );
+        GetDbIndexRunSearchFn()( queries, lut_options, word_options );
 
         if (IsMultiThreaded()) {
              x_LaunchMultiThreadedSearch(*m_InternalData);
@@ -309,36 +315,135 @@ CBlastPrelimSearch::CheckInternalData()
     return retval;
 }
 
+
 BlastHSPResults*
 CBlastPrelimSearch::ComputeBlastHSPResults(BlastHSPStream* stream,
                                            Uint4 max_num_hsps,
-                                           bool* rm_hsps) const
+					   bool* rm_hsps,
+					   vector<bool> *rm_hsps_info) const
 {
+    bool any_query_hsp_limited = false;
     auto_ptr<const CBlastOptionsMemento> opts_memento
         (m_Options->CreateSnapshot());
 
     _ASSERT(m_InternalData->m_QueryInfo->num_queries > 0);
-    Boolean removed_hsps = FALSE;
+    Boolean *removed_hsps = new Boolean [ m_InternalData->m_QueryInfo->num_queries ];
     SBlastHitsParameters* hit_param = NULL;
     SBlastHitsParametersNew(opts_memento->m_HitSaveOpts,
                             opts_memento->m_ExtnOpts,
                             opts_memento->m_ScoringOpts,
                             &hit_param);
     BlastHSPResults* retval =
-        Blast_HSPResultsFromHSPStreamWithLimit(stream,
+        Blast_HSPResultsFromHSPStreamWithLimitEx(stream,
            (Uint4) m_InternalData->m_QueryInfo->num_queries,
            hit_param,
            max_num_hsps,
-           &removed_hsps);
-    if (rm_hsps) {
-        *rm_hsps = removed_hsps == FALSE ? false : true;
+           removed_hsps);
+    if( rm_hsps_info){ 
+	rm_hsps_info->reserve(m_InternalData->m_QueryInfo->num_queries );
+        for( int query_index = 0 ; query_index < m_InternalData->m_QueryInfo->num_queries ; query_index ++ ){
+	  (*rm_hsps_info)[ query_index ] = removed_hsps[query_index] == FALSE ? false : true;
+	  if( (*rm_hsps_info)[ query_index ] ) any_query_hsp_limited = true;
+        }
     }
+    delete [] removed_hsps;
+    if( rm_hsps ) *rm_hsps = any_query_hsp_limited ;
     // applications assume the HSPLists in the HSPResults are
     // sorted in order of worsening best e-value
     Blast_HSPResultsSortByEvalue(retval);
     return retval;
 }
 
+bool CBlastPrelimSearch::Run( vector<list<CRef<CStd_seg> > >  & l )
+{
+	Run();
+	return x_BuildStdSegList(l);
+}
+
+void s_FixNumIdent(BlastHSPList *hsp_list, bool gapped_calculation)
+{
+	   BlastHSP* hsp;
+	   int i;
+
+	   for (i=0; i < hsp_list->hspcnt; i++)
+	   {
+	      hsp = hsp_list->hsp_array[i];
+	      if (gapped_calculation)
+	    	  hsp->num_ident = -1;
+	   }
+}
+
+void s_GetBitScores(BlastHitList * hit_list, bool gapped_calculation, const BlastScoreBlk * sbp)
+{
+
+	 for (int i = 0; i < hit_list->hsplist_count; i++)
+	 {
+	    	BlastHSPList* hsp_list = hit_list->hsplist_array[i];
+		    if (!hsp_list)
+		    	continue;
+
+		    Blast_HSPListGetBitScores(hsp_list, gapped_calculation, sbp);
+		    s_FixNumIdent(hsp_list, gapped_calculation);
+	 }
+}
+
+// Results is trimmed by Blast Hits Save Options if set
+bool CBlastPrelimSearch::x_BuildStdSegList( vector<list<CRef<CStd_seg> > >  & l )
+{
+	if(m_InternalData->m_HspStream.Empty())
+	{
+		_TRACE("HSP Stream is empty");
+		return false;
+	}
+
+	if(NULL != m_DbInfo)
+	{
+		m_DbAdapter.Reset(new CLocalDbAdapter(*m_DbInfo));
+	}
+
+	if(m_DbAdapter.Empty())
+	{
+		_TRACE("This method does not support CBlastPrelimSearch constructed with BlastSeqSrc");
+		return false;
+	}
+
+	BlastHSPStream * hsp_stream  = m_InternalData->m_HspStream->GetPointer();
+	if(NULL == hsp_stream)
+	{
+		_TRACE("NULL HSP Stream Pointer");
+		return false;
+	}
+
+	IBlastSeqInfoSrc * s_seqInfoSrc = m_DbAdapter->MakeSeqInfoSrc();
+	EBlastProgramType program = hsp_stream->program;
+
+	BlastHSPResults * results = ComputeBlastHSPResults(hsp_stream );
+
+	if(NULL == results)
+		return false;
+
+	int num_queries = results->num_queries;
+
+	BlastHitList ** q_list_ptr = results->hitlist_array;
+	CRef<ILocalQueryData> local_query_data = m_QueryFactory->MakeLocalQueryData(m_Options.GetPointer());
+	l.resize(num_queries);
+	const BlastScoreBlk * sbp = m_InternalData->m_ScoreBlk->GetPointer();
+	bool gapped_cal = m_Options->GetGappedMode();
+	for(int i=0; i < num_queries; i++)
+	{
+		CConstRef<CSeq_loc> query_loc = local_query_data->GetSeq_loc(i);
+		TSeqPos	query_length = local_query_data->GetSeqLength(i);
+		BlastHitList * hit_list = q_list_ptr[i];
+		if(NULL != hit_list)
+		{
+
+			s_GetBitScores(hit_list, gapped_cal, sbp);
+			BLASTPrelminSearchHitListToStdSeg(program, hit_list, *query_loc, query_length, s_seqInfoSrc, l[i]);
+		}
+	}
+
+	return true;
+}
 END_SCOPE(blast)
 END_NCBI_SCOPE
 
