@@ -1,4 +1,4 @@
-/*  $Id: remote_blast.cpp 334322 2011-09-06 14:50:26Z fongah2 $ 
+/*  $Id: remote_blast.cpp 368805 2012-07-12 16:14:48Z rafanovi $ 
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1140,6 +1140,7 @@ void CRemoteBlast::SetQueries(CRef<objects::CPssmWithParameters> pssm)
     string psi_program("blastp");
     string old_service("plain");
     string new_service("psi");
+    string delta_service("delta_blast");
     
     if (m_QSR->GetProgram() != psi_program) {
         NCBI_THROW(CBlastException, eNotSupported,
@@ -1152,7 +1153,8 @@ void CRemoteBlast::SetQueries(CRef<objects::CPssmWithParameters> pssm)
     }
     
     if ((m_QSR->GetService() != old_service) &&
-        (m_QSR->GetService() != new_service)) {
+        (m_QSR->GetService() != new_service) &&
+        (m_QSR->GetService() != delta_service)) {
         
         // Allowing "psi" allows the matrix to be set, then replaced.
         
@@ -1166,8 +1168,10 @@ void CRemoteBlast::SetQueries(CRef<objects::CPssmWithParameters> pssm)
     
     m_QSR->SetQueries(*queries_p);
     m_NeedConfig = ENeedConfig(m_NeedConfig & (~ eQueries));
-    
-    m_QSR->SetService(new_service);
+
+    if(m_QSR->GetService() != delta_service) {
+        m_QSR->SetService(new_service);
+    }
 }
 
 string CRemoteBlast::GetErrors(void)
@@ -1842,7 +1846,7 @@ NetworkProgram2BlastProgramType(const string& program, const string& service)
 
         if (program == "blastp") {
             retval = eBlastTypeRpsBlast;
-        } else if (program == "tblastn") {
+        } else if (program == "tblastn" || program == "blastx") {
             retval = eBlastTypeRpsTblastn;
         } else {
             abort();
@@ -2075,6 +2079,18 @@ CRef<CSearchResultSet> CRemoteBlast::GetResultSet()
             }
         }
 
+        if((m_Service == "rpsblast") || (m_Service == "rpstblastn") )
+        {
+        	const string kTmp("Matrix: ");
+        	 NON_CONST_ITERATE(list<string>, itr, search_stats) {
+        		 if (NStr::Find(*itr, kTmp) != NPOS) {
+        			 NStr::ReplaceInPlace(*itr, kTmp, kEmptyStr);
+        			 m_CBOH->SetOptions().SetMatrixName((*itr).c_str());
+        			 break;
+        		 }
+        	 }
+        }
+
         /* Get the Karlin-Altschul parameters */
         bool found_gapped = false, found_ungapped = false;
         pair<double, double> lambdas, Ks, Hs;
