@@ -1,4 +1,4 @@
-/*  $Id: blast_input_aux.cpp 314251 2011-07-21 13:01:52Z maning $
+/*  $Id: blast_input_aux.cpp 366497 2012-06-14 16:55:59Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_input_aux.cpp 314251 2011-07-21 13:01:52Z maning $";
+    "$Id: blast_input_aux.cpp 366497 2012-06-14 16:55:59Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -86,10 +86,12 @@ GetQueryBatchSize(EProgram program, bool is_ungapped /* = false */)
 
     switch (program) {
     case eBlastn:
-        retval = 1000000;
+        retval = 100000;
         break;
-    case eMegablast:
     case eDiscMegablast:
+	retval = 500000;
+	break;
+    case eMegablast:
         retval = 5000000;
         break;
     case eTblastn:
