@@ -1,4 +1,4 @@
-/*  $Id: cmdline_flags.cpp 350250 2012-01-18 21:22:09Z rafanovi $
+/*  $Id: cmdline_flags.cpp 368006 2012-06-29 19:48:05Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: cmdline_flags.cpp 350250 2012-01-18 21:22:09Z rafanovi $";
+    "$Id: cmdline_flags.cpp 368006 2012-06-29 19:48:05Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -194,6 +194,8 @@ const string kArgOutputSearchStrategy("export_search_strategy");
 
 const string kArgUseIndex("use_index");
 const string kArgIndexName("index_name");
+const string kArgOldStyleIndex("old_style_index");
+const string kDfltArgOldStyleIndex("true");
 
 const string kArgEntrezQuery("entrez_query");
 
