@@ -1,4 +1,4 @@
-/* $Id: blast_engine.c 345770 2011-11-30 13:58:31Z madden $
+/* $Id: blast_engine.c 370872 2012-08-02 17:19:24Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -55,7 +55,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_engine.c 345770 2011-11-30 13:58:31Z madden $";
+    "$Id: blast_engine.c 370872 2012-08-02 17:19:24Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_engine.h>
@@ -84,8 +84,8 @@ static char const rcsid[] =
 
 NCBI_XBLAST_EXPORT const int   kBlastMajorVersion = 2;
 NCBI_XBLAST_EXPORT const int   kBlastMinorVersion = 2;
-NCBI_XBLAST_EXPORT const int   kBlastPatchVersion = 26;
-NCBI_XBLAST_EXPORT const char* kBlastReleaseDate = "Sep-21-2011";
+NCBI_XBLAST_EXPORT const int   kBlastPatchVersion = 27;
+NCBI_XBLAST_EXPORT const char* kBlastReleaseDate = "Aug-12-2012";
 
 /** Structure to be passed to s_BlastSearchEngineCore, containing pointers 
     to various preallocated structures and arrays. */
@@ -252,7 +252,7 @@ static Int2 s_GetNextSubjectChunk(BLAST_SequenceBlk* subject,
     (subject->chunk)++;
 
     /* if no chunking is performed */
-    if (backup->offset == 0 && backup->next == backup->full_range.right) {
+    if (backup->offset == 0 && residual == 0 && backup->next == backup->full_range.right) {
         subject->seq_ranges = backup->soft_ranges;
         subject->num_seq_ranges = backup->num_soft_ranges;
         return SUBJECT_SPLIT_OK;
@@ -883,7 +883,8 @@ s_BlastSetUpAuxStructures(const BlastSeqSrc* seq_src,
     Boolean blastp = (lookup_wrap->lut_type == eAaLookupTable ||
                      lookup_wrap->lut_type == eCompressedAaLookupTable);
     Boolean rpsblast = (lookup_wrap->lut_type == eRPSLookupTable);
-    Boolean indexed_mb_lookup = (lookup_wrap->lut_type == eIndexedMBLookupTable);
+    // Boolean indexed_mb_lookup = (lookup_wrap->lut_type == eIndexedMBLookupTable);
+    Boolean indexed_mb_lookup = (lookup_wrap->read_indexed_db != 0);
     Boolean phi_lookup = (lookup_wrap->lut_type == ePhiLookupTable ||
                          lookup_wrap->lut_type == ePhiNaLookupTable);
     Boolean smith_waterman = 
@@ -901,8 +902,10 @@ s_BlastSetUpAuxStructures(const BlastSeqSrc* seq_src,
 
     if (smith_waterman) {
         aux_struct->WordFinder = NULL;
+    /*
     } else if (indexed_mb_lookup) {
         aux_struct->WordFinder = MB_IndexedWordFinder;
+    */
     } else if (phi_lookup) {
         aux_struct->WordFinder = PHIBlastWordFinder;
     } else if (blastp) {
@@ -911,9 +914,17 @@ s_BlastSetUpAuxStructures(const BlastSeqSrc* seq_src,
     } else if (rpsblast) {
         aux_struct->WordFinder = BlastRPSWordFinder;
     } else {
-        BlastChooseNucleotideScanSubject(lookup_wrap);
-        BlastChooseNaExtend(lookup_wrap);
-        aux_struct->WordFinder = BlastNaWordFinder;
+        if( lookup_wrap->lut_type != eIndexedMBLookupTable ) {
+            BlastChooseNucleotideScanSubject(lookup_wrap);
+            BlastChooseNaExtend(lookup_wrap);
+        }
+
+        if( indexed_mb_lookup ) {
+            aux_struct->WordFinder = MB_IndexedWordFinder;
+        }
+        else {
+            aux_struct->WordFinder = BlastNaWordFinder;
+        }
     }
     
     aux_struct->offset_pairs = 
@@ -1099,6 +1110,10 @@ BLAST_PreliminarySearchEngine(EBlastProgramType program_number,
     const Boolean kNucleotide = (program_number == eBlastTypeBlastn ||
                                 program_number == eBlastTypePhiBlastn);
 
+    T_MB_IdbCheckOid check_index_oid = 
+        (T_MB_IdbCheckOid)lookup_wrap->check_index_oid;
+    Int4 last_vol_idx = LAST_VOL_IDX_INIT;
+
     BlastInitialWordParametersNew(program_number, word_options, 
       hit_params, lookup_wrap, sbp, query_info, 
       BlastSeqSrcGetAvgSeqLen(seq_src), &word_params);
@@ -1144,6 +1159,12 @@ BLAST_PreliminarySearchEngine(EBlastProgramType program_number,
        Int4 stat_length;
        if (seq_arg.oid == BLAST_SEQSRC_ERROR)
            break;
+
+       if( check_index_oid != 0 && 
+               check_index_oid( seq_arg.oid, &last_vol_idx ) == eNoResults ) {
+           continue;
+       }
+
        if (BlastSeqSrcGetSequence(seq_src, &seq_arg) < 0)
            continue;
        if (db_length == 0) {
@@ -1202,6 +1223,14 @@ BLAST_PreliminarySearchEngine(EBlastProgramType program_number,
                             query_info, sbp, score_params, seq_src, 
                             seq_arg.seq->gen_code_string);
                if (status) {
+                  /* Tell the indexing library that this thread is done with
+                     preliminary search.
+                  */
+                  if( check_index_oid != 0 ) {
+                    ((T_MB_IdxEndSearchIndication)( 
+                        lookup_wrap->end_search_indication))( last_vol_idx );
+                  }
+         
                   BlastSeqSrcReleaseSequence(seq_src, &seq_arg);
                   return status;
                }
@@ -1262,6 +1291,14 @@ BLAST_PreliminarySearchEngine(EBlastProgramType program_number,
       }
     }
     
+    /* Tell the indexing library that this thread is done with
+       preliminary search.
+    */
+    if( check_index_oid != 0 ) {
+        ((T_MB_IdxEndSearchIndication)( 
+            lookup_wrap->end_search_indication))( last_vol_idx );
+    }
+
     hsp_list = Blast_HSPListFree(hsp_list);  /* in case we were interrupted */
     BlastSequenceBlkFree(seq_arg.seq);
     itr = BlastSeqSrcIteratorFree(itr);
