@@ -1,4 +1,4 @@
-/*  $Id: mkindex_app.cpp 219421 2011-01-10 17:07:33Z morgulis $
+/*  $Id: mkindex_app.cpp 373198 2012-08-27 16:27:09Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -116,6 +116,10 @@ void CMkIndexApplication::Init()
             "stride", "stride",
             "distance between stored database positions",
             CArgDescriptions::eInteger );
+    arg_desc->AddDefaultKey(
+            "old_style_index", "boolean",
+            "Use old style index (deprecated)",
+            CArgDescriptions::eBoolean, "true" );
     arg_desc->SetConstraint( 
             "verbosity",
             &(*new CArgAllow_Strings, "quiet", "normal", "verbose") );
@@ -148,6 +152,8 @@ int CMkIndexApplication::Run()
     CDbIndex::SOptions options = CDbIndex::DefaultSOptions();
     std::string verbosity = GetArgs()["verbosity"].AsString();
 
+    bool old_style( GetArgs()["old_style_index"].AsBoolean() );
+
     if( verbosity == "quiet" ) {
         options.report_level = REPORT_QUIET;
     }else if( verbosity == "verbose" ) {
@@ -177,7 +183,7 @@ int CMkIndexApplication::Run()
         else options.stride = GetArgs()["stride"].AsInteger();
     }
 
-    if( GetArgs()["ws_hint"] )
+    if( GetArgs()["ws_hint"] ) {
         if( options.legacy ) {
             ERR_POST( Warning << "-ws_hint has no effect upon "
                                  "legacy index creation" );
@@ -193,12 +199,14 @@ int CMkIndexApplication::Run()
 
             options.ws_hint = ws_hint;
         }
+    }
 
     unsigned int vol_num = 0;
 
     CDbIndex::TSeqNum start, orig_stop( kMax_UI4 ), stop = 0;
     string ofname_base = 
         GetArgs()["show_filters"] ? "" : GetArgs()["output"].AsString();
+    string odir_name( CFile( ofname_base ).GetDir() );
     CSequenceIStream * seqstream = 0;
     string iformat = GetArgs()["iformat"].AsString();
 
@@ -221,14 +229,16 @@ int CMkIndexApplication::Run()
                 return 0;
             }
 
-            if( GetArgs()["db_mask"] ) {
-                seqstream = new CSequenceIStreamBlastDB( 
-                        GetArgs()["input"].AsString(), true,
-                        GetArgs()["db_mask"].AsInteger() );
-            }
-            else {
-                seqstream = new CSequenceIStreamBlastDB( 
-                        GetArgs()["input"].AsString(), false );
+            if( old_style ) {
+                if( GetArgs()["db_mask"] ) {
+                    seqstream = new CSequenceIStreamBlastDB( 
+                            GetArgs()["input"].AsString(), true,
+                            GetArgs()["db_mask"].AsInteger() );
+                }
+                else {
+                    seqstream = new CSequenceIStreamBlastDB( 
+                            GetArgs()["input"].AsString(), false );
+                }
             }
         }
         else {
@@ -246,17 +256,108 @@ int CMkIndexApplication::Run()
         exit( 1 );
     }
 
+    if( !old_style && iformat == "fasta" ) {
+        ERR_POST( Error << "new style index requires input format 'blastdb'" );
+        exit( 1 );
+    }
+
+    if( !old_style && iformat == "blastdb" ) {
+        typedef std::vector< std::string > TStrVec;
+        TStrVec db_vols;
+
+        // Enumerate BLAST database volumes.
+        {
+            std::string ifname( GetArgs()["input"].AsString() );
+            CSeqDB db( ifname, CSeqDB::eNucleotide, 0, 0, false );
+            db.FindVolumePaths( db_vols, true );
+        }
+
+        bool enable_mask( GetArgs()["db_mask"] );
+        int filter( enable_mask ? GetArgs()["db_mask"].AsInteger() : 0 );
+
+        ITERATE( TStrVec, dbvi, db_vols ) {
+            seqstream = 
+                new CSequenceIStreamBlastDB( *dbvi, enable_mask, filter );
+            CDbIndex::TSeqNum start, orig_stop( kMax_UI4 ), stop = 0;
+            Uint4 vol_num_seq( 0 );
+
+            {
+                CSeqDB db( *dbvi, CSeqDB::eNucleotide, 0, 0, false );
+                vol_num_seq = db.GetNumOIDs();
+            }
+
+            Uint4 num_seq( 0 ), num_vol( 0 );
+            vol_num = 0;
+            std::string dbv_name( 
+                    CFile::ConcatPath( odir_name, CFile( *dbvi ).GetName() ) );
+            
+            do {
+                start = stop;
+                stop = orig_stop;
+                ostringstream os;
+                os << dbv_name << "." << setfill( '0' ) << setw( 2 ) 
+                   << vol_num++ << ".idx";
+                cerr << "creating " << os.str() << "..." << flush;
+                CDbIndex::MakeIndex( 
+                        *seqstream, os.str(), start, stop, options );
+                num_seq += (stop - start);
+
+                if( start == stop ) cerr << "removed (empty)" << endl;
+                else{ 
+                    ++num_vol; 
+                    cerr << "done" << endl;
+                    ERR_POST( Info << 
+                              "generated index volume with OIDs: " <<
+                              start << "--" << stop );
+                }
+            }
+            while( start != stop );
+
+            if( num_seq != vol_num_seq ) {
+                ERR_POST( Error << 
+                          "number of sequence reported by BLAST database"
+                          " volume (" << vol_num_seq << ") is not the same"
+                          " as in the index (" << num_seq << ")" );
+                return 1;
+            }
+
+            CIndexSuperHeader< 
+                CIndexSuperHeader_Base::INDEX_FORMAT_VERSION_1 > shdr( 
+                        num_seq, num_vol );
+            shdr.Save( dbv_name + ".shd" );
+            ERR_POST( Info << 
+                      "index generated for BLAST database volume " <<
+                      dbv_name << " with " << num_seq << " sequences" );
+            delete seqstream;
+        }
+
+        return 0;
+    }
+
+    Uint4 num_seq( 0 ), num_vol( 0 );
+
     do { 
         start = stop;
         stop = orig_stop;
         ostringstream os;
         os << ofname_base << "." << setfill( '0' ) << setw( 2 ) 
            << vol_num++ << ".idx";
-        cerr << "creating " << os.str() << endl;
+        cerr << "creating " << os.str() << "..." << flush;
         CDbIndex::MakeIndex( 
                 *seqstream,
                 os.str(), start, stop, options );
+        num_seq += (stop - start);
+
+        if( start == stop ) cerr << "removed (empty)" << endl;
+        else{ ++num_vol; cerr << "done" << endl; }
     }while( start != stop );
 
+    if( !old_style ) {
+        CIndexSuperHeader< 
+            CIndexSuperHeader_Base::INDEX_FORMAT_VERSION_1 > shdr(
+                    num_seq, num_vol );
+        shdr.Save( ofname_base + ".shd" );
+    }
+
     return 0;
 }
