@@ -1,5 +1,5 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: blast_format.cpp 348029 2011-12-22 17:32:54Z maning $";
+static char const rcsid[] = "$Id: blast_format.cpp 367113 2012-06-20 19:26:08Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 /*
@@ -245,6 +245,12 @@ CBlastFormat::PrintProlog()
     }
 
     m_Outfile << "\n\n";
+    if (m_Program == "deltablast") {
+        CBlastFormatUtil::BlastPrintReference(m_IsHTML, kFormatLineLength, 
+                              m_Outfile, CReference::eDeltaBlast);
+        m_Outfile << "\n";
+    }
+
     if (m_Megablast)
         CBlastFormatUtil::BlastPrintReference(m_IsHTML, kFormatLineLength, 
                                           m_Outfile, CReference::eMegaBlast);
@@ -254,27 +260,30 @@ CBlastFormat::PrintProlog()
 
     if (m_Megablast && m_IndexedMegablast)
     {
-        m_Outfile << "\n\n";
+        m_Outfile << "\n";
         CBlastFormatUtil::BlastPrintReference(m_IsHTML, kFormatLineLength, 
                               m_Outfile, CReference::eIndexedMegablast);
     }
 
-    if (m_Program == "psiblast") {
-        m_Outfile << "\n\n";
+    if (m_Program == "psiblast" || m_Program == "deltablast") {
+        m_Outfile << "\n";
         CBlastFormatUtil::BlastPrintReference(m_IsHTML, kFormatLineLength, 
                               m_Outfile, CReference::eCompAdjustedMatrices);
     }
     if (m_Program == "psiblast" || m_Program == "blastp") {
-        m_Outfile << "\n\n";
+        m_Outfile << "\n";
         CBlastFormatUtil::BlastPrintReference(m_IsHTML, kFormatLineLength, 
                               m_Outfile, CReference::eCompBasedStats,
                               (bool)(m_Program == "psiblast"));
     }
 
-    if (m_Program == "deltablast" && !m_DomainDbInfo.empty()) {
-        m_Outfile << "\n\n" << "Conserved Domain ";
-        CBlastFormatUtil::PrintDbReport(m_DomainDbInfo, kFormatLineLength, 
+    if (m_Program == "deltablast" || !m_DomainDbInfo.empty()) {
+        m_Outfile << "\n\n";
+        if (!m_DomainDbInfo.empty()) {
+        	m_Outfile << "\n\n" << "Conserved Domain ";
+        	CBlastFormatUtil::PrintDbReport(m_DomainDbInfo, kFormatLineLength, 
                                         m_Outfile, true);
+        }
     }
     else {
         m_Outfile << "\n\n";
@@ -294,12 +303,13 @@ CBlastFormat::x_PrintOneQueryFooter(const blast::CBlastAncillaryData& summary)
         (m_Program == "psiblast" || m_Program == "deltablast")
         ? summary.GetPsiUngappedKarlinBlk() 
         : summary.GetUngappedKarlinBlk();
+    const Blast_GumbelBlk *gbp = summary.GetGumbelBlk();
     m_Outfile << "\n";
     if (kbp_ungap) {
         CBlastFormatUtil::PrintKAParameters(kbp_ungap->Lambda, 
                                             kbp_ungap->K, kbp_ungap->H,
                                             kFormatLineLength, m_Outfile,
-                                            false);
+                                            false, gbp);
     }
 
     const Blast_KarlinBlk *kbp_gap = 
@@ -311,7 +321,7 @@ CBlastFormat::x_PrintOneQueryFooter(const blast::CBlastAncillaryData& summary)
         CBlastFormatUtil::PrintKAParameters(kbp_gap->Lambda, 
                                             kbp_gap->K, kbp_gap->H,
                                             kFormatLineLength, m_Outfile,
-                                            true);
+                                            true, gbp);
     }
 
     m_Outfile << "\n";
@@ -397,8 +407,7 @@ void
 CBlastFormat::x_DisplayDeflines(CConstRef<CSeq_align_set> aln_set, 
                         unsigned int itr_num,
                         blast::CPsiBlastIterationState::TSeqIds& prev_seqids,
-                        int skip_from,
-                        int skip_to)
+                        int additional)
 {
 
     if (itr_num != numeric_limits<unsigned int>::max() && 
@@ -430,8 +439,8 @@ CBlastFormat::x_DisplayDeflines(CConstRef<CSeq_align_set> aln_set,
     } else {
         CShowBlastDefline showdef(*aln_set, *m_Scope, 
                                   kFormatLineLength,
-                                  m_NumSummary);
-        x_ConfigCShowBlastDefline(showdef, skip_from, skip_to);
+                                  m_NumSummary + additional);
+        x_ConfigCShowBlastDefline(showdef, -1, -1);
         showdef.DisplayBlastDefline(m_Outfile);
     }
     m_Outfile << "\n";
@@ -545,6 +554,8 @@ CBlastFormat::x_PrintTabularReport(const blast::CSearchResults& results,
              ? CBlastTabularInfo::eComma : CBlastTabularInfo::eTab);
         CBlastTabularInfo tabinfo(m_Outfile, m_CustomOutputFormatSpec, kDelim);
         tabinfo.SetParseLocalIds(m_BelieveQuery);
+        if (ncbi::NStr::ToLower(m_Program) == string("blastn"))
+        	tabinfo.SetNoFetch(true);
 
         if (m_FormatType == CFormattingArgs::eTabularWithComments) {
             string strProgVersion =
@@ -610,10 +621,11 @@ CBlastFormat::x_PrintIgTabularReport(const blast::CIgBlastResults& results)
         tabinfo.SetIgAnnotation(annots, m_IgOptions->m_IsProtein);
 
         tabinfo.PrintHeader(strProgVersion, *(bhandle.GetBioseqCore()),
-                                m_DbName, results.GetRID(), 
+                                m_DbName, 
+                                m_IgOptions->m_DomainSystem,
+                                results.GetRID(), 
                                 numeric_limits<unsigned int>::max(),
-                                aln_set, subject_bioseq, 
-                                m_IgOptions->m_DomainSystem);
+                                aln_set, subject_bioseq);
  
         int j = 1;
         for (; itr != aln_set->Get().end(); ++itr) {
@@ -624,10 +636,11 @@ CBlastFormat::x_PrintIgTabularReport(const blast::CIgBlastResults& results)
         }
     } else {
         tabinfo.PrintHeader(strProgVersion, *(bhandle.GetBioseqCore()),
-                                m_DbName, results.GetRID(), 
+                                m_DbName, 
+                                m_IgOptions->m_DomainSystem,
+                                results.GetRID(), 
                                 numeric_limits<unsigned int>::max(),
-                                0, subject_bioseq, 
-                                m_IgOptions->m_DomainSystem);
+                                0, subject_bioseq);
     }
     if (m_IsHTML) {
         m_Outfile << "\n</pre></body></html>\n";
@@ -762,9 +775,12 @@ CBlastFormat::PrintOneResultSet(const blast::CSearchResults& results,
     // other output types will need a bioseq handle
     CBioseq_Handle bhandle = m_Scope->GetBioseqHandle(*results.GetSeqId(),
                                                       CScope::eGetBioseq_All);
-    // If this assertion fails, we're not able to get the query, most likely a
-    // bug
-    _ASSERT(bhandle);
+    // If we're not able to get the query, most likely a bug. SB-981 , GP-2207
+    if( !bhandle  ){
+        string message = "Failed to resolve SeqId: "+results.GetSeqId()->AsFastaString();
+	ERR_POST(message);
+        NCBI_THROW(CException, eUnknown, message);
+    }
     CConstRef<CBioseq> bioseq = bhandle.GetBioseqCore();
 
     // print the preamble for this query
@@ -914,16 +930,6 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
                                             m_Outfile, m_BelieveQuery,
                                             m_IsHTML, kIsTabularOutput,
                                             results.GetRID());
-    if (m_IsBl2Seq) {
-        m_Outfile << "\n";
-        // FIXME: this might be configurable in the future
-        const bool kBelieveSubject = false; 
-        CConstRef<CBioseq> subject_bioseq = x_CreateSubjectBioseq();
-        CBlastFormatUtil::AcknowledgeBlastSubject(*subject_bioseq, 
-                                                  kFormatLineLength, 
-                                                  m_Outfile, kBelieveSubject, 
-                                                  m_IsHTML, kIsTabularOutput);
-    }
 
     // quit early if there are no hits
     if ( !results.HasAlignments() ) {
@@ -944,12 +950,8 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
     // print 1-line summaries
     if ( !m_IsBl2Seq ) {
         CPsiBlastIterationState::TSeqIds prev_ids = CPsiBlastIterationState::TSeqIds();
-        int skip_from = 0; 
-        int skip_to = 0;
-       /* int skip_to = results.m_NumActualV +results.m_NumActualD + results.m_NumActualJ;
-          if (m_IgOptions->m_Db[0]->GetDatabaseName() == m_DbName) 
-           skip_from = results.m_NumActualV; //keep V if the blast DB is the same as V germlines*/
-        x_DisplayDeflines(aln_set, numeric_limits<unsigned int>::max(), prev_ids, skip_from, skip_to);
+        int additional =  results.m_NumActualV +results.m_NumActualD + results.m_NumActualJ;
+        x_DisplayDeflines(aln_set, numeric_limits<unsigned int>::max(), prev_ids, additional);
     }
 
     //-------------------------------------------------
@@ -994,6 +996,7 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
     
     if (m_IsHTML) {
           flags += CDisplaySeqalign::eHtml;
+          flags += CDisplaySeqalign::eHyperLinkSlaveSeqid;
     }
 
     list < CRef<CDisplaySeqalign::DomainInfo> >  domain;
@@ -1062,11 +1065,23 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
     results.GetSubjectMasks(subj_masks);
     display.SetSubjectMasks(subj_masks);
 
+    if (m_IsHTML) {
+     
+        m_Outfile << "\n<CENTER><b><FONT color=\"green\">Alignments</FONT></b></CENTER>" 
+                  << endl;
+
+    } else {
+        m_Outfile << "\nAlignments" << endl;
+    }
+       
     display.DisplaySeqalign(m_Outfile);
 
     // print the ancillary data for this query
 
     x_PrintOneQueryFooter(*results.GetAncillaryData());
+    if (m_IsHTML) {
+        m_Outfile << "<hr>" << endl;
+    }
 }
 
 void 
