@@ -1,4 +1,4 @@
-/*  $Id: blastfmtutil.cpp 345774 2011-11-30 14:20:00Z fongah2 $
+/*  $Id: blastfmtutil.cpp 363312 2012-05-15 20:37:17Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,7 +31,7 @@
  */
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: blastfmtutil.cpp 345774 2011-11-30 14:20:00Z fongah2 $";
+static char const rcsid[] = "$Id: blastfmtutil.cpp 363312 2012-05-15 20:37:17Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -91,6 +91,8 @@ CBlastFormatUtil::BlastPrintReference(bool html, size_t line_len,
         }
     } else if (pub == blast::CReference::eIndexedMegablast) {
         reference += " for database indexing";
+    } else if (pub == blast::CReference::eDeltaBlast) {
+        reference += " for DELTA-BLAST";
     }
 
     ostringstream str;
@@ -500,7 +502,7 @@ CBlastFormatUtil::GetWholeAlnSeqStrings(string & query,
 								 	   int slave_gen_code)
 {
 	CAlnVec aln_vec(ds, scope);
-
+	aln_vec.SetAaCoding(CSeq_data::e_Ncbieaa);
 	s_GetQueryAndSubjectStrings(aln_vec, query, subject, master_gen_code, slave_gen_code);
 }
 
@@ -517,6 +519,7 @@ CBlastFormatUtil::GetWholeAlnSeqStrings(string & query,
 						   	   	   	   	int query_frame)
 {
 	CAlnVec aln_vec(ds, scope);
+	aln_vec.SetAaCoding(CSeq_data::e_Ncbieaa);
 
 	s_GetQueryAndSubjectStrings(aln_vec, query, subject, master_gen_code, slave_gen_code);
 
