@@ -1,4 +1,4 @@
-/* $Id: blastxml_format.cpp 348637 2012-01-03 14:26:31Z madden $
+/* $Id: blastxml_format.cpp 363312 2012-05-15 20:37:17Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,7 @@
 /// Formatting of BLAST results in XML form, using the BLAST XML specification.
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: blastxml_format.cpp 348637 2012-01-03 14:26:31Z madden $";
+static char const rcsid[] = "$Id: blastxml_format.cpp 363312 2012-05-15 20:37:17Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -666,6 +666,8 @@ s_GetBlastPublication(EProgram program)
         publication = CReference::ePhiBlast; break;
     case ePSIBlast:
         publication = CReference::eCompBasedStats; break;
+    case eDeltaBlast:
+        publication = CReference::eDeltaBlast; break;
     default:
         publication = CReference::eGappedBlast; break;
     }
