@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: igblast.cpp 347049 2011-12-13 19:02:41Z maning $";
+    "$Id: igblast.cpp 359118 2012-04-10 19:00:13Z maning $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 /* ===========================================================================
  *
@@ -78,7 +78,7 @@ CIgAnnotationInfo::CIgAnnotationInfo(CConstRef<CIgBlastOptions> &ig_opt)
 
     // read domain info from pdm or ndm file
     const string suffix = (ig_opt->m_IsProtein) ? ".pdm." : ".ndm.";
-    string fn(SeqDB_ResolveDbPath("internal_data/" + ig_opt->m_Origin + "/" 
+    string fn(SeqDB_ResolveDbPath(ig_opt->m_IgDataPath + "/" + ig_opt->m_Origin + "/" 
                                + ig_opt->m_Origin + suffix + ig_opt->m_DomainSystem));
     if (fn == "") {
         NCBI_THROW(CBlastException,  eInvalidArgument, 
@@ -104,6 +104,7 @@ CIgAnnotationInfo::CIgAnnotationInfo(CConstRef<CIgBlastOptions> &ig_opt)
     }
 
     // read frame info from aux files
+    if (ig_opt->m_IsProtein) return;
     fn = ig_opt->m_AuxFilename;
     s_ReadLinesFromFile(fn, lines);
     if (lines.size() == 0) {
