@@ -1,4 +1,4 @@
-/*  $Id: blasthits_unit_test.cpp 195768 2010-06-25 17:12:38Z maning $
+/*  $Id: blasthits_unit_test.cpp 346540 2011-12-07 17:48:50Z fongah2 $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -590,17 +590,16 @@ BOOST_AUTO_TEST_CASE(BlastTargetSequence)
                                                 BLAST_SequenceBlk* * subject_blk)
     {
         const int kLength = 12;
-        const Uint1 kQuerySeq[kLength] = 
-           { 11, 19, 13, 1, 9, 22, 6, 10, 7, 12, 20, 10 };
+        const Uint1 kQuerySeq[kLength + 2] =
+           { 0, 11, 19, 13, 1, 9, 22, 6, 10, 7, 12, 20, 10, 0 };
         const Uint1 kSubjectSeq[3*kLength] = 
            { 2, 8, 4, 4, 8, 4, 1, 1, 10, 8, 15, 2, 1, 8, 1, 8, 8, 15, 8, 8, 8,
              1, 5, 1, 4, 4, 2, 1, 1, 1, 8, 4, 4, 4, 1, 4 };
 
         BlastSeqBlkNew(query_blk);
         BlastSeqBlkSetSequence(*query_blk, 
-                               (Uint1*) BlastMemDup(kQuerySeq, kLength), 
+                               (Uint1*) BlastMemDup(kQuerySeq, kLength+2),
                                kLength);
-        (*query_blk)->sequence = (*query_blk)->sequence_start;
         BlastSeqBlkNew(subject_blk);
         BlastSeqBlkSetSequence(*subject_blk, 
                                (Uint1*) BlastMemDup(kSubjectSeq, 3*kLength), 
@@ -864,8 +863,8 @@ BOOST_AUTO_TEST_CASE(BlastTargetSequence)
         hit_params->cutoffs[0].cutoff_score = hit_params->cutoff_score_min;
 
         s_SetupHSPForGappedReevaluateTest(&hsp);
-        Blast_HSPReevaluateWithAmbiguitiesGapped(hsp, query_blk->sequence, 
-            subject_seq, hit_params, scoring_params, sbp);
+        Blast_HSPReevaluateWithAmbiguitiesGapped(hsp, query_blk->sequence, query_blk->length,
+            subject_seq, subj_length, hit_params, scoring_params, sbp);
 
         // With e-value 1, the low-scoring front piece of the alignment is 
         // cut off.
@@ -879,8 +878,8 @@ BOOST_AUTO_TEST_CASE(BlastTargetSequence)
         hit_params->cutoffs[0].cutoff_score = hit_params->cutoff_score_min;
         
         s_SetupHSPForGappedReevaluateTest(&hsp);
-        Blast_HSPReevaluateWithAmbiguitiesGapped(hsp, query_blk->sequence, 
-            subject_seq, hit_params, scoring_params, sbp);
+        Blast_HSPReevaluateWithAmbiguitiesGapped(hsp, query_blk->sequence, query_blk->length,
+            subject_seq, subj_length, hit_params, scoring_params, sbp);
         // With e-value 10, the front piece of the alignment is left, and the 
         // remainder is cut off.
         checkReevaluateResultsGapped(hsp);
@@ -950,7 +949,7 @@ BOOST_AUTO_TEST_CASE(BlastTargetSequence)
         hit_params->cutoffs[0].cutoff_score = hit_params->cutoff_score_min;
 
         Boolean delete_hsp = 
-            Blast_HSPReevaluateWithAmbiguitiesGapped(hsp, query, subject, 
+            Blast_HSPReevaluateWithAmbiguitiesGapped(hsp, query, kLength, subject, kLength,
                                                      hit_params, score_params,
                                                      sbp);
         BOOST_REQUIRE(delete_hsp == TRUE);
@@ -1145,13 +1144,13 @@ BOOST_AUTO_TEST_CASE(BlastTargetSequence)
         program = eBlastTypePhiBlastp;
         BOOST_REQUIRE_EQUAL(kHspCountStart, 
                              Blast_HSPListPurgeHSPsWithCommonEndpoints(program, 
-                                                                       hsp_list));
+                                                                       hsp_list, FALSE));
 
         // Now check that for a non-PHI program the routine works properly.
         program = eBlastTypeBlastp;
         BOOST_REQUIRE_EQUAL(kHspCountEnd, 
                              Blast_HSPListPurgeHSPsWithCommonEndpoints(program, 
-                                                                       hsp_list));
+                                                                       hsp_list, FALSE));
 
         BlastHSP** hsp_array = hsp_list->hsp_array;
         for (int index = 0; index < kHspCountEnd; ++index) {
