@@ -1,4 +1,4 @@
-/*  $Id: remote_blast_unit_test.cpp 331346 2011-08-16 14:53:07Z camacho $
+/*  $Id: remote_blast_unit_test.cpp 372443 2012-08-17 14:53:31Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -523,7 +523,8 @@ BOOST_AUTO_TEST_CASE(SubmitNullQueries) {
 BOOST_AUTO_TEST_CASE_TIMEOUT(CheckPrimerBlastRID, 45);
 BOOST_AUTO_TEST_CASE(CheckPrimerBlastRID) {
     // Permanent RID provided by Jian
-    const string rid("1295554329-7380-64666159365.BLASTQ3");
+    const string rid("TNNF2YHZ016"); 
+
     CRemoteBlast rmt_blaster(rid);
     
     BOOST_REQUIRE_EQUAL(rid, rmt_blaster.GetRID());
@@ -570,7 +571,7 @@ BOOST_AUTO_TEST_CASE(CheckColoRID) {
 BOOST_AUTO_TEST_CASE(GetErrorsFromFailedRID) {
     // Uncomment to redirect to test system
     //CAutoEnvironmentVariable tmp_env("BLAST4_CONN_SERVICE_NAME", "blast4_test");
-    const string rid("1214512158-10611-186074495131.BLASTQ23"); // Permanent RID
+    const string rid("2MSX0X8M01N" );
     CRemoteBlast rmt_blaster(rid);
     //rmt_blaster.SetVerbose();
     
@@ -580,7 +581,8 @@ BOOST_AUTO_TEST_CASE(GetErrorsFromFailedRID) {
     BOOST_REQUIRE_EQUAL(rid, rmt_blaster.GetRID());
     BOOST_REQUIRE_EQUAL(true, rmt_blaster.CheckDone());
     BOOST_REQUIRE_EQUAL(kEmptyStr, rmt_blaster.GetWarnings());
-    const string error("Substitution scores 2 and -1 are not supported");
+    
+    const string error("CPU usage limit was exceeded");
     BOOST_REQUIRE(NStr::FindNoCase(rmt_blaster.GetErrors(), error) != NPOS);
     BOOST_REQUIRE_EQUAL(CRemoteBlast::eStatus_Failed, 
                         rmt_blaster.CheckStatus());
@@ -606,23 +608,20 @@ BOOST_AUTO_TEST_CASE(RetrieveInvalidRID) {
 BOOST_AUTO_TEST_CASE(RetrieveRIDWithError) {
     // Uncomment to redirect to test system
     //CAutoEnvironmentVariable tmp_env("BLAST4_CONN_SERVICE_NAME", "blast4_test");
-    const string rid("1213977851-19717-200807286580.BLASTQ23");
+    const string rid("2RCK3CZR01N");
     CRemoteBlast rmt_blaster(rid);
     //rmt_blaster.SetVerbose();
 
     BOOST_REQUIRE_EQUAL(rid, rmt_blaster.GetRID());
     BOOST_REQUIRE_EQUAL(true, rmt_blaster.CheckDone());
-    //cerr << "Errors: '" << rmt_blaster.GetErrors() << "'" << endl;
-    BOOST_REQUIRE(NStr::Find(rmt_blaster.GetErrors(), "Error: File") != NPOS);
-    BOOST_REQUIRE(NStr::Find(rmt_blaster.GetErrors(), "not found.") != NPOS);
-    BOOST_REQUIRE_EQUAL(CRemoteBlast::eStatus_Failed,
-                        rmt_blaster.CheckStatus());
+    BOOST_REQUIRE(NStr::Find(rmt_blaster.GetErrors(), "CPU usage limit was exceeded, resulting in SIGXCPU") != NPOS);
+    BOOST_REQUIRE_EQUAL(CRemoteBlast::eStatus_Failed, rmt_blaster.CheckStatus());
 }
 
 BOOST_AUTO_TEST_CASE(RetrieveRIDWithSIGXCPU) {
     // Uncomment to redirect to test system
     //CAutoEnvironmentVariable tmp_env("BLAST4_CONN_SERVICE_NAME", "blast4_test");
-    const string rid("1219425735-25988-94487186920.BLASTQ1");
+    const string rid("2RCWHBTS01N");
     CRemoteBlast rmt_blaster(rid);
     //rmt_blaster.SetVerbose();
 
@@ -784,7 +783,7 @@ BOOST_AUTO_TEST_CASE(GetDatabaseInfo)
     BOOST_REQUIRE(dbinfo.NotEmpty());
     BOOST_REQUIRE(dbinfo->GetDatabase() == *blastdb);
 
-    const string swissprot_title("Non-redundant SwissProt sequences");
+    const string swissprot_title("Non-redundant UniProtKB/SwissProt sequences.");
     BOOST_REQUIRE_EQUAL(swissprot_title, dbinfo->GetDescription());
 
     BOOST_REQUIRE(dbinfo->GetTotal_length() > (Int8)7e+7);
@@ -803,9 +802,11 @@ BOOST_AUTO_TEST_CASE(GetDatabaseInfo)
     BOOST_REQUIRE(dbinfo.NotEmpty());
 
     BOOST_REQUIRE(dbinfo->GetDatabase() == *blastdb);
-    BOOST_REQUIRE(dbinfo->GetDescription().empty());
-    BOOST_REQUIRE_EQUAL(string("12/12/2002 14:17:00"),
-                         dbinfo->GetLast_updated());
+    //cerr << MSerial_AsnText << * dbinfo << endl;
+    BOOST_REQUIRE_EQUAL("ecoli", dbinfo->GetDescription());
+    // Last_updated is now the date in which the statistics are collected?
+    //BOOST_REQUIRE_EQUAL(string("12/12/2002 14:17:00"),
+    //                     dbinfo->GetLast_updated());
     BOOST_REQUIRE_EQUAL((Int8)400, dbinfo->GetNum_sequences());
     BOOST_REQUIRE_EQUAL((Int8)4662239, dbinfo->GetTotal_length());
 
