@@ -1,4 +1,4 @@
-/*  $Id: blastn_app.cpp 343332 2011-11-04 13:18:04Z fongah2 $
+/*  $Id: blastn_app.cpp 366367 2012-06-13 20:39:43Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-	"$Id: blastn_app.cpp 343332 2011-11-04 13:18:04Z fongah2 $";
+	"$Id: blastn_app.cpp 366367 2012-06-13 20:39:43Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -111,8 +111,7 @@ int CBlastnApp::Run(void)
         CBlastInputSourceConfig iconfig(dlconfig, query_opts->GetStrand(),
                                      query_opts->UseLowercaseMasks(),
                                      query_opts->GetParseDeflines(),
-                                     query_opts->GetRange(),
-                                     !m_CmdLineArgs->ExecuteRemotely());
+                                     query_opts->GetRange());
         iconfig.SetQueryLocalIdMode();
         CBlastFastaInputSource fasta(m_CmdLineArgs->GetInputStream(), iconfig);
         CBlastInput input(&fasta, m_CmdLineArgs->GetQueryBatchSize());
@@ -128,9 +127,8 @@ int CBlastnApp::Run(void)
         // Initialize the megablast database index now so we can know whether an indexed search will be run.
         // This is only important for the reference in the report, but would be done anyway.
         if (opt.GetUseIndex() && !m_CmdLineArgs->ExecuteRemotely()) {
-            BlastSeqSrc* seqsrc = db_adapter->MakeSeqSrc();
             CRef<CBlastOptions> my_options(&(opts_hndl->SetOptions()));
-            CSetupFactory::InitializeMegablastDbIndex(seqsrc, my_options);
+            CSetupFactory::InitializeMegablastDbIndex(my_options);
         }
 
         /*** Get the formatting options ***/
