@@ -1,4 +1,4 @@
-/*  $Id: deltablast_app.cpp 350247 2012-01-18 21:21:23Z rafanovi $
+/*  $Id: deltablast_app.cpp 366367 2012-06-13 20:39:43Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-        "$Id: deltablast_app.cpp 350247 2012-01-18 21:21:23Z rafanovi $";
+        "$Id: deltablast_app.cpp 366367 2012-06-13 20:39:43Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -184,8 +184,7 @@ int CDeltaBlastApp::Run(void)
         CBlastInputSourceConfig iconfig(dlconfig, query_opts->GetStrand(),
                                      query_opts->UseLowercaseMasks(),
                                      query_opts->GetParseDeflines(),
-                                     query_opts->GetRange(),
-                                     !m_CmdLineArgs->ExecuteRemotely());
+                                     query_opts->GetRange());
         iconfig.SetQueryLocalIdMode();
         CBlastFastaInputSource fasta(m_CmdLineArgs->GetInputStream(), iconfig);
         size_t query_batch_size = m_CmdLineArgs->GetQueryBatchSize();
@@ -255,6 +254,7 @@ int CDeltaBlastApp::Run(void)
             CRef<blast::CSearchResultSet> domain_results;
 
             CRef<CDeltaBlast> deltablast;
+            CRef<CPssmWithParameters> pssm;
 
             if (m_CmdLineArgs->ExecuteRemotely()) {
 
@@ -265,6 +265,7 @@ int CDeltaBlastApp::Run(void)
                           m_CmdLineArgs->ProduceDebugRemoteOutput(),
                           m_CmdLineArgs->GetClientId());
                 results = rmt_blast->GetResultSet();
+                pssm = rmt_blast->GetPSSM();
             } else {
 
                 // Run locally
@@ -277,11 +278,10 @@ int CDeltaBlastApp::Run(void)
                                                  delta_opts));
                 deltablast->SetNumberOfThreads(m_CmdLineArgs->GetNumThreads());
                 results = deltablast->Run();
+                domain_results = deltablast->GetDomainResults();
+                pssm = deltablast->GetPssm();
             }
 
-            // get pssm
-            CRef<CPssmWithParameters> pssm = deltablast->GetPssm();
-
             // deltablast computed pssm does not have query title, so
             // it must be added if pssm is requested
             if (m_CmdLineArgs->SaveCheckpoint()
@@ -297,9 +297,10 @@ int CDeltaBlastApp::Run(void)
 
                 SavePssmToFile(pssm);
 
-                domain_results = deltablast->GetDomainResults();
-                blast::CSearchResultSet::const_iterator domain_it =
-                    domain_results->begin();
+                blast::CSearchResultSet::const_iterator domain_it;
+                if (m_CmdLineArgs->GetShowDomainHits()) {
+                    domain_it = domain_results->begin();
+                }
 
                 if (fmt_args->ArchiveFormatRequested(args)) {
                     formatter.WriteArchive(*queries, *opts_hndl, *results);
@@ -324,9 +325,8 @@ int CDeltaBlastApp::Run(void)
             }
             else {
 
-
                 // if more than 1 iterations are requested, then
-                // do PSI-BLAST iterations
+                // do PSI-BLAST iterations, this is not allowed for remote blast
 
                 // print domain search results if requested
                 // query_batch_size == 1 if number of iteratins > 1
