@@ -1,4 +1,4 @@
-/*  $Id: psiblast_app.cpp 347205 2011-12-14 20:08:44Z boratyng $
+/*  $Id: psiblast_app.cpp 366367 2012-06-13 20:39:43Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-        "$Id: psiblast_app.cpp 347205 2011-12-14 20:08:44Z boratyng $";
+        "$Id: psiblast_app.cpp 366367 2012-06-13 20:39:43Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -394,8 +394,7 @@ int CPsiBlastApp::Run(void)
             CBlastInputSourceConfig iconfig(dlconfig, query_opts->GetStrand(),
                                          query_opts->UseLowercaseMasks(),
                                          query_opts->GetParseDeflines(),
-                                         query_opts->GetRange(),
-                                         !m_CmdLineArgs->ExecuteRemotely());
+                                         query_opts->GetRange());
             iconfig.SetQueryLocalIdMode();
             CRef<CBlastFastaInputSource> fasta(new CBlastFastaInputSource(
                                          m_CmdLineArgs->GetInputStream(),
