@@ -1,4 +1,4 @@
-/*  $Id: rpsblast_app.cpp 343332 2011-11-04 13:18:04Z fongah2 $
+/*  $Id: rpsblast_app.cpp 368619 2012-07-10 17:13:04Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-	"$Id: rpsblast_app.cpp 343332 2011-11-04 13:18:04Z fongah2 $";
+	"$Id: rpsblast_app.cpp 368619 2012-07-10 17:13:04Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -112,8 +112,7 @@ int CRPSBlastApp::Run(void)
         CBlastInputSourceConfig iconfig(dlconfig, query_opts->GetStrand(),
                                      query_opts->UseLowercaseMasks(),
                                      query_opts->GetParseDeflines(),
-                                     query_opts->GetRange(),
-                                     !m_CmdLineArgs->ExecuteRemotely());
+                                     query_opts->GetRange());
         iconfig.SetQueryLocalIdMode();
         CBlastFastaInputSource fasta(m_CmdLineArgs->GetInputStream(), iconfig);
         CBlastInput input(&fasta, m_CmdLineArgs->GetQueryBatchSize());
@@ -167,7 +166,7 @@ int CRPSBlastApp::Run(void)
             }
             else
             {
-            	CLocalRPSBlast  local_search (query_batch, args[kArgDb].AsString(), opts_hndl, args[kArgNumThreads].AsInteger() );
+            	CLocalRPSBlast  local_search (query_batch, db_args->GetDatabaseName(), opts_hndl, args[kArgNumThreads].AsInteger() );
             	results = local_search.Run();
             }
 
