@@ -1,4 +1,4 @@
-/*  $Id: tblastn_app.cpp 343332 2011-11-04 13:18:04Z fongah2 $
+/*  $Id: tblastn_app.cpp 366367 2012-06-13 20:39:43Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-	"$Id: tblastn_app.cpp 343332 2011-11-04 13:18:04Z fongah2 $";
+	"$Id: tblastn_app.cpp 366367 2012-06-13 20:39:43Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -117,8 +117,7 @@ int CTblastnApp::Run(void)
             CBlastInputSourceConfig iconfig(dlconfig, query_opts->GetStrand(),
                                          query_opts->UseLowercaseMasks(),
                                          query_opts->GetParseDeflines(),
-                                         query_opts->GetRange(),
-                                         !m_CmdLineArgs->ExecuteRemotely());
+                                         query_opts->GetRange());
             iconfig.SetQueryLocalIdMode();
             fasta.Reset(new CBlastFastaInputSource(
                                          m_CmdLineArgs->GetInputStream(),
