@@ -1,4 +1,4 @@
-/*  $Id: makeprofiledb.cpp 351672 2012-01-31 17:51:11Z ivanov $
+/*  $Id: makeprofiledb.cpp 373329 2012-08-28 19:33:38Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -390,7 +390,7 @@ void CMakeProfileDBApp::x_SetupArgDescriptions(void)
                             "Create Index Files",
                             CArgDescriptions::eBoolean, kDefaultOutIndexFile);
 
-    arg_desc->SetCurrentGroup("Scoremat w residue frequencies only options (for building pssm)");
+    arg_desc->SetCurrentGroup("Used only if scoremat files do not contain PSSM scores, ignored otherwise.");
     arg_desc->AddOptionalKey(kArgGapOpen, "gap_open_penalty",
                             "Cost to open a gap",
                             CArgDescriptions::eInteger);
