@@ -1,4 +1,4 @@
-/* $Id: msvc_configure.cpp 283161 2011-05-18 13:48:40Z gouriano $
+/* $Id: msvc_configure.cpp 356440 2012-03-14 13:55:58Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -293,7 +293,12 @@ void CMsvcConfigure::AnalyzeDefines(
     }
 #else
     signature += "--";
-    signature += HOST;
+    if (CMsvc7RegSettings::GetMsvcPlatform() < CMsvc7RegSettings::eUnix) {
+        signature += "i386-pc-";
+        signature += CMsvc7RegSettings::GetRequestedArchs();
+    } else {
+        signature += HOST;
+    }
     signature += "-";
     if (CMsvc7RegSettings::GetMsvcPlatform() < CMsvc7RegSettings::eUnix) {
         signature += GetApp().GetEnvironment().Get("COMPUTERNAME");
