@@ -154,6 +154,8 @@ void CMsvcProjectGenerator::Generate(CProjItem& prj)
 
     if (CMsvc7RegSettings::GetMsvcVersion() >= CMsvc7RegSettings::eMsvc1000) {
         GenerateMsbuild(collector, project_context, prj);
+        GetApp().RegisterProjectWatcher(
+            project_context.ProjectName(), prj.m_SourcesBaseDir, prj.m_Watchers);
         return;
     }
 
@@ -925,9 +927,17 @@ void CMsvcProjectGenerator::GenerateMsbuild(
             string cfg_condition("'$(Configuration)|$(Platform)'=='");
             cfg_condition += c->GetConfigFullName() + "|" + CMsvc7RegSettings::GetMsvcPlatformName() + "'";
 
+            string outputfile(msvc_tool.Linker()->OutputFile()), targetname, targetext;
+            bool customtargetname =  false;
+            if (!outputfile.empty()) {
+                targetname = CDirEntry(outputfile).GetBase();
+                targetext  = CDirEntry(outputfile).GetExt();
+                customtargetname =  !targetname.empty() && (targetname.find('$') == string::npos);
+            }
+
             __SET_PROPGROUP_ELEMENT(t, "OutDir",          msvc_tool.Configuration()->OutputDirectory(), cfg_condition);
             __SET_PROPGROUP_ELEMENT(t, "IntDir",          msvc_tool.Configuration()->IntermediateDirectory(), cfg_condition);
-            __SET_PROPGROUP_ELEMENT(t, "TargetName",      project_context.ProjectId(), cfg_condition);
+            __SET_PROPGROUP_ELEMENT(t, "TargetName",      customtargetname ? targetname : project_context.ProjectId(), cfg_condition);
             __SET_PROPGROUP_ELEMENT(t, "LinkIncremental", msvc_tool.Linker()->LinkIncremental(), cfg_condition);
             string prop = msvc_tool.Linker()->GenerateManifest();
 #if __USE_DISABLED_CFGS__
@@ -938,6 +948,9 @@ void CMsvcProjectGenerator::GenerateMsbuild(
 #endif
             __SET_PROPGROUP_ELEMENT(t, "GenerateManifest", prop, cfg_condition);
             __SET_PROPGROUP_ELEMENT(t, "EmbedManifest", msvc_tool.Linker()->EmbedManifest(), cfg_condition);
+            if (customtargetname) {
+                __SET_PROPGROUP_ELEMENT(t, "TargetExt", targetext, cfg_condition);
+            }
         }
     }
 
@@ -1511,7 +1524,7 @@ void CMsvcProjectGenerator::GenerateMsbuildFilters(
         project_path += ".filters";
 
         filters.SetProjectLevelTagExceptTargetOrImportType().SetProjectLevelTagExceptTargetOrImportType().push_back(filter_list);
-        SaveIfNewer(project_path, filters);
+        SaveIfNewer(project_path, filters, "<UniqueIdentifier>");
     }
 }
 
