@@ -1,7 +1,7 @@
 #ifndef PROJECT_TREE_BUILDER__MSVC_PRJ_UTILS__HPP
 #define PROJECT_TREE_BUILDER__MSVC_PRJ_UTILS__HPP
 
-/* $Id: msvc_prj_utils.hpp 260520 2011-03-23 16:33:34Z gouriano $
+/* $Id: msvc_prj_utils.hpp 366536 2012-06-14 20:27:27Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -103,15 +103,23 @@ void SaveToXmlFile  (const string&               file_path,
                      const CSerialObject& project);
 
 /// Save CVisualStudioProject class instance to file only if no such file 
-//  or contents of this file will be different from already present file.
+///  or contents of this file will be different from already present file.
 ///
 /// @param file_path
 ///   Path to file project will be saved to.
 /// @param project
 ///   Project to save.
-void SaveIfNewer    (const string&               file_path, 
+void SaveIfNewer    (const string&        file_path, 
                      const CSerialObject& project);
+
+/// Save object, ignoring certain tags
+void SaveIfNewer    (const string&        file_path, 
+                     const CSerialObject& project,
+                     const string& ignore);
 #endif //NCBI_COMPILER_MSVC
+bool PromoteIfDifferent(const string& present_path, 
+                        const string& candidate_path,
+                        const string& ignore);
 
 /// Consider promotion candidate to present 
 bool PromoteIfDifferent(const string& present_path, 
@@ -283,6 +291,8 @@ public:
     
     static string    GetConfigNameKeyword(void);
     static string    GetVcprojExt(void);
+
+    static string GetTopBuilddir(void);
 private:
     static EMsvcVersion   sm_MsvcVersion;
     static EMsvcPlatform  sm_MsvcPlatform;
