@@ -1,4 +1,4 @@
-/* $Id: proj_builder_app.cpp 346490 2011-12-07 15:13:54Z gouriano $
+/* $Id: proj_builder_app.cpp 366541 2012-06-14 20:29:03Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -402,7 +402,7 @@ struct PIsExcludedByDisuse
 //-----------------------------------------------------------------------------
 CProjBulderApp::CProjBulderApp(void)
 {
-    SetVersion( CVersionInfo(3,7,1) );
+    SetVersion( CVersionInfo(3,7,4) );
     m_ScanningWholeTree = false;
     m_Dll = false;
     m_AddMissingLibs = false;
@@ -985,7 +985,8 @@ void CProjBulderApp::GenerateUnixProjects(CProjectItemsTree& projects_tree)
         return;
     }
     GetApp().RegisterGeneratedFile( m_Solution );
-    ofs << "# This file was generated by PROJECT_TREE_BUILDER" << endl;
+    ofs << "# This file was generated by PROJECT_TREE_BUILDER v"
+        <<  GetVersion().Print() << endl;
     ofs << "# on " << CTime(CTime::eCurrent).AsString() << endl << endl;
 
 // see CXX-950
@@ -1001,6 +1002,11 @@ void CProjBulderApp::GenerateUnixProjects(CProjectItemsTree& projects_tree)
     ofs << endl;
 #endif
 
+    ofs << "MINPUT=" << CDirEntry(m_Solution).GetName() << endl << endl;
+    ofs << "# Use empty MTARGET to build a project;" << endl;
+    ofs << "# MTARGET=clean - to clean, or MTARGET=purge - to purge" << endl;
+    ofs << "MTARGET =" << endl << endl;
+
     if (m_ExtSrcRoot.empty()) {
        ofs << "top_srcdir=" << m_Root << endl;
     } else {
@@ -1011,11 +1017,10 @@ void CProjBulderApp::GenerateUnixProjects(CProjectItemsTree& projects_tree)
     ofs << "MFLAGS_NR = $(MFLAGS)" << endl;
     ofs << "SKIP_PRELIMINARIES= sources= configurables=configurables.null" << endl;
 
-// all dirs -------------------------------------------------------------
+// all dirs -----------------------------------------------------------------
     list<string> all_dirs;
     ITERATE(CProjectItemsTree::TProjects, p, projects_tree.m_Projects) {
-        if (p->first.Type() == CProjKey::eMsvc ||
-            p->first.Type() == CProjKey::eDataSpec) {
+        if (p->first.Type() == CProjKey::eDataSpec) {
             continue;
         }
         all_dirs.push_back(
@@ -1034,11 +1039,6 @@ void CProjBulderApp::GenerateUnixProjects(CProjectItemsTree& projects_tree)
     ofs << "include $(top_srcdir)/src/build-system/Makefile.meta.$(is_gmake)" << endl;
     ofs << endl;
 
-    ofs << "MINPUT=" << CDirEntry(m_Solution).GetName() << endl << endl;
-    ofs << "# Use empty MTARGET to build a project;" << endl;
-    ofs << "# MTARGET=clean - to clean, or MTARGET=purge - to purge" << endl;
-    ofs << "MTARGET =" << endl << endl;
-
     string dotreal(".real");
     string dotfiles(".files");
 
@@ -1050,8 +1050,7 @@ void CProjBulderApp::GenerateUnixProjects(CProjectItemsTree& projects_tree)
             LOG_POST(Info << "For reference only: " << CreateProjectName(p->first));
             continue;
         }
-        if (p->first.Type() == CProjKey::eMsvc ||
-            p->first.Type() == CProjKey::eDataSpec) {
+        if (p->first.Type() == CProjKey::eDataSpec) {
             continue;
         }
         ofs << " \\" <<endl << "    " << CreateProjectName(p->first);
@@ -1089,8 +1088,11 @@ void CProjBulderApp::GenerateUnixProjects(CProjectItemsTree& projects_tree)
 // all sources --------------------------------------------------------------
     ofs << "all_dataspec =";
     ITERATE(CProjectItemsTree::TProjects, p, projects_tree.m_Projects) {
-        if (p->first.Type() == CProjKey::eMsvc ||
-            p->first.Type() == CProjKey::eDataSpec) {
+        if (p->second.m_MakeType == eMakeType_Excluded ||
+            p->second.m_MakeType == eMakeType_ExcludedByReq) {
+            continue;
+        }
+        if (p->first.Type() == CProjKey::eDataSpec) {
             continue;
         }
         if (p->second.m_DatatoolSources.empty()) {
@@ -1106,8 +1108,48 @@ void CProjBulderApp::GenerateUnixProjects(CProjectItemsTree& projects_tree)
     ofs << "all_files" << dotreal << " :" << " $(all_dataspec:%=%" << dotreal << ")";
     ofs << endl << endl;
 
+// all apps -----------------------------------------------------------------
+    ofs << "all_apps =";
+    ITERATE(CProjectItemsTree::TProjects, p, projects_tree.m_Projects) {
+        if (p->second.m_MakeType == eMakeType_Excluded ||
+            p->second.m_MakeType == eMakeType_ExcludedByReq) {
+            continue;
+        }
+        if (p->first.Type() == CProjKey::eApp) {
+            ofs << " \\" <<endl << "    " << CreateProjectName(p->first);
+        }
+    }
+    ofs << endl << endl;
+
+// all Unix -------------------------------------------------------------
+    ofs << "all_unix =";
+    ITERATE(CProjectItemsTree::TProjects, p, projects_tree.m_Projects) {
+        if (p->second.m_MakeType == eMakeType_Excluded ||
+            p->second.m_MakeType == eMakeType_ExcludedByReq) {
+            continue;
+        }
+        if (p->first.Type() == CProjKey::eMsvc) {
+            ofs << " \\" <<endl << "    " << CreateProjectName(p->first);
+        }
+    }
+    ofs << endl << endl;
+
+// all excluded -------------------------------------------------------------
+    ofs << "all_excluded =";
+    ITERATE(CProjectItemsTree::TProjects, p, projects_tree.m_Projects) {
+        if (p->first.Type() == CProjKey::eDataSpec) {
+            continue;
+        }
+        if (p->second.m_MakeType == eMakeType_Excluded ||
+            p->second.m_MakeType == eMakeType_ExcludedByReq) {
+            ofs << " \\" <<endl << "    " << CreateProjectName(p->first);
+        }
+    }
+    ofs << endl << endl;
+
 // CompositeProjectTags -----------------------------------------------------
 // (add always)
+    vector<string> existing_composite_names;
     ITERATE(set<string>, r, m_RegisteredProjectTags) {
         m_CompositeProjectTags[*r] = *r;
     }
@@ -1119,8 +1161,11 @@ void CProjBulderApp::GenerateUnixProjects(CProjectItemsTree& projects_tree)
             vector<string> matching;
 
             ITERATE(CProjectItemsTree::TProjects, p, projects_tree.m_Projects) {
-                if (p->first.Type() == CProjKey::eMsvc ||
-                    p->first.Type() == CProjKey::eDataSpec) {
+                if (p->second.m_MakeType == eMakeType_Excluded ||
+                    p->second.m_MakeType == eMakeType_ExcludedByReq) {
+                    continue;
+                }
+                if (p->first.Type() == CProjKey::eDataSpec) {
                     continue;
                 }
                 if (IsAllowedProjectTag(p->second, &composite_filter)) {
@@ -1128,6 +1173,7 @@ void CProjBulderApp::GenerateUnixProjects(CProjectItemsTree& projects_tree)
                 }
             }
             if (!matching.empty()) {
+                existing_composite_names.push_back(composite_name);
                 ofs << composite_name << "_projects =";
                 ITERATE(vector<string>, c, matching) {
                     ofs << " \\" <<endl << "    " << *c;
@@ -1146,6 +1192,86 @@ void CProjBulderApp::GenerateUnixProjects(CProjectItemsTree& projects_tree)
         }
     }
 
+
+// help and list targets ----------------------------------------------------
+    ofs << "help :"
+        << endl << "\t@echo Build all projects"
+        << endl << "\t@echo \"    make -f $(MINPUT) -j 12 ptb_all\""
+        << endl << "\t@echo Build a project, for example, xncbi.lib"
+        << endl << "\t@echo \"    make -f $(MINPUT) -j 12 xncbi.lib\""
+        << endl << "\t@echo Clean project intermediate files"
+        << endl << "\t@echo \"    make -f $(MINPUT) -j 12 xncbi.lib MTARGET=clean\""
+        << endl << "\t@echo Clean project intermediate and output files"
+        << endl << "\t@echo \"    make -f $(MINPUT) -j 12 xncbi.lib MTARGET=purge\""
+        << endl << "\t@echo Target lists: "
+        << endl << "\t@echo \"    list-all         - list all targets\""
+        << endl << "\t@echo \"    list-apps        - list all applications\""
+        << endl << "\t@echo \"    list-libs        - list all libraries\""
+        << endl << "\t@echo \"    list-unix        - list all native Unix projects\""
+        << endl << "\t@echo \"    list-excluded    - list 'excluded' targets\""
+        << endl << "\t@echo \"    list-tags        - list composite targets\""
+        << endl << "\t@echo \"    list-tag-TagName - list all targets in a composite target TagName\"";
+    ofs << endl << endl;
+
+    ofs << "list-all :"
+        << endl << "\t@echo"
+        << endl << "\t@echo"
+        << endl << "\t@echo --------------------------------------"
+        << endl << "\t@echo APPLICATIONS"
+        << endl << "\t@echo"
+        << endl << "\t@for i in $(all_apps); do echo $$i; done"
+        << endl << "\t@echo"
+        << endl << "\t@echo"
+        << endl << "\t@echo --------------------------------------"
+        << endl << "\t@echo LIBRARIES"
+        << endl << "\t@echo"
+        << endl << "\t@for i in $(all_libraries); do echo $$i; done";
+
+    if (!existing_composite_names.empty()) {
+       ofs 
+            << endl << "\t@echo"
+            << endl << "\t@echo"
+            << endl << "\t@echo --------------------------------------"
+            << endl << "\t@echo COMPOSITE TARGETS"
+            << endl << "\t@echo";
+        ITERATE(vector<string>, c, existing_composite_names) {
+            ofs  << endl << "\t@echo " << *c;
+        }
+    }
+    ofs
+        << endl << "\t@echo"
+        << endl << "\t@echo"
+        << endl << "\t@echo --------------------------------------"
+        << endl << "\t@echo DIRECTORIES"
+        << endl << "\t@echo"
+        << endl << "\t@for i in $(all_dirs); do echo $$i/; done";
+    ofs << endl << endl;
+    ofs << "list-apps :"
+        << endl << "\t@for i in $(all_apps); do echo $$i; done";
+    ofs << endl << endl;
+    ofs << "list-libs :"
+        << endl << "\t@for i in $(all_libraries); do echo $$i; done";
+    ofs << endl << endl;
+    ofs << "list-unix :"
+        << endl << "\t@for i in $(all_unix); do echo $$i; done";
+    ofs << endl << endl;
+    ofs << "list-excluded :"
+        << endl << "\t@for i in $(all_excluded); do echo $$i; done";
+    ofs << endl << endl;
+
+    ofs << "list-tags :";
+    if (!existing_composite_names.empty()) {
+        ITERATE(vector<string>, c, existing_composite_names) {
+            ofs  << endl << "\t@echo " << *c;
+        }
+    }
+    ofs << endl << endl;
+    ITERATE(vector<string>, c, existing_composite_names) {
+        ofs << "list-tag-" << *c << " :"
+            << endl << "\t@for i in $(" << *c << "_projects); do echo $$i; done";
+        ofs << endl << endl;
+    }
+
 // --------------------------------------------------------------------------
     string datatool_key;
     string datatool( GetDatatoolId() );
@@ -1159,22 +1285,22 @@ void CProjBulderApp::GenerateUnixProjects(CProjectItemsTree& projects_tree)
     
     ITERATE(CProjectItemsTree::TProjects, p, projects_tree.m_Projects) {
 
-        // exclude MSVC projects
-        if (p->first.Type() == CProjKey::eMsvc ||
-            p->first.Type() == CProjKey::eDataSpec) {
+        if (p->first.Type() == CProjKey::eDataSpec) {
             continue;
         }
 
         bool isLibrary = p->first.Type() == CProjKey::eLib;
         bool hasDataspec = !p->second.m_DatatoolSources.empty();
-        string target, target_app, target_lib;
+        string target, target_app, target_lib, target_user;
         list<string> dependencies;
         CProjectItemsTree::TProjects::const_iterator n;
 
         target = CreateProjectName(p->first);
-        target_app = target_lib = "\"\"";
+        target_app = target_lib = target_user = "\"\"";
         if (p->first.Type() == CProjKey::eApp) {
             target_app = p->second.m_Name;
+        } else if (p->first.Type() == CProjKey::eMsvc) {
+            target_user = p->second.m_Name;
         } else if (p->first.Type() == CProjKey::eLib) {
             target_lib = p->second.m_Name;
         } else {
@@ -1242,6 +1368,7 @@ void CProjBulderApp::GenerateUnixProjects(CProjectItemsTree& projects_tree)
         copy(lib_guid.begin(), lib_guid.end(), back_inserter(dependencies));
         dependencies.sort();
         dependencies.unique();
+        CProjectTreeBuilder::VerifyBuildOrder( p->second, dependencies, projects_tree);
 
         if (isLibrary && !m_AllDllBuild) {
             list<string> new_dependencies;
@@ -1262,8 +1389,7 @@ void CProjBulderApp::GenerateUnixProjects(CProjectItemsTree& projects_tree)
         }
 
 // collect paths ------------------------------------------------------------
-        if (p->first.Type() != CProjKey::eMsvc &&
-            p->first.Type() != CProjKey::eDataSpec) {
+        if (p->first.Type() != CProjKey::eDataSpec) {
 
             path_to_target[rel_path].push_back(target);
             if (p->second.m_MakeType != eMakeType_Excluded &&
@@ -1351,6 +1477,7 @@ void CProjBulderApp::GenerateUnixProjects(CProjectItemsTree& projects_tree)
         ofs << " $(MAKE) $(MFLAGS)"
             << " APP_PROJ=" << target_app
             << " LIB_PROJ=" << target_lib
+            << " UNIX_PROJ=" << target_user
             << " $(MTARGET) $(SKIP_PRELIMINARIES)" << endl << endl;
  #endif
         if (hasDataspec) {
@@ -1916,8 +2043,13 @@ void CProjBulderApp::VerifyArguments(void)
     }
 
     m_IncDir = GetProjectTreeInfo().m_Compilers;
-    m_IncDir = CDirEntry::ConcatPath(m_IncDir,GetRegSettings().m_CompilersSubdir);
-    m_IncDir = CDirEntry::ConcatPath(m_IncDir, GetBuildType().GetTypeStr());
+    if (CMsvc7RegSettings::GetMsvcPlatform() == CMsvc7RegSettings::eUnix) {
+        m_IncDir = CDirEntry(m_Solution).GetDir();
+        m_IncDir = CDirEntry::ConcatPath(m_IncDir,"..");
+    } else {
+        m_IncDir = CDirEntry::ConcatPath(m_IncDir,GetRegSettings().m_CompilersSubdir);
+        m_IncDir = CDirEntry::ConcatPath(m_IncDir, GetBuildType().GetTypeStr());
+    }
     m_IncDir = CDirEntry::ConcatPath(m_IncDir, "inc");
     m_IncDir = CDirEntry::ConcatPath(m_IncDir, CMsvc7RegSettings::GetConfigNameKeyword());
 }
@@ -2071,7 +2203,7 @@ const SProjectTreeInfo& CProjBulderApp::GetProjectTreeInfo(void)
     PTB_INFO("Project tree root: " << m_Root);
 
     // all possible project tags
-    const string& tagsfile = CDirEntry::ConvertToOSPath(
+    string tagsfile = CDirEntry::ConvertToOSPath(
         GetConfig().Get("ProjectTree", "ProjectTags"));
     if (!tagsfile.empty()) {
         string fileloc(CDirEntry::ConcatPath(m_ProjectTreeInfo->m_Root, tagsfile));
@@ -2080,9 +2212,19 @@ const SProjectTreeInfo& CProjBulderApp::GetProjectTreeInfo(void)
         }
         LoadProjectTags(fileloc);
     }
+
+    //dependencies
+    string depsfile = FindDepGraph(m_ProjectTreeInfo->m_Root);
+    if (depsfile.empty() && !m_ExtSrcRoot.empty()) {
+        depsfile = FindDepGraph(m_ExtSrcRoot);
+    }
+    if (!depsfile.empty()) {
+        PTB_INFO("Library dependencies graph: " << depsfile);
+        LoadDepGraph(depsfile);
+    }
     
     /// <include> branch of tree
-    const string& include = GetConfig().Get("ProjectTree", "include");
+    string include = GetConfig().Get("ProjectTree", "include");
     m_ProjectTreeInfo->m_Include = 
             CDirEntry::ConcatPath(m_ProjectTreeInfo->m_Root, 
                                   include);
@@ -2091,7 +2233,7 @@ const SProjectTreeInfo& CProjBulderApp::GetProjectTreeInfo(void)
     
 
     /// <src> branch of tree
-    const string& src = GetConfig().Get("ProjectTree", "src");
+    string src = GetConfig().Get("ProjectTree", "src");
     m_ProjectTreeInfo->m_Src = 
             CDirEntry::ConcatPath(m_ProjectTreeInfo->m_Root, 
                                   src);
@@ -2108,7 +2250,7 @@ const SProjectTreeInfo& CProjBulderApp::GetProjectTreeInfo(void)
         new CProjectsLstFileFilter(m_ProjectTreeInfo->m_Src, subtree));
 
     /// <compilers> branch of tree
-    const string& compilers = GetConfig().Get("ProjectTree", "compilers");
+    string compilers = GetConfig().Get("ProjectTree", "compilers");
     m_ProjectTreeInfo->m_Compilers = 
             CDirEntry::ConcatPath(m_ProjectTreeInfo->m_Root, 
                                   compilers);
@@ -2117,7 +2259,7 @@ const SProjectTreeInfo& CProjBulderApp::GetProjectTreeInfo(void)
                    (m_ProjectTreeInfo->m_Compilers);
 
     /// ImplicitExcludedBranches - all subdirs will be excluded by default
-    const string& implicit_exclude_str 
+    string implicit_exclude_str 
         = GetConfig().Get("ProjectTree", "ImplicitExclude");
     list<string> implicit_exclude_list;
     NStr::Split(implicit_exclude_str, 
@@ -2132,7 +2274,7 @@ const SProjectTreeInfo& CProjBulderApp::GetProjectTreeInfo(void)
     }
 
     /// <projects> branch of tree (scripts\projects)
-    const string& projects = CDirEntry::ConvertToOSPath(
+    string projects = CDirEntry::ConvertToOSPath(
         GetConfig().Get("ProjectTree", "projects"));
     m_ProjectTreeInfo->m_Projects = 
             CDirEntry::ConcatPath(m_ProjectTreeInfo->m_Root, 
@@ -2302,6 +2444,86 @@ void CProjBulderApp::LoadProjectTags(const string& filename)
     m_RegisteredProjectTags.insert("internal");
 }
 
+string CProjBulderApp::FindDepGraph(const string& root) const
+{
+    list<string> locations;
+    string locstr(GetConfig().Get("ProjectTree", "DepGraph"));
+    NStr::Split(locstr, LIST_SEPARATOR, locations);
+    for (list<string>::const_iterator l = locations.begin(); l != locations.end(); ++l) {
+        CDirEntry fileloc(CDirEntry::ConcatPath(root,CDirEntry::ConvertToOSPath(*l)));
+        if (fileloc.Exists() && fileloc.IsFile()) {
+            return fileloc.GetPath();
+        }
+    }
+    return kEmptyStr;
+}
+
+void   CProjBulderApp::LoadDepGraph(const string& filename)
+{
+    CNcbiIfstream ifs(filename.c_str(), IOS_BASE::in);
+    if ( ifs.is_open() ) {
+        string line;
+        while ( NcbiGetlineEOL(ifs, line) ) {
+            if (line.empty() || line[0] == '#') {
+                continue;
+            }
+            list<string> values;
+            NStr::Split(line, " ", values);
+            if (values.size() > 2) {
+                list<string>::const_iterator l= values.begin();
+                string first  = *l++;
+                string second = *l++;
+                string third  = *l++;
+                if (second == "includes") {
+                    m_GraphDepIncludes[first].insert(third);
+                } else if (second == "needs") {
+                    m_GraphDepPrecedes[first].insert(third);
+                }
+            }
+        }
+    }
+
+    vector< set<string> > graph;
+    for (map<string, set<string> >::const_iterator d= m_GraphDepPrecedes.begin();
+            d!= m_GraphDepPrecedes.end(); ++d) {
+        InsertDep(graph, d->first);
+    }
+    for (size_t s= 0; s<graph.size(); ++s) {
+        for (set<string>::const_iterator l = graph[s].begin(); l != graph[s].end(); ++l) {
+            m_GraphDepRank[*l] = s;
+        }
+    }
+}
+
+void  CProjBulderApp::InsertDep(vector< set<string> >& graph, const string& dep)
+{
+    const set<string>& dependents = m_GraphDepPrecedes[dep];
+    size_t graphset=0;
+    for (set<string>::const_iterator d = dependents.begin(); d != dependents.end(); ++d) {
+        for (bool found=false; !found; ) {
+            for (size_t s= 0; !found && s<graph.size(); ++s) {
+                if (graph[s].find(*d) != graph[s].end()) {
+                    graphset = max(graphset,s);
+                    found = true;
+                }
+            }
+            if (!found) {
+                InsertDep(graph, *d);
+            }
+        }
+    }
+    if (!dependents.empty()) {
+        ++graphset;
+    }
+    if (graphset < graph.size()) {
+        graph[graphset].insert(dep);
+    } else {
+        set<string> t;
+        t.insert(dep);
+        graph.push_back(t);
+    }
+}
+
 string CProjBulderApp::ProcessLocationMacros(string raw_data)
 {
     string data(raw_data), raw_macro, macro, definition;
@@ -2420,7 +2642,7 @@ END_NCBI_SCOPE
 
 USING_NCBI_SCOPE;
 
-int main(int argc, const char* argv[])
+int NcbiSys_main(int argc, TXChar* argv[])
 {
     // Execute main application function
     CDiagContext::SetLogTruncate(true);
