@@ -1,4 +1,4 @@
-/* $Id: proj_item.cpp 345374 2011-11-25 15:52:25Z gouriano $
+/* $Id: proj_item.cpp 366536 2012-06-14 20:27:27Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -175,6 +175,7 @@ void CProjItem::Clear(void)
     m_IsBundle = false;
     m_External = false;
     m_StyleObjcpp = false;
+    m_MkName.clear();
 }
 
 
@@ -213,6 +214,7 @@ void CProjItem::SetFrom(const CProjItem& item)
     m_IsBundle = item.m_IsBundle;
     m_External = item.m_External;
     m_StyleObjcpp = item.m_StyleObjcpp;
+    m_MkName = item.m_MkName;
 }
 
 string CProjItem::GetPath(void) const
@@ -230,7 +232,9 @@ string CProjItem::GetPath(void) const
         path += ".dll";
         break;
     case CProjKey::eMsvc:
-        path += ".msvc";
+        if (CMsvc7RegSettings::GetMsvcPlatform() != CMsvc7RegSettings::eUnix) {
+            path += ".msvc";
+        }
         break;
     case CProjKey::eDataSpec:
         path += ".dataspec";
