@@ -1,4 +1,4 @@
-/*  $Id: cgiapp.cpp 350022 2012-01-17 20:55:55Z rafanovi $
+/*  $Id: cgiapp.cpp 372363 2012-08-16 18:59:19Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -82,6 +82,25 @@ NCBI_PARAM_DEF_EX(bool, CGI, Allow_Sigpipe, false, eParam_NoThread,
                   CGI_ALLOW_SIGPIPE);
 typedef NCBI_PARAM_TYPE(CGI, Allow_Sigpipe) TParamAllowSigpipe;
 
+// Log any broken connection with status 299 rather than 499.
+NCBI_PARAM_DEF_EX(bool, CGI, Client_Connection_Interruption_Okay, false,
+                  eParam_NoThread,
+                  CGI_CLIENT_CONNECTION_INTERRUPTION_OKAY);
+
+// Severity for logging broken connection errors.
+NCBI_PARAM_ENUM_ARRAY(EDiagSev, CGI, Client_Connection_Interruption_Severity)
+{
+    {"Info", eDiag_Info},
+    {"Warning", eDiag_Warning},
+    {"Error", eDiag_Error},
+    {"Critical", eDiag_Critical},
+    {"Fatal", eDiag_Fatal},
+    {"Trace", eDiag_Trace}
+};
+NCBI_PARAM_ENUM_DEF_EX(EDiagSev, CGI, Client_Connection_Interruption_Severity,
+                       eDiag_Critical, eParam_NoThread,
+                       CGI_CLIENT_CONNECTION_INTERRUPTION_SEVERITY);
+
 
 ///////////////////////////////////////////////////////
 // IO streams with byte counting for CGI applications
@@ -108,19 +127,16 @@ ERW_Result CCGIStreamReader::Read(void*   buf,
                                   size_t  count,
                                   size_t* bytes_read)
 {
-    streamsize x_read = CStreamUtils::Readsome(m_IStr, (char*)buf, count);
+    size_t x_read = (size_t)CStreamUtils::Readsome(m_IStr, (char*)buf, count);
     ERW_Result result;
-    if (x_read < 0) {
-        result = eRW_Error;
-    }
-    else if (x_read > 0) {
+    if (x_read > 0  ||  count == 0) {
         result = eRW_Success;
     }
     else {
-        result = eRW_Eof;
+        result = m_IStr.eof() ? eRW_Eof : eRW_Error;
     }
     if (bytes_read) {
-        *bytes_read = x_read < 0 ? 0 : x_read;
+        *bytes_read = x_read;
     }
     return result;
 }
@@ -172,6 +188,11 @@ CCgiApplication* CCgiApplication::Instance(void)
 }
 
 
+extern "C" void SigTermHandler(int)
+{
+}
+
+
 int CCgiApplication::Run(void)
 {
     // Value to return from this method Run()
@@ -191,6 +212,14 @@ int CCgiApplication::Run(void)
     // Disable SIGPIPE if not allowed.
     if ( !TParamAllowSigpipe::GetDefault() ) {
         signal(SIGPIPE, SIG_IGN);
+        struct sigaction sigterm,  sigtermold;
+        memset(&sigterm, 0, sizeof(sigterm));
+        sigterm.sa_handler = SigTermHandler;
+        sigterm.sa_flags = SA_RESETHAND;
+        if (sigaction(SIGTERM, &sigterm, &sigtermold) == 0
+            &&  sigtermold.sa_handler != SIG_DFL) {
+            sigaction(SIGTERM, &sigtermold, 0);
+        }
     }
 
     // Compose diagnostics prefix
@@ -217,6 +246,9 @@ int CCgiApplication::Run(void)
         GetDiagContext().SetAppState(eDiagAppState_RequestBegin);
 
         m_Context.reset( CreateContext() );
+        _ASSERT(m_Context.get());
+        m_Context->CheckStatus();
+
         ConfigureDiagnostics(*m_Context);
         x_AddLBCookie();
         try {
@@ -253,6 +285,7 @@ int CCgiApplication::Run(void)
                 GetDiagContext().SetAppState(eDiagAppState_RequestEnd);
                 if (result != 0) {
                     SetHTTPStatus(500);
+                    m_ErrorStatus = true;
                 } else {
                     if (m_Cache.get()) {
                         m_Context->GetResponse().Flush();
@@ -283,6 +316,16 @@ int CCgiApplication::Run(void)
                                                e.GetStatusMessage());
             result = 0;
         }
+
+#ifdef NCBI_OS_MSWIN
+        // Logging - on MSWin this must be done before flushing the output.
+        if ( is_stat_log  &&  !skip_stat_log ) {
+            stat->Reset(start_time, result);
+            stat->Submit(stat->Compose());
+        }
+        is_stat_log = false;
+#endif
+
         _TRACE("CCgiApplication::Run: flushing");
         m_Context->GetResponse().Flush();
         _TRACE("CCgiApplication::Run: return " << result);
@@ -308,16 +351,27 @@ int CCgiApplication::Run(void)
             }
         }}
 
-        // Exception reporting
-        NCBI_REPORT_EXCEPTION_X(13, "(CGI) CCgiApplication::Run", e);
+        // Exception reporting. Use different severity for broken connection.
+        ios_base::failure* fex = dynamic_cast<ios_base::failure*>(&e);
+        CNcbiOstream* os = m_Context.get() ? m_Context->GetResponse().GetOutput() : NULL;
+        if (fex  &&  os  &&  !os->good()) {
+            if ( !TClientConnIntOk::GetDefault() ) {
+                ERR_POST_X(13, Severity(TClientConnIntSeverity::GetDefault()) <<
+                    "Connection interrupted");
+            }
+        }
+        else {
+            NCBI_REPORT_EXCEPTION_X(13, "(CGI) CCgiApplication::Run", e);
+        }
     }
 
+#ifndef NCBI_OS_MSWIN
     // Logging
     if ( is_stat_log  &&  !skip_stat_log ) {
         stat->Reset(start_time, result);
-        string msg = stat->Compose();
-        stat->Submit(msg);
+        stat->Submit(stat->Compose());
     }
+#endif
 
     x_OnEvent(eEndRequest, 120);
     x_OnEvent(eExit, result);
@@ -571,7 +625,8 @@ CCgiApplication::CCgiApplication(void)
    m_ArgContextSync(false),
    m_IsResultReady(true),
    m_ShouldExit(false),
-   m_RequestStartPrinted(false)
+   m_RequestStartPrinted(false),
+   m_ErrorStatus(false)
 {
     // CGI applications should use /log for logging by default
     CDiagContext::SetUseRootLog();
@@ -605,7 +660,12 @@ int CCgiApplication::OnException(exception& e, CNcbiOstream& os)
     // Discriminate between different types of error
     string status_str = "500 Server Error";
     string message = "";
+
+    // Save current HTTP status. Later it may be changed to 299 or 499
+    // depending on this value.
+    m_ErrorStatus = CDiagContext::GetRequestContext().GetRequestStatus() >= 400;
     SetHTTPStatus(500);
+
     CException* ce = dynamic_cast<CException*> (&e);
     if ( ce ) {
         message = ce->GetMsg();
@@ -631,6 +691,11 @@ int CCgiApplication::OnException(exception& e, CNcbiOstream& os)
         message = e.what();
     }
 
+    // Don't try to write to a broken output
+    if ( !os.good() ) {
+        return -1;
+    }
+
     try {
         // HTTP header
         os << "Status: " << status_str << HTTP_EOL;
@@ -648,7 +713,6 @@ int CCgiApplication::OnException(exception& e, CNcbiOstream& os)
             }
         }
 
-
         // Check for problems in sending the response
         if ( !os.good() ) {
             ERR_POST_X(4, "CCgiApplication::OnException() failed to send error page"
@@ -706,6 +770,7 @@ void CCgiApplication::x_OnEvent(EEvent event, int status)
 
             // Set default HTTP status code (reset above by PrintRequestStart())
             SetHTTPStatus(200);
+            m_ErrorStatus = false;
 
             const string& phid = CDiagContext::GetRequestContext().GetHitID();
             // Check if ncbi_st cookie is set
@@ -754,10 +819,30 @@ void CCgiApplication::x_OnEvent(EEvent event, int status)
         }
     case eEndRequest:
         {
+            CDiagContext& ctx = GetDiagContext();
+            CRequestContext& rctx = ctx.GetRequestContext();
+            // If an error status has been set by ProcessRequest, don't try
+            // to check the output stream and change the status to 299/499.
+            if ( !m_ErrorStatus ) {
+                // Log broken connection as 299/499 status
+                CNcbiOstream* os = m_Context.get() ?
+                    m_Context->GetResponse().GetOutput() : NULL;
+                if (os  &&  !os->good()) {
+                    if (TClientConnIntOk::GetDefault()  ||
+                        m_Context->GetResponse().AcceptRangesBytes()) {
+                        rctx.SetRequestStatus(
+                            CRequestStatus::e299_PartialContentBrokenConnection);
+                    }
+                    else {
+                        rctx.SetRequestStatus(
+                            CRequestStatus::e499_BrokenConnection);
+                    }
+                }
+            }
             if ( m_RequestStartPrinted  &&
                 !CDiagContext::IsSetOldPostFormat() ) {
                 // This will also reset request context
-                GetDiagContext().PrintRequestStop();
+                ctx.PrintRequestStop();
                 m_RequestStartPrinted = false;
             }
             break;
