@@ -1,4 +1,4 @@
-/*  $Id: cgictx.cpp 342253 2011-10-26 17:00:50Z grichenk $
+/*  $Id: cgictx.cpp 372363 2012-08-16 18:59:19Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -43,6 +43,7 @@
 #include <cgi/cgiapp.hpp>
 #include <cgi/error_codes.hpp>
 
+
 #ifdef NCBI_OS_UNIX
 #  ifdef _AIX32 // version 3.2 *or higher*
 #    include <strings.h> // needed for bzero()
@@ -117,7 +118,8 @@ CCgiContext::CCgiContext(CCgiApplication&        app,
       m_Request(new CCgiRequest(args ? args : &app.GetArguments(),
                                 env  ? env  : &app.GetEnvironment(),
                                 inp, flags, ifd, errbuf_size)),
-      m_Response(out, ofd)
+      m_Response(out, ofd),
+      m_StatusCode(CCgiException::eStatusNotSet)
 {
     if (flags & CCgiRequest::fDisableTrackingCookie) {
         m_Response.DisableTrackingCookie();
@@ -132,7 +134,8 @@ CCgiContext::CCgiContext(CCgiApplication&        app,
                          CCgiRequest::TFlags     flags)
     : m_App(app),
       m_Request(new CCgiRequest()),
-      m_Response(os, -1)
+      m_Response(os, -1),
+      m_StatusCode(CCgiException::eStatusNotSet)
 {
     m_Request->Deserialize(*is,flags);
     x_InitSession(flags);
@@ -157,13 +160,25 @@ void CCgiContext::x_InitSession(CCgiRequest::TFlags flags)
     m_Request->x_SetSession(*m_Session);
     m_Response.x_SetSession(*m_Session);
     string track_cookie_value = RetrieveTrackingId();
+    bool bad_tracking_cookie = false;
     if ((flags & CCgiRequest::fSkipDiagProperties) == 0) {
-        GetDiagContext().GetRequestContext().SetSessionID(track_cookie_value);
+        try {
+            CRequestContext& ctx = GetDiagContext().GetRequestContext();
+            ctx.SetSessionID(track_cookie_value);
+            if (ctx.GetSessionID() != track_cookie_value) {
+                // Bad session-id was ignored
+                bad_tracking_cookie = true;
+            }
+        }
+        catch (CRequestContextException& e) {
+            x_SetStatus(CCgiException::e400_BadRequest, e.GetMsg());
+            bad_tracking_cookie = true;
+        }
     }
-    if( !TCGI_DisableTrackingCookie::GetDefault() ) {
-        m_Response.SetTrackingCookie(TCGI_TrackingCookieName::GetDefault(), 
+    if( !bad_tracking_cookie  &&  !TCGI_DisableTrackingCookie::GetDefault() ) {
+        m_Response.SetTrackingCookie(TCGI_TrackingCookieName::GetDefault(),
                                      track_cookie_value,
-                                     TCGI_TrackingCookieDomain::GetDefault(), 
+                                     TCGI_TrackingCookieDomain::GetDefault(),
                                      TCGI_TrackingCookiePath::GetDefault());
     }
 
@@ -291,7 +306,9 @@ const string& CCgiContext::GetSelfURL(void) const
     m_SelfURL += server;
     string port = GetRequest().GetProperty(eCgi_ServerPort);
     // Skip port if it's default for the selected scheme
-    if ((secure  &&  port == "443")  ||  (!secure  &&  port == "80")) {
+    if ((secure  &&  port == "443")  ||  (!secure  &&  port == "80")
+	||  (server.size() >= port.size() + 2  &&  NStr::EndsWith(server, port)
+	     &&  server[server.size() - port.size() - 1] == ':')) {
         port = kEmptyStr;
     }
     if ( !port.empty() ) {
@@ -435,4 +452,22 @@ string CCgiContext::RetrieveTrackingId() const
 }
 
 
+void CCgiContext::x_SetStatus(CCgiException::EStatusCode code, const string& msg) const
+{
+    m_StatusCode = code;
+    m_StatusMessage = msg;
+}
+
+
+void CCgiContext::CheckStatus(void) const
+{
+    if (m_StatusCode == CCgiException::eStatusNotSet) return;
+
+    NCBI_EXCEPTION_VAR(ex, CCgiException, eUnknown,
+        m_StatusMessage);
+    ex.SetStatus(m_StatusCode);
+    NCBI_EXCEPTION_THROW(ex);
+}
+
+
 END_NCBI_SCOPE
