@@ -1,4 +1,4 @@
-/*  $Id: ncbicgi.cpp 347853 2011-12-21 16:14:38Z grichenk $
+/*  $Id: ncbicgi.cpp 359344 2012-04-12 14:57:12Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -469,8 +469,7 @@ CCgiCookies::x_CheckField(const string&          str,
 
 
 NCBI_PARAM_DECL(string, CGI, Cookie_Name_Banned_Symbols);
-// The old default was " ,;=".
-NCBI_PARAM_DEF_EX(string, CGI, Cookie_Name_Banned_Symbols, " ()<>@,;:\\\"/[]?={}",
+NCBI_PARAM_DEF_EX(string, CGI, Cookie_Name_Banned_Symbols, " ,;=",
                   eParam_NoThread, CGI_COOKIE_NAME_BANNED_SYMBOLS);
 typedef NCBI_PARAM_TYPE(CGI, Cookie_Name_Banned_Symbols) TCookieNameBannedSymbols;
 
@@ -488,14 +487,13 @@ const char* s_GetCookieNameBannedSymbols(void)
 
 void CCgiCookies::Add(const string& str, EOnBadCookie on_bad_cookie)
 {
-    bool need_decode = (m_EncodeFlag != NStr::eUrlEnc_None) &&
-        (TCookieEncoding::GetDefault() == CCgiCookie::eCookieEnc_Url);
     NStr::EUrlDecode dec_flag = m_EncodeFlag == NStr::eUrlEnc_PercentOnly ?
         NStr::eUrlDec_Percent : NStr::eUrlDec_All;
     const char* banned_symbols = s_GetCookieNameBannedSymbols();
 
     SIZE_TYPE pos = str.find_first_not_of(" \t\n");
     for (;;) {
+        bool need_decode = true;
         SIZE_TYPE pos_beg = str.find_first_not_of(' ', pos);
         if (pos_beg == NPOS)
             return; // done
@@ -506,8 +504,7 @@ void CCgiCookies::Add(const string& str, EOnBadCookie on_bad_cookie)
             switch ( x_CheckField(name, CCgiCookie::eField_Name,
                 banned_symbols, on_bad_cookie) ) {
             case eCheck_Valid:
-                Add(need_decode ? NStr::URLDecode(name, dec_flag) : name,
-                    kEmptyStr, on_bad_cookie);
+                Add(NStr::URLDecode(name, dec_flag), kEmptyStr, on_bad_cookie);
                 break;
             case eCheck_StoreInvalid:
                 {
@@ -527,8 +524,7 @@ void CCgiCookies::Add(const string& str, EOnBadCookie on_bad_cookie)
             switch ( x_CheckField(name, CCgiCookie::eField_Name,
                 banned_symbols, on_bad_cookie) ) {
             case eCheck_Valid:
-                Add(need_decode ? NStr::URLDecode(name, dec_flag) : name,
-                    kEmptyStr, on_bad_cookie);
+                Add(NStr::URLDecode(name, dec_flag), kEmptyStr, on_bad_cookie);
                 break;
             case eCheck_StoreInvalid:
                 {
@@ -584,7 +580,11 @@ void CCgiCookies::Add(const string& str, EOnBadCookie on_bad_cookie)
                     name + "): " +
                     NStr::PrintableString(str.substr(pos_mid + 1));
             }
-            if (!valid_quotes) {
+            if ( valid_quotes ) {
+                need_decode = false;
+            }
+            else {
+                quoted_value = false;
                 // Error - missing closing quote
                 switch ( on_bad_cookie ) {
                 case eOnBadCookie_ThrowException:
@@ -629,7 +629,7 @@ void CCgiCookies::Add(const string& str, EOnBadCookie on_bad_cookie)
             x_CheckField(val, CCgiCookie::eField_Value, ";,",
             on_bad_cookie, &name);
         if ( valid_name == eCheck_Valid  &&  valid_value == eCheck_Valid ) {
-            Add(need_decode ? NStr::URLDecode(name, dec_flag) : name,
+            Add(NStr::URLDecode(name, dec_flag),
                 need_decode ? NStr::URLDecode(val, dec_flag) : val,
                 on_bad_cookie);
         }
@@ -796,7 +796,7 @@ public:
     const char* const* GetTrackingEnv(void) const { return m_TrackingEnv; }
 
 private:
-    void x_Destroy();
+    void x_Destroy(void);
     const CNcbiEnvironment* m_Env;
     char**                  m_TrackingEnv;
 };
@@ -822,24 +822,26 @@ static const char* s_TrackingVars[] =
 CTrackingEnvHolder::CTrackingEnvHolder(const CNcbiEnvironment* env)
 	: m_Env(env), m_TrackingEnv(NULL)
 {
-    if ( !m_Env )
+    if (!m_Env)
         return;
 
     try {
-        int array_size = sizeof(s_TrackingVars) / sizeof(s_TrackingVars[0]);
-        m_TrackingEnv = new char*[array_size];
-        memset(m_TrackingEnv, 0, sizeof(char*) * array_size);
+        size_t size = sizeof(s_TrackingVars) / sizeof(s_TrackingVars[0]);
+        m_TrackingEnv = new char*[size];
+        memset(m_TrackingEnv, 0, sizeof(m_TrackingEnv[0]) * size);
 
         int i = 0;
         for (const char* const* name = s_TrackingVars;  *name;  ++name) {
             const string& value = m_Env->Get(*name);
-            if ( value.empty() )
+            if (value.empty())
                 continue;
 
-            string str( *name );
-            str += '=' + value;
-            m_TrackingEnv[i] = new char[str.length() + 1];
-            strcpy( m_TrackingEnv[i++], str.c_str() );
+            string str(*name);
+            str += '=';
+            str += value;
+            size = str.length() + 1;
+            m_TrackingEnv[i] = new char[size];
+            memcpy(m_TrackingEnv[i++], str.c_str(), size);
         }
     }
     catch (...) {
@@ -849,16 +851,19 @@ CTrackingEnvHolder::CTrackingEnvHolder(const CNcbiEnvironment* env)
 }
 
 
-void CTrackingEnvHolder::x_Destroy()
+void CTrackingEnvHolder::x_Destroy(void)
 {
-    if ( !m_TrackingEnv )
+    char** env;
+    if (!(env = m_TrackingEnv))
         return;
+    m_TrackingEnv = 0;
 
-    for (char** ptr = m_TrackingEnv;  *ptr;  ++ptr) {
-        delete[] *ptr;
+    for (char** ptr = env;  *ptr;  ++ptr) {
+        char* del = *ptr;
+        *ptr = 0;
+        delete[] del;
     }
-
-    delete[] m_TrackingEnv;
+    delete[] env;
 }
 
 
@@ -1069,7 +1074,7 @@ NCBI_PARAM_ENUM_ARRAY(CCgiCookies::EOnBadCookie, CGI, On_Bad_Cookie)
     {"Store",         CCgiCookies::eOnBadCookie_Store}
 };
 NCBI_PARAM_ENUM_DEF_EX(CCgiCookies::EOnBadCookie, CGI, On_Bad_Cookie,
-                       CCgiCookies::eOnBadCookie_SkipAndError,
+                       CCgiCookies::eOnBadCookie_Store,
                        eParam_NoThread, CGI_ON_BAD_COOKIE);
 typedef NCBI_PARAM_TYPE(CGI, On_Bad_Cookie) TOnBadCookieParam;
 
@@ -1294,7 +1299,7 @@ void CCgiRequest::x_ProcessInputStream(TFlags flags, CNcbiIstream* istr, int ifd
                 if ( !NcbiStreamCopy(buf, *istr) ) {
                     NCBI_THROW2(CCgiParseException, eRead,
                                 "Failed read of HTTP request body",
-                                istr->gcount());
+                                (size_t)istr->gcount());
                 }
                 string temp = CNcbiOstrstreamToString(buf);
                 m_Content.reset(new string);
