@@ -1,4 +1,4 @@
-/* $Id: ncbi_localnet.c 328918 2011-08-04 17:45:10Z lavr $
+/* $Id: ncbi_localnet.c 360937 2012-04-27 13:47:28Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -72,6 +72,10 @@
 #  define BUFSIZE PATH_MAX
 #endif /*PATHMAX<256*/
 
+#if defined(_DEBUG)  &&  !defined(NDEBUG)
+/*#  define LOCALNET_DEBUG 1*/
+#endif /*_DEBUG && !NDEBUG*/
+
 
 static int/*bool*/ s_Inited = 0/*false*/;
 
@@ -107,7 +111,7 @@ static int/*bool*/ s_OverlapLocalIP(const SIPRange* range, size_t n)
     size_t i;
 
     for (i = 0;  i < n;  i++) {
-        SIPRange r2    = NcbiTrueIPRange(s_LocalIP + i);
+        SIPRange r2 = NcbiTrueIPRange(s_LocalIP + i);
         unsigned int a = r1.a > r2.a ? r1.a : r2.a;
         unsigned int b = r1.b < r2.b ? r1.b : r2.b;
         if (a <= b)
@@ -131,16 +135,17 @@ static void s_LoadLocalIPs(void)
         filename = 0;
     if (!filename  ||  !(fp = fopen(filename, "r"))) {
         if (filename) {
-            CORE_LOGF_ERRNO_X(1, errno != ENOENT ? eLOG_Error : eLOG_Trace,
-                              errno, ("Cannot load local IPs from `%s'", buf));
+            CORE_LOGF_ERRNO_X(1, errno == ENOENT ? eLOG_Warning : eLOG_Error,
+                              errno, ("Cannot load local IP specs from '%s'",
+                                      buf));
         }
-        CORE_LOG(eLOG_Trace, "Using default local IPs");
+        CORE_LOG(eLOG_Trace, "Using default local IP specs");
         assert(sizeof(s_LocalIP) >= sizeof(kLocalIP));
         memcpy(s_LocalIP, kLocalIP, sizeof(kLocalIP));
         return;
     }
 
-    CORE_LOGF(eLOG_Trace, ("Loading local IPs from `%s'", filename));
+    CORE_LOGF(eLOG_Trace, ("Loading local IP specs from '%s'", filename));
     memcpy(s_LocalIP, kLocalIP, sizeof(kLocalIP[0]));
     n = 1/*localhost gets always added*/;
     lineno = 0;
@@ -170,7 +175,7 @@ static void s_LoadLocalIPs(void)
             if (!*err)
                 err = c;
             CORE_LOGF_X(2, eLOG_Error,
-                        ("Invalid IP spec at line %u: `%s'",
+                        ("Local IP spec at line %u, '%s' is invalid",
                          lineno, err));
             break;
         }
@@ -184,14 +189,14 @@ static void s_LoadLocalIPs(void)
         }
         if (s_OverlapLocalIP(&local, n)) {
             CORE_LOGF_X(4, eLOG_Warning,
-                        ("Local IP spec at line %u `%s' overlaps with"
+                        ("Local IP spec at line %u, '%s' overlaps with"
                          " already defined one(s)", lineno, c));
         }
         s_LocalIP[n++] = local;
     } while (!feof(fp));
     fclose(fp);
 
-    CORE_LOGF(eLOG_Trace, ("Done loading local IPs, %u line%s, %u entr%s",
+    CORE_LOGF(eLOG_Trace, ("Done loading local IP specs, %u line%s, %u entr%s",
                            lineno, &"s"[lineno == 1], (unsigned int) n,
                            n == 1 ? "y" : "ies"));
     if (n < sizeof(s_LocalIP)/sizeof(s_LocalIP[0]))
@@ -208,28 +213,29 @@ extern void NcbiInitLocalIP(void)
 extern int/*bool*/ NcbiIsLoopbackIP(unsigned int ip)
 {
     /* 127/8 */
-    unsigned int addr = SOCK_NetToHostLong(ip);
+    if (ip) {
+        unsigned int addr = SOCK_NetToHostLong(ip);
 #if defined(IN_CLASSA) && defined(IN_CLASSA_NET) && defined(IN_CLASSA_NSHIFT)
-    return IN_CLASSA(addr)  &&
-        (addr & IN_CLASSA_NET) == (IN_LOOPBACKNET << IN_CLASSA_NSHIFT);
+        return IN_CLASSA(addr)
+            &&  (addr & IN_CLASSA_NET) == (IN_LOOPBACKNET << IN_CLASSA_NSHIFT);
 #else
-    return !((addr & 0xFF000000) ^ (INADDR_LOOPBACK-1));
+        return !((addr & 0xFF000000) ^ (INADDR_LOOPBACK-1));
 #  endif /*IN_CLASSA && IN_CLASSA_NET && IN_CLASSA_NSHIFT*/
+    }
+    return 0/*false*/;
 }
 
 
 extern int/*bool*/ NcbiIsLocalIP(unsigned int ip)
 {
-    unsigned int addr = SOCK_NetToHostLong(ip);
     size_t n;
-
     if (!s_Inited) {
         CORE_LOCK_WRITE;
         if (!s_Inited) {
             s_LoadLocalIPs();
             s_Inited = 1;
             CORE_UNLOCK;
-#if defined(_DEBUG)  &&  !defined(NDEBUG)
+#ifdef LOCALNET_DEBUG
             for (n = 0;  n < sizeof(s_LocalIP)/sizeof(s_LocalIP[0]);  n++) {
                 char buf[128];
                 const char* result =
@@ -239,15 +245,18 @@ extern int/*bool*/ NcbiIsLocalIP(unsigned int ip)
                 if (s_LocalIP[n].type == eIPRange_None)
                     break;
             }
-#endif /*_DEBUG && !NDEBUG*/
+#endif /*LOCALNET_DEBUG*/
         } else
             CORE_UNLOCK;
     }
-    for (n = 0;  n < sizeof(s_LocalIP)/sizeof(s_LocalIP[0]);  n++) {
-        if (s_LocalIP[n].type == eIPRange_None)
-            break;
-        if (NcbiIsInIPRange(s_LocalIP + n, addr))
-            return 1/*true*/;
+    if (ip) {
+        unsigned int addr = SOCK_NetToHostLong(ip);
+        for (n = 0;  n < sizeof(s_LocalIP)/sizeof(s_LocalIP[0]);  n++) {
+            if (s_LocalIP[n].type == eIPRange_None)
+                break;
+            if (NcbiIsInIPRange(s_LocalIP + n, addr))
+                return 1/*true*/;
+        }
     }
     return 0/*false*/;
 }
@@ -261,8 +270,8 @@ static int/*bool*/ s_IsPrivateIP(unsigned int ip)
     unsigned int addr = SOCK_NetToHostLong(ip);
     return
 #if defined(IN_CLASSA) && defined(IN_CLASSA_NET) && defined(IN_CLASSA_NSHIFT)
-        (IN_CLASSA(addr)  &&
-         (addr & IN_CLASSA_NET) == (IN_LOOPBACKNET << IN_CLASSA_NSHIFT))  ||
+        (IN_CLASSA(addr)
+         && (addr & IN_CLASSA_NET) == (IN_LOOPBACKNET << IN_CLASSA_NSHIFT))  ||
 #else
         !((addr & 0xFF000000) ^ (INADDR_LOOPBACK-1))  || /* 127/8 */
 #  endif /*IN_CLASSA_NET && IN_CLASSA_NSHIFT*/
@@ -286,36 +295,34 @@ static int/*bool*/ s_IsPrivateIP(unsigned int ip)
 static const char* s_SearchTrackingEnv(const char*        name,
                                        const char* const* tracking_env)
 {
-    const char* const* str;
     const char* result;
-    size_t name_len;
 
-    if ( !tracking_env ) {
+    if (!tracking_env) {
         result = getenv(name);
-#if 0/*defined(_DEBUG) && !defined(NDEBUG)*/
-        CORE_LOGF(eLOG_Trace, ("Getenv(`%s') = %s%s%s", name,
+#ifdef LOCALNET_DEBUG
+        CORE_LOGF(eLOG_Trace, ("Getenv('%s') = %s%s%s", name,
                                result ? "\""   : "",
                                result ? result : "NULL",
                                result ? "\""   : ""));
-#endif /*_DEBUG && !NDEBUG*/
+#endif /*LOCALNET_DEBUG*/
     } else {
+        size_t len = strlen(name);
+        const char* const* str;
         result = 0;
-        name_len = strlen(name);
         for (str = tracking_env;  *str;  ++str) {
-            if (strncasecmp(*str, name, name_len) == 0
-                &&  (*str)[name_len] == '=') {
-                result = &(*str)[name_len + 1];
+            if (strncasecmp(*str, name, len) == 0  &&  (*str)[len] == '=') {
+                result = &(*str)[++len];
                 break;
             }
         }
-#if 0/*defined(_DEBUG) && !defined(NDEBUG)*/
-        CORE_LOGF(eLOG_Trace, ("Tracking `%s' = %s%s%s", name,
+#ifdef LOCALNET_DEBUG
+        CORE_LOGF(eLOG_Trace, ("Tracking '%s' = %s%s%s", name,
                                result ? "\""   : "",
                                result ? result : "NULL",
                                result ? "\""   : ""));
-#endif /*_DEBUG && !NDEBUG*/
+#endif /*LOCALNET_DEBUG*/
     }
-    return result;
+    return result  &&  *(result += strspn(result, " \t")) ? result : 0;
 }
 
 
@@ -327,35 +334,41 @@ static const char* s_GetForwardedFor(const char* const* tracking_env,
     char *p, *q, *r, *s;
     unsigned int ip;
 
-    if (!f  ||  !*f)
+    if (!f)
         return 0;
     r = 0;
-    external = !(s = strdup(f + strspn(f, " \t"))) ? 1 : 0;
+    external = !(s = strdup(f)) ? 1 : 0;
     for (p = s;  p  &&  *p;  p += strspn(p, ", \t")) {
+        int/*bool*/ private_ip;
         q = p + strcspn(p, ", \t");
         if (*q) {
             *q++ = '\0';
         }
-        if (*p  &&  (ip = SOCK_gethostbyname(p)) != 0) {
-            int/*bool*/ private_ip;
-            if (!(private_ip = s_IsPrivateIP(ip))  &&  !NcbiIsLocalIP(ip)) {
-                r     = p;
-                *addr = ip;
-                break;
-            } else if (!r  ||  (!private_ip  &&  s_IsPrivateIP(*addr))) {
-                r     = p;
-                *addr = ip;
-            }
-        } else {
-            /* unresolvable name causes external mapping */
+        if (!*p  ||  !(ip = SOCK_gethostbyname(p))) {
+#ifdef LOCALNET_DEBUG
+            CORE_LOG(eLOG_Trace, "Forcing external");
+#endif /*LOCALNET_DEBUG*/
             external = 1;
+            r        = 0;
+        } else if (!(private_ip = s_IsPrivateIP(ip))  &&  !NcbiIsLocalIP(ip)) {
+            r        = p;
+            *addr    = ip;
+            break;
+        } else if (!external && (!r || (!private_ip && s_IsPrivateIP(*addr)))){
+            r        = p;
+            *addr    = ip;
         }
         p = q;
     }
     if (r) {
         memmove(s, r, strlen(r) + 1);
+        assert(*addr);
         return s;
     }
+    if (s) {
+        free(s);
+    }
+    *addr = 0;
     return external ? "" : 0;
 }
 
@@ -391,7 +404,8 @@ extern unsigned int NcbiGetCgiClientIPEx(ECgiClientIP       flag,
                                                               &probe[i].ip);
             break;
         case 2:
-            probe[i].host = s_SearchTrackingEnv("PROXIED_IP", tracking_env);
+            probe[i].host = s_SearchTrackingEnv("PROXIED_IP",
+                                                tracking_env);
             break;
         case 3:
             probe[i].host = s_SearchTrackingEnv("HTTP_X_FWD_IP_ADDR",
@@ -407,11 +421,15 @@ extern unsigned int NcbiGetCgiClientIPEx(ECgiClientIP       flag,
         if (!probe[i].ip  &&  *probe[i].host) {
             probe[i].ip = SOCK_gethostbyname(probe[i].host);
         }
-        if (NcbiIsLocalIP(probe[i].ip)) {
+        if (*probe[i].host  &&  NcbiIsLocalIP(probe[i].ip)) {
             continue;
         }
+#ifdef LOCALNET_DEBUG
+        CORE_LOG(eLOG_Trace, "External on");
+#endif /*LOCALNET_DEBUG*/
         external = 1;
-        if (!s_IsPrivateIP(probe[i].ip)) {
+        if (probe[i].ip  &&  !s_IsPrivateIP(probe[i].ip)) {
+            assert(probe[i].host);
             host = probe[i].host;
             ip   = probe[i].ip;
             break;
@@ -452,7 +470,7 @@ extern unsigned int NcbiGetCgiClientIPEx(ECgiClientIP       flag,
             if (!xhost) {
                 continue;
             }
-            if (i < 1  ||  i > 4) {
+            if (i < 1  ||  4 < i) {
                 xip = *xhost ? SOCK_gethostbyname(xhost) : 0;
             }
             if (!xip) {
@@ -465,12 +483,13 @@ extern unsigned int NcbiGetCgiClientIPEx(ECgiClientIP       flag,
             }
         }
     }
+    assert((!ip  &&  (!host  ||  !*host))  ||
+           ( ip  &&    host  &&   *host));
     if (buf  &&  buf_size) {
-        if (host) {
-            strncpy0(buf, host, buf_size - 1);
-        } else {
+        if (host  &&  (i = strlen(host)) < buf_size)
+            memcpy(buf, host, ++i);
+        else
             buf[0] = '\0';
-        }
     }
     if (forwarded_for  &&  *forwarded_for) {
         free((void*) forwarded_for);
