@@ -1,4 +1,4 @@
-/* $Id: ncbi_conn_stream.cpp 350577 2012-01-21 00:28:59Z rafanovi $
+/* $Id: ncbi_conn_stream.cpp 362481 2012-05-09 17:23:43Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,24 +33,22 @@
  */
 
 #include <ncbi_pch.hpp>
+#include <corelib/ncbiapp.hpp>
 #include "ncbi_conn_streambuf.hpp"
-#include <connect/error_codes.hpp>
 #include <connect/ncbi_conn_exception.hpp>
+#define NCBI_CONN_STREAM_EXPERIMENTAL_API 1  // Pick up MS-Win DLL linkage
 #include <connect/ncbi_conn_stream.hpp>
 #include <connect/ncbi_file_connector.h>
 #include <connect/ncbi_socket.hpp>
-#include <corelib/ncbiapp.hpp>
+#include <connect/ncbi_util.h>
 #include <stdlib.h>
 
 
-#define NCBI_USE_ERRCODE_X   Connect_Stream
-
-
 BEGIN_NCBI_SCOPE
 
 
 CConn_IOStream::CConn_IOStream(CONNECTOR connector, const STimeout* timeout,
-                               streamsize buf_size, bool tie,
+                               size_t buf_size, bool tie,
                                CT_CHAR_TYPE* ptr, size_t size) :
     CNcbiIostream(0), m_CSb(0)
 {
@@ -66,7 +64,7 @@ CConn_IOStream::CConn_IOStream(CONNECTOR connector, const STimeout* timeout,
 
 
 CConn_IOStream::CConn_IOStream(CONN conn, bool close, const STimeout* timeout,
-                               streamsize buf_size, bool tie,
+                               size_t buf_size, bool tie,
                                CT_CHAR_TYPE* ptr, size_t size) :
     CNcbiIostream(0), m_CSb(0)
 {
@@ -186,7 +184,7 @@ EIO_Status CConn_IOStream::x_IsCanceled(CONN           conn,
     CConn_IOStream* io = reinterpret_cast<CConn_IOStream*>(data);
     if (/* io && */ io->m_Canceled.NotNull()  &&  io->m_Canceled->IsCanceled())
         return eIO_Interrupt;
-    int n = (int) type - (int) eIO_Read;
+    int n = (int) type - (int) eCONN_OnRead;
     _ASSERT(n >= 0  &&  (size_t) n < sizeof(io->m_CB) / sizeof(io->m_CB[0]));
     _ASSERT((n == 0  &&  type == eCONN_OnRead)   ||
             (n == 1  &&  type == eCONN_OnWrite)  ||
@@ -197,21 +195,12 @@ EIO_Status CConn_IOStream::x_IsCanceled(CONN           conn,
 }
 
 
-static CONNECTOR x_SC(CONNECTOR socket_connector, SOCK*& sockptr)
-{
-    // HACK * HACK * HACK
-    sockptr = socket_connector ? (SOCK*) socket_connector->handle : 0;
-    return socket_connector;
-}
-
-
 CConn_SocketStream::CConn_SocketStream(const string&   host,
                                        unsigned short  port,
                                        unsigned short  max_try,
                                        const STimeout* timeout,
-                                       streamsize      buf_size)
-    : CConn_IOStream(x_SC(SOCK_CreateConnector(host.c_str(), port, max_try),
-                          m_SockPtr),
+                                       size_t          buf_size)
+    : CConn_IOStream(SOCK_CreateConnector(host.c_str(), port, max_try),
                      timeout, buf_size)
 {
     return;
@@ -225,10 +214,9 @@ CConn_SocketStream::CConn_SocketStream(const string&   host,
                                        TSOCK_Flags     flags,
                                        unsigned short  max_try,
                                        const STimeout* timeout,
-                                       streamsize      buf_size)
-    : CConn_IOStream(x_SC(SOCK_CreateConnectorEx(host.c_str(), port, max_try,
-                                                 data, size, flags),
-                          m_SockPtr),
+                                       size_t          buf_size)
+    : CConn_IOStream(SOCK_CreateConnectorEx(host.c_str(), port, max_try,
+                                            data, size, flags),
                      timeout, buf_size)
 {
     return;
@@ -238,10 +226,8 @@ CConn_SocketStream::CConn_SocketStream(const string&   host,
 CConn_SocketStream::CConn_SocketStream(SOCK            sock,
                                        EOwnership      if_to_own,
                                        const STimeout* timeout,
-                                       streamsize      buf_size)
-    : CConn_IOStream(x_SC(SOCK_CreateConnectorOnTop(sock,
-                                                    if_to_own != eNoOwnership),
-                          m_SockPtr),
+                                       size_t          buf_size)
+    : CConn_IOStream(SOCK_CreateConnectorOnTop(sock,if_to_own != eNoOwnership),
                      timeout, buf_size)
 {
     return;
@@ -255,37 +241,45 @@ static CONNECTOR s_TunneledSocketConnector(const SConnNetInfo* net_info,
 {
     EIO_Status status;
     SOCK       sock = 0;
+    bool       proxy = false;
 
     _ASSERT(net_info);
+    if ((flags & (fSOCK_LogOn | fSOCK_LogDefault)) == fSOCK_LogDefault
+        &&  net_info->debug_printout == eDebugPrintout_Data) {
+        flags &= ~fSOCK_LogDefault;
+        flags |=  fSOCK_LogOn;
+    }
     if (*net_info->http_proxy_host  &&  net_info->http_proxy_port) {
-        status = HTTP_CreateTunnel(net_info, fHTTP_DetachableTunnel
-                                   | fHTTP_NoAutoRetry, &sock);
-        if (status == eIO_Success) {
-            size_t handle_size = SOCK_OSHandleSize();
-            char*  handle      = new char[handle_size];
-            status = SOCK_GetOSHandle(sock, handle, handle_size);
+        status = HTTP_CreateTunnel(net_info, fHTTP_NoAutoRetry, &sock);
+        _ASSERT(!sock ^ !(status != eIO_Success));
+        if (status == eIO_Success
+            &&  ((flags & ~(fSOCK_LogOn | fSOCK_LogDefault))  ||  init_size)) {
+            size_t size   = SOCK_OSHandleSize();
+            char*  handle = new char[size];
+            status = SOCK_GetOSHandleEx(sock, handle, size, eTakeOwnership);
             if (status == eIO_Success) {
                 SOCK_Close(sock);
-                status = SOCK_CreateOnTopEx(handle, handle_size, &sock,
+                status = SOCK_CreateOnTopEx(handle, size, &sock,
                                             init_data, init_size, flags);
                 if (status != eIO_Success) {
-                    SOCK_CloseOSHandle(handle, handle_size);
+                    SOCK_CloseOSHandle(handle, size);
                     _ASSERT(!sock);
                 } else
                     _ASSERT(sock);
             } else {
                 SOCK_Abort(sock);
                 SOCK_Close(sock);
+                sock = 0;
             }
             delete[] handle;
-        } else
-            _ASSERT(!sock);
-        if (!sock  &&  !net_info->http_proxy_leak)
-            return 0;
+        }
+        proxy = true;
     }
-    if (!sock) {
+    if (!sock  &&  (!proxy  ||  net_info->http_proxy_leak)) {
         const char* host = (net_info->firewall  &&  *net_info->proxy_host
                             ? net_info->proxy_host : net_info->host);
+        if (!proxy  &&  net_info->debug_printout)
+            ConnNetInfo_LogEx(net_info, eLOG_Note, CORE_GetLOG());
         status = SOCK_CreateEx(host, net_info->port, net_info->timeout, &sock,
                                init_data, init_size, flags);
         _ASSERT(!sock ^ !(status != eIO_Success));
@@ -306,10 +300,8 @@ CConn_SocketStream::CConn_SocketStream(const SConnNetInfo& net_info,
                                        const void*         data,
                                        size_t              size,
                                        TSOCK_Flags         flags,
-                                       streamsize          buf_size)
-    : CConn_IOStream(x_SC(s_TunneledSocketConnector(&net_info,
-                                                    data, size, flags),
-                          m_SockPtr),
+                                       size_t              buf_size)
+    : CConn_IOStream(s_TunneledSocketConnector(&net_info, data, size, flags),
                      net_info.timeout, buf_size)
 {
     return;
@@ -322,12 +314,12 @@ static SOCK s_GrabSOCK(CSocket& socket)
     if (!sock) {
         NCBI_THROW(CIO_Exception, eInvalidArg,
                    "CConn_SocketStream::CConn_SocketStream(): "
-                   "Socket may not be empty");
+                   " Socket may not be empty");
     }
     if (socket.SetOwnership(eNoOwnership) == eNoOwnership) {
         NCBI_THROW(CIO_Exception, eInvalidArg,
                    "CConn_SocketStream::CConn_SocketStream(): "
-                   "Socket must be owned");
+                   " Socket must be owned");
     }
     socket.Reset(0/*empty*/,
                  eNoOwnership/*irrelevant*/,
@@ -338,10 +330,8 @@ static SOCK s_GrabSOCK(CSocket& socket)
 
 CConn_SocketStream::CConn_SocketStream(CSocket&        socket,
                                        const STimeout* timeout,
-                                       streamsize      buf_size)
-    : CConn_IOStream(x_SC(SOCK_CreateConnectorOnTop(s_GrabSOCK(socket),
-                                                    1/*own*/),
-                          m_SockPtr),
+                                       size_t          buf_size)
+    : CConn_IOStream(SOCK_CreateConnectorOnTop(s_GrabSOCK(socket), 1/*own*/),
                      timeout, buf_size)
 {
     return;
@@ -436,7 +426,7 @@ CConn_HttpStream::CConn_HttpStream(const string&   host,
                                    unsigned short  port,
                                    THTTP_Flags     flags,
                                    const STimeout* timeout,
-                                   streamsize      buf_size)
+                                   size_t          buf_size)
     : CConn_IOStream(s_HttpConnectorBuilder(0,
                                             0,
                                             host.c_str(),
@@ -459,7 +449,7 @@ CConn_HttpStream::CConn_HttpStream(const string&   host,
 CConn_HttpStream::CConn_HttpStream(const string&   url,
                                    THTTP_Flags     flags,
                                    const STimeout* timeout,
-                                   streamsize      buf_size)
+                                   size_t          buf_size)
     : CConn_IOStream(s_HttpConnectorBuilder(0,
                                             url.c_str(),
                                             0,
@@ -484,7 +474,7 @@ CConn_HttpStream::CConn_HttpStream(const string&       url,
                                    const string&       user_header,
                                    THTTP_Flags         flags,
                                    const STimeout*     timeout,
-                                   streamsize          buf_size)
+                                   size_t              buf_size)
     : CConn_IOStream(s_HttpConnectorBuilder(net_info,
                                             url.c_str(),
                                             0,
@@ -512,7 +502,7 @@ CConn_HttpStream::CConn_HttpStream(const SConnNetInfo* net_info,
                                    FHTTP_Cleanup       cleanup,
                                    THTTP_Flags         flags,
                                    const STimeout*     timeout,
-                                   streamsize          buf_size)
+                                   size_t              buf_size)
     : CConn_IOStream(s_HttpConnectorBuilder(net_info,
                                             0,
                                             0,
@@ -555,13 +545,7 @@ static CONNECTOR s_ServiceConnectorBuilder(const char*           service,
         net_info->timeout = &net_info->tmo;
     } else if (!timeout)
         net_info->timeout = 0;
-    CONNECTOR c = SERVICE_CreateConnectorEx(service, types,
-                                            net_info.get(), params);
-    if (!c) {
-        ERR_POST_X(1,
-                   Error << "Cannot connect to service \"" << service << '\"');
-    }
-    return c;
+    return SERVICE_CreateConnectorEx(service, types, net_info.get(), params);
 }
 
 
@@ -570,7 +554,7 @@ CConn_ServiceStream::CConn_ServiceStream(const string&         service,
                                          const SConnNetInfo*   net_info,
                                          const SSERVICE_Extra* params,
                                          const STimeout*       timeout,
-                                         streamsize            buf_size)
+                                         size_t                buf_size)
     : CConn_IOStream(s_ServiceConnectorBuilder(service.c_str(),
                                                types,
                                                net_info,
@@ -588,7 +572,7 @@ CConn_ServiceStream::CConn_ServiceStream(const string&         service,
                                          TSERV_Type            types,
                                          const SSERVICE_Extra* params,
                                          const STimeout*       timeout,
-                                         streamsize            buf_size)
+                                         size_t                buf_size)
     : CConn_IOStream(s_ServiceConnectorBuilder(service.c_str(),
                                                types,
                                                0,
@@ -601,7 +585,7 @@ CConn_ServiceStream::CConn_ServiceStream(const string&         service,
 }
 
 
-CConn_MemoryStream::CConn_MemoryStream(streamsize buf_size)
+CConn_MemoryStream::CConn_MemoryStream(size_t buf_size)
     : CConn_IOStream(MEMORY_CreateConnector(), 0, buf_size, true),
       m_Ptr(0)
 {
@@ -611,7 +595,7 @@ CConn_MemoryStream::CConn_MemoryStream(streamsize buf_size)
 
 CConn_MemoryStream::CConn_MemoryStream(BUF        buf,
                                        EOwnership owner,
-                                       streamsize buf_size)
+                                       size_t     buf_size)
     : CConn_IOStream(MEMORY_CreateConnectorEx(buf, owner == eTakeOwnership
                                               ? 1/*true*/
                                               : 0/*false*/), 0, buf_size, true,
@@ -625,7 +609,7 @@ CConn_MemoryStream::CConn_MemoryStream(BUF        buf,
 CConn_MemoryStream::CConn_MemoryStream(const void* ptr,
                                        size_t      size,
                                        EOwnership  owner,
-                                       streamsize  buf_size)
+                                       size_t      buf_size)
     : CConn_IOStream(MEMORY_CreateConnector(), 0, buf_size, true,
                      (CT_CHAR_TYPE*) ptr, size),
       m_Ptr(owner == eTakeOwnership ? ptr : 0)
@@ -650,12 +634,17 @@ void CConn_MemoryStream::ToString(string* str)
                    "CConn_MemoryStream::ToString(NULL) is not allowed");
     }
     CConn_Streambuf* sb = dynamic_cast<CConn_Streambuf*>(rdbuf());
-    streamsize size = sb ? (size_t)(tellp() - tellg()) : 0;
+    size_t size = sb ? (size_t)(tellp() - tellg()) : 0;
     str->resize(size);
     if (sb) {
-        streamsize s = sb->sgetn(&(*str)[0], size);
-        _ASSERT(size == s);
-        str->resize(s);  // NB: this is essentially a NOOP when size == s
+        size_t s = (size_t)sb->sgetn(&(*str)[0], size);
+        _ASSERT(s == size);
+#ifdef NCBI_COMPILER_WORKSHOP
+        if (s < 0) {
+            s = 0; // WS6 weirdness to sometimes return -1 from sgetn() :-/
+        }
+#endif //NCBI_COMPILER_WORKSHOP
+        str->resize(s);  // NB: just in case, essentially NOOP when s == size
     }
 }
 
@@ -667,40 +656,26 @@ void CConn_MemoryStream::ToVector(vector<char>* vec)
                    "CConn_MemoryStream::ToVector(NULL) is not allowed");
     }
     CConn_Streambuf* sb = dynamic_cast<CConn_Streambuf*>(rdbuf());
-    streamsize size = sb ? (streamsize)(tellp() - tellg()) : 0;
+    size_t size = sb ? (size_t)(tellp() - tellg()) : 0;
     vec->resize(size);
     if (sb) {
-        streamsize s = sb->sgetn(&(*vec)[0], size);
-        _ASSERT(size == s);
-        vec->resize(s);  // NB: this is essentially a NOOP when size == s
+        size_t s = (size_t)sb->sgetn(&(*vec)[0], size);
+        _ASSERT(s == size);
+#ifdef NCBI_COMPILER_WORKSHOP
+        if (s < 0) {
+            s = 0; // WS6 weirdness to sometimes return -1 from sgetn() :-/
+        }
+#endif //NCBI_COMPILER_WORKSHOP
+        vec->resize(s);  // NB: just in case, essentially NOOP when s == size
     }
 }
 
 
-char* CConn_MemoryStream::ToCStr(void)
-{
-    CConn_Streambuf* sb = dynamic_cast<CConn_Streambuf*>(rdbuf());
-    streamsize size = sb ? (size_t)(tellp() - tellg()) : 0;
-    char* str = (char*) malloc(size + 1);
-    if (!str) {
-        NCBI_THROW(CIO_Exception, eUnknown,
-                   "CConn_MemoryStream::ToCStr():  Out of memory");
-    }
-    if (sb) {
-        streamsize s = sb->sgetn(str, size);
-        _ASSERT(size == s);
-        size = s;
-    }
-    str[size] = '\0';
-    return str;
-}
-
-
 CConn_PipeStream::CConn_PipeStream(const string&         cmd,
                                    const vector<string>& args,
                                    CPipe::TCreateFlags   create_flags,
                                    const STimeout*       timeout,
-                                   streamsize            buf_size)
+                                   size_t                buf_size)
     : CConn_IOStream(PIPE_CreateConnector(cmd, args, create_flags,
                                           &m_Pipe, eNoOwnership),
                      timeout, buf_size)
@@ -720,7 +695,7 @@ CConn_PipeStream::~CConn_PipeStream()
 CConn_NamedPipeStream::CConn_NamedPipeStream(const string&   pipename,
                                              size_t          pipebufsize,
                                              const STimeout* timeout,
-                                             streamsize      buf_size)
+                                             size_t          buf_size)
     : CConn_IOStream(NAMEDPIPE_CreateConnector(pipename, pipebufsize),
                      timeout, buf_size)
 {
@@ -746,7 +721,7 @@ CConn_FtpStream::CConn_FtpStream(const string&        host,
     : CConn_IOStream(FTP_CreateConnectorSimple(host.c_str(), port,
                                                user.c_str(), pass.c_str(),
                                                path.c_str(), flag, cmcb),
-                     timeout, 0/*must be unbuffered*/, false/*thus,untied*/)
+                     timeout, 0/*must be unbuffered*/, false/*thus untied*/)
 {
     return;
 }
@@ -798,16 +773,22 @@ CConn_FTPDownloadStream::CConn_FTPDownloadStream(const string&        host,
                                                  const STimeout*      timeout)
     : CConn_FtpStream(host, user, pass, path, port, flag, cmcb, timeout)
 {
+    // Use '\n' here instead of NcbiFlush to avoid (and thus make silent)
+    // flush errors on retrieval of inexistent (or bad) files / directories..
     if (!file.empty()) {
         EIO_Status status;
         if (offset) {
-            write("REST ", 5) << NStr::UInt8ToString(offset) << NcbiFlush;
-            status = Status(eIO_Write);
+            write("REST ", 5) << NStr::UInt8ToString(offset) << '\n';
+            status  = Status(eIO_Write);
         } else
-            status = eIO_Success;
+            status  = eIO_Success;
         if (good()  &&  status == eIO_Success) {
-            write("RETR ", 5) << file << NcbiFlush;
+            bool directory = NStr::EndsWith(file, '/');
+            write(directory ? "NLST " : "RETR ", 5) << file << '\n';
+            status  = Status(eIO_Write);
         }
+        if (status != eIO_Success)
+            setstate(NcbiBadbit);
     }
 }
 
@@ -830,13 +811,27 @@ CConn_FTPUploadStream::CConn_FTPUploadStream(const string&   host,
             status = Status(eIO_Write);
         } else
             status = eIO_Success;
-        if (good()  &&  status == eIO_Success) {
+        if (good()  &&  status == eIO_Success)
             write("STOR ", 5) << file << NcbiFlush;
-        }
     }
 }
 
 
+const char* CIO_Exception::GetErrCodeString(void) const
+{
+    switch (GetErrCode()) {
+    case eTimeout:       return "eIO_Timeout";
+    case eClosed:        return "eIO_Closed";
+    case eInterrupt:     return "eIO_Interrupt";
+    case eInvalidArg:    return "eIO_InvalidArg";
+    case eNotSupported:  return "eIO_NotSupported";
+    case eUnknown:       return "eIO_Unknown";
+    default:             break;
+    }
+    return  CException::GetErrCodeString();
+}
+
+
 /* non-public class */
 class CConn_FileStream : public CConn_IOStream
 {
@@ -858,34 +853,40 @@ private:
 };
 
 
-const char* CIO_Exception::GetErrCodeString(void) const
+static bool x_IsIdentifier(const string& str)
 {
-    switch (GetErrCode()) {
-    case eTimeout:       return "eIO_Timeout";
-    case eClosed:        return "eIO_Closed";
-    case eInterrupt:     return "eIO_Interrupt";
-    case eInvalidArg:    return "eIO_InvalidArg";
-    case eNotSupported:  return "eIO_NotSupported";
-    case eUnknown:       return "eIO_Unknown";
-    default:             return  CException::GetErrCodeString();
+    const char* s = str.c_str();
+    if (!isalpha((unsigned char)(*s)))
+        return false;
+    for (++s;  *s;  ++s) {
+        if (!isalnum((unsigned char)(*s))  &&  *s != '_')
+            return false;
     }
+    return true;
 }
 
 
-extern CConn_IOStream* NcbiOpenURL(const string& url)
+extern
+NCBI_XCONNECT_EXPORT  // FIXME: To remove once the API is fully official
+CConn_IOStream* NcbiOpenURL(const string& url)
 {
-    class CInternalIniter : public CConnIniter {
+    class CPrivateIniter : public CConnIniter {
     public:
-        CInternalIniter(void)
+        CPrivateIniter(void)
         { }
     };
-    CInternalIniter init;
+    CPrivateIniter init;
 
-    AutoPtr<SConnNetInfo> net_info = ConnNetInfo_Create(0);
+    bool svc = x_IsIdentifier(url);
+
+    AutoPtr<SConnNetInfo> net_info = ConnNetInfo_Create(svc ? url.c_str() : 0);
+
+    if (svc)
+        return new CConn_ServiceStream(url, fSERV_Any, net_info.get());
 
     unsigned int   host;
     unsigned short port;
-    if (url.size() == CSocketAPI::StringToHostPort(url, &host, &port))
+    if (url.size() == CSocketAPI::StringToHostPort(url, &host, &port) && port)
         net_info->req_method = eReqMethod_Connect;
 
     if (ConnNetInfo_ParseURL(net_info.get(), url.c_str())) {
@@ -898,13 +899,17 @@ extern CConn_IOStream* NcbiOpenURL(const string& url)
         case eURL_File:
             if (*net_info->host  ||  net_info->port)
                 break; /*not supported*/
+            if (net_info->debug_printout)
+                ConnNetInfo_LogEx(net_info.get(), eLOG_Note, CORE_GetLOG());
             return new CConn_FileStream(net_info->path);
         case eURL_Ftp:
+            if (net_info->debug_printout)
+                ConnNetInfo_LogEx(net_info.get(), eLOG_Note, CORE_GetLOG());
             return new CConn_FTPDownloadStream(net_info->host,
                                                net_info->path,
                                                net_info->user,
                                                net_info->pass,
-                                               kEmptyStr,
+                                               kEmptyStr/*path*/,
                                                net_info->port,
                                                net_info->debug_printout
                                                == eDebugPrintout_Data
