@@ -1,4 +1,4 @@
-/* $Id: ncbi_conn_streambuf.cpp 344554 2011-11-16 18:11:59Z lavr $
+/* $Id: ncbi_conn_streambuf.cpp 360860 2012-04-26 18:46:08Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,9 +33,9 @@
 #include <ncbi_pch.hpp>
 #include "ncbi_conn_streambuf.hpp"
 #include <corelib/ncbidbg.hpp>
+#include <corelib/ncbi_limits.hpp>
 #include <connect/ncbi_conn_exception.hpp>
 #include <connect/error_codes.hpp>
-#include <memory>
 #include <stdlib.h>  // free()
 
 #define NCBI_USE_ERRCODE_X   Connect_Stream
@@ -69,7 +69,7 @@ string CConn_Streambuf::x_Message(const char* msg)
 
 CConn_Streambuf::CConn_Streambuf(CONNECTOR       connector,
                                  const STimeout* timeout,
-                                 streamsize      buf_size,
+                                 size_t          buf_size,
                                  bool            tie,
                                  CT_CHAR_TYPE*   ptr,
                                  size_t          size)
@@ -78,12 +78,12 @@ CConn_Streambuf::CConn_Streambuf(CONNECTOR       connector,
       x_GPos((CT_OFF_TYPE)(ptr ? size : 0)), x_PPos((CT_OFF_TYPE) size)
 {
     if (!connector) {
-        ERR_POST_X(2, x_Message("CConn_Streambuf(): NULL connector"));
+        ERR_POST_X(2, x_Message("CConn_Streambuf():  NULL connector"));
         return;
     }
     TCONN_Flags flag = tie ? 0 : fCONN_Untie;
     if ((m_Status = CONN_CreateEx(connector, flag, &m_Conn)) != eIO_Success) {
-        ERR_POST_X(3, x_Message("CConn_Streambuf(): CONN_Create() failed"));
+        ERR_POST_X(3, x_Message("CConn_Streambuf():  CONN_Create() failed"));
         if (connector->destroy)
             connector->destroy(connector);
         return;
@@ -96,7 +96,7 @@ CConn_Streambuf::CConn_Streambuf(CONNECTOR       connector,
 CConn_Streambuf::CConn_Streambuf(CONN            conn,
                                  bool            close,
                                  const STimeout* timeout,
-                                 streamsize      buf_size,
+                                 size_t          buf_size,
                                  bool            tie,
                                  CT_CHAR_TYPE*   ptr,
                                  size_t          size)
@@ -105,7 +105,7 @@ CConn_Streambuf::CConn_Streambuf(CONN            conn,
       x_GPos((CT_OFF_TYPE)(ptr ? size : 0)), x_PPos((CT_OFF_TYPE) size)
 {
     if (!m_Conn) {
-        ERR_POST_X(1, x_Message("CConn_Streambuf(): NULL connection"));
+        ERR_POST_X(1, x_Message("CConn_Streambuf():  NULL connection"));
         return;
     }
     x_Init(timeout, buf_size, ptr, size);
@@ -114,12 +114,12 @@ CConn_Streambuf::CConn_Streambuf(CONN            conn,
 
 EIO_Status CConn_Streambuf::Status(EIO_Event direction) const
 {
-    return direction == eIO_Open ? m_Status
+    return direction == eIO_Close ? m_Status
         : m_Conn ? CONN_Status(m_Conn, direction) : eIO_Closed;
 }
 
 
-void CConn_Streambuf::x_Init(const STimeout* timeout, streamsize buf_size,
+void CConn_Streambuf::x_Init(const STimeout* timeout, size_t buf_size,
                              CT_CHAR_TYPE* ptr, size_t size)
 {
     if (timeout != kDefaultTimeout) {
@@ -157,7 +157,7 @@ EIO_Status CConn_Streambuf::x_Close(bool close)
         if ((status = CONN_Status(m_Conn, eIO_Write)) != eIO_Success) {
             m_Status = status;
             if (CONN_Status(m_Conn, eIO_Open) == eIO_Success) {
-                _TRACE(x_Message("Close(): Cannot finalize implicitly"
+                _TRACE(x_Message("Close():  Cannot finalize implicitly"
                                  ", data loss may result"));
             }
         } else if (sync() != 0)
@@ -180,7 +180,7 @@ EIO_Status CConn_Streambuf::x_Close(bool close)
                 CONN_SetCallback(c, eCONN_OnClose, &cb, 0);
         }
         if (m_Close  &&  (m_Status = CONN_Close(c)) != eIO_Success) {
-            _TRACE(x_Message("Close(): CONN_Close() failed"));
+            _TRACE(x_Message("Close():  CONN_Close() failed"));
             if (status == eIO_Success)
                 status  = m_Status;
         }
@@ -212,14 +212,14 @@ CT_INT_TYPE CConn_Streambuf::overflow(CT_INT_TYPE c)
 
     if (pbase()) {
         // send buffer
-        size_t n_towrite = pptr() - pbase();
+        size_t n_towrite = (size_t)(pptr() - pbase());
         if (n_towrite) {
             size_t n_written;
             m_Status = CONN_Write(m_Conn, pbase(),
                                   n_towrite, &n_written, eIO_WritePlain);
             if (!n_written) {
                 _ASSERT(m_Status != eIO_Success);
-                ERR_POST_X(4, x_Message("overflow(): CONN_Write() failed"));
+                ERR_POST_X(4, x_Message("overflow():  CONN_Write() failed"));
                 return CT_EOF;
             }
             // update buffer content (get rid of the data just sent)
@@ -238,17 +238,17 @@ CT_INT_TYPE CConn_Streambuf::overflow(CT_INT_TYPE c)
         m_Status = CONN_Write(m_Conn, &b, 1, &n_written, eIO_WritePlain);
         if (!n_written) {
             _ASSERT(m_Status != eIO_Success);
-            ERR_POST_X(5, x_Message("overflow(): CONN_Write(1) failed"));
+            ERR_POST_X(5, x_Message("overflow():  CONN_Write(1) failed"));
             return CT_EOF;
         }
-        x_PPos += (CT_OFF_TYPE) n_written;
+        x_PPos += (CT_OFF_TYPE) 1;
         _ASSERT(n_written == 1);
         return c;
     }
 
     _ASSERT(CT_EQ_INT_TYPE(c, CT_EOF));
     if ((m_Status = CONN_Flush(m_Conn)) != eIO_Success) {
-        ERR_POST_X(9, x_Message("overflow(): CONN_Flush() failed"));
+        ERR_POST_X(9, x_Message("overflow():  CONN_Flush() failed"));
         return CT_EOF;
     }
     return CT_NOT_EOF(CT_EOF);
@@ -262,6 +262,7 @@ streamsize CConn_Streambuf::xsputn(const CT_CHAR_TYPE* buf, streamsize m)
 
     if (m <= 0)
         return 0;
+    _ASSERT((Uint8)m < numeric_limits<size_t>::max());
     size_t n = (size_t) m;
 
     m_Status = eIO_Success;
@@ -276,13 +277,13 @@ streamsize CConn_Streambuf::xsputn(const CT_CHAR_TYPE* buf, streamsize m)
                 x_written = (size_t)(epptr() - pptr());
                 if (x_written > n)
                     x_written = n;
-                if ( x_written ) {
+                if (x_written) {
                     memcpy(pptr(), buf, x_written);
                     pbump(int(x_written));
                     n_written += x_written;
                     n         -= x_written;
                     if (!n)
-                        return n_written;
+                        return (streamsize) n_written;
                     buf       += x_written;
                 }
             }
@@ -293,7 +294,7 @@ streamsize CConn_Streambuf::xsputn(const CT_CHAR_TYPE* buf, streamsize m)
                                       &x_written, eIO_WriteSupplement);
                 if (!x_written) {
                     _ASSERT(m_Status != eIO_Success);
-                    ERR_POST_X(6, x_Message("xsputn():"
+                    ERR_POST_X(6, x_Message("xsputn(): "
                                             " CONN_Write() failed"));
                     break;
                 }
@@ -309,7 +310,7 @@ streamsize CConn_Streambuf::xsputn(const CT_CHAR_TYPE* buf, streamsize m)
         m_Status = CONN_Write(m_Conn, buf, n, &x_written, eIO_WriteSupplement);
         if (!x_written) {
             _ASSERT(m_Status != eIO_Success);
-            ERR_POST_X(7, x_Message("xsputn():"
+            ERR_POST_X(7, x_Message("xsputn(): "
                                     " CONN_Write(direct) failed"));
             break;
         }
@@ -361,7 +362,7 @@ CT_INT_TYPE CConn_Streambuf::underflow(void)
     if (!n_read) {
         _ASSERT(m_Status != eIO_Success);
         if (m_Status != eIO_Closed)
-            ERR_POST_X(8, x_Message("underflow(): CONN_Read() failed"));
+            ERR_POST_X(8, x_Message("underflow():  CONN_Read() failed"));
         return CT_EOF;
     }
 
@@ -384,6 +385,7 @@ streamsize CConn_Streambuf::xsgetn(CT_CHAR_TYPE* buf, streamsize m)
 
     if (m <= 0)
         return 0;
+    _ASSERT((Uint8)m < numeric_limits<size_t>::max());
     size_t n = (size_t) m;
 
     // first, read from the memory buffer
@@ -397,14 +399,14 @@ streamsize CConn_Streambuf::xsgetn(CT_CHAR_TYPE* buf, streamsize m)
 
     while (n) {
         // next, read from the connection
-        size_t       x_read = n < (size_t) m_BufSize ? m_BufSize : n;
-        CT_CHAR_TYPE* x_buf = n < (size_t) m_BufSize ? m_ReadBuf : buf;
+        size_t       x_read = n < m_BufSize ? m_BufSize : n;
+        CT_CHAR_TYPE* x_buf = n < m_BufSize ? m_ReadBuf : buf;
         m_Status = CONN_Read(m_Conn, x_buf, x_read,
                              &x_read, eIO_ReadSupplement);
         if (!x_read) {
             _ASSERT(m_Status != eIO_Success);
             if (m_Status != eIO_Closed)
-                ERR_POST_X(10, x_Message("xsgetn(): CONN_Read() failed"));
+                ERR_POST_X(10, x_Message("xsgetn():  CONN_Read() failed"));
             break;
         }
         x_GPos += (CT_OFF_TYPE) x_read;
@@ -417,7 +419,7 @@ streamsize CConn_Streambuf::xsgetn(CT_CHAR_TYPE* buf, streamsize m)
             setg(m_ReadBuf, m_ReadBuf + x_read, m_ReadBuf + xx_read);
         } else {
             _ASSERT(x_read <= n);
-            size_t xx_read = x_read > (size_t) m_BufSize ? m_BufSize : x_read;
+            size_t xx_read = x_read > m_BufSize ? m_BufSize : x_read;
             memcpy(m_ReadBuf, buf + x_read - xx_read, xx_read);
             setg(m_ReadBuf, m_ReadBuf + xx_read, m_ReadBuf + xx_read);
         }
