@@ -1,4 +1,4 @@
-/* $Id: ncbi_conn_test.cpp 347447 2011-12-16 20:32:53Z lavr $
+/* $Id: ncbi_conn_test.cpp 366508 2012-06-14 18:04:58Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -38,13 +38,14 @@
 #include "ncbi_comm.h"
 #include "ncbi_priv.h"
 #include "ncbi_servicep.h"
-#include <algorithm>
 #include <iterator>
 
-#define __STR(x)  #x
-#define _STR(x)   __STR(x)
+#define _STR(x)           #x
+#define STRINGIFY(x)  _STR(x)
 
-#define NCBI_HELP_DESK  "NCBI Help Desk info@ncbi.nlm.nih.gov"
+#define HELP_EMAIL  (m_Email.empty()                                    \
+                     ? string("NCBI Help Desk info@ncbi.nlm.nih.gov")   \
+                     : m_Email)
 #define NCBI_FW_URL                                                     \
     "http://www.ncbi.nlm.nih.gov/IEB/ToolBox/NETWORK/firewall.html#Settings"
 
@@ -138,6 +139,7 @@ string CConnTest::x_TimeoutMsg(void)
 
 EIO_Status CConnTest::ConnStatus(bool failure, CConn_IOStream* io)
 {
+    EIO_Status status;
     string type = io ? io->GetType()        : kEmptyStr;
     string text = io ? io->GetDescription() : kEmptyStr;
     m_CheckPoint = (type
@@ -149,9 +151,10 @@ EIO_Status CConnTest::ConnStatus(bool failure, CConn_IOStream* io)
         return eIO_Unknown;
     if (!io->GetCONN())
         return eIO_Closed;
-    EIO_Status status = io->Status();
-    if (status != eIO_Success)
+    if ((status = io->Status())         != eIO_Success  ||
+        (status = io->Status(eIO_Open)) != eIO_Success) {
         return status;
+    }
     EIO_Status r_status = io->Status(eIO_Read);
     EIO_Status w_status = io->Status(eIO_Write);
     status = r_status > w_status ? r_status : w_status;
@@ -273,7 +276,7 @@ EIO_Status CConnTest::DispatcherOkay(string* reason)
     http.SetCanceledCallback(m_Canceled);
     char buf[1024];
     http.read(buf, sizeof(buf));
-    CTempString str(buf, http.gcount());
+    CTempString str(buf, (size_t) http.gcount());
     EIO_Status status = ConnStatus
         (okay != 1  ||
          NStr::FindNoCase(str, "NCBI Dispatcher Test Page") == NPOS  ||
@@ -292,7 +295,7 @@ EIO_Status CConnTest::DispatcherOkay(string* reason)
             }
             if (okay == 1) {
                 temp += "Service response was not recognized; please contact "
-                    NCBI_HELP_DESK "\n";
+                    + HELP_EMAIL + '\n';
             }
         } else
             temp += x_TimeoutMsg();
@@ -371,7 +374,7 @@ EIO_Status CConnTest::ServiceOkay(string* reason)
                 temp += str;
                 temp += "\" from your configuration\n";
             } else if (status != eIO_Timeout  ||  m_Timeout > kTimeout)
-                temp += "; please contact " NCBI_HELP_DESK "\n";
+                temp += "; please contact " + HELP_EMAIL + '\n';
         }
         if (status != eIO_Timeout) {
             const char* mapper = SERV_MapperName(iter);
@@ -475,7 +478,7 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
     if (m_Firewall) {
         temp += "This mode requires your firewall to be configured in such a"
             " way that it allows outbound connections to the port range ["
-            _STR(CONN_FWD_PORT_MIN) ".." _STR(CONN_FWD_PORT_MAX)
+            STRINGIFY(CONN_FWD_PORT_MIN) ".." STRINGIFY(CONN_FWD_PORT_MAX)
             "] (inclusive) at the two fixed NCBI hosts, 130.14.29.112"
             " and 165.112.7.12\n"
             "To set that up correctly, please have your network administrator"
@@ -571,14 +574,14 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
             temp += m_Fwd.size() + m_FwdFB.size() == 1 ? " port" : " ports";
         } else {
             status = eIO_Unknown;
-            temp = "No connection ports found, please contact " NCBI_HELP_DESK;
+            temp = "No connection ports found, please contact " + HELP_EMAIL;
         }
     } else if (status == eIO_Timeout) {
         temp = x_TimeoutMsg();
         if (m_Timeout > kTimeout)
-            temp += "You may want to contact " NCBI_HELP_DESK;
+            temp += "You may want to contact " + HELP_EMAIL;
     } else
-        temp = "Please contact " NCBI_HELP_DESK;
+        temp = "Please contact " + HELP_EMAIL;
 
     PostCheck(eFirewallConnPoints, 1/*sub*/, status, temp);
 
@@ -596,7 +599,7 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
             if (cp->status != eIO_Success) {
                 status = cp->status;
                 temp  = CSocketAPI::HostPortToString(cp->host, cp->port);
-                temp += " is not operational, please contact " NCBI_HELP_DESK;
+                temp += " is not operational, please contact " + HELP_EMAIL;
                 break;
             }
         }
@@ -604,12 +607,12 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
             if (!m_Firewall  &&  !m_FwdFB.empty()) {
                 status = eIO_Unknown;
                 temp = "Fallback ports found in non-firewall mode, please"
-                    " contact " NCBI_HELP_DESK;
+                    " contact " + HELP_EMAIL;
             } else if (m_Firewall != firewall) {
                 status = eIO_Unknown;
                 temp  = "Firewall ";
                 temp += firewall ? "wrongly" : "not";
-                temp += " acknowledged, please contact " NCBI_HELP_DESK;
+                temp += " acknowledged, please contact " + HELP_EMAIL;
             } else
                 temp.resize(2);
         }
@@ -623,6 +626,13 @@ EIO_Status CConnTest::GetFWConnections(string* reason)
 }
 
 
+static SOCK x_GetSOCK(const CConn_IOStream* fw)
+{
+    SOCK s;
+    return CONN_GetSOCK(fw->GetCONN(), &s) == eIO_Success ? s : 0;
+}
+
+
 EIO_Status CConnTest::CheckFWConnections(string* reason)
 {
     static const STimeout kZeroTmo = { 0, 0 };
@@ -738,7 +748,7 @@ EIO_Status CConnTest::CheckFWConnections(string* reason)
                 if (!fw  ||  cp->status != eIO_Success)
                     continue;
                 CSocket* s = new CSocket;
-                s->Reset(fw->GetSOCK(), eNoOwnership, eCopyTimeoutsFromSOCK);
+                s->Reset(x_GetSOCK(fw), eNoOwnership, eCopyTimeoutsFromSOCK);
                 sock.push_back(s);
                 poll.push_back(CSocketAPI::SPoll(sock.back().get(), eIO_Read));
             }
@@ -984,7 +994,7 @@ EIO_Status CConnTest::StatefulOkay(string* reason)
                     &&  (!(iter = SERV_OpenSimple(kId2))
                          ||  !SERV_GetNextInfo(iter))) {
                     temp += "The service is currently unavailable;"
-                        " you may want to contact " NCBI_HELP_DESK "\n";
+                        " you may want to contact " + HELP_EMAIL + '\n';
                 } else if (m_Fwd.empty()  &&  net_info
                            &&  net_info->firewall != eFWMode_Fallback) {
                     temp += "The most likely reason for the failure is that"
@@ -992,12 +1002,12 @@ EIO_Status CConnTest::StatefulOkay(string* reason)
                     temp += *net_info->proxy_host ? "forwarder" : "firewall";
                     temp += " is still blocking ports as reported above\n";
                 } else if (status != eIO_Timeout  ||  m_Timeout > kTimeout)
-                    temp += "Please contact " NCBI_HELP_DESK "\n";
+                    temp += "Please contact " + HELP_EMAIL + '\n';
                 SERV_Close(iter);
             }
         } else if (!str) {
             if (n  &&  net_info  &&  net_info->http_proxy_port
-                &&  NStr::strncasecmp(ry, kFWSign, n) == 0) {
+                &&  NStr::strncasecmp(ry, kFWSign, (size_t) n) == 0) {
                 temp += "NCBI Firewall";
                 if (!net_info->firewall)
                     temp += " (Connection Relay)";
@@ -1016,7 +1026,7 @@ EIO_Status CConnTest::StatefulOkay(string* reason)
             } else {
                 temp += n ? "Unrecognized" : "No";
                 temp += " response from service;"
-                    " please contact " NCBI_HELP_DESK "\n";
+                    " please contact " + HELP_EMAIL + '\n';
             }
         }
     }
