@@ -1,4 +1,4 @@
-/* $Id: ncbi_connection.c 346756 2011-12-09 17:14:08Z lavr $
+/* $Id: ncbi_connection.c 362485 2012-05-09 17:24:48Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,6 +32,7 @@
  */
 
 #include "ncbi_priv.h"
+#include "ncbi_socketp.h"
 #include <connect/ncbi_buffer.h>
 #include <connect/ncbi_connection.h>
 #include <stdlib.h>
@@ -78,7 +79,7 @@
 
 #define CONN_LOG(s_c, f_n, lvl, msg)  CONN_LOG_EX(s_c, f_n, lvl, msg, status)
 
-/* Standard macros to verify that the passed connection handle is not NULL
+/* Standard macros to verify that the connection handle is not NULL and valid.
  * NB: "retval" must be either a valid EIO_Status or 0 (no status logged)
  */
 #define CONN_NOT_NULL_EX(subcode, func_name, retval)                    \
@@ -341,6 +342,7 @@ extern EIO_Status CONN_CreateEx
             conn->c_timeout = kDefaultTimeout;
             conn->magic     = CONNECTION_MAGIC;
             if ((status = x_ReInit(conn, connector)) != eIO_Success) {
+                conn->magic = (unsigned int)(-1);
                 free(conn);
                 conn = 0;
             }
@@ -566,7 +568,8 @@ extern EIO_Status CONN_Wait
             level = eLOG_Error;
             break;
         }
-        CONN_LOG(14, Wait, level, kErrMsg[event != eIO_Read]);
+        CONN_LOG(event != eIO_Read ? 15 : 14, Wait, level,
+                 kErrMsg[event != eIO_Read]);
     }
     return status;
 }
@@ -981,7 +984,7 @@ extern EIO_Status CONN_ReadLine
             assert(done  ||  len >= size);
             if (!BUF_PushBack(&conn->buf, x_buf + i, x_read - i)) {
                 static const STimeout* timeout = 0/*dummy*/;
-                CONN_LOG_EX(15, ReadLine, eLOG_Critical,
+                CONN_LOG_EX(35, ReadLine, eLOG_Critical,
                             "Cannot pushback extra data", 0);
                 conn->state = eCONN_Bad;
                 status = eIO_Closed;
@@ -1078,3 +1081,34 @@ extern EIO_Status CONN_SetCallback
     }
     return eIO_Success;
 }
+
+
+extern EIO_Status CONN_GetSOCK(CONN conn, SOCK* sock)
+{
+    EIO_Status status;
+    CONNECTOR  x_conn;
+
+    CONN_NOT_NULL(36, GetSOCK);
+
+    if (!sock)
+        return eIO_InvalidArg;
+    *sock = 0;
+
+    /* perform open, if not opened yet */
+    if (conn->state != eCONN_Open  &&  (status = s_Open(conn)) != eIO_Success)
+        return status;
+    assert((conn->state & eCONN_Open)  &&  conn->meta.list);
+
+    x_conn = conn->meta.list;
+    if (x_conn  &&  x_conn->meta  &&  x_conn->meta->get_type  &&
+        x_conn->meta->get_type(x_conn->meta->c_get_type)
+        == g_kNcbiSockNameAbbr) {
+        /* HACK * HACK * HACK */
+        SOCK* x_sock = (SOCK*) x_conn->handle;
+        if (x_sock) {
+            *sock = *x_sock;
+            return eIO_Success;
+        }
+    }
+    return eIO_Closed;
+}
