@@ -1,4 +1,4 @@
-/* $Id: ncbi_connector.c 346756 2011-12-09 17:14:08Z lavr $
+/* $Id: ncbi_connector.c 362482 2012-05-09 17:24:01Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -94,8 +94,8 @@ extern EIO_Status METACONN_Add
         return status;
     }
 
-    connector->setup(meta, connector);
     connector->meta = meta;
+    connector->setup(connector);
     connector->next = meta->list;
     meta->list = connector;
     return eIO_Success;
