@@ -1,4 +1,4 @@
-/* $Id: ncbi_connutil.c 350573 2012-01-21 00:28:01Z rafanovi $
+/* $Id: ncbi_connutil.c 362495 2012-05-09 17:27:00Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -331,11 +331,13 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
     if (len < 3  ||  8 < len  ||  strncasecmp(str, "infinite", len) != 0) {
         if (!*str  ||  (dbl = atof(str)) < 0.0)
             dbl = DEF_CONN_TIMEOUT;
-        info->tmo.sec  = (unsigned int)  dbl;
-        info->tmo.usec = (unsigned int)((dbl - info->tmo.sec) * 1000000.0);
-        info->timeout  = &info->tmo;
+        info->tmo.sec      = (unsigned int)  dbl;
+        info->tmo.usec     = (unsigned int)((dbl - info->tmo.sec) * 1000000.0);
+        if (dbl  &&  !(info->tmo.sec | info->tmo.usec))
+            info->tmo.usec = 1/*protect underflow*/;
+        info->timeout      = &info->tmo;
     } else
-        info->timeout  = kInfiniteTimeout/*0*/;
+        info->timeout      = kInfiniteTimeout/*0*/;
 
     /* max. # of attempts to establish connection */
     REG_VALUE(REG_CONN_MAX_TRY, str, 0);
@@ -344,7 +346,7 @@ extern SConnNetInfo* ConnNetInfo_Create(const char* service)
 
     /* firewall mode */
     REG_VALUE(REG_CONN_FIREWALL, str, DEF_CONN_FIREWALL);
-    if (!*str)
+    if (!*str) /*NB: not actually necessary but faster*/
         info->firewall = eFWMode_Legacy;
     else if (strcasecmp(str, "adaptive") == 0  ||  ConnNetInfo_Boolean(str))
         info->firewall = eFWMode_Adaptive;
@@ -416,13 +418,17 @@ extern int/*bool*/ ConnNetInfo_ParseURL(SConnNetInfo* info, const char* url)
         if (len >= sizeof(info->host))
             return 0/*failure*/;
         if (s) {
-            int n;
-            if (sscanf(++s, "%hu%n", &port, &n) < 1  ||  s[n]  ||  !port)
+            long i;
+            errno = 0;
+            i = strtol(++s, &p, 10);
+            if (errno  ||  s == p  ||  !i  ||  i ^ (i & 0xFFFF)  ||  *p)
                 return 0/*failure*/;
-            info->port = port;
+            info->port = (unsigned short) i;
+        }
+        if (len) {
+            memcpy(info->host, url, len);
+            info->host[len] = '\0';
         }
-        if (len)
-            memcpy(info->host, url, ++len);
         return 1/*success*/;
     }
 
@@ -464,12 +470,13 @@ extern int/*bool*/ ConnNetInfo_ParseURL(SConnNetInfo* info, const char* url)
 
             /* port, if any */
             if ((s = (const char*) memchr(host, ':', hostlen)) != 0) {
-                int n;
+                long i;
                 hostlen = (size_t)(s - host);
-                if (sscanf(++s, "%hu%n", &port, &n) < 1
-                    ||  s + n != path  ||  !port) {
+                errno = 0;
+                i = strtol(++s, &p, 10);
+                if (errno  ||  s == p  || !i || i ^ (i & 0xFFFF) ||  p != path)
                     return 0/*failure*/;
-                }
+                port = (unsigned short) i;
             } else
                 port = 0/*default*/;
 
@@ -487,7 +494,8 @@ extern int/*bool*/ ConnNetInfo_ParseURL(SConnNetInfo* info, const char* url)
         port    = 0;
     }
 
-    pathlen = strcspn(path, "?#");
+    pathlen = (scheme == eURL_Https  ||  scheme == eURL_Http
+               ? strcspn(path, "?#") : strlen(path));
     args    = path + pathlen;
 
     if (path != url  ||  *path == '/') {
@@ -677,7 +685,7 @@ static int/*bool*/ s_ModifyUserHeader(SConnNetInfo*      info,
         /* line & taglen */
         newlinelen = (size_t)
             (eol ? eol - newline + 1 : newhdr + newhdrlen - newline);
-        if (!eot || eot >= newline + newlinelen)
+        if (!eot  ||  eot >= newline + newlinelen)
             goto ignore;
         if (!(newtaglen = (size_t)(eot - newline)))
             goto ignore;
@@ -723,7 +731,7 @@ static int/*bool*/ s_ModifyUserHeader(SConnNetInfo*      info,
             eot = strchr(line,  ':');
 
             linelen = (size_t)(eol ? eol - line + 1 : hdr + hdrlen - line);
-            if (!eot || eot >= line + linelen)
+            if (!eot  ||  eot >= line + linelen)
                 continue;
 
             taglen = (size_t)(eot - line);
@@ -1030,7 +1038,7 @@ extern int/*bool*/ ConnNetInfo_SetupStandardArgs(SConnNetInfo* info,
         ConnNetInfo_PreOverrideArg(info, kAddress, s);
     if (s != info->client_host)
         free((void*) s);
-    if (service  &&  *service) {
+    if (service) {
         if (!ConnNetInfo_PreOverrideArg(info, kService, service)) {
             ConnNetInfo_DeleteArg(info, kPlatform);
             if (!ConnNetInfo_PreOverrideArg(info, kService, service)) {
@@ -1340,7 +1348,7 @@ extern void ConnNetInfo_Destroy(SConnNetInfo* info)
  */
 
 
-static EIO_Status x_URLConnectReturn(SOCK sock, EIO_Status status)
+static EIO_Status x_URLConnectErrorReturn(SOCK sock, EIO_Status status)
 {
     if (sock) {
         SOCK_Abort(sock);
@@ -1389,7 +1397,7 @@ extern EIO_Status URL_ConnectEx
             *sock = 0;
         } else
             s = 0;
-        return x_URLConnectReturn(s, eIO_InvalidArg);
+        return x_URLConnectErrorReturn(s, eIO_InvalidArg);
     }
     s = *sock;
     *sock = 0;
@@ -1421,7 +1429,7 @@ extern EIO_Status URL_ConnectEx
                     ("[URL_Connect]  Unrecognized request method (#%u)",
                      (unsigned int) req_method));
         assert(0);
-        return x_URLConnectReturn(s, eIO_InvalidArg);
+        return x_URLConnectErrorReturn(s, eIO_InvalidArg);
     }
 
     hdr_len = 0;
@@ -1451,7 +1459,7 @@ extern EIO_Status URL_ConnectEx
                     CORE_LOGF_ERRNO_X(8, eLOG_Error, errno,
                                       ("[URL_Connect]  Out of memory (%lu)",
                                        (unsigned long) size));
-                    return x_URLConnectReturn(s, eIO_Unknown);
+                    return x_URLConnectErrorReturn(s, eIO_Unknown);
                 }
                 URL_Encode(args, args_len, &rd_len, x_args, size, &wr_len);
                 assert(args_len == rd_len);
@@ -1484,9 +1492,9 @@ extern EIO_Status URL_ConnectEx
 
         /* Content-Length: <content_length>\r\n */
         (req_method == eReqMethod_Post
-         &&  ((add_hdr =
-               sprintf(hdr_buf, "Content-Length: %lu\r\n",
-                       (unsigned long) content_length)) <= 0    ||
+         &&  ((add_hdr
+               = sprintf(hdr_buf, "Content-Length: %lu\r\n",
+                         (unsigned long) content_length)) <= 0  ||
               !BUF_Write(&buf, hdr_buf,  (size_t) add_hdr)))    ||
 
         /* <user_header> */
@@ -1506,7 +1514,7 @@ extern EIO_Status URL_ConnectEx
         BUF_Destroy(buf);
         if (temp  &&  temp != args)
             free((void*) temp);
-        return x_URLConnectReturn(s, eIO_Unknown);
+        return x_URLConnectErrorReturn(s, eIO_Unknown);
     }
     if (temp  &&  temp != args)
         free((void*) temp);
@@ -1515,27 +1523,27 @@ extern EIO_Status URL_ConnectEx
         ||  BUF_Read(buf, hdr, hdr_len) != hdr_len) {
         int x_errno = errno;
         CORE_LOGF_ERRNO_X(6, eLOG_Error, x_errno,
-                          ("[URL_Connect]  Cannot convert HTTP header to"
-                           " string for %s:%hu", host, port));
+                          ("[URL_Connect]  Cannot maintain HTTP header for"
+                           " %s:%hu", host, port));
         if (hdr)
             free(hdr);
         BUF_Destroy(buf);
-        return x_URLConnectReturn(s, eIO_Unknown);
+        return x_URLConnectErrorReturn(s, eIO_Unknown);
     }
     BUF_Destroy(buf);
 
     if (s) {
         /* resuse connection to HTTPD */
-        size_t handle_size = SOCK_OSHandleSize();
-        void*  handle      = malloc(handle_size);
-        status = SOCK_GetOSHandle(s, handle, handle_size);
+        size_t size   = SOCK_OSHandleSize();
+        void*  handle = malloc(size);
+        status = SOCK_GetOSHandleEx(s, handle, size, eTakeOwnership);
         if (status == eIO_Success) {
             SOCK_Close(s);
-            status  = SOCK_CreateOnTopEx(handle, handle_size, sock,
+            status  = SOCK_CreateOnTopEx(handle, size, sock,
                                          hdr, hdr_len, flags);
             if (status != eIO_Success) {
-                SOCK_CloseOSHandle(handle, handle_size);
-                assert(!sock);
+                SOCK_CloseOSHandle(handle, size);
+                assert(!*sock);
             }
         } else {
             SOCK_Abort(s);
