@@ -1,4 +1,4 @@
-/* $Id: ncbi_dispd.c 344679 2011-11-17 18:58:31Z lavr $
+/* $Id: ncbi_dispd.c 354443 2012-02-27 15:07:20Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,11 +36,9 @@
 #include "ncbi_dispd.h"
 #include "ncbi_lb.h"
 #include "ncbi_priv.h"
-#include <connect/ncbi_connection.h>
 #include <connect/ncbi_http_connector.h>
 #include <ctype.h>
 #include <math.h>
-#include <stdio.h>
 #include <stdlib.h>
 
 #define NCBI_USE_ERRCODE_X   Connect_Dispd
@@ -193,8 +191,9 @@ static void s_Resolve(SERV_ITER iter)
         CONN_Close(c);
     } else {
         CORE_LOGF_X(1, eLOG_Error,
-                    ("[%s]  Unable to create auxiliary HTTP %s: %s",
-                     iter->name, conn ? "connection" : "connector",
+                    ("%s%s%sUnable to create auxiliary HTTP %s: %s",
+                     &"["[!*iter->name], iter->name, *iter->name ? "]  " : "",
+                     conn ? "connection" : "connector",
                      IO_StatusStr(conn ? status : eIO_Unknown)));
         assert(0);
     }
@@ -386,9 +385,6 @@ const SSERV_VTable* SERV_DISPD_Open(SERV_ITER iter,
 {
     struct SDISPD_Data* data;
 
-    if (!iter->ismask  &&  strpbrk(iter->name, "?*"))
-        return 0/*failed to start unallowed wildcard search*/;
-
     if (!(data = (struct SDISPD_Data*) calloc(1, sizeof(*data))))
         return 0;
     iter->data = data;
