@@ -1,4 +1,4 @@
-/* $Id: ncbi_http_connector.c 344560 2011-11-16 18:23:06Z lavr $
+/* $Id: ncbi_http_connector.c 362482 2012-05-09 17:24:01Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -227,13 +227,22 @@ static EIO_Status s_Adjust(SHttpConnector* uuu,
                            EExtractMode    extract)
 {
     EIO_Status status;
+    const char* msg;
 
     assert(!uuu->sock  &&  uuu->can_connect != eCC_None);
 
-    if (retry  &&  retry->mode  &&  ++uuu->minor_fault < 3) {
+    if (!retry  ||  !retry->mode  ||  uuu->minor_fault > 5) {
+        uuu->minor_fault = 0;
+        uuu->major_fault++;
+    } else
+        uuu->minor_fault++;
+
+    if (uuu->major_fault >= uuu->net_info->max_try) {
+        msg = extract != eEM_Drop  &&  uuu->major_fault > 1
+            ? "[HTTP%s%s]  Too many failed attempts (%d), giving up" : "";
+    } else if (retry  &&  retry->mode) {
         char* url  = ConnNetInfo_URL(uuu->net_info);
         int   fail = 0;
-        /* adjust info before yet another connection attempt */
         switch (retry->mode) {
         case eRetry_Redirect:
             if (uuu->net_info->req_method == eReqMethod_Connect)
@@ -297,9 +306,9 @@ static EIO_Status s_Adjust(SHttpConnector* uuu,
                              ? "Authorization" : "Proxy authorization",
                              fail < 0 ? "not yet implemented" :
                              fail > 1 ? "prohibited" : "failed",
-                             "(<"[!retry->data],
+                             "(["[!retry->data],
                              retry->data ? retry->data : "NULL",
-                             ")>"[!retry->data]));
+                             ")]"[!retry->data]));
                 if (url)
                     free(url);
                 status = fail < 0 ? eIO_NotSupported : eIO_Closed;
@@ -325,42 +334,30 @@ static EIO_Status s_Adjust(SHttpConnector* uuu,
         }
         if (url)
             free(url);
-        if (status != eIO_Success) {
+        if (status != eIO_Success)
             uuu->can_connect = eCC_None;
-            return status;
-        }
-    } else {
-        const char* msg;
-        uuu->minor_fault = 0;
-        /* we're here because something is going wrong */
-        if (++uuu->major_fault >= uuu->net_info->max_try) {
-            msg = extract != eEM_Drop  &&  uuu->major_fault > 1
-                ? "[HTTP%s%s]  Too many failed attempts (%d), giving up" : "";
-        } else if (!uuu->adjust
-                   ||  !uuu->adjust(uuu->net_info,
-                                    uuu->user_data,
-                                    uuu->major_fault)) {
-            msg = extract != eEM_Drop  &&  uuu->major_fault > 1
-                ? "[HTTP%s%s]  Retry attempts (%d) exhausted, giving up" : "";
-        } else
-            msg = 0;
-        if (msg) {
-            if (*msg) {
-                char* url = ConnNetInfo_URL(uuu->net_info);
-                CORE_LOGF_X(1, eLOG_Error,
-                            (msg,
-                             url  &&  *url ? "; " : "",
-                             url           ? url  : "",
-                             uuu->major_fault));
-                if (url)
-                    free(url);
-            }
-            uuu->can_connect = eCC_None;
-            return eIO_Closed;
-        }
-    }
+        return status;
+    } else if (!uuu->adjust
+               ||  !uuu->adjust(uuu->net_info,
+                                uuu->user_data,
+                                uuu->major_fault)) {
+        msg = extract != eEM_Drop  &&  uuu->major_fault > 1
+            ? "[HTTP%s%s]  Retry attempts (%d) exhausted, giving up" : "";
+    } else
+        return eIO_Success;
 
-    return eIO_Success;
+    if (*msg) {
+        char* url = ConnNetInfo_URL(uuu->net_info);
+        CORE_LOGF_X(1, eLOG_Error,
+                    (msg,
+                     url  &&  *url ? "; " : "",
+                     url           ? url  : "",
+                     uuu->major_fault));
+        if (url)
+            free(url);
+    }
+    uuu->can_connect = eCC_None;
+    return eIO_Closed;
 }
 
 
@@ -437,41 +434,32 @@ static EIO_Status s_Connect(SHttpConnector* uuu,
 
     /* the re-try loop... */
     for (;;) {
-        TSOCK_Flags flags;
-
+        TSOCK_Flags flags
+            = (uuu->net_info->debug_printout == eDebugPrintout_Data
+               ? fSOCK_LogOn : fSOCK_LogDefault);
         sock = 0;
-        status = eIO_Success;
-        flags = (uuu->net_info->debug_printout == eDebugPrintout_Data
-                 ? fSOCK_LogOn : fSOCK_LogDefault);
-        if (uuu->net_info->scheme == eURL_Https) {
-            flags |= fSOCK_Secure;
-            if (uuu->net_info->req_method == eReqMethod_Connect) {
-                if (uuu->flags & fHTTP_DetachableTunnel) {
-                    status = eIO_InvalidArg;
-                    break;
-                }
-            } else if (uuu->net_info->http_proxy_host[0]  &&
-                       uuu->net_info->http_proxy_port) {
-                SConnNetInfo* net_info = ConnNetInfo_Clone(uuu->net_info);
-                if (!net_info) {
-                    status = eIO_Unknown;
-                    break;
-                }
-                net_info->scheme   = eURL_Http;
-                net_info->user[0]  = '\0';
-                net_info->pass[0]  = '\0';
-                if (!net_info->port)
-                    net_info->port = CONN_PORT_HTTPS;
-                net_info->firewall = 0/*false*/;
-                net_info->proxy_host[0] = '\0';
-                ConnNetInfo_DeleteUserHeader(net_info, kHttpHostTag);
-                status = HTTP_CreateTunnel(net_info, fHTTP_DetachableTunnel
-                                           | fHTTP_NoUpread, &sock);
-                assert((status == eIO_Success) ^ !sock);
-                ConnNetInfo_Destroy(net_info);
+        if (uuu->net_info->req_method != eReqMethod_Connect
+            &&  uuu->net_info->scheme == eURL_Https
+            &&  uuu->net_info->http_proxy_host[0]
+            &&  uuu->net_info->http_proxy_port) {
+            SConnNetInfo* net_info = ConnNetInfo_Clone(uuu->net_info);
+            if (!net_info) {
+                status = eIO_Unknown;
+                break;
             }
-        }
-
+            net_info->scheme   = eURL_Http;
+            net_info->user[0]  = '\0';
+            net_info->pass[0]  = '\0';
+            if (!net_info->port)
+                net_info->port = CONN_PORT_HTTPS;
+            net_info->firewall = 0/*false*/;
+            net_info->proxy_host[0] = '\0';
+            ConnNetInfo_DeleteUserHeader(net_info, kHttpHostTag);
+            status = HTTP_CreateTunnel(net_info, fHTTP_NoUpread, &sock);
+            assert((status == eIO_Success) ^ !sock);
+            ConnNetInfo_Destroy(net_info);
+        } else
+            status  = eIO_Success;
         if (status == eIO_Success) {
             int/*bool*/    reset_user_header;
             char*          http_user_header;
@@ -483,8 +471,9 @@ static EIO_Status s_Connect(SHttpConnector* uuu,
 
             len = BUF_Size(uuu->w_buf);
             if (uuu->net_info->req_method == eReqMethod_Connect
-                ||  (!sock  &&  (uuu->net_info->http_proxy_host[0]  &&
-                                 uuu->net_info->http_proxy_port))) {
+                ||  (uuu->net_info->scheme != eURL_Https
+                     &&  uuu->net_info->http_proxy_host[0]
+                     &&  uuu->net_info->http_proxy_port)) {
                 char* temp;
                 host = uuu->net_info->http_proxy_host;
                 port = uuu->net_info->http_proxy_port;
@@ -494,8 +483,7 @@ static EIO_Status s_Connect(SHttpConnector* uuu,
                     break;
                 }
                 if (uuu->net_info->req_method == eReqMethod_Connect) {
-                    if (uuu->flags & fHTTP_DetachableTunnel)
-                        flags |= fSOCK_KeepOnClose;
+                    /* Tunnel */
                     if (!len) {
                         args = 0;
                     } else if (!(temp = (char*) malloc(len))
@@ -508,6 +496,8 @@ static EIO_Status s_Connect(SHttpConnector* uuu,
                     } else
                         args = temp;
                 } else {
+                    /* Proxied HTTP */
+                    assert(uuu->net_info->scheme == eURL_Http);
                     if (!s_SetHttpHostTag(uuu->net_info)) {
                         status = eIO_Unknown;
                         free((void*) path);
@@ -523,6 +513,7 @@ static EIO_Status s_Connect(SHttpConnector* uuu,
                         args = 0;
                 }
             } else {
+                /* Direct HTTP[S] or tunneled HTTPS */
                 if (!s_SetHttpHostTag(uuu->net_info)) {
                     status = eIO_Unknown;
                     break;
@@ -532,6 +523,8 @@ static EIO_Status s_Connect(SHttpConnector* uuu,
                 path = uuu->net_info->path;
                 args = uuu->net_info->args;
             }
+            if (uuu->net_info->scheme == eURL_Https)
+                flags |= fSOCK_Secure;
             if (!(uuu->flags & fHTTP_NoUpread))
                 flags |= fSOCK_ReadOnWrite;
 
@@ -1032,8 +1025,8 @@ static EIO_Status s_ReadHeader(SHttpConnector* uuu,
             size_t n = BUF_Read(buf, body, size);
             if (n != size) {
                 CORE_LOGF_X(13, eLOG_Error,
-                            ("[HTTP%s%s]  Cannot read server error"
-                             " body from buffer (%lu out of %lu)",
+                            ("[HTTP%s%s]  Cannot read server error body"
+                             " from buffer (%lu out of %lu)",
                              url  &&  *url ? "; " : "",
                              url           ? url  : "",
                              (unsigned long) n, (unsigned long) size));
@@ -1294,9 +1287,9 @@ static void s_OpenHttpConnector(SHttpConnector* uuu,
         uuu->w_timeout  = timeout;
     }
 
-    /* reset the auto-reconnect feature */
-    uuu->can_connect = (uuu->flags & fHTTP_AutoReconnect
-                        ? eCC_Unlimited : eCC_Once);
+    /* reset the auto-reconnect/re-try/auth features */
+    uuu->can_connect     = (uuu->flags & fHTTP_AutoReconnect
+                            ? eCC_Unlimited : eCC_Once);
     uuu->major_fault     = 0;
     uuu->minor_fault     = 0;
     uuu->auth_done       = 0;
@@ -1334,10 +1327,8 @@ extern "C" {
                                      EIO_Event       dir);
     static EIO_Status  s_VT_Close   (CONNECTOR       connector,
                                      const STimeout* timeout);
-
-    static void        s_Setup      (SMetaConnector *meta,
-                                     CONNECTOR connector);
-    static void        s_Destroy    (CONNECTOR connector);
+    static void        s_Setup      (CONNECTOR       connector);
+    static void        s_Destroy    (CONNECTOR       connector);
 #ifdef __cplusplus
 } /* extern "C" */
 #endif /* __cplusplus */
@@ -1455,7 +1446,6 @@ static EIO_Status s_VT_Flush
 {
     SHttpConnector* uuu = (SHttpConnector*) connector->handle;
 
-    assert(connector->meta);
     if (!(uuu->flags & fHTTP_Flushable)  ||  uuu->sock) {
         /* The real flush will be performed on the first "READ" (or "CLOSE"),
          * or on "WAIT". Here, we just store the write timeout, that's all...
@@ -1513,9 +1503,11 @@ static EIO_Status s_VT_Close
 }
 
 
-static void s_Setup(SMetaConnector *meta, CONNECTOR connector)
+static void s_Setup
+(CONNECTOR connector)
 {
     SHttpConnector* uuu = (SHttpConnector*) connector->handle;
+    SMetaConnector* meta = connector->meta;
 
     /* initialize virtual table */
     CONN_SET_METHOD(meta, get_type, s_VT_GetType, connector);
@@ -1533,6 +1525,7 @@ static void s_Setup(SMetaConnector *meta, CONNECTOR connector)
 
 static void s_DestroyHttpConnector(SHttpConnector* uuu)
 {
+    assert(!uuu->sock);
     if (uuu->cleanup)
         uuu->cleanup(uuu->user_data);
     ConnNetInfo_Destroy(uuu->net_info);
@@ -1543,7 +1536,8 @@ static void s_DestroyHttpConnector(SHttpConnector* uuu)
 }
 
 
-static void s_Destroy(CONNECTOR connector)
+static void s_Destroy
+(CONNECTOR connector)
 {
     SHttpConnector* uuu = (SHttpConnector*) connector->handle;
     connector->handle = 0;
@@ -1628,10 +1622,10 @@ static EIO_Status s_CreateHttpConnector
     if (!xxx)
         return eIO_Unknown;
     if (!tunnel) {
-        if ((xxx->scheme != eURL_Unspec  && 
-             xxx->scheme != eURL_Https   &&
-             xxx->scheme != eURL_Http)   ||
-            xxx->req_method == eReqMethod_Connect) {
+        if (xxx->req_method == eReqMethod_Connect
+            ||  (xxx->scheme != eURL_Unspec  && 
+                 xxx->scheme != eURL_Https   &&
+                 xxx->scheme != eURL_Http)) {
             ConnNetInfo_Destroy(xxx);
             return eIO_InvalidArg;
         }
@@ -1639,7 +1633,6 @@ static EIO_Status s_CreateHttpConnector
             xxx->scheme  = eURL_Http;
         if ((fff = strchr(xxx->args, '#')) != 0)
             *fff = '\0';
-        flags &= ~fHTTP_DetachableTunnel;
     }
     ConnNetInfo_OverrideUserHeader(xxx, user_header);
     if (tunnel) {
@@ -1652,7 +1645,7 @@ static EIO_Status s_CreateHttpConnector
         }
         ConnNetInfo_DeleteUserHeader(xxx, "Referer:");
     }
-    x_FixupUserHeader(xxx, flags & fHTTP_NoAutomagicSID ? 1 : 0);
+    x_FixupUserHeader(xxx, flags & fHTTP_NoAutomagicSID ? 1 : tunnel);
 
     if ((flags & fHTTP_NoAutoRetry)  ||  !xxx->max_try)
         xxx->max_try = 1;
@@ -1672,14 +1665,14 @@ static EIO_Status s_CreateHttpConnector
 
     uuu->flags        = flags;
     uuu->reserved     = 0;
-    uuu->can_connect  = eCC_None;         /* will be properly set at open*/
+    uuu->can_connect  = eCC_None;         /* will be properly set at open */
 
     ConnNetInfo_GetValue(0, "HTTP_ERROR_HEADER_ONLY", val, sizeof(val), "");
     uuu->error_header = ConnNetInfo_Boolean(val);
 
     uuu->sock         = 0;
-    uuu->o_timeout    = kDefaultTimeout;  /* deliberately bad values --  */
-    uuu->w_timeout    = kDefaultTimeout;  /* must be reset prior to use  */
+    uuu->o_timeout    = kDefaultTimeout;  /* deliberately bad values..    */
+    uuu->w_timeout    = kDefaultTimeout;  /* ..must be reset prior to use */
     uuu->http         = 0;
     uuu->r_buf        = 0;
     uuu->w_buf        = 0;
@@ -1780,17 +1773,17 @@ extern EIO_Status HTTP_CreateTunnelEx
         return eIO_InvalidArg;
     *sock = 0;
     
-    if ((status = s_CreateHttpConnector(net_info,0/*user_header*/, 1/*tunnel*/,
-                                        flags | fHTTP_DropUnread, &uuu))
-        != eIO_Success) {
+    status = s_CreateHttpConnector(net_info, 0/*user_header*/, 1/*tunnel*/,
+                                   flags | fHTTP_DropUnread, &uuu);
+    if (status != eIO_Success) {
         assert(!uuu);
         return status;
     }
-    assert(uuu);
+    assert(uuu  &&  !BUF_Size(uuu->w_buf));
     if (!init_size  ||  BUF_Prepend(&uuu->w_buf, init_data, init_size)) {
         if (init_size)
             sprintf(uuu->net_info->args, "[%lu]", (unsigned long) init_size);
-        status = s_PreRead(uuu, net_info->timeout, eEM_Wait);
+        status = s_PreRead(uuu, uuu->net_info->timeout, eEM_Wait);
         if (status == eIO_Success) {
             assert(uuu->read_state == eRS_ReadBody);
             assert(uuu->code / 100 == 2);
