@@ -1,4 +1,4 @@
-/* $Id: ncbi_lbsm.c 333175 2011-09-02 16:18:24Z lavr $
+/* $Id: ncbi_lbsm.c 351842 2012-02-01 18:31:22Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -171,7 +171,8 @@ const SLBSM_Service* LBSM_LookupService(HEAP                 heap,
 }
 
 
-const SLBSM_Host* LBSM_LookupHost(HEAP heap, unsigned addr,
+const SLBSM_Host* LBSM_LookupHost(HEAP               heap,
+                                  unsigned int       addr,
                                   const SLBSM_Entry* hint)
 {
     const SLBSM_Entry* e = hint;
