@@ -1,4 +1,4 @@
-/* $Id: ncbi_local.c 350738 2012-01-23 22:02:50Z rafanovi $
+/* $Id: ncbi_local.c 351821 2012-02-01 17:34:25Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -177,8 +177,10 @@ static int/*bool*/ s_LoadServices(SERV_ITER iter)
             s[len++] = '\0';
         if (!(c = SERV_ServiceName(s)))
             break;
-        if ((iter->reverse_dns  ||  UTIL_MatchesMask(c, iter->name))  &&
-            s_LoadSingleService(c, iter)) {
+        if ((iter->reverse_dns
+             ||  (iter->ismask
+                  &&  (!*iter->name  ||  UTIL_MatchesMask(c, iter->name))))
+            &&  s_LoadSingleService(c, iter)) {
             ok = 1/*succeeded*/;
         }
         free((void*) c);
@@ -369,9 +371,6 @@ const SSERV_VTable* SERV_LOCAL_Open(SERV_ITER iter,
 {
     struct SLOCAL_Data* data;
 
-    if (!iter->ismask  &&  strpbrk(iter->name, "?*"))
-        return 0/*failed to start unallowed wildcard search*/;
-
     if (!(data = (struct SLOCAL_Data*) calloc(1, sizeof(*data))))
         return 0;
 
