@@ -1,4 +1,4 @@
-/* $Id: ncbi_namedpipe_connector.cpp 337346 2011-09-11 01:18:02Z lavr $
+/* $Id: ncbi_namedpipe_connector.cpp 362482 2012-05-09 17:24:01Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -167,9 +167,10 @@ static EIO_Status s_VT_Close
 
 
 static void s_Setup
-(SMetaConnector* meta,
- CONNECTOR       connector)
+(CONNECTOR connector)
 {
+    SMetaConnector* meta = connector->meta;
+
     // Initialize virtual table
     CONN_SET_METHOD(meta, get_type, s_VT_GetType, connector);
     CONN_SET_METHOD(meta, descr,    s_VT_Descr,   connector);
