@@ -1,4 +1,4 @@
-/* $Id: ncbi_pipe.cpp 349694 2012-01-12 16:41:21Z rafanovi $
+/* $Id: ncbi_pipe.cpp 356158 2012-03-12 18:37:09Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -41,8 +41,6 @@
 #include <connect/ncbi_util.h>
 #include <corelib/ncbi_system.hpp>
 #include <corelib/stream_utils.hpp>
-#include <memory>
-#include <stdio.h>
 
 #ifdef NCBI_OS_MSWIN
 
@@ -481,15 +479,15 @@ void CPipeHandle::x_Clear(void)
     } else {
         if (m_ChildStdIn != INVALID_HANDLE_VALUE) {
             ::CloseHandle(m_ChildStdIn);
-            m_ChildStdIn = INVALID_HANDLE_VALUE;
+            m_ChildStdIn  = INVALID_HANDLE_VALUE;
         }
         if (m_ChildStdOut != INVALID_HANDLE_VALUE) {
             ::CloseHandle(m_ChildStdOut);
-            m_ChildStdOut = INVALID_HANDLE_VALUE;
+            m_ChildStdOut  = INVALID_HANDLE_VALUE;
         }
         if (m_ChildStdErr != INVALID_HANDLE_VALUE) {
             ::CloseHandle(m_ChildStdErr);
-            m_ChildStdErr = INVALID_HANDLE_VALUE;
+            m_ChildStdErr  = INVALID_HANDLE_VALUE;
         }
     }
 }
@@ -615,8 +613,8 @@ EIO_Status CPipeHandle::Read(void* buf, size_t count, size_t* read,
         _ASSERT(bytes_avail);
         // We must read only "count" bytes of data regardless of
         // the amount available to read
-        if (bytes_avail > count) {
-            bytes_avail = count;
+        if (bytes_avail >         count) {
+            bytes_avail = (DWORD) count;
         }
         if ( !::ReadFile(fd, buf, bytes_avail, &bytes_avail, NULL) ) {
             PIPE_THROW(::GetLastError(), "Failed to read data from pipe");
@@ -653,11 +651,14 @@ EIO_Status CPipeHandle::Write(const void* buf, size_t count,
         }
 
         DWORD x_timeout = timeout ? NcbiTimeoutToMs(timeout) : INFINITE;
+        DWORD to_write  = (count > numeric_limits<DWORD>::max()
+                           ? numeric_limits<DWORD>::max()
+                           : (DWORD) count);
         DWORD bytes_written = 0;
 
         // Try to write data into the pipe within specified time.
         for (;;) {
-            BOOL ok = ::WriteFile(m_ChildStdIn, (char*)buf, count,
+            BOOL ok = ::WriteFile(m_ChildStdIn, (char*) buf, to_write,
                                   &bytes_written, NULL);
             if ( bytes_written ) {
                 break;
@@ -1944,7 +1945,7 @@ CPipe::EFinish CPipe::ExecWait(const string&           cmd,
             if (rmask & fStdIn  &&  !in_done) {
                 if (in.good()  &&  bytes_in_inbuf == 0) {
                     bytes_in_inbuf =
-                        CStreamUtils::Readsome(in, inbuf, buf_size);
+                        (size_t)CStreamUtils::Readsome(in, inbuf, buf_size);
                     total_bytes_written = 0;
                 }
 
