@@ -1,4 +1,4 @@
-/* $Id: ncbi_pipe_connector.cpp 337342 2011-09-11 01:13:46Z lavr $
+/* $Id: ncbi_pipe_connector.cpp 362482 2012-05-09 17:24:01Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -183,9 +183,10 @@ static EIO_Status s_VT_Close
 
 
 static void s_Setup
-(SMetaConnector* meta,
- CONNECTOR       connector)
+(CONNECTOR connector)
 {
+    SMetaConnector* meta = connector->meta;
+
     // Initialize virtual table
     CONN_SET_METHOD(meta, get_type, s_VT_GetType, connector);
     CONN_SET_METHOD(meta, descr,    s_VT_Descr,   connector);
@@ -200,7 +201,8 @@ static void s_Setup
 }
 
 
-static void s_Destroy(CONNECTOR connector)
+static void s_Destroy
+(CONNECTOR connector)
 {
     SPipeConnector* xxx = (SPipeConnector*) connector->handle;
     connector->handle = 0;
