@@ -1,4 +1,4 @@
-/* $Id: ncbi_sendmail.c 344022 2011-11-10 20:22:37Z lavr $
+/* $Id: ncbi_sendmail.c 360593 2012-04-24 17:54:30Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,7 +34,6 @@
 #include "ncbi_priv.h"
 #include <connect/ncbi_connutil.h>
 #include <connect/ncbi_sendmail.h>
-#include <connect/ncbi_socket.h>
 #include <ctype.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -223,7 +222,7 @@ static void x_Sendmail_InitEnv(void)
         return;
 
     if (!ConnNetInfo_GetValue(0, "MX_TIMEOUT", buf, sizeof(buf), 0)
-        ||  (tmo = atof(buf)) <= 0.0) {
+        ||  (tmo = atof(buf)) < 0.000001) {
         tmo = 120.0;
     }
     if (!ConnNetInfo_GetValue(0, "MX_PORT", buf, sizeof(buf), 0)
