@@ -1,4 +1,4 @@
-/* $Id: ncbi_socket_cxx.cpp 313288 2011-07-19 19:01:22Z lavr $
+/* $Id: ncbi_socket_cxx.cpp 366364 2012-06-13 20:36:22Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,11 +32,12 @@
  */
 
 #include <ncbi_pch.hpp>
+#include "ncbi_assert.h"                // no _ASSERT()s, keep clean from xncbi
 #include <connect/ncbi_socket_unix.hpp>
 #include <limits.h>                     // for PATH_MAX
-#if defined(NCBI_OS_MSWIN) && !defined(PATH_MAX)
+#if defined(NCBI_OS_MSWIN)  &&  !defined(PATH_MAX)
 #  define PATH_MAX 512                  // will actually use less than 32 chars
-#endif
+#endif // NCBI_OS_MSWIN && !PATH_MAX
 
 
 BEGIN_NCBI_SCOPE
@@ -46,15 +47,6 @@ BEGIN_NCBI_SCOPE
 //  CTrigger::
 //
 
-
-CTrigger::CTrigger(ESwitch log)
-    : m_Trigger(0)
-{
-    if (TRIGGER_Create(&m_Trigger, log) != eIO_Success)
-        m_Trigger = 0;
-}
-
-
 CTrigger::~CTrigger()
 {
     if (m_Trigger)
@@ -68,15 +60,6 @@ CTrigger::~CTrigger()
 //
 
 
-CSocket::CSocket(void)
-    : m_Socket(0),
-      m_IsOwned(eTakeOwnership),
-      o_timeout(0), r_timeout(0), w_timeout(0), c_timeout(0)
-{
-    return;
-}
-
-
 CSocket::CSocket(const string&   host,
                  unsigned short  port,
                  const STimeout* timeout,
@@ -89,10 +72,7 @@ CSocket::CSocket(const string&   host,
         o_timeout  = &oo_timeout;
     } else
         o_timeout  = 0;
-    if (SOCK_CreateEx(host.c_str(), port, o_timeout, &m_Socket, 0, 0, flags)
-        != eIO_Success) {
-        m_Socket = 0;
-    }
+    SOCK_CreateEx(host.c_str(), port, o_timeout, &m_Socket, 0, 0, flags);
 }
 
 
@@ -109,28 +89,24 @@ CSocket::CSocket(unsigned int    host,
         o_timeout = &oo_timeout;
     } else
         o_timeout = 0;
-    if (SOCK_ntoa(host, x_host, sizeof(x_host)) != 0  ||
-        SOCK_CreateEx(x_host, port, o_timeout, &m_Socket, 0, 0, flags)
-        != eIO_Success) {
+    if (SOCK_ntoa(host, x_host, sizeof(x_host)) != 0)
         m_Socket = 0;
-    }
+    else
+        SOCK_CreateEx(x_host, port, o_timeout, &m_Socket, 0, 0, flags);
+
 }
 
 
 CUNIXSocket::CUNIXSocket(const string&   path,
                          const STimeout* timeout,
                          TSOCK_Flags     flags)
-    : CSocket()
 {
     if (timeout && timeout != kDefaultTimeout) {
         oo_timeout = *timeout;
         o_timeout = &oo_timeout;
     } else
         o_timeout = 0;
-    if (SOCK_CreateUNIX(path.c_str(), o_timeout, &m_Socket, 0, 0, flags)
-        != eIO_Success) {
-        m_Socket = 0;
-    }
+    SOCK_CreateUNIX(path.c_str(), o_timeout, &m_Socket, 0, 0, flags);
 }
 
 
@@ -167,7 +143,7 @@ EIO_Status CSocket::Connect(const string&   host,
         SOCK_SetTimeout(m_Socket, eIO_Write, w_timeout);
         SOCK_SetTimeout(m_Socket, eIO_Close, c_timeout);        
     } else
-        m_Socket = 0;
+        assert(!m_Socket);
     return status;
 }
 
@@ -192,12 +168,12 @@ EIO_Status CUNIXSocket::Connect(const string&   path,
     }
     EIO_Status status = SOCK_CreateUNIX(path.c_str(), o_timeout,
                                         &m_Socket, 0, 0, flags);
-    if (status != eIO_Success) {
+    if (status == eIO_Success) {
         SOCK_SetTimeout(m_Socket, eIO_Read,  r_timeout);
         SOCK_SetTimeout(m_Socket, eIO_Write, w_timeout);
         SOCK_SetTimeout(m_Socket, eIO_Close, c_timeout);        
     } else
-        m_Socket = 0;
+        assert(!m_Socket);
     return status;
 }
 
@@ -288,8 +264,8 @@ const STimeout* CSocket::GetTimeout(EIO_Event event) const
             return w_timeout;
         if ( !w_timeout )
             return r_timeout;
-        return ((unsigned long) r_timeout->sec*1000000 + r_timeout->usec >
-                (unsigned long) w_timeout->sec*1000000 + w_timeout->usec)
+        return ((unsigned long) r_timeout->sec * 1000000 + r_timeout->usec >
+                (unsigned long) w_timeout->sec * 1000000 + w_timeout->usec)
             ? w_timeout : r_timeout;
     case eIO_Close:
         return c_timeout;
@@ -315,12 +291,12 @@ EIO_Status CSocket::Read(void*          buf,
 
 EIO_Status CSocket::ReadLine(string& str)
 {
+    str.erase();
     if ( !m_Socket )
         return eIO_Closed;
     EIO_Status status;
     char buf[1024];
     size_t size;
-    str.erase();
     do {
         status = SOCK_ReadLine(m_Socket, buf, sizeof(buf), &size);
         if (!size)
@@ -412,17 +388,10 @@ void CSocket::Reset(SOCK sock, EOwnership if_to_own, ECopyTimeout whence)
 //
 
 
-CDatagramSocket::CDatagramSocket(TSOCK_Flags flags)
-{
-    if (DSOCK_CreateEx(&m_Socket, flags) != eIO_Success)
-        m_Socket = 0;
-}
-
-
 EIO_Status CDatagramSocket::Connect(unsigned int   host,
                                     unsigned short port)
 {
-    char addr[32];
+    char addr[40];
     if (host  &&  SOCK_ntoa(host, addr, sizeof(addr)) != 0)
         return eIO_Unknown;
     return m_Socket
@@ -431,17 +400,6 @@ EIO_Status CDatagramSocket::Connect(unsigned int   host,
 }
 
 
-EIO_Status CDatagramSocket::Send(const void*    data,
-                                 size_t         datalen,
-                                 const string&  host,
-                                 unsigned short port)
-{
-    return m_Socket
-        ? DSOCK_SendMsg(m_Socket, host.c_str(), port, data, datalen)
-        : eIO_Closed;
-}
-
-
 EIO_Status CDatagramSocket::Recv(void*           buf,
                                  size_t          buflen,
                                  size_t*         msglen,
@@ -449,8 +407,15 @@ EIO_Status CDatagramSocket::Recv(void*           buf,
                                  unsigned short* sender_port,
                                  size_t          maxmsglen)
 {
-    if ( !m_Socket )
+    if ( !m_Socket ) {
+        if ( msglen )
+            *msglen = 0;
+        if ( sender_host )
+            *sender_host = "";
+        if ( sender_port )
+            *sender_port = 0;
         return eIO_Closed;
+    }
 
     unsigned int addr;
     EIO_Status status = DSOCK_RecvMsg(m_Socket, buf, buflen, maxmsglen,
@@ -468,84 +433,31 @@ EIO_Status CDatagramSocket::Recv(void*           buf,
 //
 
 
-CListeningSocket::CListeningSocket(void)
-    : m_Socket(0),
-      m_IsOwned(eTakeOwnership)
-{
-    return;
-}
-
-
-CListeningSocket::CListeningSocket(unsigned short port,
-                                   unsigned short backlog,
-                                   TSOCK_Flags    flags)
-    : m_Socket(0),
-      m_IsOwned(eTakeOwnership)
-{
-    if (LSOCK_CreateEx(port, backlog, &m_Socket, flags) != eIO_Success)
-        m_Socket = 0;
-}
-
-
-CUNIXListeningSocket::CUNIXListeningSocket(const string&  path,
-                                           unsigned short backlog,
-                                           TSOCK_Flags    flags)
-    : CListeningSocket()
-{
-    if (LSOCK_CreateUNIX(path.c_str(), backlog,
-                         &m_Socket, flags) != eIO_Success) {
-        m_Socket = 0;
-    }
-}
-
-
 CListeningSocket::~CListeningSocket()
 {
     Close();
 }
 
 
-EIO_Status CListeningSocket::Listen(unsigned short port,
-                                    unsigned short backlog,
-                                    TSOCK_Flags    flags)
-{
-    if ( m_Socket )
-        return eIO_Unknown;
-    EIO_Status status = LSOCK_CreateEx(port, backlog, &m_Socket, flags);
-    if (status != eIO_Success)
-        m_Socket = 0;
-    return status;
-}
-
-
-EIO_Status CUNIXListeningSocket::Listen(const string&  path,
-                                        unsigned short backlog,
-                                        TSOCK_Flags    flags)
-{
-    if ( m_Socket )
-        return eIO_Unknown;
-    EIO_Status status = LSOCK_CreateUNIX(path.c_str(), backlog,
-                                         &m_Socket, flags);
-    if (status != eIO_Success)
-        m_Socket = 0;
-    return status;
-}
-
-
 EIO_Status CListeningSocket::Accept(CSocket*&       sock,
                                     const STimeout* timeout,
                                     TSOCK_Flags     flags) const
 {
-    if ( !m_Socket )
+    if ( !m_Socket ) {
+        sock = 0;
         return eIO_Closed;
+    }
 
-    SOCK x_sock;
-    EIO_Status status = LSOCK_AcceptEx(m_Socket, timeout, &x_sock, flags);
+    SOCK       x_sock;
+    EIO_Status status;
+    status = LSOCK_AcceptEx(m_Socket, timeout, &x_sock, flags);
+    assert(!x_sock ^ !(status != eIO_Success));
     if (status == eIO_Success) {
         try {
             sock = new CSocket;
         } catch (...) {
             sock = 0;
+            SOCK_Abort(x_sock);
             SOCK_Close(x_sock);
             throw;
         }
@@ -560,13 +472,15 @@ EIO_Status CListeningSocket::Accept(CSocket&        sock,
                                     const STimeout* timeout,
                                     TSOCK_Flags     flags) const
 {
-    if ( !m_Socket )
-        return eIO_Closed;
-
-    SOCK x_sock;
-    EIO_Status status = LSOCK_AcceptEx(m_Socket, timeout, &x_sock, flags);
-    if (status == eIO_Success)
-        sock.Reset(x_sock, eTakeOwnership, eCopyTimeoutsToSOCK);
+    SOCK       x_sock;
+    EIO_Status status;
+    if ( !m_Socket ) {
+        x_sock = 0;
+        status = eIO_Closed;
+    } else
+        status = LSOCK_AcceptEx(m_Socket, timeout, &x_sock, flags);
+    assert(!x_sock ^ !(status != eIO_Success));
+    sock.Reset(x_sock, eTakeOwnership, eCopyTimeoutsToSOCK);
     return status;
 }
 
@@ -574,7 +488,7 @@ EIO_Status CListeningSocket::Accept(CSocket&        sock,
 EIO_Status CListeningSocket::Close(void)
 {
     if ( !m_Socket )
-        return eIO_Success;
+        return eIO_Closed;
 
     EIO_Status status = m_IsOwned != eNoOwnership
         ? LSOCK_Close(m_Socket) : eIO_Success;
@@ -654,10 +568,10 @@ EIO_Status CSocketAPI::Poll(vector<SPoll>&  polls,
 
 string CSocketAPI::ntoa(unsigned int host)
 {
-    char ipaddr[64];
-    if (SOCK_ntoa(host, ipaddr, sizeof(ipaddr)) != 0)
-        *ipaddr = 0;
-    return string(ipaddr);
+    char addr[40];
+    if (SOCK_ntoa(host, addr, sizeof(addr)) != 0)
+        *addr = 0;
+    return string(addr);
 }
 
 
@@ -699,7 +613,7 @@ SIZE_TYPE CSocketAPI::StringToHostPort(const string&   str,
                                        unsigned short* port)
 {
     const char* s = str.c_str();
-    const char* e = SOCK_StringToHostPort(str.c_str(), host, port);
+    const char* e = SOCK_StringToHostPort(s, host, port);
     return e ? (SIZE_TYPE)(e - s) : NPOS;
 }
 
