@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_SOCKETP__H
 #define CONNECT___NCBI_SOCKETP__H
 
-/* $Id: ncbi_socketp.h 344550 2011-11-16 18:07:11Z lavr $
+/* $Id: ncbi_socketp.h 362476 2012-05-09 17:22:21Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -58,6 +58,10 @@
  */
 #include <errno.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif /*__cplusplus*/
+
 #ifdef NCBI_OS_MSWIN
 
 typedef SOCKET TSOCK_Handle;    /* NB: same as HANDLE   */
@@ -348,4 +352,12 @@ typedef struct SOCK_tag {
 #endif /*NCBI_OS_MSWIN && _WIN64*/
 
 
+extern const char g_kNcbiSockNameAbbr[];
+
+
+#ifdef __cplusplus
+} /* extern "C" */
+#endif /*__cplusplus*/
+
+
 #endif /* CONNECT___NCBI_SOCKETP__H */
