@@ -1,4 +1,4 @@
-/* $Id: server.cpp 329206 2011-08-09 14:11:28Z ivanovp $
+/* $Id: server.cpp 364729 2012-05-30 15:10:17Z ivanovp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,11 +32,14 @@
 
 #include <ncbi_pch.hpp>
 #include <corelib/ncbi_param.hpp>
-#include <connect/server.hpp>
 #include "connection_pool.hpp"
 #include <connect/ncbi_buffer.h>
 #include <connect/error_codes.hpp>
 
+#ifdef NCBI_OS_LINUX
+# include <sys/prctl.h>
+#endif
+
 
 #define NCBI_USE_ERRCODE_X   Connect_ThrServer
 
@@ -56,6 +59,7 @@ typedef NCBI_PARAM_TYPE(server, Catch_Unhandled_Exceptions) TParamServerCatchExc
 void IServer_MessageHandler::OnRead(void)
 {
     CSocket &socket = GetSocket();
+    CServer_Connection* conn = static_cast<CServer_Connection*>(&socket);
     char read_buf[4096];
     size_t n_read;
     EIO_Status status = socket.Read(read_buf, sizeof(read_buf), &n_read);
@@ -74,7 +78,7 @@ void IServer_MessageHandler::OnRead(void)
     }
     int message_tail;
     char *buf_ptr = read_buf;
-    for ( ;n_read > 0; ) {
+    for ( ;n_read > 0  &&  conn->type == eActiveSocket; ) {
         message_tail = this->CheckMessage(&m_Buffer, buf_ptr, n_read);
         // TODO: what should we do if message_tail > n_read?
         if (message_tail < 0) {
@@ -112,6 +116,235 @@ int Server_CheckLineMessage(BUF* buffer, const void *data, size_t size,
 }
 
 
+CBlockingQueue_ForServer::TItemHandle
+CBlockingQueue_ForServer::Put(const TRequest& data)
+{
+    CMutexGuard guard(m_Mutex);
+    if (m_Queue.empty()) {
+#ifdef NCBI_HAVE_CONDITIONAL_VARIABLE
+        m_GetCond.SignalAll();
+#else
+        m_GetSem.TryWait(); // is this still needed?
+        m_GetSem.Post();
+#endif
+    }
+    TItemHandle handle(new CQueueItem(data));
+    m_Queue.push_back(handle);
+    return handle;
+}
+
+CBlockingQueue_ForServer::TItemHandle
+CBlockingQueue_ForServer::GetHandle(void)
+{
+    CMutexGuard guard(m_Mutex);
+
+    while (m_Queue.empty()) {
+#ifdef NCBI_HAVE_CONDITIONAL_VARIABLE
+        m_GetCond.WaitForSignal(m_Mutex);
+#else
+        m_GetSem.TryWait();
+        m_GetSem.Post();
+#endif
+    }
+
+    TItemHandle handle(m_Queue.front());
+    m_Queue.pop_front();
+#ifndef NCBI_HAVE_CONDITIONAL_VARIABLE
+    if (!m_Queue.empty()) {
+        m_GetSem.TryWait();
+        m_GetSem.Post();
+    }
+#endif
+
+    guard.Release(); // avoid possible deadlocks from x_SetStatus
+    handle->x_SetStatus(CQueueItemBase::eActive);
+    return handle;
+}
+
+
+CThreadInPool_ForServer::CAutoUnregGuard::CAutoUnregGuard(TThread* thr)
+    : m_Thread(thr)
+{}
+
+CThreadInPool_ForServer::CAutoUnregGuard::~CAutoUnregGuard(void)
+{
+    m_Thread->x_UnregisterThread();
+}
+
+void
+CThreadInPool_ForServer::x_UnregisterThread(void)
+{
+    m_Pool->UnRegister(*this);
+}
+
+void
+CThreadInPool_ForServer::x_HandleOneRequest(bool catch_all)
+{
+    TItemHandle handle(m_Pool->GetHandle());
+    if (catch_all) {
+        try {
+            ProcessRequest(handle);
+        } catch (std::exception& e) {
+            handle->MarkAsForciblyCaught();
+            NCBI_REPORT_EXCEPTION_X(9, "Exception from thread in pool: ", e);
+            // throw;
+        } catch (...) {
+            handle->MarkAsForciblyCaught();
+            // silently propagate non-standard exceptions because they're
+            // likely to be CExitThreadException.
+            throw;
+        }
+    }
+    else {
+        ProcessRequest(handle);
+    }
+}
+
+void*
+CThreadInPool_ForServer::Main(void)
+{
+    if (!m_Pool->m_ThrSuffix.empty()) {
+        string thr_name = CNcbiApplication::Instance()->GetProgramDisplayName();
+        thr_name += m_Pool->m_ThrSuffix;
+#ifdef NCBI_OS_LINUX
+        prctl(PR_SET_NAME, (unsigned long)thr_name.c_str(), 0, 0, 0);
+#endif
+    }
+
+    m_Pool->Register(*this);
+    CAutoUnregGuard guard(this);
+
+    bool catch_all = TParamThreadPoolCatchExceptions::GetDefault();
+    for (;;) {
+        x_HandleOneRequest(catch_all);
+    }
+
+    return NULL;
+}
+
+void
+CThreadInPool_ForServer::OnExit(void)
+{
+    m_Pool->m_ThreadCount.Add(-1);
+}
+
+void
+CThreadInPool_ForServer::ProcessRequest(TItemHandle handle)
+{
+    TCompletingHandle completer = handle;
+    ProcessRequest(completer->GetRequest());
+}
+
+
+CPoolOfThreads_ForServer::CPoolOfThreads_ForServer(unsigned int max_threads,
+                                                   const string& thr_suffix)
+    : m_MaxThreads(max_threads),
+      m_ThrSuffix(thr_suffix),
+      m_KilledAll(false)
+{
+    m_ThreadCount.Set(0);
+    m_PutQueueNum.Set(0);
+    m_GetQueueNum.Set(0);
+
+    m_Queues = (TQueue**)malloc(m_MaxThreads * sizeof(m_Queues[0]));
+    for (TACValue i = 0; i < m_MaxThreads; ++i) {
+        m_Queues[i] = new TQueue();
+    }
+}
+
+CPoolOfThreads_ForServer::~CPoolOfThreads_ForServer(void)
+{
+    try {
+        KillAllThreads(false);
+    } catch(...) {}    // Just to be sure that we will not throw from the destructor.
+
+    CAtomicCounter::TValue n = m_ThreadCount.Get();
+    if (n) {
+        ERR_POST_X(10, Warning << "CPoolOfThreads_ForServer::~CPoolOfThreads_ForServer: "
+                               << n << " thread(s) still active");
+    }
+
+    // Just in case let's deliberately not destroy all queues.
+}
+
+void
+CPoolOfThreads_ForServer::Spawn(unsigned int num_threads)
+{
+    for (unsigned int i = 0; i < num_threads; i++)
+    {
+        m_ThreadCount.Add(1);
+        NewThread()->Run();
+    }
+}
+
+void
+CPoolOfThreads_ForServer::AcceptRequest(const TRequest& req)
+{
+    Uint4 q_num = Uint4(m_PutQueueNum.Add(1)) % m_MaxThreads;
+    m_Queues[q_num]->Put(req);
+}
+
+CPoolOfThreads_ForServer::TItemHandle
+CPoolOfThreads_ForServer::GetHandle(void)
+{
+    Uint4 q_num = Uint4(m_GetQueueNum.Add(1)) % m_MaxThreads;
+    return m_Queues[q_num]->GetHandle();
+}
+
+
+class CFatalRequest_ForServer : public CStdRequest
+{
+protected:
+    void Process(void) { CThread::Exit(0); } // Kill the current thread
+};
+
+
+void
+CPoolOfThreads_ForServer::KillAllThreads(bool wait)
+{
+    m_KilledAll = true;
+
+    CRef<CStdRequest> poison(new CFatalRequest_ForServer);
+
+    for (TACValue i = 0;  i < m_MaxThreads; ++i) {
+        AcceptRequest(poison);
+    }
+    NON_CONST_ITERATE(TThreads, it, m_Threads) {
+        if (wait) {
+            (*it)->Join();
+        } else {
+            (*it)->Detach();
+        }
+    }
+    m_Threads.clear();
+}
+
+
+void
+CPoolOfThreads_ForServer::Register(TThread& thread)
+{
+    CMutexGuard guard(m_Mutex);
+    if (!m_KilledAll) {
+        m_Threads.push_back(CRef<TThread>(&thread));
+    }
+}
+
+void
+CPoolOfThreads_ForServer::UnRegister(TThread& thread)
+{
+    CMutexGuard guard(m_Mutex);
+    if (!m_KilledAll) {
+        TThreads::iterator it = find(m_Threads.begin(), m_Threads.end(),
+                                     CRef<TThread>(&thread));
+        if (it != m_Threads.end()) {
+            (*it)->Detach();
+            m_Threads.erase(it);
+        }
+    }
+}
+
+
+
 /////////////////////////////////////////////////////////////////////////////
 // Abstract class for CAcceptRequest and CServerConnectionRequest
 class CServer_Request : public CStdRequest
@@ -184,20 +417,14 @@ CAcceptRequest::CAcceptRequest(EServIO_Event event,
 
 void CAcceptRequest::x_DoProcess(void)
 {
-    if (m_ConnPool.Add(m_Connection,
-                       CServer_ConnectionPool::eActiveSocket))
-    {
+    if (m_ConnPool.Add(m_Connection, eActiveSocket)) {
         m_Connection->OnSocketEvent(eServIO_Open);
-        m_ConnPool.SetConnType(m_Connection,
-                               CServer_ConnectionPool::eInactiveSocket);
+        m_ConnPool.SetConnType(m_Connection, eInactiveSocket);
     }
     else {
         // The connection pool is full
         // This place is the only one which can call OnOverflow now
         m_Connection->OnOverflow(eOR_ConnectionPoolFull);
-        // Abort connection here to prevent it sitting in TIME_WAIT state
-        // on the server.
-        m_Connection->Abort();
         delete m_Connection;
     }
 }
@@ -254,8 +481,10 @@ void CServerConnectionRequest::Process(void)
     else {
         m_Connection->OnSocketEvent(m_Event);
     }
-    // Return socket to poll vector
-    m_ConnPool.SetConnType(m_Connection, CServer_ConnectionPool::eInactiveSocket, false);
+    if (m_Event != eServIO_Inactivity  &&  m_Event != eServIO_Delete) {
+        // Return socket to poll vector
+        m_ConnPool.SetConnType(m_Connection, eInactiveSocket);
+    }
 }
 
 
@@ -265,8 +494,7 @@ void CServerConnectionRequest::Cancel(void)
     // See comment at CServer::CreateRequest
     m_Connection->OnOverflow(eOR_RequestQueueFull);
     // Return socket to poll vector
-    m_ConnPool.SetConnType(m_Connection,
-                           CServer_ConnectionPool::eInactiveSocket);
+    m_ConnPool.SetConnType(m_Connection, eInactiveSocket);
 }
 
 
@@ -288,10 +516,6 @@ CServer_Connection::CreateRequest(EServIO_Event           event,
                                   CServer_ConnectionPool& conn_pool,
                                   const STimeout*         timeout)
 {
-    // Pull out socket from poll vector
-    // See CServerConnectionRequest::Process and
-    // CServer_ConnectionPool::GetPollAndTimerVec
-    conn_pool.SetConnType(this, CServer_ConnectionPool::eActiveSocket);
     return new CServerConnectionRequest(event, conn_pool, timeout, this);
 }
 
@@ -304,27 +528,45 @@ void CServer_Connection::OnSocketEvent(EServIO_Event event)
 {
     if (event == (EServIO_Event) -1) {
         m_Handler->OnTimer();
-    } else if (eServIO_Open == event) {
+        return;
+    }
+    switch (event) {
+    case eServIO_Open:
         m_Handler->OnOpen();
-    } else if (eServIO_OurClose == event) {
+        break;
+    case eServIO_OurClose:
         m_Handler->OnClose(IServer_ConnectionHandler::eOurClose);
         m_Open = false;
-    } else if (eServIO_ClientClose == event) {
+        break;
+    case eServIO_ClientClose:
         m_Handler->OnClose(IServer_ConnectionHandler::eClientClose);
         m_Open = false;
-    } else if (eServIO_Inactivity == event) {
+        break;
+    case eServIO_Inactivity:
         OnTimeout();
         m_Handler->OnClose(IServer_ConnectionHandler::eOurClose);
-        Abort();
-        m_Open = false;
-    } else {
+        //m_Open = false;
+        // fall through
+    case eServIO_Delete:
+        delete this;
+        break;
+    default:
         if (eServIO_Read & event)
             m_Handler->OnRead();
         if (eServIO_Write & event)
             m_Handler->OnWrite();
+        break;
     }
 }
 
+CServer_Connection::~CServer_Connection()
+{
+    static const STimeout zero_timeout = {0, 0};
+
+    // Set zero timeout to prevent the socket from sitting in
+    // TIME_WAIT state on the server.
+    SetTimeout(eIO_Close, &zero_timeout);
+}
 
 /////////////////////////////////////////////////////////////////////////////
 // CServer implementation
@@ -348,8 +590,7 @@ CServer::~CServer()
 void CServer::AddListener(IServer_ConnectionFactory* factory,
                           unsigned short port)
 {
-    m_ConnectionPool->Add(new CServer_Listener(factory, port),
-                          CServer_ConnectionPool::eListener);
+    m_ConnectionPool->Add(new CServer_Listener(factory, port), eListener);
 }
 
 
@@ -393,31 +634,52 @@ static inline bool operator <(const STimeout& to1, const STimeout& to2)
 
 void CServer::x_DoRun(void)
 {
-    m_ThreadPool->Spawn(m_Parameters->init_threads);
+    m_ThreadPool->Spawn(m_Parameters->max_threads);
 
     Init();
 
     vector<CSocketAPI::SPoll> polls;
     size_t     count;
     typedef vector<IServer_ConnectionBase*> TConnsList;
+    typedef vector<CRef<CStdRequest> > TReqsList;
     TConnsList timer_requests;
     TConnsList revived_conns;
     TConnsList to_close_conns;
-    STimeout   timer_timeout;
+    TConnsList to_delete_conns;
+    TReqsList to_add_reqs;
+    STimeout timer_timeout;
     const STimeout* timeout;
 
     while (!ShutdownRequested()) {
         bool has_timer = m_ConnectionPool->GetPollAndTimerVec(
                                            polls, timer_requests, &timer_timeout,
-                                           revived_conns, to_close_conns);
+                                           revived_conns, to_close_conns,
+                                           to_delete_conns);
 
         ITERATE(TConnsList, it, revived_conns) {
-            EIO_Event evt = (*it)->GetEventsToPollFor(NULL);
-            CreateRequest(*it, IOEventToServIOEvent(evt), m_Parameters->idle_timeout);
+            IServer_ConnectionBase* conn_base = *it;
+            EServIO_Event evt = IOEventToServIOEvent(conn_base->GetEventsToPollFor(NULL));
+            CRef<CStdRequest> req(conn_base->CreateRequest(
+                                                evt, *m_ConnectionPool,
+                                                m_Parameters->idle_timeout));
+            to_add_reqs.push_back(req);
         }
         ITERATE(TConnsList, it, to_close_conns) {
-            CreateRequest(*it, eServIO_Inactivity, m_Parameters->idle_timeout);
+            IServer_ConnectionBase* conn_base = *it;
+            CRef<CStdRequest> req(conn_base->CreateRequest(
+                                                eServIO_Inactivity, *m_ConnectionPool,
+                                                m_Parameters->idle_timeout));
+            to_add_reqs.push_back(req);
         }
+        ITERATE(TConnsList, it, to_delete_conns) {
+            IServer_ConnectionBase* conn_base = *it;
+            CRef<CStdRequest> req(conn_base->CreateRequest(
+                                                eServIO_Delete, *m_ConnectionPool,
+                                                m_Parameters->idle_timeout));
+            to_add_reqs.push_back(req);
+        }
+        x_AddRequests(to_add_reqs);
+        to_add_reqs.clear();
 
         timeout = m_Parameters->accept_timeout;
 
@@ -455,23 +717,35 @@ void CServer::x_DoRun(void)
                 ProcessTimeout();
             }
             else {
+                m_ConnectionPool->SetAllActive(timer_requests);
                 ITERATE (vector<IServer_ConnectionBase*>, it, timer_requests)
                 {
-                    CreateRequest(*it, (EServIO_Event) -1, timeout);
+                    IServer_ConnectionBase* conn_base = *it;
+                    CRef<CStdRequest> req(conn_base->CreateRequest(
+                                          (EServIO_Event)-1, *m_ConnectionPool, timeout));
+                    to_add_reqs.push_back(req);
                 }
+                x_AddRequests(to_add_reqs);
+                to_add_reqs.clear();
             }
             continue;
         }
 
+        m_ConnectionPool->SetAllActive(polls);
         ITERATE (vector<CSocketAPI::SPoll>, it, polls) {
             if (!it->m_REvent) continue;
             IServer_ConnectionBase* conn_base =
                 dynamic_cast<IServer_ConnectionBase*>(it->m_Pollable);
             _ASSERT(conn_base);
-            CreateRequest(conn_base,
-                          IOEventToServIOEvent(it->m_REvent),
-                          m_Parameters->idle_timeout);
+            CRef<CStdRequest> req(conn_base->CreateRequest(
+                                                IOEventToServIOEvent(it->m_REvent),
+                                                *m_ConnectionPool,
+                                                m_Parameters->idle_timeout));
+            if (req)
+                to_add_reqs.push_back(req);
         }
+        x_AddRequests(to_add_reqs);
+        to_add_reqs.clear();
     }
 }
 
@@ -479,9 +753,7 @@ void CServer::Run(void)
 {
     StartListening(); // detect unavailable ports ASAP
 
-    m_ThreadPool.reset(new CStdPoolOfThreads(m_Parameters->max_threads,
-                                             kMax_UInt,
-                                             m_Parameters->spawn_threshold));
+    m_ThreadPool.reset(new CPoolOfThreads_ForServer(m_Parameters->max_threads, m_ThreadSuffix));
     if (TParamServerCatchExceptions::GetDefault()) {
         try {
             x_DoRun();
@@ -518,7 +790,7 @@ void CServer::SubmitRequest(const CRef<CStdRequest>& request)
 
 void CServer::DeferConnectionProcessing(IServer_ConnectionBase* conn)
 {
-    m_ConnectionPool->SetConnType(conn, CServer_ConnectionPool::ePreDeferredSocket);
+    m_ConnectionPool->SetConnType(conn, ePreDeferredSocket);
 }
 
 
@@ -538,38 +810,16 @@ void CServer::Exit()
 }
 
 
-void CServer::CreateRequest(IServer_ConnectionBase* conn_base,
-                            EServIO_Event event,
-                            const STimeout* timeout)
+void CServer::x_AddRequests(const vector<CRef<CStdRequest> >& reqs)
 {
-    CRef<CStdRequest> request(conn_base->CreateRequest(
-                                         event, *m_ConnectionPool, timeout));
-
-    if (request) {
-        try {
-            m_ThreadPool->AcceptRequest(request);
-        } catch (CBlockingQueueException&) {
-            // The size of thread pool queue is set to kMax_UInt, so
-            // this is impossible event, but we handle it gently
-            ERR_POST_X(1, Critical << "Thread pool queue full");
-            CServer_Request* req = dynamic_cast<CServer_Request*>(
-                                                        request.GetPointer());
-            _ASSERT(req);
-            // Queue is full, drop request, indirectly dropping incoming
-            // connection (see also CAcceptRequest::Cancel)
-            // ??? What should we do if conn_base is CServerConnection?
-            // Should we close it? (see also CServerConnectionRequest::Cancel)
-            req->Cancel();
-        }
-    }
-    else {
-        _ASSERT(event == eServIO_Read);
+    ITERATE(vector<CRef<CStdRequest> >, it, reqs) {
+        m_ThreadPool->AcceptRequest(*it);
     }
 }
 
 void CServer::AddConnectionToPool(CServer_Connection* conn)
 {
-    if (!m_ConnectionPool->Add(conn, CServer_ConnectionPool::eInactiveSocket)) {
+    if (!m_ConnectionPool->Add(conn, eInactiveSocket)) {
         NCBI_THROW(CServer_Exception, ePoolOverflow,
                    "Cannot add connection, pool has overflowed.");
     }
