@@ -1,4 +1,4 @@
-/*  $Id: ddumpable.cpp 310436 2011-06-30 17:34:16Z ivanov $
+/*  $Id: ddumpable.cpp 363040 2012-05-11 19:24:28Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -212,7 +212,7 @@ void CDebugDumpContext::Log(const string& name, unsigned long value,
     Log(name, NStr::ULongToString(value), CDebugDumpFormatter::eValue, comment);
 }
 
-#if (SIZEOF_LONG < 8)
+#ifndef NCBI_INT8_IS_LONG
 void CDebugDumpContext::Log(const string& name, Int8 value,
                             const string& comment)
 {
