@@ -1,4 +1,4 @@
-/*  $Id: interprocess_lock.cpp 222093 2011-01-25 15:32:16Z gouriano $
+/*  $Id: interprocess_lock.cpp 370747 2012-08-01 15:10:11Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -52,10 +52,9 @@ BEGIN_NCBI_SCOPE
 #if   defined(NCBI_OS_UNIX)
     const int    kInvalidLockHandle = -1;
 #elif defined(NCBI_OS_MSWIN)
-    const HANDLE kInvalidLockHandle = INVALID_HANDLE_VALUE;
+    const HANDLE kInvalidLockHandle = NULL;
 #endif
 
-
 // List of all locks in the current process <name, ref_counter>.
 typedef map<string, int> TLocks;
 static CSafeStaticPtr<TLocks> s_Locks;
