@@ -1,4 +1,4 @@
-/*  $Id: ncbiapp.cpp 350059 2012-01-17 22:09:57Z rafanovi $
+/*  $Id: ncbiapp.cpp 372845 2012-08-22 19:16:15Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -330,8 +330,9 @@ void CNcbiApplication::x_TryMain(EAppDiagStream diag,
                 // handle-exceptions flag.
                 throw;
             }
-            catch (CArgException& e) {
-                NCBI_RETHROW_SAME(e, "Application's initialization failed");
+            catch (CArgException&) {
+//                NCBI_RETHROW_SAME(e, "Application's initialization failed");
+                throw;
             }
             catch (CException& e) {
                 NCBI_REPORT_EXCEPTION_X(15,
@@ -667,7 +668,7 @@ int CNcbiApplication::AppMain
     }
     catch (CArgException& e) {
         // Print USAGE and the exception error message
-        if ( m_ArgDesc.get() ) {
+        if ( e.GetErrCode() != CArgException::eNoValue &&  m_ArgDesc.get() ) {
             x_AddDefaultArgs();
             string str;
             m_ArgDesc->PrintUsage(str);
@@ -678,8 +679,9 @@ int CNcbiApplication::AppMain
                 cerr << "Error in command-line arguments. "
                     "See error logs for more details." << endl;
             }
-            cerr << string(72, '=') << endl;
+            cerr << string(72, '=') << endl << endl;
         }
+        SetDiagPostAllFlags(eDPF_Severity);
         NCBI_REPORT_EXCEPTION_X(18, "", e);
         got_exception = true;
         exit_code = 1;
@@ -1275,12 +1277,21 @@ void CNcbiApplication::x_HonorStandardSettings( IRegistry* reg)
     }
     // [NCBI.MemorySizeLimit]
     if ( !reg->Get("NCBI", "MemorySizeLimit").empty() ) {
-        int mem_size_limit = reg->GetInt("NCBI", "MemorySizeLimit", 0);
-        if (mem_size_limit < 0) {
-            NCBI_THROW(CAppException, eLoadConfig,
-                       "Configuration file error:  [NCBI.MemorySizeLimit] < 0");
+        size_t mem_size_limit = 0;
+        string s = reg->GetString("NCBI", "MemorySizeLimit", kEmptyStr);
+        size_t pos = s.find("%");
+        if (pos != NPOS) {
+            // Size in percents of total memory
+            size_t percents = NStr::StringToUInt(CTempString(s, 0, pos));
+            if (percents > 100) {
+                NCBI_THROW(CAppException, eLoadConfig,
+                           "Configuration file error:  [NCBI.HeapSizeLimit] > 100%");
+            }
+            mem_size_limit = (size_t)(GetPhysicalMemorySize() * percents / 100);
+        } else {
+            // Size in MB
+            mem_size_limit = (size_t)(NStr::StringToUInt(s) * 1024 * 1024);
         }
-        mem_size_limit *= 1024 * 1024;
         if ( !SetMemoryLimit(mem_size_limit) ) {
             ERR_POST_X(13, Warning
                            << "Failed to set memory size limit to "
