@@ -1,4 +1,4 @@
-/* $Id: ncbistre.cpp 268007 2011-03-28 16:43:56Z gouriano $
+/* $Id: ncbistre.cpp 372997 2012-08-23 20:51:17Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -42,6 +42,7 @@
 
 BEGIN_NCBI_SCOPE
 
+
 #if defined(NCBI_OS_MSWIN) && defined(_UNICODE)
 
 CNcbiIfstream::CNcbiIfstream(
@@ -49,45 +50,43 @@ CNcbiIfstream::CNcbiIfstream(
  : IO_PREFIX::ifstream(_T_XCSTRING(_Filename), _Mode, _Prot)
 {
 }
+
 void CNcbiIfstream::open(
     const char *_Filename, IOS_BASE::openmode _Mode, int _Prot)
 {
     IO_PREFIX::ifstream::open(_T_XCSTRING(_Filename), _Mode, _Prot);
 }
 
-
 CNcbiOfstream::CNcbiOfstream(
     const char *_Filename, IOS_BASE::openmode _Mode, int _Prot)
  : IO_PREFIX::ofstream(_T_XCSTRING(_Filename), _Mode, _Prot)
 {
 }
+
 void CNcbiOfstream::open(
     const char *_Filename, IOS_BASE::openmode _Mode, int _Prot)
 {
     IO_PREFIX::ofstream::open(_T_XCSTRING(_Filename), _Mode, _Prot);
 }
 
-
 CNcbiFstream::CNcbiFstream(
     const char *_Filename, IOS_BASE::openmode _Mode, int _Prot)
  : IO_PREFIX::fstream(_T_XCSTRING(_Filename), _Mode, _Prot)
 {
 }
+
 void CNcbiFstream::open(
     const char *_Filename, IOS_BASE::openmode _Mode, int _Prot)
 {
     IO_PREFIX::fstream::open(_T_XCSTRING(_Filename), _Mode, _Prot);
 }
 
-
 #endif
 
-CNcbiIstream& NcbiGetline(CNcbiIstream& is, string& str, const string& delims)
-{
-    CT_INT_TYPE ch;
-    char        buf[1024];
-    SIZE_TYPE   pos = 0;
 
+CNcbiIstream& NcbiGetline(CNcbiIstream& is, string& str, const string& delims,
+                          SIZE_TYPE* count)
+{
     str.erase();
 
     IOS_BASE::fmtflags f = is.flags();
@@ -95,57 +94,74 @@ CNcbiIstream& NcbiGetline(CNcbiIstream& is, string& str, const string& delims)
 #ifdef NO_PUBSYNC
     if ( !is.ipfx(1) ) {
         is.flags(f);
+        is.setstate(NcbiFailbit);
         return is;
     }
 #else
     CNcbiIstream::sentry s(is);
     if ( !s ) {
-        is.clear(NcbiFailbit | is.rdstate());
         is.flags(f);
+        is.setstate(NcbiFailbit);
         return is;
     }
-#endif
+#endif //NO_PUBSYNC
+    _ASSERT( is.good() );
 
-    SIZE_TYPE end = str.max_size();
-    SIZE_TYPE i = 0;
-    for (ch = is.rdbuf()->sbumpc();  !CT_EQ_INT_TYPE(ch, CT_EOF);
-         ch = is.rdbuf()->sbumpc()) {
-        i++;
+    char buf[1024];
+    SIZE_TYPE pos = 0;
+    SIZE_TYPE size = 0;
+    SIZE_TYPE max_size = str.max_size();
+    SIZE_TYPE delim_count = 0;
+    IOS_BASE::iostate iostate = NcbiGoodbit/*0*/;
+    for (;;) {
+        CT_INT_TYPE ch = is.rdbuf()->sbumpc();
+        if ( CT_EQ_INT_TYPE(ch, CT_EOF) ) {
+            iostate = NcbiEofbit;
+            break;
+        }
         SIZE_TYPE delim_pos = delims.find(CT_TO_CHAR_TYPE(ch));
         if (delim_pos != NPOS) {
             // Special case -- if two different delimiters are back to
             // back and in the same order as in delims, treat them as
             // a single delimiter (necessary for correct handling of
-            // DOS-style CRLF endings).
-            CT_INT_TYPE next = is.rdbuf()->sgetc();
-            if (!CT_EQ_INT_TYPE(next, CT_EOF)
-                &&  delims.find(CT_TO_CHAR_TYPE(next), delim_pos + 1) != NPOS){
+            // DOS/MAC-style CR/LF endings).
+            ch = is.rdbuf()->sgetc();
+            if (!CT_EQ_INT_TYPE(ch, CT_EOF)
+                &&  delims.find(CT_TO_CHAR_TYPE(ch), delim_pos + 1) != NPOS) {
                 is.rdbuf()->sbumpc();
+                delim_count = 2;
+            } else {
+                delim_count = 1;
             }
             break;
         }
-        if (i == end) {
-            is.clear(NcbiFailbit | is.rdstate());      
+        if (size == max_size) {
+            CT_INT_TYPE bk = is.rdbuf()->sungetc();
+            iostate = CT_EQ_INT_TYPE(bk, ch) ? NcbiFailbit : NcbiBadbit;
             break;
         }
 
         buf[pos++] = CT_TO_CHAR_TYPE(ch);
         if (pos == sizeof(buf)) {
             str.append(buf, pos);
-            pos = 0;
+            pos  = 0;
         }
+        size++;
     }
-    str.append(buf, pos);
-    if (CT_EQ_INT_TYPE(ch, EOF)) 
-        is.clear(NcbiEofbit | is.rdstate());      
-    if ( !i )
-        is.clear(NcbiFailbit | is.rdstate());      
+    if (pos > 0)
+        str.append(buf, pos);
+    if (count != NULL)
+        *count = size + delim_count;
 
 #ifdef NO_PUBSYNC
     is.isfx();
-#endif
-
+#endif //NO_PUBSYNC
     is.flags(f);
+    if (iostate) {
+        if (iostate == NcbiEofbit  &&  str.empty())
+            iostate |= NcbiFailbit;
+        is.clear(iostate);
+    }
     return is;
 }
 
@@ -156,33 +172,49 @@ CNcbiIstream& NcbiGetline(CNcbiIstream& is, string& str, const string& delims)
 #  endif
 #endif
 
-extern CNcbiIstream& NcbiGetline(CNcbiIstream& is, string& str, char delim)
+extern CNcbiIstream& NcbiGetline(CNcbiIstream& is, string& str, char delim,
+                                 SIZE_TYPE* count)
 {
-#if defined(NCBI_USE_OLD_IOSTREAM)
-    return NcbiGetline(is, str, string(1, delim));
-#elif defined(NCBI_COMPILER_GCC29x)
-    // The code below is normally somewhat faster than this call,
-    // which typically appends one character at a time to str;
-    // however, it blows up when built with some GCC versions.
-    return getline(is, str, delim);
+#if   defined(NCBI_USE_OLD_IOSTREAM)
+    return NcbiGetline(is, str, string(1, delim), count);
 #else
-    char buf[1024];
     str.erase();
-    while (is.good()) {
+
+    if ( !is.good() ) {
+        is.setstate(NcbiFailbit);
+        return is;
+    }
+
+    char buf[1024];
+    SIZE_TYPE size = 0;
+    SIZE_TYPE max_size = str.max_size();
+    do {
         CT_INT_TYPE nextc = is.get();
         if (CT_EQ_INT_TYPE(nextc, CT_EOF) 
             ||  CT_EQ_INT_TYPE(nextc, CT_TO_INT_TYPE(delim))) {
+            ++size;
             break;
         }
-        is.putback(nextc);
-        is.get(buf, sizeof(buf), delim);
-        str.append(buf, is.gcount());
-    }
-    if (str.empty()  &&  is.eof()) {
-        is.setstate(NcbiFailbit);
-    }
-    return is;
+        if ( !is.unget() )
+            break;
+        if (size == max_size) {
+            is.clear(NcbiFailbit);
+            break;
+        }
+        SIZE_TYPE n = max_size - size;
+        is.get(buf, n < sizeof(buf) ? n : sizeof(buf), delim);
+        n = (size_t) is.gcount();
+        str.append(buf, n);
+        size += n;
+        _ASSERT(size == str.length());
+    } while ( is.good() );
 #endif
+
+    if (is.rdstate() == NcbiEofbit  &&  str.empty())
+        is.setstate(NcbiFailbit);
+    if (count != NULL)
+        *count = size;
+    return is;
 }
 
 
@@ -198,21 +230,18 @@ const char* Endl(void)
 }
 
 
-// Get the next line taking into account platform specifics of End-of-Line
-CNcbiIstream& NcbiGetlineEOL(CNcbiIstream& is, string& str)
+// Get a line taking into account platform-specific of End-Of-Line
+CNcbiIstream& NcbiGetlineEOL(CNcbiIstream& is, string& str, SIZE_TYPE* count)
 {
-#if defined(NCBI_OS_MSWIN)
-    NcbiGetline(is, str, '\n');
-    if (!str.empty()  &&  str[str.length()-1] == '\r')
+#if   defined(NCBI_OS_MSWIN)
+    NcbiGetline(is, str, '\n', count);
+    if (!str.empty()  &&  str[str.length() - 1] == '\r')
         str.resize(str.length() - 1);
 #elif defined(NCBI_OS_DARWIN)
-    NcbiGetline(is, str, "\r\n");
+    NcbiGetline(is, str, "\r\n", count);
 #else /* assume UNIX-like EOLs */
-    NcbiGetline(is, str, '\n');
-#endif
-    // special case -- an empty line
-    if (is.fail()  &&  !is.eof()  &&  !is.gcount()  &&  str.empty())
-        is.clear(is.rdstate() & ~NcbiFailbit);
+    NcbiGetline(is, str, '\n', count);
+#endif //NCBI_OS_...
     return is;
 }
 
@@ -254,22 +283,18 @@ bool NcbiStreamCopy(CNcbiOstream& os, CNcbiIstream& is)
     }
     os << is.rdbuf();
 #endif
-    if (!os.good())
+    if (!os.good()  ||  !os.flush())
         return false;
-    os.flush();
-    if (!os.good())
-        return false;
-    if (!CT_EQ_INT_TYPE(is.peek(), CT_EOF)) {
-        os.clear(NcbiFailbit);
-        return false;
-    }
-    return true;
+    if (CT_EQ_INT_TYPE(is.peek(), CT_EOF))
+        return is.fail() ? false : true;
+    os.clear(NcbiFailbit);
+    return false;
 }
 
 
 CNcbiOstrstreamToString::operator string(void) const
 {
-    SIZE_TYPE length = m_Out.pcount();
+    SIZE_TYPE length = (size_t)m_Out.pcount();
     if ( length == 0 )
         return string();
     const char* str = m_Out.str();
@@ -476,8 +501,8 @@ istream& istream::read(char *s, streamsize n)
 EEncodingForm ReadIntoUtf8(
     CNcbiIstream&     input,
     CStringUTF8*      result,
-    EEncodingForm     ef             /*  = eEncodingForm_Unknown*/,
-    EReadUnknownNoBOM what_if_no_bom /* = eNoBOM_GuessEncoding*/
+    EEncodingForm     ef             /* = eEncodingForm_Unknown */,
+    EReadUnknownNoBOM what_if_no_bom /* = eNoBOM_GuessEncoding  */
 )
 {
     EEncodingForm ef_bom = eEncodingForm_Unknown;
