@@ -1,4 +1,4 @@
-/*  $Id: rwstreambuf.cpp 329061 2011-08-06 20:01:26Z lavr $
+/* $Id: rwstreambuf.cpp 357069 2012-03-20 13:05:00Z ivanovp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,6 +33,7 @@
 #include <ncbi_pch.hpp>
 #include <corelib/ncbidbg.hpp>
 #include <corelib/ncbiexpt.hpp>
+#include <corelib/ncbi_limits.hpp>
 #include <corelib/reader_writer.hpp>
 #include <corelib/rwstream.hpp>
 #include <corelib/error_codes.hpp>
@@ -91,9 +92,9 @@
 BEGIN_NCBI_SCOPE
 
 
-const char* g_RW_ResultToString(ERW_Result res)
+const char* g_RW_ResultToString(ERW_Result result)
 {
-    static const char* const res_str[eRW_Eof - eRW_NotImplemented + 1] = {
+    static const char* const kResultStr[eRW_Eof - eRW_NotImplemented + 1] = {
         "eRW_NotImplemented",
         "eRW_Success",
         "eRW_Timeout",
@@ -101,8 +102,8 @@ const char* g_RW_ResultToString(ERW_Result res)
         "eRW_Eof"
     };
 
-    _ASSERT(eRW_NotImplemented <= res  &&  res <= eRW_Eof);
-    return res_str[res - eRW_NotImplemented];
+    _ASSERT(eRW_NotImplemented <= result  &&  result <= eRW_Eof);
+    return kResultStr[result - eRW_NotImplemented];
 }
 
 static const streamsize kDefaultBufSize = 4096;
@@ -162,9 +163,9 @@ CRWStreambuf::~CRWStreambuf()
 }
 
 
-CNcbiStreambuf* CRWStreambuf::setbuf(CT_CHAR_TYPE* s, streamsize n)
+CNcbiStreambuf* CRWStreambuf::setbuf(CT_CHAR_TYPE* s, streamsize m)
 {
-    if (!s  &&  !n) {
+    if (!s  &&  !m) {
         return this;
     }
 
@@ -177,20 +178,17 @@ CNcbiStreambuf* CRWStreambuf::setbuf(CT_CHAR_TYPE* s, streamsize n)
 
     delete[] m_pBuf;
 
+    size_t n = (size_t) m;
     if ( !n ) {
         _ASSERT(kDefaultBufSize > 1);
-        n = kDefaultBufSize << (m_Reader  &&  m_Writer ? 1 : 0);
+        n = (size_t) kDefaultBufSize << (m_Reader  &&  m_Writer ? 1 : 0);
     }
-
     if ( !s ) {
-        if (n != 1)
-            s = m_pBuf = new CT_CHAR_TYPE[n];
-        else
-            s = &x_Buf;
+        s          = n == 1 ? &x_Buf : (m_pBuf = new CT_CHAR_TYPE[n]);
     }
 
     if ( m_Reader ) {
-        m_BufSize  = n == 1 ? 1 : n >> (m_Reader  &&  m_Writer ? 1 : 0);
+        m_BufSize  = n == 1 ? 1      : n >> (m_Reader  &&  m_Writer ? 1 : 0);
         m_ReadBuf  = s;
     } else {
         m_BufSize  = 0;
@@ -199,7 +197,7 @@ CNcbiStreambuf* CRWStreambuf::setbuf(CT_CHAR_TYPE* s, streamsize n)
     setg(m_ReadBuf, m_ReadBuf, m_ReadBuf);
 
     if ( m_Writer ) {
-        m_WriteBuf = n == 1 ? 0 : s + m_BufSize;
+        m_WriteBuf = n == 1 ? 0      : s + m_BufSize;
     } else {
         m_WriteBuf = 0;
     }
@@ -260,21 +258,22 @@ CT_INT_TYPE CRWStreambuf::overflow(CT_INT_TYPE c)
 
     _ASSERT(CT_EQ_INT_TYPE(c, CT_EOF));
 
-    ERW_Result res = eRW_Error;
+    ERW_Result result = eRW_Error;
     RWSTREAMBUF_HANDLE_EXCEPTIONS(
-        res = m_Writer->Flush(),
+        result = m_Writer->Flush(),
         7, "CRWStreambuf::overflow(): IWriter::Flush()",
         (void) 0);
-    switch (res) {
+    switch (result) {
     case eRW_Error:
     case eRW_Eof:
         x_Err    = true;
         x_ErrPos = x_GetPPos();
         return CT_EOF;
     default:
-        x_Err = false;
-        return CT_NOT_EOF(CT_EOF);
+        break;
     }
+    x_Err = false;
+    return CT_NOT_EOF(CT_EOF);
 }
 
 
@@ -285,6 +284,7 @@ streamsize CRWStreambuf::xsputn(const CT_CHAR_TYPE* buf, streamsize m)
 
     if (m <= 0)
         return 0;
+    _ASSERT((Uint8)m < numeric_limits<size_t>::max());
     size_t n = (size_t) m;
 
     ERW_Result result = eRW_Success;
@@ -306,7 +306,7 @@ streamsize CRWStreambuf::xsputn(const CT_CHAR_TYPE* buf, streamsize m)
                     n_written += x_written;
                     n         -= x_written;
                     if ( !n )
-                        return n_written;
+                        return (streamsize) n_written;
                     buf       += x_written;
                 }
             }
@@ -388,7 +388,7 @@ CT_INT_TYPE CRWStreambuf::underflow(void)
         n_read = 0);
     if ( !n_read )
         return CT_EOF;
-    _ASSERT(n_read <= (size_t) m_BufSize);
+    _ASSERT(n_read <= m_BufSize);
 
     // update input buffer with the data just read
     x_GPos += (CT_OFF_TYPE) n_read;
@@ -407,10 +407,11 @@ streamsize CRWStreambuf::xsgetn(CT_CHAR_TYPE* buf, streamsize m)
 
     if (m <= 0)
         return 0;
+    _ASSERT((Uint8)m < numeric_limits<size_t>::max());
     size_t n = (size_t) m;
 
     // first, read from the memory buffer
-    size_t n_read = gptr() ?  egptr() - gptr() : 0;
+    size_t n_read = gptr() ? (size_t)(egptr() - gptr()) : 0;
     if (n_read > n)
         n_read = n;
     memcpy(buf, gptr(), n_read);
@@ -420,8 +421,8 @@ streamsize CRWStreambuf::xsgetn(CT_CHAR_TYPE* buf, streamsize m)
 
     while ( n ) {
         // next, read directly from the device
-        size_t      to_read = n < (size_t) m_BufSize ? m_BufSize : n;
-        CT_CHAR_TYPE* x_buf = n < (size_t) m_BufSize ? m_ReadBuf : buf;
+        size_t      to_read = n < m_BufSize ? m_BufSize : n;
+        CT_CHAR_TYPE* x_buf = n < m_BufSize ? m_ReadBuf : buf;
         ERW_Result   result = eRW_Success;
         size_t       x_read;
 
@@ -442,7 +443,7 @@ streamsize CRWStreambuf::xsgetn(CT_CHAR_TYPE* buf, streamsize m)
             setg(m_ReadBuf, m_ReadBuf + x_read, m_ReadBuf + xx_read);
         } else {
             _ASSERT(x_read <= n);
-            size_t xx_read = x_read > (size_t) m_BufSize ? m_BufSize : x_read;
+            size_t xx_read = x_read > m_BufSize ? m_BufSize : x_read;
             memcpy(m_ReadBuf, buf + x_read - xx_read, xx_read);
             setg(m_ReadBuf, m_ReadBuf + xx_read, m_ReadBuf + xx_read);
         }
@@ -479,8 +480,9 @@ streamsize CRWStreambuf::showmanyc(void)
     case eRW_Success:
         return count;
     default:
-        return -1;
+        break;
     }
+    return -1;
 }
 
 
