@@ -1,4 +1,4 @@
-/*  $Id: test_boost.cpp 352677 2012-02-08 18:44:35Z rafanovi $
+/*  $Id: test_boost.cpp 369798 2012-07-23 21:11:05Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -38,6 +38,7 @@
 #include <corelib/ncbimisc.hpp>
 #include <corelib/ncbiapp.hpp>
 #include <corelib/ncbi_system.hpp>
+#include <corelib/ncbi_safe_static.hpp>
 
 #ifndef BOOST_TEST_NO_LIB
 #  define BOOST_TEST_NO_LIB
@@ -99,6 +100,7 @@ BEGIN_NCBI_SCOPE
 
 const char* kTestsDisableSectionName = "UNITTESTS_DISABLE";
 const char* kTestsToFixSectionName = "UNITTESTS_TOFIX";
+const char* kTestsTimeoutSectionName = "UNITTESTS_TIMEOUT_MULT";
 const char* kTestConfigGlobalValue = "GLOBAL";
 
 #define DUMMY_TEST_FUNCTION_NAME  DummyTestFunction
@@ -378,6 +380,7 @@ class CNcbiTestApplication : public CNcbiApplication
 {
 public:
     CNcbiTestApplication(void);
+    ~CNcbiTestApplication(void);
 
     virtual void Init  (void);
     virtual int  Run   (void);
@@ -820,6 +823,9 @@ CNcbiTestsTreeBuilder::FixUnitsOrder(void)
 }
 
 
+static CNcbiTestApplication* s_TestApp = NULL;
+
+
 inline
 CNcbiTestApplication::CNcbiTestApplication(void)
     : m_RunCalled(false),
@@ -838,13 +844,19 @@ CNcbiTestApplication::CNcbiTestApplication(void)
     SetDiagFilter(eDiagFilter_Post, "!(106.11)");
 }
 
+CNcbiTestApplication::~CNcbiTestApplication(void)
+{
+    if (m_ReportOut.good())
+        but::results_reporter::set_stream(cerr);
+}
+
 /// Application for unit tests
 static CNcbiTestApplication&
 s_GetTestApp(void)
 {
-    static CNcbiTestApplication s_TestApp;
-
-    return s_TestApp;
+    if (!s_TestApp)
+        s_TestApp = new CNcbiTestApplication();
+    return *s_TestApp;
 }
 
 void
@@ -1332,7 +1344,42 @@ CNcbiTestApplication::x_ReadConfiguration(void)
             }
         }
         else {
-            ERR_POST_X(2, Warning << "Invalid test case name: '"
+            ERR_POST_X(4, Warning << "Invalid test case name: '"
+                                  << test_name << "'");
+        }
+    }
+
+    reg_entries.clear();
+    registry.EnumerateEntries(kTestsTimeoutSectionName, &reg_entries);
+    // Adjust timeouts of test units
+    ITERATE(list<string>, it, reg_entries) {
+        const string& test_name = *it;
+        string reg_value = registry.Get(kTestsTimeoutSectionName, test_name);
+
+        but::test_unit* tu = GetTestUnit(test_name);
+        if (tu) {
+            list<CTempString> koef_lst;
+            NStr::Split(reg_value, ";", koef_lst);
+            ITERATE(list<CTempString>, it_koef, koef_lst) {
+                CTempString koef_str, koef_cond;
+                if (NStr::SplitInTwo(*it_koef, ":", koef_str, koef_cond)) {
+                    if (x_CalcConfigValue(koef_cond)) {
+                        double koef = NStr::StringToDouble(koef_str,
+                                                NStr::fAllowLeadingSpaces
+                                                | NStr::fAllowTrailingSpaces);
+                        tu->p_timeout.set(Uint4(tu->p_timeout.get() * koef));
+                        break;
+                    }
+                }
+                else {
+                    ERR_POST_X(6, "Bad format of TIMEOUT_MULT string: '"
+                                  << reg_value << "'");
+                    break;
+                }
+            }
+        }
+        else {
+            ERR_POST_X(5, Warning << "Invalid test case name: '"
                                   << test_name << "'");
         }
     }
@@ -1423,7 +1470,7 @@ CNcbiTestApplication::SetTestErrored(but::test_case* tc)
         m_HasTestErrors = true;
 }
 
-inline void
+void
 CNcbiTestApplication::AdjustTestTimeout(but::test_unit* tu)
 {
     m_CurUnitTimeout = tu->p_timeout.get();
@@ -1444,7 +1491,7 @@ CNcbiTestApplication::AdjustTestTimeout(but::test_unit* tu)
     }
 }
 
-inline string
+string
 CNcbiTestApplication::GetTestResultString(but::test_unit* tu)
 {
     string result;
@@ -2051,5 +2098,7 @@ main(int argc, char* argv[])
 
     results_reporter::make_report();
 
+    delete NCBI_NS_NCBI::s_TestApp;
+    NCBI_NS_NCBI::GetDiagContext().SetExitCode(result);
     return result;
 }
