@@ -1,4 +1,4 @@
-/*  $Id: dbapi_driver_conn_params.cpp 271894 2011-04-04 14:56:25Z rafanovi $
+/*  $Id: dbapi_driver_conn_params.cpp 372544 2012-08-20 14:36:41Z ucko $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -718,11 +718,16 @@ CCPPToolkitConnParams::~CCPPToolkitConnParams(void)
 CDBConnParams::EServerType
 CCPPToolkitConnParams::GetServerType(void) const
 {
-    const string server_name = GetThis().GetServerName();
+    EServerType type = GetServerType(GetThis().GetServerName());
+    return (type == eUnknown) ? CDBConnParamsDelegate::GetServerType() : type;
+}
 
+
+CDBConnParams::EServerType
+CCPPToolkitConnParams::GetServerType(const CTempString& server_name)
+{
     // Artificial intelligence ...
-    if (NStr::CompareNocase(server_name, 0, 3, "MS_") == 0
-        || NStr::CompareNocase(server_name, 0, 13, "DBAPI_MS_TEST") == 0
+    if (NStr::CompareNocase(server_name, 0, 13, "DBAPI_MS_TEST") == 0
         || NStr::CompareNocase(server_name, 0, 5, "MSSQL") == 0
         || NStr::CompareNocase(server_name, 0, 5, "MSDEV") == 0
         || NStr::CompareNocase(server_name, 0, 7, "OAMSDEV") == 0
@@ -732,34 +737,30 @@ CCPPToolkitConnParams::GetServerType(void) const
         || NStr::CompareNocase(server_name, 0, 5, "GPIPE") == 0
         || NStr::CompareNocase(server_name, 0, 7, "MAPVIEW") == 0
         || NStr::CompareNocase(server_name, 0, 5, "MSSNP") == 0
-        || NStr::CompareNocase(server_name, 0, 4, "STRC") == 0
-        || NStr::CompareNocase(server_name, 0, 4, "SUBS") == 0
         )
     {
         return eMSSqlServer;
-    } else if ( NStr::CompareNocase(server_name, "TAPER") == 0
-        || NStr::CompareNocase(server_name, "THALBERG") == 0
+    } else if ( NStr::CompareNocase(server_name, 0, 5, "GLUCK") == 0
         || NStr::CompareNocase(server_name, 0, 8, "SCHUMANN") == 0
-        || NStr::CompareNocase(server_name, 0, 8, "CLEMENTI") == 0
-        || NStr::CompareNocase(server_name, 0, 10, "DBAPI_DEV1") == 0
-        || NStr::CompareNocase(server_name, 0, 6, "BARTOK") == 0
+        || NStr::CompareNocase(server_name, 0, 9, "DBAPI_DEV") == 0
         || NStr::CompareNocase(server_name, 0, 8, "SCHUBERT") == 0
-        || NStr::CompareNocase(server_name, 0, 14, "DBAPI_SYB_TEST") == 0
+        || NStr::CompareNocase(server_name, 0, 9, "DBAPI_SYB") == 0
         )
     {
         return eSybaseSQLServer;
     } else if ( NStr::CompareNocase(server_name, 0, 7, "LINK_OS") == 0
         || NStr::CompareNocase(server_name, 0, 7, "MAIL_OS") == 0
         || NStr::CompareNocase(server_name, 0, 9, "PUBSEQ_OS") == 0
-        || NStr::CompareNocase(server_name, 0, 7, "TEST_OS") == 0
-        || NStr::CompareNocase(server_name, 0, 8, "TRACE_OS") == 0
-        || NStr::CompareNocase(server_name, 0, 7, "TROS_OS") == 0
+        || NStr::CompareNocase(server_name, 0, 6, "IDFLOW") == 0
+        || NStr::CompareNocase(server_name, 0, 6, "IDLOAD") == 0
+        || NStr::CompareNocase(server_name, 0, 6, "IDPROD") == 0
+        || NStr::CompareNocase(server_name, 0, 4, "IDQA") == 0
         )
     {
         return eSybaseOpenServer;
     }
 
-    return CDBConnParamsDelegate::GetServerType();
+    return eUnknown;
 }
 
 
