@@ -1,4 +1,4 @@
-/*  $Id: seq_id_tree.cpp 313248 2011-07-19 17:01:22Z vasilche $
+/*  $Id: seq_id_tree.cpp 371868 2012-08-13 15:10:25Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1965,7 +1965,7 @@ void CSeq_id_General_Tree::FindMatch(const CSeq_id_Handle& id,
     }
     else {
         const string& s = obj_id.GetStr();
-        int n = NStr::StringToNumeric(s);
+        int n = NStr::StringToNonNegativeInt(s);
         if ( n >= 0 && NStr::IntToString(n) == s ) {
             CSeq_id seq_id2;
             CDbtag& dbtag2 = seq_id2.SetGeneral();
