@@ -1,7 +1,7 @@
 #ifndef OBJECTS_OBJMGR___SEQ_ID_TREE__HPP
 #define OBJECTS_OBJMGR___SEQ_ID_TREE__HPP
 
-/*  $Id: seq_id_tree.hpp 176876 2009-11-23 18:13:01Z vasilche $
+/*  $Id: seq_id_tree.hpp 371868 2012-08-13 15:10:25Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -304,9 +304,11 @@ public:
             return !(*this == b);
         }
         bool operator<(const TKey& b) const {
-            return m_Hash < b.m_Hash || m_Hash == b.m_Hash &&
-                (m_Version < b.m_Version || m_Version == b.m_Version &&
-                 NStr::CompareNocase(m_Prefix, b.m_Prefix) < 0);
+            return m_Hash < b.m_Hash ||
+                (m_Hash == b.m_Hash &&
+                 (m_Version < b.m_Version ||
+                  (m_Version == b.m_Version &&
+                   NStr::CompareNocase(m_Prefix, b.m_Prefix) < 0)));
         }
 
         bool SameHash(const TKey& b) const {
@@ -719,6 +721,10 @@ public:
     virtual CSeq_id_Handle FindInfo(const CSeq_id& id) const;
     virtual CSeq_id_Handle FindOrCreate(const CSeq_id& id);
 
+    // Get the list of matching seq-id (int id = str id).
+    virtual bool HaveMatch(const CSeq_id_Handle& id) const;
+    virtual void FindMatch(const CSeq_id_Handle& id,
+                           TSeq_id_MatchList& id_list) const;
     virtual void FindMatchStr(const string& sid,
                               TSeq_id_MatchList& id_list) const;
 
