@@ -1,4 +1,4 @@
-/*  $Id: data_loader.cpp 311373 2011-07-11 19:16:41Z grichenk $
+/*  $Id: data_loader.cpp 372972 2012-08-23 18:45:19Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -125,6 +125,19 @@ CDataLoader::GetRecords(const CSeq_id_Handle& /*idh*/,
 }
 
 
+CDataLoader::TTSE_LockSet
+CDataLoader::GetRecordsNoBlobState(const CSeq_id_Handle& idh,
+                                   EChoice choice)
+{
+    try {
+        return GetRecords(idh, choice);
+    }
+    catch ( CBlobStateException& /* ignored */ ) {
+        return TTSE_LockSet();
+    }
+}
+
+
 CDataLoader::TTSE_LockSet
 CDataLoader::GetDetailedRecords(const CSeq_id_Handle& idh,
                                 const SRequestDetails& details)
@@ -205,7 +218,7 @@ CDataLoader::TBlobId CDataLoader::GetBlobIdFromString(const string& /*str*/) con
 
 void CDataLoader::GetIds(const CSeq_id_Handle& idh, TIds& ids)
 {
-    TTSE_LockSet locks = GetRecords(idh, eBioseqCore);
+    TTSE_LockSet locks = GetRecordsNoBlobState(idh, eBioseqCore);
     ITERATE(TTSE_LockSet, it, locks) {
         CConstRef<CBioseq_Info> bs_info = (*it)->FindMatchingBioseq(idh);
         if ( bs_info ) {
@@ -243,7 +256,7 @@ string CDataLoader::GetLabel(const CSeq_id_Handle& idh)
 int CDataLoader::GetTaxId(const CSeq_id_Handle& idh)
 {
     int ret = -1;
-    TTSE_LockSet locks = GetRecords(idh, eBioseqCore);
+    TTSE_LockSet locks = GetRecordsNoBlobState(idh, eBioseqCore);
     ITERATE(TTSE_LockSet, it, locks) {
         CConstRef<CBioseq_Info> bs_info = (*it)->FindMatchingBioseq(idh);
         if ( bs_info ) {
@@ -255,6 +268,36 @@ int CDataLoader::GetTaxId(const CSeq_id_Handle& idh)
 }
 
 
+TSeqPos CDataLoader::GetSequenceLength(const CSeq_id_Handle& idh)
+{
+    TSeqPos ret = kInvalidSeqPos;
+    TTSE_LockSet locks = GetRecordsNoBlobState(idh, eBioseqCore);
+    ITERATE(TTSE_LockSet, it, locks) {
+        CConstRef<CBioseq_Info> bs_info = (*it)->FindMatchingBioseq(idh);
+        if ( bs_info ) {
+            ret = bs_info->GetBioseqLength();
+            break;
+        }
+    }
+    return ret;
+}
+
+
+CSeq_inst::TMol CDataLoader::GetSequenceType(const CSeq_id_Handle& idh)
+{
+    CSeq_inst::TMol ret = CSeq_inst::eMol_not_set;
+    TTSE_LockSet locks = GetRecordsNoBlobState(idh, eBioseqCore);
+    ITERATE(TTSE_LockSet, it, locks) {
+        CConstRef<CBioseq_Info> bs_info = (*it)->FindMatchingBioseq(idh);
+        if ( bs_info ) {
+            ret = bs_info->GetInst_Mol();
+            break;
+        }
+    }
+    return ret;
+}
+
+
 void CDataLoader::GetAccVers(const TIds& ids, TLoaded& loaded, TIds& ret)
 {
     int count = ids.size();
@@ -266,8 +309,10 @@ void CDataLoader::GetAccVers(const TIds& ids, TLoaded& loaded, TIds& ret)
             continue;
         }
         GetIds(ids[i], seq_ids);
-        ret[i] = CScope::x_GetAccVer(seq_ids);
-        loaded[i] = true;
+        if ( !seq_ids.empty() ) {
+            ret[i] = CScope::x_GetAccVer(seq_ids);
+            loaded[i] = true;
+        }
     }
 }
 
@@ -283,8 +328,10 @@ void CDataLoader::GetGis(const TIds& ids, TLoaded& loaded, TGis& ret)
             continue;
         }
         GetIds(ids[i], seq_ids);
-        ret[i] = CScope::x_GetGi(seq_ids);
-        loaded[i] = true;
+        if ( !seq_ids.empty() ) {
+            ret[i] = CScope::x_GetGi(seq_ids);
+            loaded[i] = true;
+        }
     }
 }
 
@@ -319,17 +366,66 @@ void CDataLoader::GetTaxIds(const TIds& ids, TLoaded& loaded, TTaxIds& ret)
             continue;
         }
         
-        ret[i] = -1;
-        TTSE_LockSet locks = GetRecords(ids[i], eBioseqCore);
+        TTSE_LockSet locks = GetRecordsNoBlobState(ids[i], eBioseqCore);
         ITERATE(TTSE_LockSet, it, locks) {
             CConstRef<CBioseq_Info> bs_info =
                 (*it)->FindMatchingBioseq(ids[i]);
             if ( bs_info ) {
                 ret[i] = bs_info->GetTaxId();
+                loaded[i] = true;
+                break;
+            }
+        }
+    }
+}
+
+
+void CDataLoader::GetSequenceLengths(const TIds& ids, TLoaded& loaded,
+                                     TSequenceLengths& ret)
+{
+    int count = ids.size();
+    _ASSERT(ids.size() == loaded.size());
+    _ASSERT(ids.size() == ret.size());
+    for ( int i = 0; i < count; ++i ) {
+        if ( loaded[i] ) {
+            continue;
+        }
+        
+        TTSE_LockSet locks = GetRecordsNoBlobState(ids[i], eBioseqCore);
+        ITERATE(TTSE_LockSet, it, locks) {
+            CConstRef<CBioseq_Info> bs_info =
+                (*it)->FindMatchingBioseq(ids[i]);
+            if ( bs_info ) {
+                ret[i] = bs_info->GetBioseqLength();
+                loaded[i] = true;
+                break;
+            }
+        }
+    }
+}
+
+
+void CDataLoader::GetSequenceTypes(const TIds& ids, TLoaded& loaded,
+                                   TSequenceTypes& ret)
+{
+    int count = ids.size();
+    _ASSERT(ids.size() == loaded.size());
+    _ASSERT(ids.size() == ret.size());
+    for ( int i = 0; i < count; ++i ) {
+        if ( loaded[i] ) {
+            continue;
+        }
+        
+        TTSE_LockSet locks = GetRecordsNoBlobState(ids[i], eBioseqCore);
+        ITERATE(TTSE_LockSet, it, locks) {
+            CConstRef<CBioseq_Info> bs_info =
+                (*it)->FindMatchingBioseq(ids[i]);
+            if ( bs_info ) {
+                ret[i] = bs_info->GetInst_Mol();
+                loaded[i] = true;
                 break;
             }
         }
-        loaded[i] = true;
     }
 }
 
@@ -432,6 +528,7 @@ SRequestDetails CDataLoader::ChoiceToDetails(EChoice choice) const
         break;
     case eBlob:
     case eBioseq:
+    case eBioseqCore:
         sequence = true;
         // internal only
         details.m_AnnotBlobType = SRequestDetails::fAnnotBlobInternal;
