@@ -1,4 +1,4 @@
-/*  $Id: scope.cpp 352026 2012-02-02 18:21:55Z vasilche $
+/*  $Id: scope.cpp 366487 2012-06-14 16:53:31Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -677,5 +677,75 @@ void CScope::GetTaxIds(TTaxIds* results,
 }
 
 
+TSeqPos CScope::GetSequenceLength(const CSeq_id& id,
+                                  EForceLoad force_load)
+{
+    return GetSequenceLength(CSeq_id_Handle::GetHandle(id), force_load);
+}
+
+
+TSeqPos CScope::GetSequenceLength(const CSeq_id_Handle& id,
+                                  EForceLoad force_load)
+{
+    return m_Impl->GetSequenceLength(id, force_load == eForceLoad);
+}
+
+
+void CScope::GetSequenceLengths(TSequenceLengths* results,
+                                const TSeq_id_Handles& ids,
+                                EForceLoad force_load)
+{
+    if ( !results ) {
+        NCBI_THROW(CCoreException, eNullPtr,
+                   "CScope::GetSequenceLengths: null results pointer");
+    }
+    return m_Impl->GetSequenceLengths(*results, ids, force_load == eForceLoad);
+}
+
+
+CScope::TSequenceLengths CScope::GetSequenceLengths(const TSeq_id_Handles& ids,
+                                                    EForceLoad force_load)
+{
+    TSequenceLengths results;
+    GetSequenceLengths(&results, ids, force_load);
+    return results;
+}
+
+
+CSeq_inst::TMol CScope::GetSequenceType(const CSeq_id& id,
+                                        EForceLoad force_load)
+{
+    return GetSequenceType(CSeq_id_Handle::GetHandle(id), force_load);
+}
+
+
+CSeq_inst::TMol CScope::GetSequenceType(const CSeq_id_Handle& id,
+                                        EForceLoad force_load)
+{
+    return m_Impl->GetSequenceType(id, force_load == eForceLoad);
+}
+
+
+void CScope::GetSequenceTypes(TSequenceTypes* results,
+                              const TSeq_id_Handles& ids,
+                              EForceLoad force_load)
+{
+    if ( !results ) {
+        NCBI_THROW(CCoreException, eNullPtr,
+                   "CScope::GetSequenceTypes: null results pointer");
+    }
+    return m_Impl->GetSequenceTypes(*results, ids, force_load == eForceLoad);
+}
+
+
+CScope::TSequenceTypes CScope::GetSequenceTypes(const TSeq_id_Handles& ids,
+                                                EForceLoad force_load)
+{
+    TSequenceTypes results;
+    GetSequenceTypes(&results, ids, force_load);
+    return results;
+}
+
+
 END_SCOPE(objects)
 END_NCBI_SCOPE
