@@ -1,4 +1,4 @@
-/*  $Id: scope_impl.cpp 352026 2012-02-02 18:21:55Z vasilche $
+/*  $Id: scope_impl.cpp 371183 2012-08-06 17:24:48Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -2500,6 +2500,7 @@ int CScope_Impl::GetTaxId(const CSeq_id_Handle& idh, bool force_load)
             x_FindBioseq_Info(idh, CScope::eGetBioseq_Resolved, match);
         if ( info ) {
             if ( info->HasBioseq() ) {
+                TBioseq_Lock bioseq = info->GetLock(null);
                 ret = info->GetObjectInfo().GetTaxId();
             }
         }
@@ -2788,7 +2789,7 @@ void CScope_Impl::GetGis(TGIs& ret,
                          bool force_load)
 {
     int count = ids.size(), remaining = count;
-    ret.assign(count, -1);
+    ret.assign(count, 0);
     vector<bool> loaded(count);
     if ( !force_load ) {
         for ( int i = 0; i < count; ++i ) {
@@ -2900,7 +2901,7 @@ void CScope_Impl::GetTaxIds(TTaxIds& ret,
                             bool force_load)
 {
     int count = ids.size(), remaining = count;
-    ret.assign(count, 0);
+    ret.assign(count, -1);
     vector<bool> loaded(count);
     if ( !force_load ) {
         for ( int i = 0; i < count; ++i ) {
@@ -2931,6 +2932,7 @@ void CScope_Impl::GetTaxIds(TTaxIds& ret,
                                       match);
                 if ( info ) {
                     if ( info->HasBioseq() ) {
+                        TBioseq_Lock bioseq = info->GetLock(null);
                         ret[i] = info->GetObjectInfo().GetTaxId();
                         loaded[i] = true;
                         --remaining;
@@ -2954,7 +2956,161 @@ void CScope_Impl::GetTaxIds(TTaxIds& ret,
         }
     }
 }
+
+
+TSeqPos CScope_Impl::GetSequenceLength(const CSeq_id_Handle& idh,
+                                       bool force_load)
+{
+    TConfReadLockGuard rguard(m_ConfLock);
+
+    if ( !force_load ) {
+        SSeqMatch_Scope match;
+        CRef<CBioseq_ScopeInfo> info =
+            x_FindBioseq_Info(idh, CScope::eGetBioseq_Resolved, match);
+        if ( info ) {
+            if ( info->HasBioseq() ) {
+                TBioseq_Lock bioseq = info->GetLock(null);
+                return info->GetObjectInfo().GetBioseqLength();
+            }
+            return kInvalidSeqPos;
+        }
+    }
+    
+    // Unknown bioseq, try to find in data sources
+    for (CPriority_I it(m_setDataSrc); it; ++it) {
+        CPrefetchManager::IsActive();
+        TSeqPos length = it->GetDataSource().GetSequenceLength(idh);
+        if ( length != kInvalidSeqPos ) {
+            return length;
+        }
+    }
+    return kInvalidSeqPos;
+}
                                   
 
+CSeq_inst::TMol CScope_Impl::GetSequenceType(const CSeq_id_Handle& idh,
+                                             bool force_load)
+{
+    TConfReadLockGuard rguard(m_ConfLock);
+
+    if ( !force_load ) {
+        SSeqMatch_Scope match;
+        CRef<CBioseq_ScopeInfo> info =
+            x_FindBioseq_Info(idh, CScope::eGetBioseq_Resolved, match);
+        if ( info ) {
+            if ( info->HasBioseq() ) {
+                TBioseq_Lock bioseq = info->GetLock(null);
+                return info->GetObjectInfo().GetInst_Mol();
+            }
+            return CSeq_inst::eMol_not_set;
+        }
+    }
+    
+    // Unknown bioseq, try to find in data sources
+    for (CPriority_I it(m_setDataSrc); it; ++it) {
+        CPrefetchManager::IsActive();
+        CSeq_inst::TMol type = it->GetDataSource().GetSequenceType(idh);
+        if ( type != CSeq_inst::eMol_not_set ) {
+            return type;
+        }
+    }
+    return CSeq_inst::eMol_not_set;
+}
+                                  
+
+void CScope_Impl::GetSequenceLengths(TSequenceLengths& ret,
+                                     const TIds& ids,
+                                     bool force_load)
+{
+    int count = ids.size(), remaining = count;
+    ret.assign(count, kInvalidSeqPos);
+    vector<bool> loaded(count);
+    
+    TConfReadLockGuard rguard(m_ConfLock);
+    
+    if ( !force_load ) {
+        for ( int i = 0; i < count; ++i ) {
+            if ( loaded[i] ) {
+                continue;
+            }
+            SSeqMatch_Scope match;
+            CRef<CBioseq_ScopeInfo> info =
+                x_FindBioseq_Info(ids[i],
+                                  CScope::eGetBioseq_Resolved,
+                                  match);
+            if ( info ) {
+                if ( info->HasBioseq() ) {
+                    TBioseq_Lock bioseq = info->GetLock(null);
+                    ret[i] = info->GetObjectInfo().GetBioseqLength();
+                    loaded[i] = true;
+                    --remaining;
+                }
+            }
+        }
+    }
+    
+    // Unknown bioseq, try to find in data sources
+    for (CPriority_I it(m_setDataSrc); it; ++it) {
+        if ( !remaining ) {
+            break;
+        }
+        CPrefetchManager::IsActive();
+        it->GetDataSource().GetSequenceLengths(ids, loaded, ret);
+#ifdef NCBI_COMPILER_WORKSHOP
+        std::count(loaded.begin(), loaded.end(), false, remaining);
+#else
+        remaining = std::count(loaded.begin(), loaded.end(), false);
+#endif
+    }
+}
+
+
+void CScope_Impl::GetSequenceTypes(TSequenceTypes& ret,
+                                   const TIds& ids,
+                                   bool force_load)
+{
+    int count = ids.size(), remaining = count;
+    ret.assign(count, CSeq_inst::eMol_not_set);
+    vector<bool> loaded(count);
+    
+    TConfReadLockGuard rguard(m_ConfLock);
+    
+    if ( !force_load ) {
+        for ( int i = 0; i < count; ++i ) {
+            if ( loaded[i] ) {
+                continue;
+            }
+            SSeqMatch_Scope match;
+            CRef<CBioseq_ScopeInfo> info =
+                x_FindBioseq_Info(ids[i],
+                                  CScope::eGetBioseq_Resolved,
+                                  match);
+            if ( info ) {
+                if ( info->HasBioseq() ) {
+                    TBioseq_Lock bioseq = info->GetLock(null);
+                    ret[i] = info->GetObjectInfo().GetInst_Mol();
+                    loaded[i] = true;
+                    --remaining;
+                }
+            }
+        }
+    }
+    
+    // Unknown bioseq, try to find in data sources
+    for (CPriority_I it(m_setDataSrc); it; ++it) {
+        if ( !remaining ) {
+            break;
+        }
+        CPrefetchManager::IsActive();
+        it->GetDataSource().GetSequenceTypes(ids, loaded, ret);
+#ifdef NCBI_COMPILER_WORKSHOP
+        std::count(loaded.begin(), loaded.end(), false, remaining);
+#else
+        remaining = std::count(loaded.begin(), loaded.end(), false);
+#endif
+    }
+}
+
+
 END_SCOPE(objects)
 END_NCBI_SCOPE
