@@ -1,4 +1,4 @@
-/*  $Id: seq_vector.cpp 205662 2010-09-21 13:44:24Z vasilche $
+/*  $Id: seq_vector.cpp 309636 2011-06-27 14:44:39Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -988,8 +988,8 @@ void CSeqVector::x_GetPacked2naSeqData(string& dst_str,
                     dataPos = seg.GetRefPosition() +
                         (src_pos - seg.GetPosition());
                 }
-                _ASSERT(!randomizer && dst_coding == CSeq_data::e_Ncbi2na ||
-                        randomizer && dst_coding == CSeq_data::e_Ncbi4na);
+                _ASSERT((!randomizer && dst_coding == CSeq_data::e_Ncbi2na) ||
+                        (randomizer && dst_coding == CSeq_data::e_Ncbi4na));
                 x_AppendAnyTo2(dst_str, dst_c, dst_pos,
                                data, dataPos, count, table, reverse,
                                randomizer, src_pos);
