@@ -1,4 +1,4 @@
-/*  $Id: tse_info.cpp 347580 2011-12-19 20:03:34Z vasilche $
+/*  $Id: tse_info.cpp 370666 2012-07-31 20:32:41Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -44,6 +44,7 @@
 #include <objmgr/impl/snp_annot_info.hpp>
 #include <objmgr/impl/annot_type_index.hpp>
 #include <objmgr/impl/handle_range.hpp>
+#include <objmgr/impl/handle_range_map.hpp>
 
 #include <objects/seqset/Seq_entry.hpp>
 
@@ -122,7 +123,8 @@ SIdAnnotObjs::SIdAnnotObjs(const SIdAnnotObjs& _DEBUG_ARG(objs))
 
 
 CTSE_Info::CTSE_Info(void) 
-    : m_InternalBioObjNumber(0)
+    : m_InternalBioObjNumber(0),
+      m_MasterSeqSegmentsLoaded(false)
 {
     x_Initialize();
 
@@ -132,8 +134,8 @@ CTSE_Info::CTSE_Info(void)
 
 CTSE_Info::CTSE_Info(const TBlobId& blob_id,
                      TBlobVersion blob_version)
-    : m_InternalBioObjNumber(0)
-
+    : m_InternalBioObjNumber(0),
+      m_MasterSeqSegmentsLoaded(false)
 {
     x_Initialize();
 
@@ -148,8 +150,8 @@ CTSE_Info::CTSE_Info(CSeq_entry& entry,
                      TBlobState blob_state,
                      const TBlobId& blob_id,
                      TBlobVersion blob_version)
-    : m_InternalBioObjNumber(0)
-
+    : m_InternalBioObjNumber(0),
+      m_MasterSeqSegmentsLoaded(false)
 {
     x_Initialize();
 
@@ -166,8 +168,8 @@ CTSE_Info::CTSE_Info(CSeq_entry& entry,
 
 CTSE_Info::CTSE_Info(CSeq_entry& entry,
                      TBlobState blob_state)
-    : m_InternalBioObjNumber(0)
-
+    : m_InternalBioObjNumber(0),
+      m_MasterSeqSegmentsLoaded(false)
 {
     x_Initialize();
 
@@ -182,8 +184,8 @@ CTSE_Info::CTSE_Info(CSeq_entry& entry,
 
 CTSE_Info::CTSE_Info(const CTSE_Lock& tse)
     : m_BaseTSE(new SBaseTSE(tse)),
-      m_InternalBioObjNumber(0)
-
+      m_InternalBioObjNumber(0),
+      m_MasterSeqSegmentsLoaded(false)
 {
     x_Initialize();
 
@@ -741,6 +743,27 @@ CTSE_Info::GetSegSetMaster(void) const
 }
 
 
+CConstRef<CMasterSeqSegments> CTSE_Info::GetMasterSeqSegments(void) const
+{
+    if ( !m_MasterSeqSegmentsLoaded ) {
+        TAnnotLockWriteGuard guard(m_AnnotLock);
+        if ( !m_MasterSeqSegmentsLoaded ) {
+            CConstRef<CBioseq_Info> master_seq = GetSegSetMaster();
+            if ( master_seq ) {
+                try {
+                    m_MasterSeqSegments = new CMasterSeqSegments(*master_seq);
+                }
+                catch ( CException& exc ) {
+                    ERR_POST("Segment set cannot be initialized: "<<exc);
+                }
+            }
+            m_MasterSeqSegmentsLoaded = true;
+        }
+    }
+    return m_MasterSeqSegments;
+}
+
+
 SSeqMatch_TSE CTSE_Info::GetSeqMatch(const CSeq_id_Handle& id) const
 {
     SSeqMatch_TSE ret;
