@@ -1,4 +1,4 @@
-/*  $Id: tse_split_info.cpp 340880 2011-10-13 16:35:46Z vasilche $
+/*  $Id: tse_split_info.cpp 372853 2012-08-22 19:51:04Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -310,7 +310,6 @@ void CTSE_Split_Info::x_UpdateAnnotIndex(void)
 
 void CTSE_Split_Info::x_UpdateAnnotIndex(CTSE_Chunk_Info& chunk)
 {
-    CMutexGuard guard(m_SeqIdToChunksMutex);
     if ( !chunk.m_AnnotIndexEnabled ) {
         NON_CONST_ITERATE ( TTSE_Set, it, m_TSE_Set ) {
             CTSE_Info& tse = *it->first;
@@ -338,10 +337,19 @@ CTSE_Split_Info::x_FindChunk(const CSeq_id_Handle& id) const
 // load requests
 void CTSE_Split_Info::x_GetRecords(const CSeq_id_Handle& id, bool bioseq) const
 {
-    CMutexGuard guard(m_SeqIdToChunksMutex);
-    for ( TSeqIdToChunks::const_iterator iter = x_FindChunk(id);
-          iter != m_SeqIdToChunks.end() && iter->first == id; ++iter ) {
-        GetChunk(iter->second).x_GetRecords(id, bioseq);
+    vector< CConstRef<CTSE_Chunk_Info> > chunks;
+    {{
+        CMutexGuard guard(m_SeqIdToChunksMutex);
+        for ( TSeqIdToChunks::const_iterator iter = x_FindChunk(id);
+              iter != m_SeqIdToChunks.end() && iter->first == id; ++iter ) {
+            const CTSE_Chunk_Info& chunk = GetChunk(iter->second);
+            if ( !chunk.IsLoaded() ) {
+                chunks.push_back(ConstRef(&chunk));
+            }
+        }
+    }}
+    ITERATE ( vector< CConstRef<CTSE_Chunk_Info> >, it, chunks ) {
+        (*it)->x_GetRecords(id, bioseq);
     }
 }
 
