@@ -173,6 +173,7 @@ void CDeflineGenerator::x_SetFlags (
     m_IsPDB = false;
     m_ThirdParty = false;
     m_WGSMaster = false;
+    m_TSAMaster = false;
 
     m_GeneralStr.clear();
     m_PatentCountry.clear();
@@ -225,11 +226,11 @@ void CDeflineGenerator::x_SetFlags (
     m_IsAA = bsh.IsAa();
 
     if (bsh.IsSetInst()) {
-        const CSeq_inst& inst = bsh.GetInst();
-        if (inst.IsSetRepr()) {
-            TSEQ_REPR repr = inst.GetRepr();
+        if (bsh.IsSetInst_Repr()) {
+            TSEQ_REPR repr = bsh.GetInst_Repr();
             m_IsSeg = (repr == CSeq_inst::eRepr_seg);
             m_IsDelta = (repr == CSeq_inst::eRepr_delta);
+            m_IsVirtual = (repr == CSeq_inst::eRepr_virtual);
         }
     }
 
@@ -252,6 +253,11 @@ void CDeflineGenerator::x_SetFlags (
                         if( (type & CSeq_id::fAcc_master) != 0 ) {
                             m_WGSMaster = true;
                         }
+                    } else if ( (type & NCBI_ACCN(division_mask)) == NCBI_ACCN(tsa) ) 
+                    {
+                        if( (type & CSeq_id::fAcc_master) != 0 && m_IsVirtual ) {
+                            m_TSAMaster = true;
+                        }
                     } else if ((type & NCBI_ACCN(division_mask)) == NCBI_ACCN(refseq_chromosome)) {
                         m_IsNC = true;
                     } else if ((type & NCBI_ACCN(division_mask)) == NCBI_ACCN(refseq_mrna)) {
@@ -598,8 +604,9 @@ string CDeflineGenerator::x_TitleFromBioSrc (void)
     string chr, cln, mp, pls, stn, sfx;
 
     if (! m_Strain.empty()) {
-        if (! x_EndsWithStrain (m_Taxname, m_Strain)) {
-            stn = " strain " + m_Strain.substr (0, m_Strain.find(';'));
+        string add = m_Strain.substr (0, m_Strain.find(';'));
+        if (! x_EndsWithStrain (m_Taxname, add)) {
+            stn = " strain " + add;
         }
     }
     if (! m_Chromosome.empty()) {
@@ -1013,9 +1020,8 @@ CConstRef<CSeq_feat> CDeflineGenerator::x_GetLongestProtein (
     CScope& scope = bsh.GetScope();
 
     if (bsh.IsSetInst ()) {
-        const CSeq_inst& inst = bsh.GetInst ();
-        if (inst.IsSetLength ()) {
-            seq_len = inst.GetLength ();
+        if (bsh.IsSetInst_Length ()) {
+            seq_len = bsh.GetInst_Length ();
         }
     }
 
@@ -1300,7 +1306,8 @@ string CDeflineGenerator::x_TitleFromProtein (
             }
             */
 
-            if (NStr::CompareNocase (result, "hypothetical protein") == 0) {
+            if (NStr::CompareNocase (result, "hypothetical protein") == 0 ||
+                NStr::CompareNocase (result, "uncharacterized protein") == 0) {
                 gene = x_GetGeneRefViaCDS (mapped_cds);
                 if (gene) {
                     const CGene_ref& grp = *gene;
@@ -1671,6 +1678,20 @@ string CDeflineGenerator::x_SetSuffix (
                 suffix += ", whole genome shotgun sequence";
             }
             break;
+        case NCBI_TECH(tsa):
+            if (m_MIBiomol == NCBI_BIOMOL(mRNA)) {
+                if (m_TSAMaster) {
+                    if (title.find ("whole genome shotgun sequencing project")
+                        == NPOS){
+                        suffix = ", whole genome shotgun sequencing project";
+                    }            
+                } else {
+                    if (title.find ("mRNA sequence") == NPOS){
+                        suffix = ", mRNA sequence";
+                    }            
+                }
+            }
+            break;
         default:
             break;
     }
