@@ -1,4 +1,4 @@
-/*  $Id: feature.cpp 352026 2012-02-02 18:21:55Z vasilche $
+/*  $Id: feature.cpp 358361 2012-04-02 13:05:23Z dicuccio $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -163,18 +163,18 @@ static void s_GetCdregionLabel
             const CSeq_id& id = GetId(feat.GetProduct(), scope);            
             CBioseq_Handle hnd = scope->GetBioseqHandle(id);
             if (hnd) {
-                const CBioseq& seq = *hnd.GetCompleteBioseq();
-            
-                // Now look for a CProt_ref feature in seq and
-                // if found call GetLabel() on the CProt_ref
-                CTypeConstIterator<CSeqFeatData> it = ConstBegin(seq);
-                for (;it; ++it) {
-                    if (it->IsProt()) {
-                        it->GetProt().GetLabel(tlabel);
-                        return;
-                    }
+
+                for (CFeat_CI feat_it(hnd,
+                                      SAnnotSelector()
+                                      .IncludeFeatType(CSeqFeatData::e_Prot));
+                     feat_it;  ++feat_it) {
+                    feat_it->GetData().GetProt().GetLabel(tlabel);
+                    return;
                 }
             }
+            else {
+                ERR_POST(Error << "cannot find sequence: " + id.AsFastaString());
+            }
         } catch (CObjmgrUtilException&) {}
     }
     
@@ -1252,9 +1252,10 @@ namespace {
     }
 
     
-    static const int kSameTypeParentQuality  = 1000;
+    static const int kBetterTypeParentQuality= 1000;
     static const int kByLocusParentQuality   =  750;
-    static const int kWorseTypeParentQuality =  500;
+    static const int kSameTypeParentQuality  =  500;
+    static const int kWorseTypeParentQuality =  kSameTypeParentQuality;
 
     static
     int sx_GetParentTypeQuality(CSeqFeatData::ESubtype parent,
@@ -1262,10 +1263,10 @@ namespace {
     {
         int d_child = sx_GetRootDistance(child);
         int d_parent = sx_GetRootDistance(parent);
-        if ( d_parent <= d_child ) {
+        if ( d_parent < d_child ) {
             // parent candidate is higher than child
-            // return value <= kSameTypeParentQuality
-            return kSameTypeParentQuality - (d_child - d_parent);
+            // return value <= kBetterTypeParentQuality
+            return kBetterTypeParentQuality - (d_child - d_parent);
         }
         else {
             // parent candidate is not higher than child
