@@ -1,4 +1,4 @@
-/* $Id: tabular.cpp 347537 2011-12-19 16:45:43Z maning $
+/* $Id: tabular.cpp 367113 2012-06-20 19:26:08Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 /// One line is printed for each alignment with tab-delimited fielalnVec. 
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: tabular.cpp 347537 2011-12-19 16:45:43Z maning $";
+static char const rcsid[] = "$Id: tabular.cpp 367113 2012-06-20 19:26:08Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -130,6 +130,7 @@ CBlastTabularInfo::CBlastTabularInfo(CNcbiOstream& ostr, const string& format,
     x_ResetFields();
     x_SetFieldDelimiter(delim);
     SetParseLocalIds(parse_local_ids);
+    SetNoFetch(false);
 }
 
 CBlastTabularInfo::~CBlastTabularInfo()
@@ -333,7 +334,8 @@ int CBlastTabularInfo::SetFields(const CSeq_align& align,
     const int kSubjectRow = 1;
 
     int num_ident = -1;
-    
+    const bool kNoFetchSequence = GetNoFetch();
+
     // First reset all fields.
     x_ResetFields();
 
@@ -473,6 +475,7 @@ int CBlastTabularInfo::SetFields(const CSeq_align& align,
         alnVec.Reset(new CAlnVec(ds, scope));
     }    
 
+    alnVec->SetAaCoding(CSeq_data::e_Ncbieaa);
 
     int align_length = 0, num_gaps = 0, num_gap_opens = 0;
     if (x_IsFieldRequested(eAlignmentLength) ||
@@ -492,9 +495,9 @@ int CBlastTabularInfo::SetFields(const CSeq_align& align,
         x_IsFieldRequested(ePositives) ||
         x_IsFieldRequested(ePercentPositives) ||
         x_IsFieldRequested(eBTOP) ||
-        x_IsFieldRequested(eNumIdentical) ||
-        x_IsFieldRequested(eMismatches) ||
-        x_IsFieldRequested(ePercentIdentical)) {
+        (x_IsFieldRequested(eNumIdentical) && !kNoFetchSequence) ||
+        (x_IsFieldRequested(eMismatches) && !kNoFetchSequence) ||
+        (x_IsFieldRequested(ePercentIdentical) && !kNoFetchSequence)) {
 
         alnVec->SetGapChar('-');
         alnVec->GetWholeAlnSeqString(0, m_QuerySeq);
@@ -917,11 +920,11 @@ void
 CIgBlastTabularInfo::PrintHeader(const string& program_version, 
        const CBioseq& bioseq, 
        const string& dbname, 
+       const string& domain_sys,
        const string& rid /* = kEmptyStr */,
        unsigned int iteration /* = numeric_limits<unsigned int>::max() */,
        const CSeq_align_set* align_set /* = 0 */,
-       CConstRef<CBioseq> subj_bioseq /* = CConstRef<CBioseq>() */,
-       const string& domain_sys)
+       CConstRef<CBioseq> subj_bioseq /* = CConstRef<CBioseq>() */)
 {
     x_PrintQueryAndDbNames(program_version, bioseq, dbname, rid, iteration, subj_bioseq);
     m_Ostream << "# Domain classification requested: " << domain_sys << endl;
@@ -1248,8 +1251,8 @@ void CIgBlastTabularInfo::x_PrintIgGenes(bool isHtml, const string& header) cons
         m_Ostream << header << "V(D)J junction details (V end, ";
         if (m_ChainType == "VH")  m_Ostream << "V-D junction, D region, D-J junction, ";
         else m_Ostream << "V-J junction, ";
-        m_Ostream << "J start.  Note that overlapping nucleotides at VDJ junction, if any, are shown in parentheses (i.e., (TACT)) and"
-                  << " are not included under V, D, or J region itself)" << endl;
+        m_Ostream << "J start).  Note that possible overlapping nucleotides at VDJ junction (i.e, nucleotides that could be assigned to either joining gene segment) are indicated in parentheses (i.e., (TACT)) but"
+                  << " are not included under V, D, or J gene itself" << endl;
     }
 
     x_PrintPartialQuery(max(b0, a1 - 5), a1, isHtml); m_Ostream << m_FieldDelimiter;
@@ -1266,11 +1269,9 @@ void CIgBlastTabularInfo::x_PrintIgGenes(bool isHtml, const string& header) cons
         m_Ostream << "</tr>\n</table></pre>";
 
         m_Ostream << "*: Overlapping nucleotides may exist"
-                  << " at some V-D-J junction (i.e, nucleotides"
-                  << " that could be contributed by either joining segment "
-                  << " due to homology-directed rearrangement). "
-                  << "Such bases will be shown inside a parenthesis (i.e., (TACAT))"
-                  << " and will not be included under V, D or J region itself.\n";
+                  << " at V-D-J junction (i.e, nucleotides that could be assigned to either joining gene segment).\n"
+                  << " Such bases are indicated inside a parenthesis (i.e., (TACAT))"
+                  << " but are not included under V, D or J gene itself.\n";
     }
     m_Ostream << endl << endl;
 };
