@@ -1,4 +1,4 @@
-/*  $Id: aln_printer_unit_test.cpp 347538 2011-12-19 16:46:07Z boratyng $
+/*  $Id: aln_printer_unit_test.cpp 367110 2012-06-20 19:20:23Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -204,7 +204,7 @@ BOOST_AUTO_TEST_CASE(TestPhylipInterleaved)
                               "----------------------------------") != NPOS);
 
 
-    BOOST_REQUIRE(output.find("unnamed_p --------------------------------MPQWA"
+    BOOST_REQUIRE(output.find("serine__o --------------------------------MPQWA"
                               "NPVPAIA--GAAPVVITSARAAISAGVDEA---G") != NPOS);
 
     // last line
