@@ -1,4 +1,4 @@
-/*  $Id: aln_builders.cpp 348472 2011-12-29 16:52:24Z grichenk $
+/*  $Id: aln_builders.cpp 369883 2012-07-24 17:34:06Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -54,13 +54,6 @@ MergePairwiseAlns(CPairwiseAln& existing,
     SubtractAlnRngCollections(addition, // minuend
                               existing, // subtrahend
                               difference);
-    int gap_flags = CPairwiseAln::TAlnRngColl::fAllowAbutting |
-        CPairwiseAln::TAlnRngColl::fAllowMixedDir |
-        CPairwiseAln::TAlnRngColl::fAllowOverlap;
-    CPairwiseAln::TAlnRngColl gaps_coll(addition.GetInsertions(),
-        gap_flags);
-    CPairwiseAln::TAlnRngColl gaps_truncated(gap_flags);
-    SubtractAlnRngCollections(gaps_coll, existing, gaps_truncated);
 #ifdef _TRACE_MergeAlnRngColl
     cerr << endl;
     cerr << "existing:" << endl << existing << endl;
@@ -70,7 +63,16 @@ MergePairwiseAlns(CPairwiseAln& existing,
     ITERATE(CPairwiseAln, rng_it, difference) {
         existing.insert(*rng_it);
     }
-    existing.AddInsertions(gaps_truncated);
+    if ((flags & CAlnUserOptions::fIgnoreInsertions) == 0) {
+        int gap_flags = CPairwiseAln::TAlnRngColl::fAllowAbutting |
+            CPairwiseAln::TAlnRngColl::fAllowMixedDir |
+            CPairwiseAln::TAlnRngColl::fAllowOverlap;
+        CPairwiseAln::TAlnRngColl gaps_coll(addition.GetInsertions(),
+            gap_flags);
+        CPairwiseAln::TAlnRngColl gaps_truncated(gap_flags);
+        SubtractAlnRngCollections(gaps_coll, existing, gaps_truncated);
+        existing.AddInsertions(gaps_truncated);
+    }
 #ifdef _TRACE_MergeAlnRngColl
     cerr << "result = existing + difference:" << endl << existing << endl;
 #endif
@@ -125,13 +127,6 @@ private:
             SubtractAlnRngCollections(*addition,  // minuend
                                       existing,   // subtrahend
                                       *truncated);// difference
-            int gap_flags = CPairwiseAln::TAlnRngColl::fAllowAbutting |
-                CPairwiseAln::TAlnRngColl::fAllowMixedDir |
-                CPairwiseAln::TAlnRngColl::fAllowOverlap;
-            CPairwiseAln::TAlnRngColl gaps_coll(addition->GetInsertions(),
-                gap_flags);
-            CPairwiseAln::TAlnRngColl gaps_truncated(gap_flags);
-            SubtractAlnRngCollections(gaps_coll, existing, gaps_truncated);
 
 #ifdef _TRACE_MergeAlnRngColl
             cerr << endl;
@@ -139,8 +134,21 @@ private:
             cerr << "addition:" << endl << *addition << endl;
             cerr << "truncated = addition - existing:" << endl << *truncated << endl;
 #endif
-            addition = truncated;
-            addition->AddInsertions(gaps_truncated);
+
+            if ((m_MergeFlags & CAlnUserOptions::fIgnoreInsertions) == 0) {
+                int gap_flags = CPairwiseAln::TAlnRngColl::fAllowAbutting |
+                    CPairwiseAln::TAlnRngColl::fAllowMixedDir |
+                    CPairwiseAln::TAlnRngColl::fAllowOverlap;
+                CPairwiseAln::TAlnRngColl gaps_coll(addition->GetInsertions(),
+                    gap_flags);
+                CPairwiseAln::TAlnRngColl gaps_truncated(gap_flags);
+                SubtractAlnRngCollections(gaps_coll, existing, gaps_truncated);
+                addition = truncated;
+                addition->AddInsertions(gaps_truncated);
+            }
+            else {
+                addition = truncated;
+            }
         }
     }
 
@@ -263,11 +271,13 @@ private:
             if (next_rng_it != addition.end()) {
                 next_rng_pos = next_rng_it->GetFirstFrom();
             }
-            // Add all gaps up to the next non-gap range
-            while (gap_it != gaps.end()  &&
-                (gap_it->GetFirstFrom() <= next_rng_pos  ||  next_rng_pos < 0)) {
-                (*aln_it)->AddInsertion(*gap_it);
-                gap_it++;
+            if ((m_MergeFlags & CAlnUserOptions::fIgnoreInsertions) == 0) {
+                // Add all gaps up to the next non-gap range
+                while (gap_it != gaps.end()  &&
+                    (gap_it->GetFirstFrom() <= next_rng_pos  ||  next_rng_pos < 0)) {
+                    (*aln_it)->AddInsertion(*gap_it);
+                    gap_it++;
+                }
             }
         }
     }
@@ -305,7 +315,6 @@ void BuildAln(const TMergedVec& merged_vec,
 
     // Resize the container
     out_aln.SetPairwiseAlns().resize(total_number_of_rows);
-    
 
     // Copy pairwises
     TDim row = 0;
@@ -331,12 +340,20 @@ void
 s_TranslateAnchorToAlnCoords(CPairwiseAln& out_anchor_pw, ///< output must be empty
                              const CPairwiseAln& anchor_pw)
 {
+    if ( anchor_pw.empty() ) return;
     CPairwiseAln::TPos aln_pos = 0; /// Start at 0
+    CPairwiseAln::TPos aln_len = anchor_pw.GetFirstLength();
+    bool direct = anchor_pw.begin()->IsFirstDirect();
+
     // There should be no gaps on anchor
     _ASSERT(anchor_pw.GetInsertions().empty());
     ITERATE (CPairwiseAln::TAlnRngColl, it, anchor_pw) {
         CPairwiseAln::TAlnRng ar = *it;
-        ar.SetFirstFrom(aln_pos);
+        ar.SetFirstFrom(direct ? aln_pos : aln_len - aln_pos - ar.GetLength());
+        if ( !direct ) {
+            ar.SetDirect(!ar.IsDirect());
+            ar.SetFirstDirect(true);
+        }
         out_anchor_pw.insert(ar);
         aln_pos += ar.GetLength();
     }
@@ -344,19 +361,23 @@ s_TranslateAnchorToAlnCoords(CPairwiseAln& out_anchor_pw, ///< output must be em
 
 
 void
-s_TranslatePairwiseToAlnCoords(CPairwiseAln& out_pw,   ///< output pairwise (needs to be empty)
-                               const CPairwiseAln& pw, ///< input pairwise to translate from
-                               const CPairwiseAln& tr) ///< translating (aln segments) pairwise
+s_TranslatePairwiseToAlnCoords(CPairwiseAln& out_pw,   // output pairwise (needs to be empty)
+                               const CPairwiseAln& pw, // input pairwise to translate from
+                               const CPairwiseAln& tr, // translating (aln segments) pairwise
+                               bool direct) // new anchor has the same direction as the original one?
 {
     // Shift between the old anchor and the alignment.
-    CPairwiseAln::TPos anchor_shift = 0;
     const CPairwiseAln::TAlignRangeVector& gaps = pw.GetInsertions();
     CPairwiseAln::TAlignRangeVector::const_iterator gap_it = gaps.begin();
     ITERATE (CPairwiseAln, it, pw) {
         CPairwiseAln::TAlnRng ar = *it;
-        CPairwiseAln::TPos pos = tr.GetFirstPosBySecondPos(ar.GetFirstFrom());
-        anchor_shift = ar.GetFirstFrom() - pos;
+        CPairwiseAln::TPos pos =
+            tr.GetFirstPosBySecondPos(direct ? ar.GetFirstFrom() : ar.GetFirstTo());
         ar.SetFirstFrom(pos);
+        if ( !direct ) {
+            ar.SetDirect(!ar.IsDirect());
+            ar.SetFirstDirect(true);
+        }
         out_pw.insert(ar);
         if (gap_it != gaps.end()) {
             CPairwiseAln::const_iterator next_it = it;
@@ -368,7 +389,13 @@ s_TranslatePairwiseToAlnCoords(CPairwiseAln& out_pw,   ///< output pairwise (nee
                     // Need to specify direction since the source point is out of
                     // anchor ranges and will produce -1.
                     CPairwiseAln::TPos new_gap_pos =
-                        tr.GetFirstPosBySecondPos(gap_rg.GetFirstFrom(), CPairwiseAln::eForward);
+                        tr.GetFirstPosBySecondPos(gap_rg.GetFirstFrom(),
+                        direct ? CPairwiseAln::eForward : CPairwiseAln::eBackwards);
+                    if ( !direct ) {
+                        ++new_gap_pos;
+                        gap_rg.SetDirect(!gap_rg.IsDirect());
+                        gap_rg.SetFirstDirect(true);
+                    }
                     gap_rg.SetFirstFrom(new_gap_pos);
                     out_pw.AddInsertion(gap_rg);
                     gap_it++;
@@ -378,7 +405,23 @@ s_TranslatePairwiseToAlnCoords(CPairwiseAln& out_pw,   ///< output pairwise (nee
     }
     while (gap_it != gaps.end()) {
         CPairwiseAln::TAlnRng gap_rg = *gap_it;
-        gap_rg.SetFirstFrom(gap_rg.GetFirstFrom() - anchor_shift);
+        CPairwiseAln::TPos new_gap_pos =
+            tr.GetFirstPosBySecondPos(gap_rg.GetFirstFrom(),
+            CPairwiseAln::eForward);
+        // If there are no ranges ahead, try to find the last one before the gap.
+        if (new_gap_pos == -1) {
+            new_gap_pos = tr.GetFirstPosBySecondPos(
+                gap_rg.GetFirstFrom(),
+                CPairwiseAln::eBackwards) + 1;
+        }
+        else if ( !direct ) {
+            new_gap_pos++;
+        }
+        gap_rg.SetFirstFrom(new_gap_pos);
+        if ( !direct ) {
+            gap_rg.SetDirect(!gap_rg.IsDirect());
+            gap_rg.SetFirstDirect(true);
+        }
         out_pw.AddInsertion(gap_rg);
         gap_it++;
     }
@@ -396,10 +439,14 @@ s_TranslateToAlnCoords(CAnchoredAln& anchored_aln,
     /// Fix the anchor pairwise, so it's expressed in aln coords:
     const CPairwiseAln& anchor_pw = *pairwises[anchor_row];
 
+    int flags = anchor_pw.GetFlags();
+    flags &= ~(CPairwiseAln::fDirect | CPairwiseAln::fReversed);
+
     CRef<CPairwiseAln> new_anchor_pw(new CPairwiseAln(pseudo_seqid,
                                                       anchor_pw.GetSecondId(),
-                                                      anchor_pw.GetFlags()));
+                                                      flags));
     s_TranslateAnchorToAlnCoords(*new_anchor_pw, anchor_pw);
+    bool direct = (new_anchor_pw->begin()->IsFirstDirect() == anchor_pw.begin()->IsFirstDirect());
 
     /// Translate non-anchor pairwises to aln coords:
     for (TDim row = 0;  row < (TDim)pairwises.size();  ++row) {
@@ -407,10 +454,12 @@ s_TranslateToAlnCoords(CAnchoredAln& anchored_aln,
             pairwises[row].Reset(new_anchor_pw);
         } else {
             const CPairwiseAln& pw = *pairwises[row];
+            flags = pw.GetFlags();
+            flags &= ~(CPairwiseAln::fDirect | CPairwiseAln::fReversed);
             CRef<CPairwiseAln> new_pw(new CPairwiseAln(pseudo_seqid,
                                                        pw.GetSecondId(),
-                                                       pw.GetFlags()));
-            s_TranslatePairwiseToAlnCoords(*new_pw, pw, *new_anchor_pw);
+                                                       flags));
+            s_TranslatePairwiseToAlnCoords(*new_pw, pw, *new_anchor_pw, direct);
             pairwises[row].Reset(new_pw);
         }
     }            
