@@ -1,4 +1,4 @@
-/*  $Id: aln_converters.cpp 348472 2011-12-29 16:52:24Z grichenk $
+/*  $Id: aln_converters.cpp 372361 2012-08-16 18:58:42Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -157,6 +157,7 @@ ConvertDensegToPairwiseAln(CPairwiseAln& pairwise_aln,
     CDense_seg::TNumseg seg;
     int pos_1, pos_2;
     TSignedSeqPos last_to_1 = 0;
+
     for (seg = 0, pos_1 = row_1, pos_2 = row_2;
          seg < numseg;
          ++seg, pos_1 += dim, pos_2 += dim) {
@@ -164,7 +165,7 @@ ConvertDensegToPairwiseAln(CPairwiseAln& pairwise_aln,
         TSignedSeqPos from_2 = starts[pos_2];
         TSeqPos len = lens[seg];
 
-        /// determinte the direction
+        // determinte the direction
         bool direct = true;
         bool first_direct = true;
         if (strands) {
@@ -178,10 +179,14 @@ ConvertDensegToPairwiseAln(CPairwiseAln& pairwise_aln,
              direction == CAlnUserOptions::eDirect :
              direction == CAlnUserOptions::eReverse)) {
 
-            /// base-width adjustments
+            // base-width adjustments
             const int& base_width_1 = pairwise_aln.GetFirstBaseWidth();
             const int& base_width_2 = pairwise_aln.GetSecondBaseWidth();
             if (mixed  ||  base_width_1 > 1) {
+                // Tell the pairwise alignment we are already using genomic
+                // coordinates.
+                pairwise_aln.SetUsingGenomic();
+
                 if (base_width_1 > 1) {
                     from_1 *= base_width_1;
                 }
@@ -191,21 +196,36 @@ ConvertDensegToPairwiseAln(CPairwiseAln& pairwise_aln,
                 len *= 3;
             }
 
-            /// if not a gap, insert it to the collection
+            // if not a gap, insert it to the collection
             if (from_1 >= 0  &&  from_2 >= 0)  {
-                /// insert the range
+                // insert the range
                 pairwise_aln.insert(CPairwiseAln::TAlnRng(
                     from_1, from_2, len, direct, first_direct));
-                last_to_1 = from_1 + len;
+                last_to_1 = first_direct ? from_1 + len : from_1;
             }
             else if (from_1 < 0  &&  from_2 >= 0) {
                 // Store gaps
+                TSignedSeqPos ins_pos = last_to_1;
+                if (!first_direct  &&  ins_pos == 0) {
+                    // Special case: first is on minus strand and starts with a gap.
+                    // Instead of using '0' for insertion position, place it after
+                    // the first non-gap segment.
+                    CDense_seg::TNumseg skip_seg = 1;
+                    while (seg + skip_seg < numseg) {
+                        int p = starts[pos_1 + skip_seg*dim];
+                        if (p >= 0) {
+                            ins_pos = p + lens[seg + skip_seg];
+                            break;
+                        }
+                        skip_seg++;
+                    }
+                }
                 pairwise_aln.AddInsertion(CPairwiseAln::TAlnRng(
-                    last_to_1, from_2, len, direct, first_direct));
+                    ins_pos, from_2, len, direct, first_direct));
             }
             else if (from_1 >= 0) {
                 // Adjust next possible gap start
-                last_to_1 = from_1 + len;
+                last_to_1 = first_direct ? from_1 + len : from_1;
             }
         }
     }
@@ -245,7 +265,7 @@ ConvertPackedsegToPairwiseAln(CPairwiseAln& pairwise_aln,
         bool present_2 = presents[pos_2];
         TSeqPos len = lens[seg];
 
-        /// determinte the direction
+        // determinte the direction
         bool direct = true;
         bool first_direct = true;
         if (strands) {
@@ -259,10 +279,14 @@ ConvertPackedsegToPairwiseAln(CPairwiseAln& pairwise_aln,
              direction == CAlnUserOptions::eDirect :
              direction == CAlnUserOptions::eReverse)) {
 
-            /// base-width adjustments
+            // base-width adjustments
             const int& base_width_1 = pairwise_aln.GetFirstBaseWidth();
             const int& base_width_2 = pairwise_aln.GetSecondBaseWidth();
             if (mixed  ||  base_width_1 > 1) {
+                // Tell the pairwise alignment we are already using genomic
+                // coordinates.
+                pairwise_aln.SetUsingGenomic();
+
                 if (base_width_1 > 1) {
                     from_1 *= base_width_1;
                 }
@@ -272,12 +296,12 @@ ConvertPackedsegToPairwiseAln(CPairwiseAln& pairwise_aln,
                 len *= 3;
             }
                 
-            /// if not a gap, insert it to the collection
+            // if not a gap, insert it to the collection
             if (present_1  &&  present_2)  {
-                /// insert the range
+                // insert the range
                 pairwise_aln.insert(CPairwiseAln::TAlnRng(
                     from_1, from_2, len, direct, first_direct));
-                last_to_1 = from_1 + len;
+                last_to_1 = first_direct ? from_1 + len : from_1;
             }
             else if (!present_1  &&  present_2) {
                 // Store gaps
@@ -286,7 +310,7 @@ ConvertPackedsegToPairwiseAln(CPairwiseAln& pairwise_aln,
             }
             else if (present_1) {
                 // Adjust next possible gap start
-                last_to_1 = from_1 + len;
+                last_to_1 = first_direct ? from_1 + len : from_1;
             }
         }
     }
@@ -303,6 +327,21 @@ ConvertStdsegToPairwiseAln(CPairwiseAln& pairwise_aln,
 {
     _ALNMGR_ASSERT(row_1 >=0  &&  row_2 >= 0);
 
+    TSeqPos last_to_1 = 0;
+
+    int guessed_width_1 = 0;
+    int guessed_width_2 = 0;
+
+    // Check global strand of the first row - we'll need it for gaps.
+    bool global_first_direct = true;
+    ITERATE (CSeq_align::TSegs::TStd, std_it, stds) {
+        const CStd_seg::TLoc& loc = (*std_it)->GetLoc();
+        if ( !loc[row_1]->GetTotalRange().Empty() ) {
+            global_first_direct = !loc[row_1]->IsReverseStrand();
+            break;
+        }
+    }
+
     ITERATE (CSeq_align::TSegs::TStd, std_it, stds) {
 
         const CStd_seg::TLoc& loc = (*std_it)->GetLoc();
@@ -314,11 +353,12 @@ ConvertStdsegToPairwiseAln(CPairwiseAln& pairwise_aln,
 
         TSeqPos len_1 = rng_1.GetLength();
         TSeqPos len_2 = rng_2.GetLength();
+        bool first_direct = !loc[row_1]->IsReverseStrand();
+        bool direct = first_direct != loc[row_2]->IsReverseStrand();
 
         if (len_1 > 0  &&  len_2 > 0) {
 
-            bool first_direct = loc[row_1]->IsReverseStrand();
-            bool direct = first_direct == loc[row_2]->IsReverseStrand();
+            last_to_1 = first_direct ? rng_1.GetFrom() + len_1 : rng_1.GetFrom();
             if (direction == CAlnUserOptions::eBothDirections  ||
                 (direct ?
                  direction == CAlnUserOptions::eDirect :
@@ -333,6 +373,14 @@ ConvertStdsegToPairwiseAln(CPairwiseAln& pairwise_aln,
                 bool row_1_is_protein = prot_to_nuc_diff < nuc_to_nuc_diff;
                 bool row_2_is_protein = nuc_to_prot_diff < nuc_to_nuc_diff;
 
+                if (nuc_to_nuc_diff != 0) {
+                    // If the alignment is mixed, using genomic coordinates.
+                    pairwise_aln.SetUsingGenomic();
+                }
+
+                guessed_width_1 = row_1_is_protein ? 3 : 1;
+                guessed_width_2 = row_2_is_protein ? 3 : 1;
+
                 CPairwiseAln::TAlnRng aln_rng;
                 aln_rng.SetDirect(direct);
                 aln_rng.SetFirstDirect(first_direct);
@@ -393,6 +441,30 @@ ConvertStdsegToPairwiseAln(CPairwiseAln& pairwise_aln,
                 }
             }
         }
+        else if (len_2 > 0) {
+            // Gap in the second row - add insertion
+            TSignedSeqPos from_1 = last_to_1;
+            TSignedSeqPos from_2 = rng_2.GetFrom();
+            TSignedSeqPos len = len_2;
+            // If we've seen a normal segment, widths should be set (but may be
+            // different from base widths from the pairwise_aln). Otherwise we
+            // have to use pairwise alignment.
+            if (guessed_width_1 == 0) {
+                guessed_width_1 = pairwise_aln.GetFirstBaseWidth();
+            }
+            if (guessed_width_2 == 0) {
+                guessed_width_2 = pairwise_aln.GetSecondBaseWidth();
+            }
+            from_1 *= guessed_width_1;
+            from_2 *= guessed_width_2;
+            len *= guessed_width_2;
+            bool gap_direct = global_first_direct != loc[row_2]->IsReverseStrand();
+            pairwise_aln.AddInsertion(CPairwiseAln::TAlnRng(
+                from_1, from_2, len, gap_direct, global_first_direct));
+        }
+        else if (len_1 > 0) {
+            last_to_1 = first_direct ? rng_1.GetFrom() + len_1 : rng_1.GetFrom();
+        }
     }
 }
 
@@ -420,7 +492,7 @@ ConvertDendiagToPairwiseAln(CPairwiseAln& pairwise_aln,
         TSeqPos from_2 = dd.GetStarts()[row_2];
         TSeqPos len = dd.GetLen();
 
-        /// determinte the strands
+        // determinte the strands
         bool direct = true;
         bool first_direct = true;
         if (dd.IsSetStrands()) {
@@ -434,10 +506,14 @@ ConvertDendiagToPairwiseAln(CPairwiseAln& pairwise_aln,
              direction == CAlnUserOptions::eDirect :
              direction == CAlnUserOptions::eReverse)) {
 
-            /// base-width adjustments
+            // base-width adjustments
             const int& base_width_1 = pairwise_aln.GetFirstBaseWidth();
             const int& base_width_2 = pairwise_aln.GetSecondBaseWidth();
             if (mixed  ||  base_width_1 > 1) {
+                // Tell the pairwise alignment we are already using genomic
+                // coordinates.
+                pairwise_aln.SetUsingGenomic();
+
                 if (base_width_1 > 1) {
                     from_1 *= base_width_1;
                 }
@@ -447,7 +523,7 @@ ConvertDendiagToPairwiseAln(CPairwiseAln& pairwise_aln,
                 len *= 3;
             }
 
-            /// insert the range
+            // insert the range
             pairwise_aln.insert(CPairwiseAln::TAlnRng(
                 from_1, from_2, len, direct, first_direct));
 
@@ -466,9 +542,13 @@ ConvertSparseToPairwiseAln(CPairwiseAln& pairwise_aln,
 {
     typedef CPairwiseAln::TAlnRngColl TAlnRngColl;
 
-    _ALNMGR_ASSERT(row_1 == 0); /// TODO: Hanlde case when the anchor is not sparse_aln's anchor.
+    // Sparse-segs are not intended to store mixed alignments.
+    _ALNMGR_ASSERT(pairwise_aln.GetFirstId()->GetBaseWidth() ==
+        pairwise_aln.GetSecondId()->GetBaseWidth());
+
+    _ALNMGR_ASSERT(row_1 == 0); // TODO: Hanlde case when the anchor is not sparse_aln's anchor.
     if (row_1 == 0) {
-        if (row_2 == 0) { /// Anchor aligned to itself
+        if (row_2 == 0) { // Anchor aligned to itself
             bool first_row = true;
             ITERATE(CSparse_seg::TRows, aln_it, sparse_seg.GetRows()) {
                 TAlnRngColl curr_row;
@@ -495,7 +575,7 @@ ConvertSparseToPairwiseAln(CPairwiseAln& pairwise_aln,
                     }
                 }                    
             }
-        } else { /// Regular row
+        } else { // Regular row
             _ALNMGR_ASSERT(row_2 > 0  &&  row_2 <= sparse_seg.CheckNumRows());
 
             const CSparse_align& sa = *sparse_seg.GetRows()[row_2 - 1];
@@ -528,12 +608,15 @@ ConvertSplicedToPairwiseAln(CPairwiseAln& pairwise_aln,
     _ALNMGR_ASSERT((row_1 == 0  ||  row_1 == 1)  &&  (row_2 == 0  ||  row_2 == 1));
 
     bool prot = spliced_seg.GetProduct_type() == CSpliced_seg::eProduct_type_protein;
+    // With spliced-seg we always know how to translate coordinates and use
+    // genomic ones.
+    pairwise_aln.SetUsingGenomic();
 
     ITERATE (CSpliced_seg::TExons, exon_it, spliced_seg.GetExons()) {
 
         const CSpliced_exon& exon = **exon_it;
             
-        /// Determine strands
+        // Determine strands
         if (spliced_seg.CanGetProduct_strand()  &&  exon.CanGetProduct_strand()  &&
             spliced_seg.GetProduct_strand() != exon.GetProduct_strand()) {
             NCBI_THROW(CSeqalignException, eInvalidAlignment,
@@ -561,7 +644,7 @@ ConvertSplicedToPairwiseAln(CPairwiseAln& pairwise_aln,
         bool direct = product_plus == genomic_plus;
         bool first_direct = row_1 == 0 ? product_plus : genomic_plus;
 
-        /// Determine positions
+        // Determine positions
         TSeqPos product_start;
         if (prot) {
             product_start = exon.GetProduct_start().GetProtpos().GetAmin() * 3;
@@ -631,7 +714,7 @@ ConvertSplicedToPairwiseAln(CPairwiseAln& pairwise_aln,
                     first_direct));
             }
         } else {
-            /// Iterate trhough exon chunks
+            // Iterate trhough exon chunks
             TSeqPos last_product_to = 0;
             TSeqPos last_genomic_to = 0;
             ITERATE (CSpliced_exon::TParts, chunk_it, exon.GetParts()) {
@@ -663,17 +746,17 @@ ConvertSplicedToPairwiseAln(CPairwiseAln& pairwise_aln,
                 TSeqPos gpos = genomic_plus ? genomic_pos : genomic_pos - genomic_len + 1;
                 if (row_1 == 0  &&  row_2 == 0) {
                     if (product_len != 0) {
-                        /// insert the range
+                        // insert the range
                         pairwise_aln.insert(CPairwiseAln::TAlnRng(
                             ppos, ppos, product_len, true, first_direct));
-                        last_product_to = ppos + product_len;
+                        last_product_to = first_direct ? ppos + product_len : ppos;
                     }
                 } else if (row_1 == 1  &&  row_2 == 1) {
                     if (genomic_len != 0) {
-                        /// insert the range
+                        // insert the range
                         pairwise_aln.insert(CPairwiseAln::TAlnRng(
                             gpos, gpos, genomic_len, true, first_direct));
-                        last_genomic_to = gpos + genomic_len;
+                        last_genomic_to = first_direct ? gpos + genomic_len : gpos;
                     }
                 } else {
                     if (product_len != 0  &&  product_len == genomic_len  &&
@@ -681,16 +764,18 @@ ConvertSplicedToPairwiseAln(CPairwiseAln& pairwise_aln,
                          (direct ?
                           direction == CAlnUserOptions::eDirect :
                           direction == CAlnUserOptions::eReverse))) {
-                        /// insert the range
+                        // insert the range
                         if (row_1 == 0) {
                             pairwise_aln.insert(CPairwiseAln::TAlnRng(
                                 ppos, gpos, genomic_len, direct, first_direct));
+                            last_product_to = first_direct ? ppos + product_len : ppos;
+                            last_genomic_to = direct ? gpos + genomic_len : gpos;
                         } else {
                             pairwise_aln.insert(CPairwiseAln::TAlnRng(
                                 gpos, ppos, genomic_len, direct, first_direct));
+                            last_product_to = direct ? ppos + product_len : ppos;
+                            last_genomic_to = first_direct ? gpos + genomic_len : gpos;
                         }
-                        last_product_to = ppos + product_len;
-                        last_genomic_to = gpos + genomic_len;
                     }
                     else {
                         // Gap on the first row?
@@ -772,6 +857,11 @@ ConvertSeqLocsToPairwiseAln(CPairwiseAln& aln,
     if ( !wid2 ) {
         wid2 = 1;
     }
+    if (wid1 == 3  ||  wid2 == 3) {
+        // We know we are translating everything to genomic coordinates.
+        aln.SetUsingGenomic();
+    }
+
     CSeq_loc_CI it1(loc_1);
     CSeq_loc_CI it2(loc_2);
     TSeqPos lshift1 = 0;
@@ -884,6 +974,7 @@ void SeqLocMapperToPairwiseAligns(const objects::CSeq_loc_Mapper_Base& mapper,
                 TAlnSeqIdIRef dst_id(Ref(new CAlnSeqId(*dst_idh.GetSeqId())));
                 dst_id->SetBaseWidth(mapper.GetWidthById(dst_idh));
                 aln = new CPairwiseAln(src_id, dst_id);
+                aln->SetUsingGenomic(); // All coordinates are already genomic.
                 aln_map[dst_idh] = aln;
                 aligns.push_back(aln);
             }
