@@ -1,4 +1,4 @@
-/*  $Id: sparse_aln.cpp 348472 2011-12-29 16:52:24Z grichenk $
+/*  $Id: sparse_aln.cpp 372361 2012-08-16 18:58:42Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -39,7 +39,6 @@
 #include <objects/seqalign/Sparse_align.hpp>
 #include <objects/seqfeat/Genetic_code_table.hpp>
 
-
 #define NCBI_USE_ERRCODE_X   Objtools_Aln_Sparse
 
 
@@ -50,7 +49,10 @@ USING_SCOPE(ncbi::objects);
 CSparseAln::CSparseAln(const CAnchoredAln& anchored_aln,
                        objects::CScope& scope)
     : m_Scope(&scope),
-      m_GapChar('-')
+      m_GapChar('-'),
+      m_NaCoding(CSeq_data::e_not_set),
+      m_AaCoding(CSeq_data::e_not_set),
+      m_AnchorDirect(true)
 {
     x_Build(anchored_aln);
 }
@@ -146,8 +148,12 @@ void CSparseAln::x_Build(const CAnchoredAln& src_align)
         CRef<CPairwiseAln> dst(new CPairwiseAln(
             pw.GetFirstId(), pw.GetSecondId(), pw.GetFlags()));
         TSeqPos last_to = 0;
+        bool first_direct = true;
+        bool second_direct = true;
         while (seg_it != pw.end()) {
             CPairwiseAln::TAlignRange rg = *seg_it;
+            first_direct = rg.IsFirstDirect();
+            second_direct = rg.IsDirect();
             ++seg_it;
             // Check if there are gaps before the new segment's end.
             while (gap != gaps.end()  &&
@@ -156,6 +162,9 @@ void CSparseAln::x_Build(const CAnchoredAln& src_align)
                     // Insertion in this row - align to the anchor.
                     CPairwiseAln::TAlignRange ins(gap->from + shift,
                         gap->second_from, gap->len, gap->direct);
+                    // Reset flags to match row orientation.
+                    ins.SetFirstDirect(first_direct);
+                    ins.SetDirect(second_direct);
                     dst->push_back(ins);
                 }
                 else if (gap->from > rg.GetFirstFrom()) {
@@ -185,6 +194,8 @@ void CSparseAln::x_Build(const CAnchoredAln& src_align)
             if (gap->row == row) {
                 CPairwiseAln::TAlignRange ins(gap->from + shift,
                     last_to, gap->len, gap->direct);
+                ins.SetFirstDirect(first_direct);
+                ins.SetDirect(second_direct);
                 dst->push_back(ins);
             }
             shift = gap->shift + gap->len;
@@ -216,6 +227,9 @@ void CSparseAln::x_Build(const CAnchoredAln& src_align)
         ext.UpdateIndex();
         m_SecondRanges[row] = ext.GetSecondRange();
     }
+
+    const CPairwiseAln& anch_pw = *m_Aln->GetPairwiseAlns()[m_Aln->GetAnchorRow()];
+    m_AnchorDirect = anch_pw.empty() || anch_pw.begin()->IsFirstDirect();
 }
 
 
@@ -298,7 +312,7 @@ bool CSparseAln::IsPositiveStrand(TNumrow row) const
 {
     _ASSERT(row >= 0  &&  row < GetDim());
     _ASSERT( !m_Aln->GetPairwiseAlns()[row]->IsSet(CPairwiseAln::fMixedDir) );
-    return m_Aln->GetPairwiseAlns()[row]->IsSet(CPairwiseAln::fDirect);
+    return m_Aln->GetPairwiseAlns()[row]->IsSet(CPairwiseAln::fDirect) == m_AnchorDirect;
 }
 
 
@@ -306,7 +320,7 @@ bool CSparseAln::IsNegativeStrand(TNumrow row) const
 {
     _ASSERT(row >= 0  &&  row < GetDim());
     _ASSERT( !m_Aln->GetPairwiseAlns()[row]->IsSet(CPairwiseAln::fMixedDir) );
-    return m_Aln->GetPairwiseAlns()[row]->IsSet(CPairwiseAln::fReversed);
+    return m_Aln->GetPairwiseAlns()[row]->IsSet(CPairwiseAln::fReversed) == m_AnchorDirect;
 }
 
 
@@ -404,7 +418,26 @@ CSeqVector& CSparseAln::x_GetSeqVector(TNumrow row) const
              CBioseq_Handle::eStrand_Minus);
         m_SeqVectors[row].Reset(new CSeqVector(vec));
     }
-    return *m_SeqVectors[row];
+
+    CSeqVector& seq_vec = *m_SeqVectors[row];
+    if ( seq_vec.IsNucleotide() ) {
+        if (m_NaCoding != CSeq_data::e_not_set) {
+            seq_vec.SetCoding(m_NaCoding);
+        }
+        else {
+            seq_vec.SetIupacCoding();
+        }
+    }
+    else if ( seq_vec.IsProtein() ) {
+        if (m_AaCoding != CSeq_data::e_not_set) {
+            seq_vec.SetCoding(m_AaCoding);
+        }
+        else {
+            seq_vec.SetIupacCoding();
+        }
+    }
+
+    return seq_vec;
 }
 
 
@@ -421,6 +454,8 @@ void CSparseAln::TranslateNAToAA(const string& na,
         aa.resize(na_size / 3 + (na_remainder ? 1 : 0));
     }
 
+    if ( na.empty() ) return;
+
     size_t aa_i = 0;
     int state = 0;
     for (size_t na_i = 0;  na_i < na_size; ) {
@@ -452,6 +487,7 @@ string& CSparseAln::GetSeqString(TNumrow row,
     if (width > 1) {
         seq_from /= 3;
         seq_to /= 3;
+        force_translation = false; // do not translate AAs.
     }
     if (seq_to >= seq_from) {
         CSeqVector& seq_vector = x_GetSeqVector(row);
@@ -465,6 +501,10 @@ string& CSparseAln::GetSeqString(TNumrow row,
             TSeqPos vec_size = seq_vector.size();
             seq_vector.GetSeqData(vec_size - seq_to - 1, vec_size - seq_from, buffer);
         }
+
+        if ( force_translation ) {
+            TranslateNAToAA(buffer, buffer);
+        }
     }
     return buffer;
 }
@@ -472,11 +512,16 @@ string& CSparseAln::GetSeqString(TNumrow row,
 
 string& CSparseAln::GetSeqString(TNumrow row,
                                  string &buffer,
-                                 const TRange &seq_range,
+                                 const TRange &rq_seq_range,
                                  bool force_translation) const
 {
     _ASSERT(row >= 0  &&  row < GetDim());
 
+    TRange seq_range = rq_seq_range;
+    if ( seq_range.IsWhole() ) {
+        seq_range = GetSeqRange(row);
+    }
+
     return GetSeqString(row,
                         buffer,
                         seq_range.GetFrom(), seq_range.GetTo(),
@@ -486,106 +531,98 @@ string& CSparseAln::GetSeqString(TNumrow row,
 
 string& CSparseAln::GetAlnSeqString(TNumrow row,
                                     string &buffer,
-                                    const TSignedRange &aln_range,
+                                    const TSignedRange &rq_aln_range,
                                     bool force_translation) const
 {
     _ASSERT(row >= 0  &&  row < GetDim());
 
     bool translate = force_translation  ||  IsTranslated();
 
+    TSignedRange aln_range(rq_aln_range);
+    if ( aln_range.IsWhole() ) {
+        aln_range = GetSeqAlnRange(row);
+    }
+
     buffer.erase();
+    if (aln_range.GetLength() <= 0) {
+        return buffer;
+    }
 
-    if(aln_range.GetLength() > 0)   {
-        const CPairwiseAln& pairwise_aln = *m_Aln->GetPairwiseAlns()[row];
-        _ASSERT( !pairwise_aln.empty() );
-        if (pairwise_aln.empty()) {
-            string errstr = "Invalid (empty) row (" + NStr::IntToString(row) + ").  Seq id \"" +
-                GetSeqId(row).AsFastaString() + "\".";
-            NCBI_THROW(CAlnException, eInvalidRequest, errstr);
+    const CPairwiseAln& pairwise_aln = *m_Aln->GetPairwiseAlns()[row];
+    if (pairwise_aln.empty()) {
+        string errstr = "Invalid (empty) row (" + NStr::IntToString(row) + ").  Seq id \"" +
+            GetSeqId(row).AsFastaString() + "\".";
+        NCBI_THROW(CAlnException, eInvalidRequest, errstr);
+    }
+
+    CSeqVector& seq_vector = x_GetSeqVector(row);
+    TSeqPos vec_size = seq_vector.size();
+
+    const int base_width = pairwise_aln.GetSecondBaseWidth();
+
+    // buffer holds sequence for "aln_range", 0 index corresonds to aln_range.GetFrom()
+    size_t size = aln_range.GetLength();
+    if (translate) {
+        size /= 3;
+    }
+    buffer.resize(size, m_GapChar);
+
+    string s; // current segment sequence
+    CSparse_CI it(*this, row, IAlnSegmentIterator::eSkipInserts, aln_range);
+
+    while ( it )   {
+        const IAlnSegment::TSignedRange& aln_r = it->GetAlnRange(); // in alignment
+        const IAlnSegment::TSignedRange& row_r = it->GetRange(); // on sequence
+        if ( row_r.Empty() ) {
+            ++it;
+            continue;
         }
 
-        CSeqVector& seq_vector = x_GetSeqVector(row);
-        TSeqPos vec_size = seq_vector.size();
-
-        const int base_width = pairwise_aln.GetSecondBaseWidth();
-
-        // buffer holds sequence for "aln_range", 0 index corresonds to aln_range.GetFrom()
-        size_t size = aln_range.GetLength();
-        if (translate) {
-            size /= 3;
-        }
-        buffer.resize(size, ' ');
-
-        // check whether we have a gap at start position
-        size_t prev_to_open = (pairwise_aln.GetFirstFrom() > aln_range.GetFrom()) ? string::npos : 0;
-
-        string s;
-        CSparse_CI it(pairwise_aln, IAlnSegmentIterator::eSkipGaps, aln_range);
-
-        //LOG_POST_X(1, "GetAlnSeqString(" << row << ") ==========================================" );
-        while (it)   {
-            const IAlnSegment::TSignedRange& aln_r = it->GetAlnRange(); // in alignment
-            const IAlnSegment::TSignedRange& r = it->GetRange(); // on sequence
-
-            size_t off;
-            //LOG_POST_X(2, "Aln [" << aln_r.GetFrom() << ", " << aln_r.GetTo() << "], Seq  "
-            //                      << r.GetFrom() << ", " << r.GetTo());
-            if (base_width == 1) {
-                // TODO performance issue - waiting for better API
-                if (IsPositiveStrand(row)) {
-                    seq_vector.GetSeqData(r.GetFrom(), r.GetToOpen(), s);
-                } else {
-                    seq_vector.GetSeqData(vec_size - r.GetToOpen(),
-                                          vec_size - r.GetFrom(), s);
-                }
-                if (translate) {
-                    TranslateNAToAA(s, s);
-                }
-                off = aln_r.GetFrom() - aln_range.GetFrom();
-                if (translate) {
-                    off /= 3;
-                }
+        size_t off;
+        if (base_width == 1) {
+            // TODO performance issue - waiting for better API
+            if (IsPositiveStrand(row)) {
+                seq_vector.GetSeqData(row_r.GetFrom(), row_r.GetToOpen(), s);
             } else {
-                _ASSERT(base_width == 3);
-                IAlnSegment::TSignedRange prot_r = r;
-                prot_r.SetFrom(r.GetFrom() / 3);
-                prot_r.SetLength(r.GetLength() < 3 ? 1 : r.GetLength() / 3);
+                seq_vector.GetSeqData(vec_size - row_r.GetToOpen(),
+                                        vec_size - row_r.GetFrom(), s);
+            }
+            if (translate) {
+                TranslateNAToAA(s, s);
+            }
+            off = aln_r.GetFrom() - aln_range.GetFrom();
+            if (translate) {
+                off /= 3;
+            }
+        }
+        else {
+            _ASSERT(base_width == 3);
+            IAlnSegment::TSignedRange prot_r = row_r;
+            if (prot_r.GetLength() > 0) {
+                prot_r.SetFrom(row_r.GetFrom() / 3);
+                prot_r.SetLength(row_r.GetLength() < 3 ? 1 : row_r.GetLength() / 3);
                 if (IsPositiveStrand(row)) {
                     seq_vector.GetSeqData(prot_r.GetFrom(),
-                                          prot_r.GetToOpen(), s);
-                } else {
+                                            prot_r.GetToOpen(), s);
+                }
+                else {
                     seq_vector.GetSeqData(vec_size - prot_r.GetToOpen(),
                                           vec_size - prot_r.GetFrom(), s);
                 }
-                off = (aln_r.GetFrom() - aln_range.GetFrom()) / 3;
             }
-            /*if(it->IsReversed())    {
-                std::reverse(s.begin(), s.end());
-            }*/
+            off = (aln_r.GetFrom() - aln_range.GetFrom()) / 3;
+        }
 
-            if(prev_to_open == string::npos) {
-                // we have a gap at the start position
-                buffer.replace(0, off, off, m_GapChar);
-            } else {   // this is not the first segement
-                off = max(prev_to_open, off);
-                int gap_size = off - prev_to_open;
-                buffer.replace(prev_to_open, gap_size, gap_size, m_GapChar);
-            }
-
-            size_t len = min(buffer.size() - off, s.size());
-
-            _ASSERT(off + len <= buffer.size());
+        size_t len = min(size - off, s.size());
+        _ASSERT(off + len <= size);
 
+        if ( m_AnchorDirect ) {
             buffer.replace(off, len, s, 0, len);
-            prev_to_open = off + len;
-            ++it;
         }
-        int fill_len = size - prev_to_open;
-        if(prev_to_open != string::npos  &&  fill_len > 0  &&  pairwise_aln.GetFirstTo() > aln_range.GetTo()) {
-            // there is gap on the right
-            buffer.replace(prev_to_open, fill_len, fill_len, m_GapChar);
+        else {
+            buffer.replace(size - off - len, len, s, 0, len);
         }
-        //LOG_POST_X(3, buffer);
+        ++it;
     }
     return buffer;
 }
@@ -603,7 +640,7 @@ CSparseAln::CreateSegmentIterator(TNumrow row,
             GetSeqId(row).AsFastaString() + "\".";
         NCBI_THROW(CAlnException, eInvalidRequest, errstr);
     }
-    return new CSparse_CI(*m_Aln->GetPairwiseAlns()[row], flag, range);
+    return new CSparse_CI(*this, row, flag, range);
 }
 
 
