@@ -1,4 +1,4 @@
-/*  $Id: seq_writer.cpp 311249 2011-07-11 14:12:16Z camacho $
+/*  $Id: seq_writer.cpp 355436 2012-03-06 13:45:27Z camacho $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: seq_writer.cpp 311249 2011-07-11 14:12:16Z camacho $";
+    "$Id: seq_writer.cpp 355436 2012-03-06 13:45:27Z camacho $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -203,6 +203,9 @@ void CSeqFormatter::DumpAll(CSeqDB& blastdb, CSeqFormatterConfig config)
     CRef<CBioseq> bioseq;
     for (int i=0; blastdb.CheckOrFindOID(i); i++) {
          bioseq.Reset(blastdb.GetBioseq(i));
+         if (bioseq.Empty()) {
+             continue;
+         }
          if (config.m_UseCtrlA) {
              s_ReplaceCtrlAsInTitle(bioseq);
          }
