@@ -1,4 +1,4 @@
-/*  $Id: seqdbalias.cpp 351397 2012-01-27 20:05:15Z rafanovi $
+/*  $Id: seqdbalias.cpp 365378 2012-06-05 17:09:46Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -43,7 +43,7 @@
 /// Implemented for: UNIX, MS-Windows
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: seqdbalias.cpp 351397 2012-01-27 20:05:15Z rafanovi $";
+static char const rcsid[] = "$Id: seqdbalias.cpp 365378 2012-06-05 17:09:46Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -260,7 +260,7 @@ void CSeqDBAliasNode::x_ResolveNames(char prot_nucl, CSeqDBLockHold & locked)
     
     for(i = 0; i < m_DBList.size(); i++) {
         // skip local DB search only if absolute path is given
-        if(m_DBList[i].GetBasePathS()[0] == CDirEntry::GetPathSeparator()) {
+        if(m_DBList[i].GetBasePathS().find(CDirEntry::GetPathSeparator()) != string::npos) {
             m_SkipLocal[i] = true;
         }
 
