@@ -1,4 +1,4 @@
-/*  $Id: build_db.cpp 350008 2012-01-17 20:01:10Z rafanovi $
+/*  $Id: build_db.cpp 368592 2012-07-10 14:40:31Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -31,7 +31,7 @@
   Code to build a database given various sources of sequence data.
 */
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: build_db.cpp 350008 2012-01-17 20:01:10Z rafanovi $";
+static char const rcsid[] = "$Id: build_db.cpp 368592 2012-07-10 14:40:31Z rafanovi $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -697,7 +697,9 @@ CFastaBioseqSource::CFastaBioseqSource(CNcbiIstream & fasta_file,
     
     typedef CFastaReader::EFlags TFlags;
     
-    int iflags = CFastaReader::fAllSeqIds | CFastaReader::fForceType;
+    int iflags = CFastaReader::fAllSeqIds |
+                 CFastaReader::fForceType |
+                 CFastaReader::fParseGaps;
     
     if (is_protein) {
         iflags |= CFastaReader::fAssumeProt;
