@@ -1,4 +1,4 @@
-/*  $Id: writedb_convert.cpp 200354 2010-08-06 17:58:25Z camacho $
+/*  $Id: writedb_convert.cpp 282672 2011-05-13 17:53:00Z maning $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,7 @@
 /// class for WriteDB.
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: writedb_convert.cpp 200354 2010-08-06 17:58:25Z camacho $";
+static char const rcsid[] = "$Id: writedb_convert.cpp 282672 2011-05-13 17:53:00Z maning $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -502,17 +502,12 @@ void WriteDB_Ncbi4naToBinary(const char * ncbi4na,
         } else {
             // Check each element, accumulate ambiguity data.
             
-            if (! (b1 && b2)) {
-                _ASSERT(b1 != 0);
-                
-                if ((i*2+1) < base_length) {
-                    // the second base is only available if the
-                    // sequence contains a whole number of bytes
-                    // (i.e. has an even length).
-                    _ASSERT(b2 != 0);
-                } else {
-                    _ASSERT(b2 == 0);
-                }
+            if (! b1) {
+                b1 = 0xF; // replace gap with 'N'
+            }
+
+            if (! b2 && (i*2+1) < base_length) {
+                b2 = 0xF; // the last base is the sentinel
             }
             
             half |= ambiguities.Check(b1, i*2) << 2;
