@@ -1,4 +1,4 @@
-/*  $Id: blast_services_test.cpp 220534 2011-01-21 15:04:59Z camacho $
+/*  $Id: blast_services_test.cpp 372807 2012-08-22 17:18:58Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -131,7 +131,7 @@ BOOST_AUTO_TEST_CASE(OneBadDbValidityCheck)
     bool found = remote_svc.IsValidBlastDb("nt dummy", false);
     BOOST_REQUIRE(found == false);
 }
-
+#ifdef UNIT_TEST_REPEAT_DB
 BOOST_AUTO_TEST_CASE(GetRepeatsFilteringDatabases)
 {
     CBlastServices remote_svc;
@@ -188,7 +188,7 @@ BOOST_AUTO_TEST_CASE(GetRepeatsFilteringDatabases)
         BOOST_REQUIRE_MESSAGE(itr->second, msg);
     }
 }
-
+#endif
 BOOST_AUTO_TEST_CASE(GetWindowMaskedTaxIds)
 {
     // Uncomment to redirect to test system
@@ -242,7 +242,7 @@ BOOST_AUTO_TEST_CASE(GetDatabaseInfo)
     BOOST_REQUIRE(dbinfo.NotEmpty());
     BOOST_REQUIRE(dbinfo->GetDatabase() == *blastdb);
 
-    const string swissprot_title("Non-redundant SwissProt sequences");
+    const string swissprot_title("Non-redundant UniProtKB/SwissProt sequences.");
     BOOST_REQUIRE_EQUAL(swissprot_title, dbinfo->GetDescription());
 
     BOOST_REQUIRE(dbinfo->GetTotal_length() > (Int8)7e+7);
@@ -261,9 +261,9 @@ BOOST_AUTO_TEST_CASE(GetDatabaseInfo)
     BOOST_REQUIRE(dbinfo.NotEmpty());
 
     BOOST_REQUIRE(dbinfo->GetDatabase() == *blastdb);
-    BOOST_REQUIRE(dbinfo->GetDescription().empty());
-    BOOST_REQUIRE_EQUAL(string("12/12/2002 14:17:00"),
-                         dbinfo->GetLast_updated());
+    //BOOST_REQUIRE(dbinfo->GetDescription().empty());
+    //BOOST_REQUIRE_EQUAL(string("12/12/2002 14:17:00"),
+    //                     dbinfo->GetLast_updated());
     BOOST_REQUIRE_EQUAL((Int8)400, dbinfo->GetNum_sequences());
     BOOST_REQUIRE_EQUAL((Int8)4662239, dbinfo->GetTotal_length());
 
