@@ -1,4 +1,4 @@
-/* $Id: cleanup.cpp 347817 2011-12-21 13:09:53Z kornbluh $
+/* $Id: cleanup.cpp 366493 2012-06-14 16:55:00Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -378,6 +378,8 @@ const char* const CCleanupChange::sm_ChangeDesc[eNumberofChangeTypes + 1] = {
     "Trim Flanking Quotes",
     "Clean Bioseq Title",
     "Decode XML",
+    "Remove Dup BioSource",
+    "Clean Org-ref",
 
     // set when any other change is made.
     "Change Other", 
