@@ -369,6 +369,7 @@ private:
     void x_SetFrameFromLoc( CCdregion &cdregion, const CSeq_loc &location );
 
     void x_CleanupECNumber( string &ec_num );
+    void x_CleanupECNumberList( CProt_ref::TEc & ec_num_list );
 
     void x_CleanupAndRepairInference( string &inference );
 
@@ -434,6 +435,15 @@ private:
 
     void x_DecodeXMLMarkChanged( std::string & str );
 
+private:
+    void x_SortSeqDescs( CSeq_entry & seq_entry );
+
+    void x_RemoveDupBioSource( CBioseq & bioseq );
+    void x_RemoveDupBioSource( CBioseq_set & bioseq_set );
+
+    void x_RemoveProtDescThatDupsProtName( CProt_ref & prot );
+    void x_RemoveRedundantComment( CGene_ref& gene, CSeq_feat& seq_feat );
+
 protected:
 
     CRef<CCleanupChange>  m_Changes;
@@ -445,6 +455,7 @@ protected:
     bool                  m_IsGpipe;
 
     friend class CAutogeneratedCleanup;
+    friend class CAutogeneratedExtendedCleanup;
 };
 
 
