@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_DATA_LOADERS_BLASTDB___CACHED_SEQUENCE__HPP
 #define OBJTOOLS_DATA_LOADERS_BLASTDB___CACHED_SEQUENCE__HPP
 
-/*  $Id: cached_sequence.hpp 219674 2011-01-12 20:09:13Z vasilche $
+/*  $Id: cached_sequence.hpp 368104 2012-07-02 20:05:54Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -75,7 +75,7 @@ public:
     ///
     /// @param idmap
     ///   A map from CSeq_id_Handle to OID.
-    void RegisterIds(CBlastDbDataLoader::TIds & idmap);
+    void RegisterIds(CBlastDbDataLoader::TIdMap & idmap);
     
     /// A list of 'chunk' objects, generic sequence related data elements.
     typedef vector< CRef<CTSE_Chunk_Info> > TCTSE_Chunk_InfoVector;
