@@ -1,4 +1,4 @@
-/*  $Id: reader_id2_base.cpp 347885 2011-12-21 18:20:59Z vasilche $
+/*  $Id: reader_id2_base.cpp 370295 2012-07-27 16:38:16Z vasilche $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -176,11 +176,13 @@ struct SId2LoadedSet
     typedef pair<int, TBlob_ids> TBlob_idsInfo;
     typedef map<CSeq_id_Handle, TBlob_idsInfo> TBlob_idSet;
     typedef map<CBlob_id, CConstRef<CID2_Reply_Data> > TSkeletons;
+    typedef map<CBlob_id, int> TBlobStates;
 
     TStringSet  m_Seq_idsByString;
     TSeq_idSet  m_Seq_ids;
     TBlob_idSet m_Blob_ids;
     TSkeletons  m_Skeletons;
+    TBlobStates m_BlobStates;
 };
 
 
@@ -391,7 +393,7 @@ bool CId2ReaderBase::LoadSeq_idTaxId(CReaderRequestResult& result,
 
     if ( !ids->IsLoadedTaxId() ) {
         m_AvoidRequest |= fAvoidRequest_for_Seq_id_taxid;
-        return false;
+        return true; // repeat
     }
 
     return true;
@@ -1191,7 +1193,7 @@ void CId2ReaderBase::x_SetContextData(CID2_Request& request)
     }
     if ( rctx.IsSetHitID() ) {
         CRef<CID2_Param> param(new CID2_Param);
-        param->SetName("log:phid");
+        param->SetName("log:ncbi_phid");
         param->SetValue().push_back(rctx.GetHitID());
         request.SetParams().Set().push_back(param);
     }
@@ -1564,6 +1566,34 @@ CId2ReaderBase::x_GetMessageError(const CID2_Reply& reply)
 }
 
 
+CReader::TBlobState
+CId2ReaderBase::x_GetBlobState(const CID2_Reply& reply,
+                               TErrorFlags* errors_ptr)
+{
+    TBlobState blob_state = 0;
+    TErrorFlags errors = x_GetMessageError(reply);
+    if ( errors_ptr ) {
+        *errors_ptr = errors;
+    }
+    if ( errors & fError_no_data ) {
+        blob_state |= CBioseq_Handle::fState_no_data;
+        if ( errors & fError_restricted ) {
+            blob_state |= CBioseq_Handle::fState_confidential;
+        }
+        if ( errors & fError_withdrawn ) {
+            blob_state |= CBioseq_Handle::fState_withdrawn;
+        }
+    }
+    if ( errors & fError_warning_dead ) {
+        blob_state |= CBioseq_Handle::fState_dead;
+    }
+    if ( errors & fError_warning_suppressed ) {
+        blob_state |= CBioseq_Handle::fState_suppress_perm;
+    }
+    return blob_state;
+}
+
+
 void CId2ReaderBase::x_ProcessReply(CReaderRequestResult& result,
                                     SId2LoadedSet& loaded_set,
                                     const CID2_Reply& reply)
@@ -1801,17 +1831,11 @@ void CId2ReaderBase::x_ProcessGetBlobId(
 {
     const CSeq_id& seq_id = reply.GetSeq_id();
     CSeq_id_Handle idh = CSeq_id_Handle::GetHandle(seq_id);
-    TErrorFlags errors = x_GetMessageError(main_reply);
-    if ( errors & fError_no_data ) {
-        int state = CBioseq_Handle::fState_no_data;
-        if ( errors & fError_restricted ) {
-            state |= CBioseq_Handle::fState_confidential;
-        }
-        if ( errors & fError_withdrawn ) {
-            state |= CBioseq_Handle::fState_withdrawn;
-        }
+    TErrorFlags errors;
+    TBlobState blob_state = x_GetBlobState(main_reply, &errors);
+    if ( blob_state & CBioseq_Handle::fState_no_data ) {
         CLoadLockBlob_ids ids(result, idh, 0);
-        ids->SetState(state);
+        ids->SetState(blob_state);
         SetAndSaveSeq_idBlob_ids(result, idh, 0, ids);
         return;
     }
@@ -1822,6 +1846,9 @@ void CId2ReaderBase::x_ProcessGetBlobId(
     }
     const CID2_Blob_Id& src_blob_id = reply.GetBlob_id();
     CBlob_id blob_id = GetBlobId(src_blob_id);
+    if ( blob_state ) {
+        loaded_set.m_BlobStates[blob_id] |= blob_state;
+    }
     TContentsMask mask = 0;
     {{ // TODO: temporary logic, this info should be returned by server
         if ( blob_id.GetSubSat() == CID2_Blob_Id::eSub_sat_main ) {
@@ -1921,16 +1948,9 @@ void CId2ReaderBase::x_ProcessGetBlob(
         return;
     }
 
-    TErrorFlags errors = x_GetMessageError(main_reply);
-    if ( errors & fError_no_data ) {
-        int state = CBioseq_Handle::fState_no_data;
-        if ( errors & fError_restricted ) {
-            state |= CBioseq_Handle::fState_confidential;
-        }
-        if ( errors & fError_withdrawn ) {
-            state |= CBioseq_Handle::fState_withdrawn;
-        }
-        blob.SetBlobState(state);
+    TBlobState blob_state = x_GetBlobState(main_reply);
+    if ( blob_state & CBioseq_Handle::fState_no_data ) {
+        blob.SetBlobState(blob_state);
         SetAndSaveNoBlob(result, blob_id, chunk_id, blob);
         _ASSERT(CProcessor::IsLoaded(blob_id, chunk_id, blob));
         return;
@@ -1938,6 +1958,9 @@ void CId2ReaderBase::x_ProcessGetBlob(
 
     if ( !reply.IsSetData() ) {
         // assume only blob info reply
+        if ( blob_state ) {
+            loaded_set.m_BlobStates[blob_id] |= blob_state;
+        }
         return;
     }
 
@@ -1957,15 +1980,6 @@ void CId2ReaderBase::x_ProcessGetBlob(
         return;
     }
 
-    TBlobState blob_state = 0;
-    if ( errors & fError_warning_dead ) {
-        //ERR_POST_X(7, "blob.SetBlobState(CBioseq_Handle::fState_dead)");
-        blob_state = CBioseq_Handle::fState_dead;
-    }
-    if ( errors & fError_warning_suppressed ) {
-        //ERR_POST_X(8, "blob.SetBlobState(CBioseq_Handle::fState_suppress_perm)");
-        blob_state = CBioseq_Handle::fState_suppress_perm;
-    }
     if ( blob_state ) {
         m_Dispatcher->SetAndSaveBlobState(result, blob_id, blob, blob_state);
     }
@@ -1992,7 +2006,7 @@ void CId2ReaderBase::x_ProcessGetBlob(
 void CId2ReaderBase::x_ProcessGetSplitInfo(
     CReaderRequestResult& result,
     SId2LoadedSet& loaded_set,
-    const CID2_Reply& /*main_reply*/,
+    const CID2_Reply& main_reply,
     const CID2S_Reply_Get_Split_Info& reply)
 {
     TChunkId chunk_id = CProcessor::kMain_ChunkId;
@@ -2019,6 +2033,21 @@ void CId2ReaderBase::x_ProcessGetSplitInfo(
         return;
     }
 
+    TBlobState blob_state = x_GetBlobState(main_reply);
+    if ( blob_state & CBioseq_Handle::fState_no_data ) {
+        blob.SetBlobState(blob_state);
+        SetAndSaveNoBlob(result, blob_id, chunk_id, blob);
+        _ASSERT(CProcessor::IsLoaded(blob_id, chunk_id, blob));
+        return;
+    }
+    {{
+        SId2LoadedSet::TBlobStates::iterator iter =
+            loaded_set.m_BlobStates.find(blob_id);
+        if ( iter != loaded_set.m_BlobStates.end() ) {
+            blob_state |= iter->second;
+        }
+    }}
+
     CConstRef<CID2_Reply_Data> skel;
     {{
         SId2LoadedSet::TSkeletons::iterator iter =
@@ -2028,6 +2057,10 @@ void CId2ReaderBase::x_ProcessGetSplitInfo(
         }
     }}
 
+    if ( blob_state ) {
+        m_Dispatcher->SetAndSaveBlobState(result, blob_id, blob, blob_state);
+    }
+
     dynamic_cast<const CProcessor_ID2&>
         (m_Dispatcher->GetProcessor(CProcessor::eType_ID2))
         .ProcessData(result, blob_id, blob->GetBlobState(), chunk_id,
