@@ -1,4 +1,4 @@
-/*  $Id: request_result.cpp 347885 2011-12-21 18:20:59Z vasilche $
+/*  $Id: request_result.cpp 368697 2012-07-11 16:45:03Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -198,6 +198,12 @@ bool CLoadInfoSeq_ids::IsLoadedTaxId(void)
     if ( m_TaxIdLoaded ) {
         return true;
     }
+    if ( IsLoaded() && (m_State & CBioseq_Handle::fState_no_data) ) {
+        // update no taxid for unknown sequences
+        m_TaxId = 0;
+        m_TaxIdLoaded = true;
+        return true;
+    }
     return false;
 }
 
