@@ -1,4 +1,4 @@
-/*  $Id: comment_item.cpp 347918 2011-12-21 20:07:23Z kans $
+/*  $Id: comment_item.cpp 371260 2012-08-07 14:57:55Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -30,6 +30,9 @@
 *
 */
 #include <ncbi_pch.hpp>
+
+#include <sstream>
+
 #include <corelib/ncbistd.hpp>
 
 #include <objects/seqfeat/Seq_feat.hpp>
@@ -134,6 +137,20 @@ CCommentItem::CCommentItem(const CSeq_feat& feat, CBioseqContext& ctx) :
     }       
 }
 
+CCommentItem::CCommentItem(const CUser_object & userObject, CBioseqContext& ctx) :
+    CFlatItem(&ctx), 
+    m_CommentInternalIndent(0),
+    m_First(false), 
+    m_NeedPeriod(true)
+{
+    swap(m_First, sm_FirstComment);
+    x_SetObject(userObject);
+    x_GatherInfo(ctx);
+    if ( x_IsCommentEmpty() ) {
+        x_SetSkip();
+    }
+}
+
 
 void CCommentItem::Format
 (IFormatter& formatter,
@@ -408,6 +425,8 @@ CCommentItem::TRefTrackStatus CCommentItem::GetRefTrackStatus
             retval = eRefTrackStatus_Model;
         } else if (NStr::EqualNocase(status, "WGS")) {
             retval = eRefTrackStatus_WGS;
+        } else if (NStr::EqualNocase(status, "TSA")) {
+            retval = eRefTrackStatus_TSA;
         }
 
         if ( st != 0  &&  retval != eRefTrackStatus_Unknown ) {
@@ -422,7 +441,8 @@ CCommentItem::TRefTrackStatus CCommentItem::GetRefTrackStatus
 string CCommentItem::GetStringForRefTrack
 (const CUser_object& uo,
  const CBioseq_Handle& bsh,
- ECommentFormat format)
+ ECommentFormat format,
+ EGenomeBuildComment eGenomeBuildComment )
 {
     if ( !uo.IsSetType()  ||  !uo.GetType().IsStr()  ||
          uo.GetType().GetStr() != "RefGeneTracking") {
@@ -500,60 +520,66 @@ string CCommentItem::GetStringForRefTrack
 
     CNcbiOstrstream oss;
     if (status == eRefTrackStatus_Pipeline) {
-        oss << ( format == eFormat_Html ? kRefSeqInformationLink : kRefSeqInformation ) << ": ";
+        oss << ( format == eFormat_Html ? kRefSeqInformationLink : kRefSeqInformation ) << ":";
     } else {
         oss << status_str << ' ' 
-            << ( format == eFormat_Html ? kRefSeqLink : kRefSeq ) << ": ";
+            << ( format == eFormat_Html ? kRefSeqLink : kRefSeq ) << ":";
     }
     switch ( status ) {
     case eRefTrackStatus_Inferred:
-        oss << "This record is predicted by genome sequence analysis and is "
+        oss << " This record is predicted by genome sequence analysis and is "
             << "not yet supported by experimental evidence.";
         break;
     case eRefTrackStatus_Pipeline:
-        if ( !build_num.empty() ) {
-            oss << "Features on this sequence have been produced for build "
-                << build_num << " of the NCBI's genome annotation"
-                << " [see ";
-            if( format == eFormat_Html ) {
-                oss << "<a href=\"" << strDocLink << "\">" ;
+        if( eGenomeBuildComment == eGenomeBuildComment_Yes ) {
+            if ( !build_num.empty() ) {
+                oss << " Features on this sequence have been produced for build "
+                    << build_num << " of the NCBI's genome annotation"
+                    << " [see ";
+                if( format == eFormat_Html ) {
+                    oss << "<a href=\"" << strDocLink << "\">" ;
+                }
+                oss << "documentation";
+                if( format == eFormat_Html ) {
+                    oss << "</a>";
+                }
+                oss << "].";
+            } else {
+                oss << " NCBI contigs are derived from assembled genomic sequence data.~"
+                    << "Also see:~"
+                    << "    Documentation of NCBI's Annotation Process~ ";
             }
-            oss << "documentation";
-            if( format == eFormat_Html ) {
-                oss << "</a>";
-            }
-            oss << "].";
-        } else {
-            oss << "NCBI contigs are derived from assembled genomic sequence data."
-                << "~Also see:~"
-                << "    Documentation of NCBI's Annotation Process~ ";
         }
         break;
     case eRefTrackStatus_Provisional:
         if (collaborator.empty()) {
-            oss << "This record has not yet been subject to final NCBI review.";
+            oss << " This record has not yet been subject to final NCBI review.";
         } else {
-            oss << "This record is based on preliminary "
+            oss << " This record is based on preliminary "
                 "annotation provided by " << collaborator << '.';
         }
         break;
     case eRefTrackStatus_Predicted:
-        oss << "This record has not been reviewed and the function is unknown.";
+        oss << " This record has not been reviewed and the function is unknown.";
         break;
     case eRefTrackStatus_Validated:
-        oss << "This record has undergone validation or preliminary review.";
+        oss << " This record has undergone validation or preliminary review.";
         break;
     case eRefTrackStatus_Reviewed:
-        oss << "This record has been curated by " 
+        oss << " This record has been curated by " 
             << (collaborator.empty() ? "NCBI staff" : collaborator) << '.';
         break;
     case eRefTrackStatus_Model:
-        oss << "This record is predicted by automated computational analysis.";
+        oss << " This record is predicted by automated computational analysis.";
         break;
     case eRefTrackStatus_WGS:
-        oss << "This record is provided to represent a collection of "
+        oss << " This record is provided to represent a collection of "
             << "whole genome shotgun sequences.";
         break;
+    case eRefTrackStatus_TSA:
+        oss << " This record is provided to represent a collection of "
+            << "transcriptome shotgun assembly sequences.";
+        break;
     default:
         break;
     }
@@ -676,6 +702,80 @@ string CCommentItem::GetStringForWGS(CBioseqContext& ctx)
     return CNcbiOstrstreamToString(text);
 }
 
+string CCommentItem::GetStringForTSA(CBioseqContext& ctx)
+{
+    static const string default_str = "?";
+
+    if (!ctx.IsTSAMaster()) {
+        return kEmptyStr;
+    }
+
+    const string& tsaaccn = ctx.GetTSAMasterAccn();
+    const string& tsaname = ctx.GetTSAMasterName();
+
+    if (NStr::IsBlank(tsaaccn)  ||  NStr::IsBlank(tsaname)) {
+        return kEmptyStr;
+    }
+
+    const string* taxname = &default_str;
+    for (CSeqdesc_CI it(ctx.GetHandle(), CSeqdesc::e_Source); it; ++it) {
+        const CBioSource& src = it->GetSource();
+        if (src.IsSetOrg()  &&  src.GetOrg().IsSetTaxname()  &&
+            !NStr::IsBlank(src.GetOrg().GetTaxname()) ) {
+            taxname = &(src.GetOrg().GetTaxname());
+        }
+    }
+
+    const string* first = &default_str, *last = &default_str;
+    for (CSeqdesc_CI it(ctx.GetHandle(), CSeqdesc::e_User); it; ++it) {
+        const CUser_object& uo = it->GetUser();
+        if (uo.IsSetType()  &&  uo.GetType().IsStr()  &&
+            NStr::EqualNocase(uo.GetType().GetStr(), "TSA-mRNA-List")) {
+            if (uo.HasField("Accession_first")) {
+                const CUser_field& uf = uo.GetField("Accession_first");
+                if (uf.IsSetData()  &&  uf.GetData().IsStr()  &&
+                    !NStr::IsBlank(uf.GetData().GetStr()) ) {
+                    first = &(uf.GetData().GetStr());
+                }
+            } else if (uo.HasField("TSA_accession_first")) {
+                const CUser_field& uf = uo.GetField("TSA_accession_first");
+                if (uf.IsSetData()  &&  uf.GetData().IsStr()  &&
+                    !NStr::IsBlank(uf.GetData().GetStr()) ) {
+                    first = &(uf.GetData().GetStr());
+                }
+            }
+            if (uo.HasField("Accession_last")) {
+                const CUser_field& uf = uo.GetField("Accession_last");
+                if (uf.IsSetData()  &&  uf.GetData().IsStr()  &&
+                    !NStr::IsBlank(uf.GetData().GetStr())) {
+                    last = &(uf.GetData().GetStr());
+                }
+            } else if (uo.HasField("TSA_accession_last")) {
+                const CUser_field& uf = uo.GetField("TSA_accession_last");
+                if (uf.IsSetData()  &&  uf.GetData().IsStr()  &&
+                    !NStr::IsBlank(uf.GetData().GetStr())) {
+                    last = &(uf.GetData().GetStr());
+                }
+            }
+        }
+    }
+
+    string version = (tsaname.length() == 15) ? 
+        tsaname.substr(7, 2) : tsaname.substr(4, 2);
+
+    CNcbiOstrstream text;
+    text << "The " << *taxname 
+         << " transcriptome shotgun assembly (TSA) project has the project accession " 
+         << tsaaccn << ".  This version of the project (" << version 
+         << ") has the accession number " << tsaname << ",";
+    if (*first != *last) {
+        text << " and consists of sequences " << *first << "-" << *last << ".";
+    } else {
+        text << " and consists of sequence " << *first << ".";
+    }
+
+    return CNcbiOstrstreamToString(text);
+}
 
 string CCommentItem::GetStringForMolinfo(const CMolInfo& mi, CBioseqContext& ctx)
 {
@@ -766,7 +866,7 @@ string CCommentItem::GetStringForHTGS(CBioseqContext& ctx)
                  << "* are represented as runs of N. The order of the pieces~"
                  << "* is believed to be correct as given, however the sizes~"
                  << "* of the gaps between them are based on estimates that have~"
-                 << "* provided by the submittor.";
+                 << "* provided by the submitter.";
         }
         text << "~* This sequence will be replaced~"
              << "* by the finished sequence as soon as it is available and~"
@@ -785,9 +885,25 @@ string CCommentItem::GetStringForHTGS(CBioseqContext& ctx)
 }
 
 static
-string s_HtmlWrapModelEvidenceName( const string &name )
+string s_HtmlWrapModelEvidenceName( const SModelEvidance& me )
 {
-    return "<a href=\"" + strLinkBaseNuc + name + "?report=graph\">" + name + "</a>";
+    stringstream strm;
+    strm << "<a href=\"" << strLinkBaseNuc;
+    if( me.gi > 0 ) {
+        strm << me.gi;
+    } else {
+        strm << me.name;
+    }
+    strm << "?report=graph";
+    if( (me.span.first >= 0) && (me.span.second >= me.span.first) ) {
+        const Int8 kPadAmount = 500;
+        // The "+1" is because we display 1-based to user and in URL
+        strm << "&v=" << max<Int8>(me.span.first + 1 - kPadAmount, 1) 
+             << ":" << (me.span.second + 1 + kPadAmount); // okay if second number goes over end of sequence
+    }
+    strm << "\">" << me.name << "</a>";
+
+    return strm.str();
 }
 
 string CCommentItem::GetStringForModelEvidance
@@ -800,7 +916,7 @@ string CCommentItem::GetStringForModelEvidance
 
     CNcbiOstrstream text;
 
-    const string me_name = ( bHtml ? s_HtmlWrapModelEvidenceName(me.name) : me.name );
+    const string me_name = ( bHtml ? s_HtmlWrapModelEvidenceName(me) : me.name );
 
     text << "MODEL " << *refseq << ":  " << "This record is predicted by "
          << "automated computational analysis. This record is derived from "
@@ -936,6 +1052,11 @@ void CCommentItem::x_GatherInfo(CBioseqContext& ctx)
         const CSeq_feat* feat = dynamic_cast<const CSeq_feat*>(obj);
         if (feat != NULL) {
             x_GatherFeatInfo(*feat, ctx);
+        } else {
+            const CUser_object * userObject = dynamic_cast<const CUser_object*>(obj);
+            if(userObject != NULL) {
+                x_GatherUserObjInfo(*userObject);
+            }
         }
     }
 }
@@ -951,7 +1072,7 @@ string s_HtmlizeStructuredCommentData( const bool is_html, const string &label_s
     if( label_str == "GOLD Stamp ID" && NStr::StartsWith(data_str, "Gi") ) {
         CNcbiOstrstream result;
         result << "<a href=\"http://genomesonline.org/cgi-bin/GOLD/bin/GOLDCards.cgi?goldstamp=" << data_str 
-               << ".html\">" << data_str << "</a>";
+               << "\">" << data_str << "</a>";
         return CNcbiOstrstreamToString(result);
     } else {
         // normalize case: nothing to do
@@ -1062,7 +1183,7 @@ void CCommentItem::x_GatherDescInfo(const CSeqdesc& desc)
                 str = desc.GetComment();
                 TrimSpacesAndJunkFromEnds(str, true);
                 ConvertQuotes(str);
-                if( ! NStr::EndsWith(str, ".") ) {
+                if( ! NStr::EndsWith(str, ".") && ! NStr::EndsWith(str, "/") && ! NStr::EndsWith(str, "~") ) {
                     str += '.';
                 }
             }
@@ -1143,6 +1264,17 @@ void CCommentItem::x_GatherFeatInfo(const CSeq_feat& feat, CBioseqContext& ctx)
     x_SetCommentWithURLlinks(kEmptyStr, feat.GetComment(), kEmptyStr, ePeriod_Add);
 }
 
+void CCommentItem::x_GatherUserObjInfo(const CUser_object& userObject )
+{
+    // make sure the user object is really of type StructuredComment
+    const CUser_object::TType &type = userObject.GetType();
+    if( type.IsStr() && type.GetStr() == "StructuredComment" ) {
+        s_GetStrForStructuredComment( userObject.GetData(),  
+            m_Comment, m_CommentInternalIndent, IsFirst(), GetContext()->Config().DoHTML() );
+        SetNeedPeriod( false );
+    }
+}
+
 
 void CCommentItem::x_SetSkip(void)
 {
@@ -1381,7 +1513,7 @@ void CHistComment::x_GatherInfo(CBioseqContext& ctx)
 
     switch ( m_Type ) {
     case eReplaced_by:
-        if( ctx.IsWGSMaster() ) {
+        if( ctx.IsWGSMaster() || ctx.IsTSAMaster() ) {
             x_SetComment(s_CreateHistCommentString(
                 "[WARNING] On",
                 "this project was updated. The new version is",
