@@ -1,4 +1,4 @@
-/*  $Id: context.cpp 346027 2011-12-02 13:43:47Z kornbluh $
+/*  $Id: context.cpp 372931 2012-08-23 16:50:20Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -40,6 +40,8 @@
 #include <objects/seq/Seg_ext.hpp>
 #include <objects/seq/Delta_ext.hpp>
 #include <objects/seq/Delta_seq.hpp>
+#include <objects/seq/Annot_descr.hpp>
+#include <objects/seq/Annotdesc.hpp>
 #include <objects/seqset/Bioseq_set.hpp>
 #include <objects/seqloc/Textseq_id.hpp>
 #include <objects/seqloc/Patent_seq_id.hpp>
@@ -56,6 +58,7 @@
 #include <objmgr/seq_map_ci.hpp>
 #include <objmgr/feat_ci.hpp>
 #include <objmgr/bioseq_ci.hpp>
+#include <objmgr/annot_ci.hpp>
 
 #include <objtools/format/context.hpp>
 
@@ -97,10 +100,13 @@ CBioseqContext::CBioseqContext
     m_RefseqInfo(0),
     m_IsGbGenomeProject(false),  // GenBank Genome project data (AE)
     m_IsNcbiCONDiv(false),       // NCBI CON division (CH)
+    m_IsNcbiGenomes(false),
     m_IsPatent(false),
     m_IsGI(false),
     m_IsWGS(false),
     m_IsWGSMaster(false),
+    m_IsTSA(false),
+    m_IsTSAMaster(false),
     m_IsHup(false),
     m_Gi(0),
     m_ShowGBBSource(false),
@@ -108,7 +114,8 @@ CBioseqContext::CBioseqContext
     m_HasOperon(false),
     m_HasMultiIntervalGenes(true), // true is the safe choice if we're not sure
     m_IsGenomeAssembly(false),
-    m_eUnverified(eUnverified_None),
+    m_fUnverified(fUnverified_None),
+    m_ShowAnnotCommentAsCOMMENT(false),
     m_FFCtx(ffctx),
     m_Master(mctx),
     m_TLSeqEntryCtx(tlsec)
@@ -148,10 +155,13 @@ CBioseqContext::CBioseqContext
     m_RefseqInfo(0),
     m_IsGbGenomeProject(false),  // GenBank Genome project data (AE)
     m_IsNcbiCONDiv(false),       // NCBI CON division (CH)
+    m_IsNcbiGenomes(false),
     m_IsPatent(false),
     m_IsGI(false),
     m_IsWGS(false),
     m_IsWGSMaster(false),
+    m_IsTSA(false),
+    m_IsTSAMaster(false),
     m_IsHup(false),
     m_Gi(0),
     m_ShowGBBSource(false),
@@ -159,7 +169,8 @@ CBioseqContext::CBioseqContext
     m_HasOperon(false),
     m_HasMultiIntervalGenes(true), // true is the safe choice if we're not sure
     m_IsGenomeAssembly(false),
-    m_eUnverified(eUnverified_None),
+    m_fUnverified(fUnverified_None),
+    m_ShowAnnotCommentAsCOMMENT(false),
     m_FFCtx(ffctx),
     m_Master(mctx),
     m_TLSeqEntryCtx(tlsec)
@@ -197,8 +208,8 @@ void CBioseqContext::x_Init(const CBioseq_Handle& seq, const CSeq_loc* user_loc)
     _ASSERT(seq.IsSetInst());
 
     // NB: order of execution is important
-    x_SetId();
     m_Repr = x_GetRepr();
+    x_SetId();
     m_Mol  = seq.GetInst_Mol();
     m_Molinfo.Reset(x_GetMolInfo());
 
@@ -224,6 +235,8 @@ void CBioseqContext::x_Init(const CBioseq_Handle& seq, const CSeq_loc* user_loc)
     m_Encode.Reset(x_GetEncode());
 
     x_SetDataFromUserObjects();
+
+    x_SetDataFromAnnot();
     
     m_HasOperon = x_HasOperon();
 
@@ -402,8 +415,6 @@ void CBioseqContext::x_SetDataFromUserObjects(void)
                     }
                 }
             } else if(NStr::EqualNocase(uo.GetType().GetStr(), "Unverified")) {
-                m_eUnverified = eUnverified_SequenceOrAnnotation;
-                // see if it's an alternate kind of unverified
                 if( uo.IsSetData() ) {
                     ITERATE( CUser_object::TData, field_iter, uo.GetData() ) {
                         const CUser_field &field = **field_iter;
@@ -417,10 +428,62 @@ void CBioseqContext::x_SetDataFromUserObjects(void)
                             continue;
                         }
                         if( NStr::EqualNocase( field.GetData().GetStr(), "Organism") ) {
-                            m_eUnverified = eUnverified_Organism;
+                            m_fUnverified |= fUnverified_Organism;
+                        } else if( NStr::EqualNocase( field.GetData().GetStr(), "Features") ) {
+                            m_fUnverified |= fUnverified_SequenceOrAnnotation;
                         }
                     }
                 }
+                // default in the past was to use feature
+                if( m_fUnverified == fUnverified_None ) {
+                    m_fUnverified = fUnverified_SequenceOrAnnotation;
+                }
+            }
+        }
+    }
+}
+
+void CBioseqContext::x_SetDataFromAnnot(void)
+{
+    CAnnot_CI annot_ci(m_Handle);
+    for( ; annot_ci; ++annot_ci ) {
+        if( ! annot_ci->Seq_annot_IsSetDesc() ) {
+            continue;
+        }
+
+        const CSeq_annot::TDesc & desc = annot_ci->Seq_annot_GetDesc();
+        ITERATE( CSeq_annot::TDesc::Tdata, one_desc_iter, desc.Get() ) {
+            const CAnnotdesc & one_desc = **one_desc_iter;
+            if( ! one_desc.IsUser() ) {
+                continue;
+            }
+
+            // we finally got down to an annot desc user object.  See if it indicates any
+            // relevant information
+            const CUser_object & user_obj = one_desc.GetUser();
+            if( ! user_obj.IsSetType() || ! user_obj.GetType().IsStr() || 
+                ! user_obj.IsSetData() ||
+                user_obj.GetType().GetStr() != "AnnotDescCommentPolicy" )
+            {
+                continue;
+            }
+
+            // check policy flags
+            ITERATE( CUser_object::TData, policy_field_iter, user_obj.GetData() ) {
+                const CUser_field & policy_field = **policy_field_iter;
+                if( ! policy_field.IsSetLabel() || ! policy_field.GetLabel().IsStr() ||
+                    ! policy_field.IsSetData() ||
+                    policy_field.GetLabel().GetStr() != "Policy" )
+                {
+                    continue;
+                }
+
+                if( policy_field.GetData().IsStr() ) { 
+                    const string & policy_str = policy_field.GetData().GetStr();
+                    if( policy_str == "ShowInComment" ) {
+                        m_ShowAnnotCommentAsCOMMENT = true;
+                    }
+                }
             }
         }
     }
@@ -531,6 +594,9 @@ void CBioseqContext::x_SetId(void)
                 if ( !NStr::CompareCase(id.GetGeneral().GetDb(), "BankIt") ) {
                     m_IsTPA = bTpaAssemblyPresent;
                 }
+                if( NStr::Equal(id.GetGeneral().GetDb(), "NCBI_GENOMES") ) {
+                    m_IsNcbiGenomes = true;
+                }
             }
             break;
         case CSeq_id::e_Gibbsq:
@@ -564,6 +630,21 @@ void CBioseqContext::x_SetId(void)
             }
         } 
 
+        // TSA
+        m_IsTSA = m_IsTSA  ||  (acc_div == CSeq_id::eAcc_tsa);
+        
+        if ( m_IsTSA  &&  !acc.empty() ) {
+            size_t len = acc.length();
+            m_IsTSAMaster = 
+                ( ((len == 12  ||  len == 15)  &&  NStr::EndsWith(acc, "000000"))  ||
+                  (len == 13  &&  NStr::EndsWith(acc, "0000000")) ) &&
+                ( m_Repr == CSeq_inst::eRepr_virtual );
+            if ( m_IsTSAMaster ) {
+                m_TSAMasterAccn = acc;
+                m_TSAMasterName = tsip->CanGetName() ? tsip->GetName() : kEmptyStr;
+            }
+        } 
+
         // GBB source
         m_ShowGBBSource = m_ShowGBBSource  ||  (acc_info == CSeq_id::eAcc_gsdb_dirsub);
     }
@@ -875,6 +956,7 @@ void CMasterContext::x_SetBaseName(void)
 CTopLevelSeqEntryContext::CTopLevelSeqEntryContext( const CSeq_entry_Handle &entry_handle )
 {
     m_CanSourcePubsBeFused = false;
+    m_HasSmallGenomeSet = false;
 
     CBioseq_CI bioseq_iter( entry_handle.GetScope(), *entry_handle.GetSeq_entryCore() );
     for( ; bioseq_iter; ++bioseq_iter ) {
@@ -918,6 +1000,19 @@ CTopLevelSeqEntryContext::CTopLevelSeqEntryContext( const CSeq_entry_Handle &ent
             }
         }
     }
+
+    // check all Bioseq-sets, if any
+    if( entry_handle.IsSet() ) {
+        CSeq_entry_CI seq_entry_ci( entry_handle );
+        for( ; seq_entry_ci; ++seq_entry_ci ) {
+            if( seq_entry_ci->IsSet() && seq_entry_ci->GetSet().CanGetClass() &&
+                seq_entry_ci->GetSet().GetClass() == CBioseq_set::eClass_small_genome_set ) 
+            {
+                m_HasSmallGenomeSet = true;
+                break;
+            }
+        }
+    }
 }
 
 
