@@ -1,4 +1,4 @@
-/*  $Id: genbank_formatter.cpp 345628 2011-11-29 12:54:48Z kornbluh $
+/*  $Id: genbank_formatter.cpp 366473 2012-06-14 16:48:38Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -58,6 +58,7 @@
 #include <objtools/format/items/basecount_item.hpp>
 #include <objtools/format/items/sequence_item.hpp>
 #include <objtools/format/items/wgs_item.hpp>
+#include <objtools/format/items/tsa_item.hpp>
 #include <objtools/format/items/primary_item.hpp>
 #include <objtools/format/items/contig_item.hpp>
 #include <objtools/format/items/genome_item.hpp>
@@ -232,7 +233,7 @@ void CGenbankFormatter::FormatLocus
 
     string units = "bp";
     if ( !ctx.IsProt() ) {
-        if ( ctx.IsWGSMaster()  && ! ctx.IsRSWGSNuc() ) {
+        if ( ( ctx.IsWGSMaster() || ctx.IsTSAMaster() )  && ! ctx.IsRSWGSNuc() ) {
             units = "rc";
         }
     } else {
@@ -949,10 +950,11 @@ CGenbankFormatter::x_LocusHtmlPrefix( string &first_line, CBioseqContext& ctx )
     {{
         // we split the if-statement into little local vars for ease of reading
         const bool is_wgs_master = ( ctx.IsWGSMaster() && ctx.GetTech() == CMolInfo::eTech_wgs );
+        const bool is_tsa_master = ( ctx.IsTSAMaster() && ctx.GetTech() == CMolInfo::eTech_tsa && ctx.GetBiomol() == CMolInfo::eBiomol_mRNA );
         const bool do_contig_style = ctx.DoContigStyle();
         const bool show_contig = ( (ctx.IsSegmented()  &&  ctx.HasParts())  ||
                                    (ctx.IsDelta()  &&  ! ctx.IsDeltaLitOnly()) );
-        if( ! is_wgs_master && (do_contig_style || ( ctx.Config().ShowContigAndSeq() && show_contig )) ) {
+        if( ! is_wgs_master && ! is_tsa_master && (do_contig_style || ( ctx.Config().ShowContigAndSeq() && show_contig )) ) {
             has_contig = true;
         }
     }}
@@ -1258,7 +1260,7 @@ string s_GetLinkFeatureKey(
 
     // location
     string strLocation;
-    if( item.GetFeat().GetLocation().IsInt() ) {
+    if( item.GetFeat().GetLocation().IsInt() || item.GetFeat().GetLocation().IsPnt() ) {
         strLocation += "?from=";
         strLocation += NStr::IntToString( iFrom );
         strLocation += "&amp;to=";
@@ -1747,23 +1749,107 @@ void CGenbankFormatter::FormatWGS
 
     const bool bHtml = wgs.GetContext()->Config().DoHTML();
 
+    // Get first and last id (sanitized for html, if necessary)
     list<string> l;
     string first_id = wgs.GetFirstID();
     if( bHtml ) {
         TryToSanitizeHtml( first_id );
     }
+    string last_id;
+    bool first_id_equals_second_id = false;
     if ( wgs.GetFirstID() == wgs.GetLastID() ) {
-        Wrap(l, tag, first_id);
+        last_id = first_id;
+        first_id_equals_second_id = true;
     } else {
-        string last_id = wgs.GetLastID();
+        last_id = wgs.GetLastID();
         if( bHtml ) {
             TryToSanitizeHtml( last_id );
         }
-        Wrap(l, tag, first_id + "-" + last_id);
     }
+
+    string wgs_line = ( first_id_equals_second_id ? first_id : first_id + "-" + last_id );
+
+    // surround wgs_line with a link, if necessary
+    if( bHtml ) {
+        string link;
+        if( first_id_equals_second_id ) {
+            link = "http://www.ncbi.nlm.nih.gov/nuccore/" + first_id;
+        } else {
+            string url_arg;
+            if( CWGSItem::eWGS_Projects == wgs.GetType() ) {
+                if( first_id.length() > 2 && first_id[2] == '_' ) {
+                    url_arg = first_id.substr(0, 9);
+                } else {
+                    url_arg = first_id.substr(0, 6);
+                }
+                link = "http://www.ncbi.nlm.nih.gov/Traces/wgs?val=" + url_arg;
+            } else {
+                link = "http://www.ncbi.nlm.nih.gov/nuccore?term=" + first_id + ":" + last_id + "[PACC]";
+            }
+        }
+        wgs_line = "<a href=\"" + link + "\">" + wgs_line + "</a>";
+    }
+
+    Wrap( l, tag, wgs_line, ePara, bHtml );
+
     text_os.AddParagraph(l, wgs.GetObject());
 }
 
+///////////////////////////////////////////////////////////////////////////
+//
+// TSA
+
+void CGenbankFormatter::FormatTSA
+(const CTSAItem& tsa,
+ IFlatTextOStream& text_os)
+{
+    string tag;
+
+    switch ( tsa.GetType() ) {
+    case CTSAItem::eTSA_Projects:
+        tag = "TSA";
+        break;
+
+    default:
+        return;
+    }
+
+    const bool bHtml = tsa.GetContext()->Config().DoHTML();
+
+    list<string> l;
+    string first_id = tsa.GetFirstID();
+    if( bHtml ) {
+        TryToSanitizeHtml( first_id );
+    }
+    string id_range;
+    if ( tsa.GetFirstID() == tsa.GetLastID() ) {
+        id_range = first_id;
+    } else {
+        string last_id = tsa.GetLastID();
+        id_range = first_id + "-" + last_id;
+    }
+
+    if( bHtml ) {
+        TryToSanitizeHtml( id_range );
+
+        string tsa_master = tsa.GetContext()->GetTSAMasterName();
+        if( tsa_master.length() > 2 && tsa_master[2] == '_' ) {
+            tsa_master = tsa_master.substr(0, 9);
+        } else {
+            tsa_master = tsa_master.substr(0, 6);
+        }
+        TryToSanitizeHtml(tsa_master);
+        if( ! tsa_master.empty() ) {
+            id_range = "<a href=\"http://www.ncbi.nlm.nih.gov/Traces/wgs?val=" + tsa_master + "\">" + id_range + "</a>";
+        }
+    }
+
+    Wrap(l, tag, id_range, ePara, bHtml);
+
+    text_os.AddParagraph(l, tsa.GetObject());
+}
+
+
 
 ///////////////////////////////////////////////////////////////////////////
 //
