@@ -1,4 +1,4 @@
-/*  $Id: gff3_formatter.cpp 337096 2011-09-08 18:01:35Z ludwigf $
+/*  $Id: gff3_formatter.cpp 363540 2012-05-17 14:22:23Z whlavina $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -57,6 +57,9 @@
 
 #define NCBI_USE_ERRCODE_X   Objtools_Fmt_GFF
 
+//#define GFF3_USE_CIGAR_FORMATTER 1
+
+
 BEGIN_NCBI_SCOPE
 BEGIN_SCOPE(objects)
 USING_SCOPE(sequence);
@@ -123,6 +126,7 @@ protected:
                             TSeqPos seg_len);
 
 private:
+    const CAlignmentItem&       m_Alignment;
     IFlatTextOStream&           m_Out;
     CGFF3_Formatter&            m_GFF3_Fmt;
     auto_ptr<CNcbiOstrstream>   m_Attrs;
@@ -131,9 +135,11 @@ private:
 
 
 CGFF3_CIGAR_Formatter::CGFF3_CIGAR_Formatter(CGFF3_Formatter&       gff3,
-                                             const CAlignmentItem& aln,
-                                             IFlatTextOStream&     text_os)
-    : CCIGAR_Formatter(aln),
+                                             const CAlignmentItem&  aln,
+                                             IFlatTextOStream&      text_os)
+    : CCIGAR_Formatter(aln.GetAlign(),
+                       &aln.GetContext()->GetScope()),
+      m_Alignment(aln),
       m_Out(text_os),
       m_GFF3_Fmt(gff3),
       m_Attrs(new CNcbiOstrstream)
@@ -143,15 +149,12 @@ CGFF3_CIGAR_Formatter::CGFF3_CIGAR_Formatter(CGFF3_Formatter&       gff3,
 
 void CGFF3_CIGAR_Formatter::EndSubAlignment(void)
 {
-    if (GetSeq_align().GetSegs().IsDisc() && GetConfig().GffGenerateIdTags()) {
-        ++m_GFF3_Fmt.m_CurrentId; // ?????
-    }
 }
 
 
 void CGFF3_CIGAR_Formatter::StartRow(void)
 {
-    const CFlatFileConfig& config = GetConfig();
+    const CFlatFileConfig& config = m_Alignment.GetContext()->Config();
 
     // We can't use x_FormatAttr because we seem to need a mix
     // of literal pluses, which we otherwise avoid due to ambiguity,
@@ -210,7 +213,7 @@ void CGFF3_CIGAR_Formatter::AddRow(const string& cigar)
 
 void CGFF3_CIGAR_Formatter::EndRow(void)
 {
-    CBioseqContext ctx = GetContext();
+    CBioseqContext& ctx = *m_Alignment.GetContext();
     // XXX - should supply appropriate score, if any
     CSeq_loc loc(*ctx.GetPrimaryId(),
         GetRefRange().GetFrom(), GetRefRange().GetTo(),
@@ -368,9 +371,12 @@ void CGFF3_Formatter::FormatAlignment(const CAlignmentItem& aln,
 {
 #ifdef GFF3_USE_CIGAR_FORMATTER
     CGFF3_CIGAR_Formatter cigar(*this, aln, text_os);
-    cigar.FormatAlignmentRows();
+    cigar.FormatByReferenceId(*aln.GetContext()->GetPrimaryId());
 #else
     x_FormatAlignment(aln, text_os, aln.GetAlign(), true, false);
+    if ( aln.GetContext()->Config().GffGenerateIdTags() ) {
+        ++m_CurrentId;
+    }
 #endif
 }
 
@@ -435,9 +441,6 @@ void CGFF3_Formatter::x_FormatAlignment(const CAlignmentItem& aln,
                    "Conversion of alignments of type dendiag and packed "
                    "not supported in current GFF3 CIGAR output");
     }
-    if ( config.GffGenerateIdTags() ) {
-        ++m_CurrentId;
-    }
 }
 
 
@@ -728,10 +731,10 @@ void CGFF3_Formatter::x_FormatDenseg(const CAlignmentItem& aln,
             }
             if (count) {
                 if ( 0 != cigar.pcount() ) {
+                   trivial = false;
                    cigar << '+';
                 }
                 cigar << type << count;
-                trivial = false;
             }
             if (frameshift) {
                 trivial = false;
@@ -746,7 +749,7 @@ void CGFF3_Formatter::x_FormatDenseg(const CAlignmentItem& aln,
         CConstRef<CSeq_id> tgt_id =
             s_GetTargetId(alnmap.GetSeqId(tgt_row), scope);
         if ( config.GffGenerateIdTags() ) {
-            attrs << "ID=" << m_CurrentId++ << ";";
+            attrs << "ID=" << m_CurrentId << ";";
         }
         attrs << "Target=";
         // GFF3 specs require %09 escape for spaces in the Target,
