@@ -1,4 +1,4 @@
-/* $Id: inst_info_map.cpp 350179 2012-01-18 17:19:09Z rafanovi $
+/* $Id: inst_info_map.cpp 372023 2012-08-14 18:37:18Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -53,7 +53,7 @@ CInstInfoMap::GetInstitutionVoucherInfo(
     static const string s_ccap_base("http://www.ccap.ac.uk/strain_info.php?Strain_No=");
     static const string s_ccmp_base("https://ccmp.bigelow.org/node/1/strain/CCMP");
     static const string s_ccug_base("http://www.ccug.se/default.cfm?page=search_record.cfm&db=mc&s_tests=1&ccugno=");
-    static const string s_cori_base("http://ccr.coriell.org/Sections/Search/Sample_Detail.aspx?Ref=");
+    static const string s_cori_base("http://ccr.coriell.org/Sections/Search/Search.aspx?q=");
     static const string s_dsmz_base("http://www.dsmz.de/catalogues/details/culture/DSM-");
     static const string s_fsu_base("http://www.prz.uni-jena.de/data.php?fsu=");
     static const string s_kctc_base("http://www.brc.re.kr/English/_SearchView.aspx?sn=");
@@ -108,6 +108,8 @@ CInstInfoMap::GetInstitutionVoucherInfo(
         TVoucherInfoElem("KU:I",             TVoucherInfoRef(new SVoucherInfo(&s_ku_base,   false, &s_kui_pfx,    NULL,        "University of Kansas, Museum of Natural History, Ichthyology collection") ) ),
         TVoucherInfoElem("KU:IT",            TVoucherInfoRef(new SVoucherInfo(&s_ku_base,   false, &s_kuit_pfx,   NULL,        "University of Kansas, Museum of Natural History, Ichthyology tissue collection") ) ),
         TVoucherInfoElem("KWP:Ento",         TVoucherInfoRef(new SVoucherInfo(&s_uam_base,  true,  &s_colon_pfx,  NULL,        "Kenelm W. Philip Collection, University of Alaska Museum of the North, Lepidoptera collection") ) ),
+        TVoucherInfoElem("MLZ:Bird",         TVoucherInfoRef(new SVoucherInfo(&s_uam_base,  true,  &s_colon_pfx,  NULL,        "Moore Laboratory of Zoology, Occidental College, Bird Collection" ) ) ),
+        TVoucherInfoElem("MLZ:Mamm",         TVoucherInfoRef(new SVoucherInfo(&s_uam_base,  true,  &s_colon_pfx,  NULL,        "Moore Laboratory of Zoology, Occidental College, Mammal Collection" ) ) ),
         TVoucherInfoElem("MSB:Bird",         TVoucherInfoRef(new SVoucherInfo(&s_uam_base,  true,  &s_colon_pfx,  NULL,        "Museum of Southwestern Biology, Bird Collection") ) ),
         TVoucherInfoElem("MSB:Mamm",         TVoucherInfoRef(new SVoucherInfo(&s_uam_base,  true,  &s_colon_pfx,  NULL,        "Museum of Southwestern Biology, Mammal Collection") ) ),
         TVoucherInfoElem("MSB:Para",         TVoucherInfoRef(new SVoucherInfo(&s_uam_base,  true,  &s_colon_pfx,  NULL,        "Museum of Southwestern Biology, Parasitology Collection") ) ),
