@@ -1,4 +1,4 @@
-/*  $Id: item_formatter.cpp 330368 2011-08-11 15:09:07Z kornbluh $
+/*  $Id: item_formatter.cpp 360035 2012-04-19 13:43:48Z kornbluh $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -222,6 +222,16 @@ string  CFlatItemFormatter::x_FormatAccession
         }
     }
 
+    if ( ctx.IsTSA() && ! acc.GetTSAAccession().empty() ) {
+        const bool is_html = ctx.Config().DoHTML();
+        if( is_html ) {
+            acc_line << separator << "<a href=\"" << strLinkBaseNucSearch << acc.GetTSAAccession() << 
+                "\">" << acc.GetTSAAccession() << "</a>";
+        } else {
+            acc_line << separator << acc.GetTSAAccession();
+        }
+    }
+
     if (!acc.GetExtraAccessions().empty()) {
         s_FormatSecondaryAccessions(acc_line, acc.GetExtraAccessions(), separator);
     }
@@ -550,10 +560,10 @@ static void s_FixPages( string& pages )
             ++it;
         }
 
-        const bool firstNumberEmpty = (firstNumber[0]   == '\0');
-        const bool firstTextEmpty = (firstText[0]   == '\0');
-        const bool lastNumberEmpty = (lastNumber[0]   == '\0');
-        const bool lastTextEmpty = (lastText[0]   == '\0');
+        const bool firstNumberEmpty = firstNumber.empty();
+        const bool firstTextEmpty = firstText.empty();
+        const bool lastNumberEmpty = lastNumber.empty();
+        const bool lastTextEmpty = lastText.empty();
         
         if( (lastNumberEmpty  && firstTextEmpty) ||
             (firstNumberEmpty && lastTextEmpty) ) {
