@@ -1,4 +1,4 @@
-/*  $Id: qualifiers.cpp 346027 2011-12-02 13:43:47Z kornbluh $
+/*  $Id: qualifiers.cpp 361622 2012-05-03 13:26:51Z kornbluh $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -264,21 +264,6 @@ static size_t s_ComposeCodonRecognizedStr(const CTrna_ext& trna, string& recogni
     return size;
 }
 
-// makes sure str is of the pattern "[number]..[number]" (e.g. "34..405" )
-bool s_RangeStringIsPlainNumber( const string & str ) {
-
-    // join in anticodon quarantined pending collab approval
-    if( str.find("join") != string::npos ||
-        str.find("order") != string::npos ||
-        str.find("complement") != string::npos )
-    {
-            return false;
-    }
-
-    return true;
-}
-
-
 /////////////////////////////////////////////////////////////////////////////
 // CFormatQual - low-level formatted qualifier
 
@@ -337,13 +322,13 @@ CFlatStringQVal::CFlatStringQVal(const string& value,
 }
 
 
-typedef pair<const char*, ETildeStyle> TNameTildeStylePair;
-typedef CStaticArrayMap<const char*, ETildeStyle, PCase_CStr > TNameTildeStyleMap;
+typedef SStaticPair<const char*, ETildeStyle> TNameTildeStylePair;
+typedef CStaticPairArrayMap<const char*, ETildeStyle, PCase_CStr > TNameTildeStyleMap;
 static const TNameTildeStylePair kNameTildeStyleMap[] = {
-    TNameTildeStylePair("function",     eTilde_tilde),
-    TNameTildeStylePair("prot_desc",    eTilde_note),
-    TNameTildeStylePair("prot_note",    eTilde_note),
-    TNameTildeStylePair("seqfeat_note", eTilde_note)
+    { "function",     eTilde_tilde },
+    { "prot_desc",    eTilde_note },
+    { "prot_note",    eTilde_note },
+    { "seqfeat_note", eTilde_note }
 };
 DEFINE_STATIC_ARRAY_MAP(TNameTildeStyleMap, sc_NameTildeStyleMap, kNameTildeStyleMap);
 
@@ -587,14 +572,9 @@ void CFlatCodeBreakQVal::Format(TFlatQuals& q, const string& name,
             return;
         }
 
-        // Example accession where we have to iterate due to multiple pieces:
-        // NW_001468136
-        CSeq_loc_CI loc_ci( (*it)->GetLoc() );
-        for( ; loc_ci; ++loc_ci ) {
-            string pos = CFlatSeqLoc( *loc_ci.GetRangeAsSeq_loc(), ctx).GetString();
-            x_AddFQ(q, name, "(pos:" + pos + ",aa:" + aa + ')', 
-                CFormatQual::eUnquoted);
-        }
+        string pos = CFlatSeqLoc( (*it)->GetLoc(), ctx).GetString();
+        x_AddFQ(q, name, "(pos:" + pos + ",aa:" + aa + ')', 
+            CFormatQual::eUnquoted);
     }
 }
 
@@ -1312,15 +1292,6 @@ void CFlatXrefQVal::Format(TFlatQuals& q, const string& name,
             }
             // trim
             TrimSpacesAndJunkFromEnds( id, true );
-            // remove initial/final parens
-            if( ! id.empty() ) {
-                if( id[0] == '(' ) {
-                    id = id.substr(1);
-                }
-                if( id[id.length()-1] == ')') {
-                    id.resize( id.length() - 1 );
-                }
-            }
         }
         if (NStr::IsBlank(id)) {
             continue;
@@ -1568,13 +1539,7 @@ void CFlatAnticodonQVal::Format
         return;
     }
 
-    // anticodons with complement, join, etc. ( e.g. L15362 ) are not supported in release mode, until collab approval
     string locationString = CFlatSeqLoc(*m_Anticodon, ctx).GetString();
-    if( ctx.Config().IsModeRelease() ) {
-        if( ! s_RangeStringIsPlainNumber(locationString) ) {
-            return;
-        }
-    }
 
     CNcbiOstrstream text;
     text << "(pos:" ;
