@@ -1,4 +1,4 @@
-/*  $Id: reference_item.cpp 303782 2011-06-13 16:55:07Z kornbluh $
+/*  $Id: reference_item.cpp 361103 2012-04-30 14:18:40Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1205,7 +1205,7 @@ void CReferenceItem::x_CapitalizeTitleIfNecessary()
 
 // these must be in "ASCIIbetical" order; beware of the fact that
 // closing quotes sort after spaces.
-static const string sc_RemarkText[] = {
+static const char* const sc_RemarkText[] = {
   "full automatic",
   "full staff_entry",
   "full staff_review",
@@ -1216,12 +1216,13 @@ static const string sc_RemarkText[] = {
   "unannotated staff_entry",
   "unannotated staff_review"
 };
-DEFINE_STATIC_ARRAY_MAP(CStaticArraySet<string>, sc_Remarks, sc_RemarkText);
+typedef CStaticArraySet<const char*, PCase_CStr> TStaticRemarkSet;
+DEFINE_STATIC_ARRAY_MAP(TStaticRemarkSet, sc_Remarks, sc_RemarkText);
 
 
 void CReferenceItem::x_GatherRemark(CBioseqContext& ctx)
 {
-    const static string kDoiLink = "http://dx.doi.org/";
+    static const char* const kDoiLink = "http://dx.doi.org/";
 
     list<string> l;
 
@@ -1230,7 +1231,7 @@ void CReferenceItem::x_GatherRemark(CBioseqContext& ctx)
     if ( m_Pubdesc->IsSetComment()  &&  !m_Pubdesc->GetComment().empty() ) {
         const string& comment = m_Pubdesc->GetComment();
         
-        if ( sc_Remarks.find(comment) == sc_Remarks.end() ) {
+        if ( sc_Remarks.find(comment.c_str()) == sc_Remarks.end() ) {
             l.push_back(comment);
         }
     }
