@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___UTILS_HPP
 #define OBJTOOLS_FORMAT___UTILS_HPP
 
-/*  $Id: utils.hpp 303782 2011-06-13 16:55:07Z kornbluh $
+/*  $Id: utils.hpp 366477 2012-06-14 16:49:45Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -113,13 +113,17 @@ const string& GetTechString(int tech);
 
 struct SModelEvidance
 {
-    string name;
-    string method;
-    bool mrnaEv;
-    bool estEv;
+    typedef std::pair<Int8, Int8> TSpanType;
+
+    string     name;
+    string     method;
+    bool       mrnaEv;
+    bool       estEv;
+    int        gi;
+    TSpanType  span; // start, then end. 0-based
 
     SModelEvidance(void) :
-        name(kEmptyStr), method(kEmptyStr), mrnaEv(false), estEv(false)
+        name(kEmptyStr), method(kEmptyStr), mrnaEv(false), estEv(false), gi(-1), span(-1, -1)
     {}
 };
 
