@@ -1,4 +1,4 @@
-/*  $Id: agp_read.cpp 346740 2011-12-09 16:48:38Z ivanov $
+/*  $Id: agp_read.cpp 372932 2012-08-23 16:50:52Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -113,6 +113,11 @@ void AgpRead(CNcbiIstream& is,
             continue;
         }
 
+        // remove Windows-style endline, if it exists
+        if( ! line.empty() && *line.rbegin() == '\r' ) {
+            line.resize( line.size() - 1 );
+        }
+
         // split into fields, as delimited by tabs
         fields.clear();
         NStr::Tokenize(line, "\t", fields);
