@@ -1,4 +1,4 @@
-/*  $Id: agp_util.cpp 348453 2011-12-29 15:31:46Z sapojnik $
+/*  $Id: agp_util.cpp 356094 2012-03-12 14:14:32Z sapojnik $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,6 +33,11 @@
 #include <ncbi_pch.hpp>
 #include <objtools/readers/agp_util.hpp>
 
+#ifndef i2s
+#define i2s(x) NStr::NumericToString(x)
+#define s2i(x) NStr::StringToNumeric(x)
+#endif
+
 BEGIN_NCBI_SCOPE
 
 //// class CAgpErr
@@ -196,7 +201,7 @@ int CAgpErr::AppliesTo(int mask)
 // (unless they follow a previous error message).
 void CAgpErr::Msg(int code, const string& details, int appliesTo)
 {
-    // Append warnings to preexisting errors.
+     // Append warnings to the previously reported errors.
     // To collect all warnings, override Msg() in the derived class.
     if(code<E_Last || m_apply_to) {
         m_apply_to |= appliesTo;
@@ -419,8 +424,10 @@ int CAgpRow::FromString(const string& line)
                     return CAgpErr::E_ColumnCount;
                 }
                 else
-                */
+
+                not important enough:
                 m_AgpErr->Msg( CAgpErr::W_GapLineMissingCol9);
+                */
             }
             if( m_agp_version == eAgpVersion_2_0 && cols.size()==8 ) {
                 // just to make sure no out-of-bounds array accesses
@@ -1137,7 +1144,8 @@ void CAgpErrEx::PrintAllMessages(CNcbiOstream& out)
     for(int i=W_First; i<W_Last; i++) {
         out << GetPrintableCode(i) << "\t" << GetMsg(i);
         if(i==W_GapLineMissingCol9) {
-            out << " (only the total count is printed unless you specify: -only " << GetPrintableCode(i) << ")";
+            out << " (no longer reported)";
+            //out << " (only the total count is printed unless you specify: -only " << GetPrintableCode(i) << ")";
         }
         out << "\n";
     }
@@ -1188,6 +1196,18 @@ void CAgpErrEx::PrintLine(CNcbiOstream& ostr,
     ostr<< linenum  << ":" << line << "\n";
 }
 
+void CAgpErrEx::PrintLineXml(CNcbiOstream& ostr,
+    const string& filename, int linenum, const string& content,
+    bool two_lines_involved)
+{
+    string attr="num=\""+i2s(linenum)+"\"";
+    if(filename.size()) attr+=" filename=\"" + NStr::XmlEncode(filename) + "\"";
+    if(two_lines_involved) attr+=" two_lines=\"true\"";
+
+    ostr << " <line " << attr << ">" << NStr::XmlEncode(content) << "</line>\n";
+
+}
+
 void CAgpErrEx::PrintMessage(CNcbiOstream& ostr, int code,
         const string& details)
 {
@@ -1197,9 +1217,27 @@ void CAgpErrEx::PrintMessage(CNcbiOstream& ostr, int code,
     << ": " << FormatMessage( GetMsg(code), details ) << "\n";
 }
 
+void CAgpErrEx::PrintMessageXml(CNcbiOstream& ostr, int code, const string& details, int appliesTo)
+{
+    ostr<< " <message severity=\"" << (
+        (code<W_First || code>W_Last) ? "ERROR" :
+        (code==W_ShortGap || code==W_AssumingVersion) ? "NOTE" : "WARNING"
+    ) << "\"";
+    if(code <=E_LastToSkipLine) ostr << " line_skipped=\"1\"";
+    ostr<<">\n";
+
+    ostr << " <code>"     << GetPrintableCode(code) << "</code>\n";
+    if(appliesTo & CAgpErr::fAtPpLine  ) ostr << " <line_num>" << m_line_num_pp    << "</line_num>\n";
+    if(appliesTo & CAgpErr::fAtPrevLine) ostr << " <line_num>" << m_line_num_prev  << "</line_num>\n";
+    if(appliesTo & CAgpErr::fAtThisLine) ostr << " <line_num>current</line_num>\n";
+    ostr << " <text>" << NStr::XmlEncode( FormatMessage( GetMsg(code), details ) ) << "</text>\n";
+
+    ostr << "</message>\n";
+}
+
 
 //// class CAgpErrEx - constructor
-CAgpErrEx::CAgpErrEx(CNcbiOstream* out) : m_out(out)
+CAgpErrEx::CAgpErrEx(CNcbiOstream* out, bool use_xml) : m_use_xml(use_xml), m_out(out)
 {
     m_messages = new CNcbiOstrstream();
     m_MaxRepeat = 0; // no limit
@@ -1216,12 +1254,16 @@ CAgpErrEx::CAgpErrEx(CNcbiOstream* out) : m_out(out)
 
     memset(m_MustSkip , 0, sizeof(m_MustSkip ));
     ResetTotals();
+
     // errors that are "silenced" by default (only the count is printed)
     m_MustSkip[W_GapLineMissingCol9]=true;
-    m_MustSkip[W_ExtraTab          ]=true;
-    m_MustSkip[W_CompIsWgsTypeIsNot]=true;
-    m_MustSkip[W_CompIsNotWgsTypeIs]=true;
-    m_MustSkip[W_ShortGap          ]=true;
+    if(!use_xml) // perhaps, we should have a separate parameter for hiding these...
+    {
+        m_MustSkip[W_ExtraTab          ]=true;
+        m_MustSkip[W_CompIsWgsTypeIsNot]=true;
+        m_MustSkip[W_CompIsNotWgsTypeIs]=true;
+        m_MustSkip[W_ShortGap          ]=true;
+    }
 
     // A "random check" to make sure enum and msg[] are not out of skew.
     //cerr << sizeof(msg)/sizeof(msg[0]) << "\n";
@@ -1253,10 +1295,6 @@ void CAgpErrEx::ResetTotals()
 
 void CAgpErrEx::Msg(int code, const string& details, int appliesTo)
 {
-    // Ignore a possibly spurious errors generated after
-    // an unacceptable line (e.g. a line with wrong # of columns)
-    //if(m_invalid_prev && (appliesTo&CAgpErrEx::fAtSkipAfterBad) ) return;
-
     // Suppress some messages while still counting them
     m_MsgCount[code]++;
     if( m_MustSkip[code]) {
@@ -1272,11 +1310,17 @@ void CAgpErrEx::Msg(int code, const string& details, int appliesTo)
     if(appliesTo & CAgpErr::fAtPpLine) {
         // Print the line before previous if it was not printed
         if( !m_pp_printed && m_line_pp.size() ) {
-            // if( !m_two_lines_involved ) ??
-            *m_out << "\n";
-            PrintLine(*m_out,
-                m_filenum_pp>=0 ? m_InputFiles[m_filenum_pp] : NcbiEmptyString,
-                m_line_num_pp, m_line_pp);
+            if(m_use_xml) {
+                PrintLineXml(*m_out,
+                    m_filenum_pp>=0 ? m_InputFiles[m_filenum_pp] : NcbiEmptyString,
+                    m_line_num_pp, m_line_pp, m_two_lines_involved);
+            }
+            else {
+                *m_out << "\n";
+                PrintLine(*m_out,
+                    m_filenum_pp>=0 ? m_InputFiles[m_filenum_pp] : NcbiEmptyString,
+                    m_line_num_pp, m_line_pp);
+            }
         }
         m_pp_printed=true;
     }
@@ -1284,24 +1328,39 @@ void CAgpErrEx::Msg(int code, const string& details, int appliesTo)
     if(appliesTo & CAgpErr::fAtPrevLine) {
         // Print the previous line if it was not printed
         if( !m_prev_printed && m_line_prev.size() ) {
-            if( !m_two_lines_involved ) *m_out << "\n";
-            PrintLine(*m_out,
-                m_filenum_prev>=0 ? m_InputFiles[m_filenum_prev] : NcbiEmptyString,
-                m_line_num_prev, m_line_prev);
+            if(m_use_xml) {
+                PrintLineXml(*m_out,
+                    m_filenum_prev>=0 ? m_InputFiles[m_filenum_prev] : NcbiEmptyString,
+                    m_line_num_prev, m_line_prev, m_two_lines_involved);
+            }
+            else {
+                if( !m_two_lines_involved ) *m_out << "\n";
+                PrintLine(*m_out,
+                    m_filenum_prev>=0 ? m_InputFiles[m_filenum_prev] : NcbiEmptyString,
+                    m_line_num_prev, m_line_prev);
+            }
         }
         m_prev_printed=true;
     }
     if(appliesTo & CAgpErr::fAtThisLine) {
         // Accumulate messages
-        PrintMessage(*m_messages, code, details);
+        if(m_use_xml) {
+            PrintMessageXml(*m_messages, code, details, appliesTo);
+        }
+        else {
+            PrintMessage(*m_messages, code, details);
+        }
     }
     else {
         // Print it now (useful for appliesTo==CAgpErr::fAtPrevLine)
-
-        // E_NoValidLines
-        if(appliesTo==fAtNone && m_InputFiles.size() ) *m_out << m_InputFiles.back() << ":\n";
-
-        PrintMessage(*m_out, code, details);
+        if(m_use_xml) {
+            PrintMessageXml(*m_out, code, details, appliesTo);
+        }
+        else {
+            // E_NoValidLines
+            if(appliesTo==fAtNone && m_InputFiles.size() ) *m_out << m_InputFiles.back() << ":\n";
+            PrintMessage(*m_out, code, details);
+        }
     }
 
     if( (appliesTo&CAgpErr::fAtPrevLine) && (appliesTo&CAgpErr::fAtThisLine) ) m_two_lines_involved=true;
@@ -1310,10 +1369,24 @@ void CAgpErrEx::Msg(int code, const string& details, int appliesTo)
 void CAgpErrEx::LineDone(const string& s, int line_num, bool invalid_line)
 {
     if( m_messages->pcount() ) {
-        if( !m_two_lines_involved ) *m_out << "\n";
+        if(m_use_xml) {
+            PrintLineXml(*m_out, m_filename, line_num, s, m_two_lines_involved);
+        }
+        else {
+            if( !m_two_lines_involved ) *m_out << "\n";
+            PrintLine(*m_out, m_filename, line_num, s);
+        }
 
-        PrintLine(*m_out, m_filename, line_num, s);
-        *m_out << (string)CNcbiOstrstreamToString(*m_messages);
+        if(m_use_xml) {
+            string m;
+            NStr::Replace((string)CNcbiOstrstreamToString(*m_messages),
+              "<line_num>current</line_num>",
+              "<line_num>"+i2s(line_num)+"</line_num>", m);
+            *m_out << m;
+        }
+        else {
+            *m_out << (string)CNcbiOstrstreamToString(*m_messages);
+        }
         delete m_messages;
         m_messages = new CNcbiOstrstream;
 
@@ -1428,23 +1501,47 @@ void CAgpErrEx::PrintMessageCounts(CNcbiOstream& ostr, int from, int to, bool re
             ostr << "Internal error in CAgpErrEx::PrintMessageCounts().";
         }
     }
-    if(from<to) ostr<< setw(7) << "Count" << " Code  Description\n"; // code?
-    for(int i=from; i<to; i++) {
-        if( m_MsgCount[i] ) {
-            ostr<< setw(7) << m_MsgCount[i] << "  "
-                    << GetPrintableCode(i) << "  "
-                    << GetMsg(i) << "\n";
+
+    if(m_use_xml) {
+        for(int i=from; i<to; i++) {
+            if( m_MsgCount[i] ) {
+                ostr << "<msg_summary>\n";
+                ostr << " <code>" << GetPrintableCode(i)            << "</code>\n";
+                ostr << " <text>" << NStr::XmlEncode(GetMsg(i))     << "</text>\n";
+                ostr << " <cnt>"  << m_MsgCount[i]                  << "</cnt>\n";
+                ostr << "</msg_summary>\n";
+            }
         }
-        // ouside of previous "if" because one hint may apply to one or more of several consequitive warnings
-        // (such as W_CompIsWgsTypeIsNot and W_CompIsNotWgsTypeIs)
-        if(hints && (*hints).find(i)!=(*hints).end() ) {
-            ostr << "         " << (*hints)[i] << "\n";
+        // lines that we failed to parse because of syntax errors
+        ostr << " <invalid_lines>"  << m_lines_skipped << "</invalid_lines>\n";
+    }
+    else {
+        if(from<to) ostr<< setw(7) << "Count" << " Code  Description\n"; // code?
+        for(int i=from; i<to; i++) {
+            if( m_MsgCount[i] ) {
+                ostr<< setw(7) << m_MsgCount[i] << "  "
+                        << GetPrintableCode(i) << "  "
+                        << GetMsg(i) << "\n";
+            }
+            // ouside of previous "if" because one hint may apply to one or more of several consequitive warnings
+            // (such as W_CompIsWgsTypeIsNot and W_CompIsNotWgsTypeIs)
+            if(hints && (*hints).find(i)!=(*hints).end() ) {
+                ostr << "         " << (*hints)[i] << "\n";
+            }
+        }
+        if(m_lines_skipped && report_lines_skipped) {
+          ostr << "\nNOTE: " << m_lines_skipped <<
+            " invalid lines were skipped (not subjected to all the checks, not included in most of the counts below).\n";
         }
     }
-    if(m_lines_skipped && report_lines_skipped) {
-      ostr << "\nNOTE: " << m_lines_skipped <<
-        " invalid lines were skipped (not subjected to all the checks, not included in most of the counts below).\n";
-    }
+}
+
+void CAgpErrEx::PrintTotalsXml(CNcbiOstream& ostr, int e_count, int w_count, int note_count, int skipped_count)
+{
+    ostr << " <notes>"    << note_count    << "</notes>\n";
+    ostr << " <warnings>" << w_count       << "</warnings>\n";
+    ostr << " <errors>"   << e_count       << "</errors>\n";
+    ostr << " <skipped>"  << skipped_count << "</skipped>\n";
 }
 
 void CAgpErrEx::PrintTotals(CNcbiOstream& ostr, int e_count, int w_count, int skipped_count)
