@@ -1,4 +1,4 @@
-/*  $Id: agp_validate_reader.cpp 348682 2012-01-03 18:11:30Z sapojnik $
+/*  $Id: agp_validate_reader.cpp 356544 2012-03-14 20:02:47Z sapojnik $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -71,6 +71,7 @@ void CAgpValidateReader::Reset(bool for_chr_from_scaf)
 
   memset(m_CompOri, 0, sizeof(m_CompOri));
   memset(m_GapTypeCnt, 0, sizeof(m_GapTypeCnt));
+  m_ln_ev_flags2count.clear();
 
   if(for_chr_from_scaf) {
     NCBI_ASSERT(m_explicit_scaf, "m_explicit_scaf is false in CAgpValidateReader::Reset(true)");
@@ -171,6 +172,8 @@ void CAgpValidateReader::OnGapOrComponent()
       if(m_explicit_scaf && m_is_chr) {
         m_AgpErr->Msg(CAgpErrEx::E_WithinScafGap);
       }
+
+      m_ln_ev_flags2count[m_this_row->linkage_evidence_flags]++;
     }
     else {
       if(!m_at_beg && !m_prev_row->IsGap()) {
@@ -547,7 +550,115 @@ void CAgpValidateReader::OnObjectChange()
 }
 
 #define ALIGN_W(x) setw(w) << resetiosflags(IOS_BASE::left) << (x)
-void CAgpValidateReader::x_PrintTotals() // without comment counts
+#define ALIGN_M_W(x) setw(m_w) << resetiosflags(IOS_BASE::left) << (x)
+
+/** Output the count as text or as xml.
+ *  The text label could be transformed into XML tag as follows:
+ *  - uppercase first letter of each word;
+ *  - strip all spaces and non-alphabetic chars;
+ *  - if the first character of the label was non-alphanum:
+ *      prefix the tag with last_tag,
+ *    else: save the tag to last_tag (for possible future use).
+ */
+class XPrintTotalsItem
+{
+public:
+  CNcbiOstream& m_out;
+  bool m_use_xml;
+  int m_w;
+
+  bool m_strip_attrs;
+  string last_tag;
+  string m_eol_text; // usually "\n", but could be gap or comp types, e.g. "(w)", "(W:35, D:8)"
+
+  XPrintTotalsItem(CNcbiOstream& out, bool use_xml, int w) : m_out(out), m_use_xml(use_xml), m_w(w)
+  {
+    m_eol_text="\n";
+    m_strip_attrs=true; // "  clone   , linkage yes" => attribute type="clone"
+  }
+
+  // print a value line
+  void line(const string& label, const string& value, string xml_tag=NcbiEmptyString)
+  {
+    if(!m_use_xml) {
+      m_out << label;
+      if( label.find("***")==NPOS ) {
+        m_out << ALIGN_M_W(value);
+      }
+      else {
+        // no aligning for: ***single component + gap(s), ***no components, gaps only
+        m_out << value;
+      }
+
+      m_out << m_eol_text;
+      m_eol_text="\n"; // reset to default (AFTER printing whatever was there)
+    }
+    else {
+      if(xml_tag.size()==0) {
+        // false = add as a suffix
+        bool add_label_as_attribute = ( last_tag.size() && last_tag[last_tag.size()-1]=='=' );
+
+        if(!add_label_as_attribute || m_strip_attrs) {
+          bool uc = !add_label_as_attribute;
+
+          for(string::const_iterator it = label.begin();  it != label.end(); ++it) {
+            if(isalpha(*it)) {
+              xml_tag+= uc ? toupper(*it): tolower(*it);
+              uc = false;
+            }
+            else {
+              if(*it==',') break; // "***no components, gaps only: " -> "NoComponents"
+              uc = !add_label_as_attribute;
+            }
+          }
+        }
+        else {
+          // attribute pattern="AADB[02037555..02037659].1"
+          xml_tag = NStr::XmlEncode( NStr::TruncateSpaces(label));
+          if(xml_tag.size() && xml_tag[xml_tag.size()-1]==':') {
+            xml_tag.resize( xml_tag.size()-1 );
+            NStr::TruncateSpacesInPlace(xml_tag);
+          }
+        }
+        m_strip_attrs=true; // reset to default
+
+        if( add_label_as_attribute ) {
+          xml_tag = last_tag + "\""+ xml_tag + "\"";
+        }
+        else if(isalpha(label[0])) {
+          // may use as a prefix later
+          last_tag = xml_tag;
+        }
+        else {
+          // use the prior tag as a prefix, e.g. 'Objects' + 'WithSingleComponent'
+          xml_tag = last_tag + xml_tag;
+        }
+      }
+
+      m_out << " <" << xml_tag << ">" << NStr::XmlEncode(value);
+
+      // strip any attributes from the closing tag
+      SIZE_TYPE pos=xml_tag.find(" ");
+      if(pos!=NPOS) xml_tag.resize(pos);
+
+      m_out << "</" << xml_tag << ">\n";
+    }
+  }
+
+  void line(const string& label, int value, const string& xml_tag=NcbiEmptyString)
+  {
+    line(label, NStr::NumericToString(value), xml_tag);
+  }
+
+  // print an empty line or static text; no-op in XML
+  void line(const string& s=NcbiEmptyString)
+  {
+    if( !m_use_xml ) m_out << s << m_eol_text;
+    m_eol_text="\n"; // reset to default (AFTER printing whatever was there)
+  }
+};
+
+void CAgpValidateReader::x_PrintTotals(CNcbiOstream& out, bool use_xml) // without comment counts
 {
   //// Counts of errors and warnings
   int e_count=m_AgpErr->CountTotals(CAgpErrEx::E_Last);
@@ -559,38 +670,50 @@ void CAgpValidateReader::x_PrintTotals() // without comment counts
         e_count==m_AgpErr->CountTotals(CAgpErrEx::E_NoValidLines)
     ) return; // all files are empty, no need to say it again
 
-    cout << "\n";
-    m_AgpErr->PrintTotals(cout, e_count, w_count, m_AgpErr->m_msg_skipped);
-    if(m_AgpErr->m_MaxRepeatTopped) {
-      cout << " (to print all: -limit 0; to skip some: -skip CODE)";
+    CAgpErrEx::TMapCcodeToString hints;
+    if(use_xml) {
+      // jira/browse/GP-594: [iinsignificant warning are] making it hard for the naive user to know what to fix
+      // w_count -= m_AgpErr->CountTotals(CAgpErrEx::W_GapLineMissingCol9);
+      int note_count = m_AgpErr->CountTotals(CAgpErrEx::W_ShortGap) + m_AgpErr->CountTotals(CAgpErrEx::W_AssumingVersion);
+      m_AgpErr->PrintTotalsXml(out, e_count, w_count-note_count, note_count, m_AgpErr->m_msg_skipped);
     }
-    cout << ".";
-    if(m_AgpErr->m_MaxRepeat && (e_count+w_count) ) {
-      cout << "\n";
-      CAgpErrEx::TMapCcodeToString hints;
-      if(!m_CheckCompNames && (
-        m_AgpErr->CountTotals(CAgpErrEx::W_CompIsWgsTypeIsNot) ||
-        m_AgpErr->CountTotals(CAgpErrEx::W_CompIsNotWgsTypeIs)
-      ) ) {
-          // W_CompIsNotWgsTypeIs is the last numerically, so the hint whiil get printed
-          // after one or both of the above warnings
-          hints[CAgpErrEx::W_CompIsNotWgsTypeIs] =
-              "(Use -g to print lines with WGS component_id/component_type mismatch.)";
+    else {
+      out << "\n";
+      m_AgpErr->PrintTotals(out, e_count, w_count, m_AgpErr->m_msg_skipped);
+      if(m_AgpErr->m_MaxRepeatTopped) {
+        out << " (to print all: -limit 0; to skip some: -skip CODE)";
       }
-      if(m_AgpErr->CountTotals(CAgpErrEx::W_ShortGap)) {
-          hints[CAgpErrEx::W_ShortGap] = "(Use -show "+
-              CAgpErrEx::GetPrintableCode(CAgpErrEx::W_ShortGap)+
-              " to print lines with short gaps.)";
+      out << ".";
+      if(m_AgpErr->m_MaxRepeat && (e_count+w_count) ) {
+        out << "\n";
+        if(!m_CheckCompNames && (
+          m_AgpErr->CountTotals(CAgpErrEx::W_CompIsWgsTypeIsNot) ||
+          m_AgpErr->CountTotals(CAgpErrEx::W_CompIsNotWgsTypeIs)
+        ) ) {
+            // W_CompIsNotWgsTypeIs is the last numerically, so the hint whiil get printed
+            // after one or both of the above warnings
+            hints[CAgpErrEx::W_CompIsNotWgsTypeIs] =
+                "(Use -g to print lines with WGS component_id/component_type mismatch.)";
+        }
+        if(m_AgpErr->CountTotals(CAgpErrEx::W_ShortGap)) {
+            hints[CAgpErrEx::W_ShortGap] = "(Use -show "+
+                CAgpErrEx::GetPrintableCode(CAgpErrEx::W_ShortGap)+
+                " to print lines with short gaps.)";
+        }
       }
-      m_AgpErr->PrintMessageCounts(cout, CAgpErrEx::CODE_First, CAgpErrEx::CODE_Last, true, &hints);
     }
+    if(use_xml || (m_AgpErr->m_MaxRepeat && (e_count+w_count)) )
+      m_AgpErr->PrintMessageCounts(out, CAgpErrEx::CODE_First, CAgpErrEx::CODE_Last, true, &hints);
   }
   if(m_ObjCount==0) {
-    // cout << "No valid AGP lines.\n";
+    // out << "No valid AGP lines.\n";
     return;
   }
 
-  cout << "\n";
+  // w: width for right alignment
+  int w = NStr::IntToString((unsigned)(m_CompId2Spans.size())).size(); // +1;
+  XPrintTotalsItem xprint(out, use_xml, w);
+  xprint.line();
 
   //// Prepare component/gap types and counts for later printing
   string s_comp, s_gap;
@@ -613,69 +736,95 @@ void CAgpValidateReader::x_PrintTotals() // without comment counts
 
   //// Various counts of AGP elements
 
-  // w: width for right alignment
-  int w = NStr::IntToString((unsigned)(m_CompId2Spans.size())).size(); // +1;
-
-  cout << "\n"
-    "Objects                : "<<ALIGN_W(m_ObjCount           )<<"\n"
-    "- with single component: "<<ALIGN_W(m_SingleCompObjects  )<<"\n";
+  if(use_xml) out << "<stats>\n";
+  xprint.line();
+  xprint.line( "Objects                : ", m_ObjCount);
+  xprint.line( "- with single component: ", m_SingleCompObjects);
   if(m_SingleCompObjects_withGaps) {
-    cout << "  *** single component + gap(s): " << m_SingleCompObjects_withGaps<< "\n";
+    xprint.line( "  *** single component + gap(s): ", m_SingleCompObjects_withGaps, "SingleCompObjects_withGaps");
   }
-  cout << "\n"
-    "Scaffolds              : "<<ALIGN_W(m_ScaffoldCount      )<<"\n"
-    "- with single component: "<<ALIGN_W(m_SingleCompScaffolds)<<"\n";
+
+  xprint.line();
+  xprint.line( "Scaffolds              : ", m_ScaffoldCount);
+  xprint.line( "- with single component: ", m_SingleCompScaffolds);
   if(m_SingleCompScaffolds_withGaps) {
-    cout << "  ***single component + gap(s): " << m_SingleCompScaffolds_withGaps<< "\n";
+    // note: skipping ALIGN_W
+    xprint.line("  ***single component + gap(s): ", m_SingleCompScaffolds_withGaps, "SingleCompScaffolds_withGaps");
   }
   if(m_NoCompScaffolds) {
-    cout << "***no components, gaps only: " << m_NoCompScaffolds << "\n";
+    // note: skipping ALIGN_W
+    xprint.line("***no components, gaps only: ", m_NoCompScaffolds);
   }
-  cout << "\n";
+  xprint.line();
 
 
-  cout<<
-    "Components                   : "<< ALIGN_W(m_CompCount);
-  if(m_CompCount) {
-    if( s_comp.size() ) {
+
+  if( s_comp.size() ) {
+    if(use_xml) {
+      string comp_by_type = ", " + s_comp;
+      // ", W: 1234, D: 5678" => <Comp type="W">1234</Comp>\n<Comp type="D">5678</Comp>
+      NStr::ReplaceInPlace(comp_by_type, ", ", "</Comp>\n <Comp type=\"");
+      NStr::ReplaceInPlace(comp_by_type, ":", "\">");
+      // move "</Comp>\n" from start to end of string
+      comp_by_type = comp_by_type.substr(8) + comp_by_type.substr(0, 8);
+      out << comp_by_type;
+    }
+    else {
       if( NStr::Find(s_comp, ",")!=NPOS ) {
         // (W: 1234, D: 5678)
-        cout << " (" << s_comp << ")";
+        xprint.m_eol_text = " (" + s_comp + ")\n";
       }
       else {
         // One type of components: (W) or (invalid type)
-        cout << " (" << s_comp.substr( 0, NStr::Find(s_comp, ":") ) << ")";
+        xprint.m_eol_text = " (" + s_comp.substr( 0, NStr::Find(s_comp, ":") ) + ")\n";
       }
     }
-    cout << "\n"
-      "  orientation +              : " << ALIGN_W(m_CompOri[CCompVal::ORI_plus ]) << "\n"
-      "  orientation -              : " << ALIGN_W(m_CompOri[CCompVal::ORI_minus]) << "\n"
-      "  orientation ? (formerly 0) : " << ALIGN_W(m_CompOri[CCompVal::ORI_zero ]) << "\n"
-      "  orientation na             : " << ALIGN_W(m_CompOri[CCompVal::ORI_na   ]) << "\n";
+  }
+  xprint.line("Components                   : ", m_CompCount, string("Components type_counts=\"")+s_comp+"\"");
+
+  if(m_CompCount) {
+    xprint.line("  orientation +              : ", m_CompOri[CCompVal::ORI_plus ], "CompOri val=\"+\"");
+    xprint.line("  orientation -              : ", m_CompOri[CCompVal::ORI_minus], "CompOri val=\"-\"");
+    xprint.line("  orientation ? (formerly 0) : ", m_CompOri[CCompVal::ORI_zero ], "CompOri val=\"? (formerly 0)\"");
+    xprint.line("  orientation na             : ", m_CompOri[CCompVal::ORI_na   ], "CompOri val=\"na\"");
   }
   if( m_comp2range_coll->size() &&
       m_AgpErr->CountTotals(CAgpErrEx::G_NsWithinCompSpan)==0 &&
       m_CompCount > m_AgpErr->CountTotals(CAgpErrEx::G_InvalidCompId)
   ) {
-    cout << "Component spans in AGP are consistent with FASTA\n(i.e. do not include or intersect runs of Ns longer than 10 bp)." << endl;
+    xprint.line("Component spans in AGP are consistent with FASTA\n(i.e. do not include or intersect runs of Ns longer than 10 bp).");
   }
 
 
-  cout << "\n" << "Gaps                   : " << ALIGN_W(m_GapCount);
+  xprint.line();
   if(m_GapCount) {
     // Print (N) if all components are of one type,
     //        or (N: 1234, U: 5678)
     if( s_gap.size() ) {
-      if( NStr::Find(s_gap, ",")!=NPOS ) {
-        // (N: 1234, U: 5678)
-        cout << " (" << s_gap << ")";
+      if(use_xml) {
+        string gap_u_n = ", " + s_gap;
+        // ", U: 1234, N: 5678" => <Gap u_n="U">1234</Gap>\n<Gap u_n="N">5678</Gap>
+        NStr::ReplaceInPlace(gap_u_n, ", ", "</Gap>\n <Gap u_n=\"");
+        NStr::ReplaceInPlace(gap_u_n, ":", "\">");
+        // move "</Gap>\n" from start to end of string
+        gap_u_n = gap_u_n.substr(7) + gap_u_n.substr(0, 7);
+        out << gap_u_n;
       }
       else {
-        // One type of gaps: (N)
-        cout << " (" << s_gap.substr( 0, NStr::Find(s_gap, ":") ) << ")";
+        if( NStr::Find(s_gap, ",")!=NPOS ) {
+          // (N: 1234, U: 5678)
+          xprint.m_eol_text =  " (" + s_gap + ")\n";
+        }
+        else {
+          // One type of gaps: (N)
+          xprint.m_eol_text =  " (" + s_gap.substr( 0, NStr::Find(s_gap, ":") ) + ")\n";
+        }
       }
     }
+  }
+  xprint.line("Gaps                   : ", m_GapCount, string("Gaps u_n_counts=\"")+s_gap+"\"");
 
+  if(m_GapCount) {
     int linkageYesCnt =
       m_GapTypeCnt[CAgpRow::eGapCount+CAgpRow::eGapClone   ]+
       m_GapTypeCnt[CAgpRow::eGapCount+CAgpRow::eGapFragment]+
@@ -686,42 +835,75 @@ void CAgpValidateReader::x_PrintTotals() // without comment counts
     int doNotBreakCnt= linkageYesCnt + m_GapTypeCnt[CAgpRow::eGapFragment];
     int breakCnt     = linkageNoCnt  - m_GapTypeCnt[CAgpRow::eGapFragment];
 
-    cout<< "\n- do not break scaffold: "<<ALIGN_W(doNotBreakCnt);
+    xprint.line("- do not break scaffold: ", doNotBreakCnt, "GapsWithinScaf");
     if(doNotBreakCnt) {
+      xprint.last_tag="GapsWithinScaf_byType linkage=\"yes\" type=";
       if(m_GapTypeCnt[CAgpRow::eGapCount+CAgpRow::eGapClone   ])
-        cout<< "\n  clone   , linkage yes: "<<
-              ALIGN_W(m_GapTypeCnt[CAgpRow::eGapCount+CAgpRow::eGapClone   ]);
+        xprint.line("  clone   , linkage yes: ", m_GapTypeCnt[CAgpRow::eGapCount+CAgpRow::eGapClone   ]);
       if(m_GapTypeCnt[CAgpRow::eGapCount+CAgpRow::eGapFragment])
-        cout<< "\n  fragment, linkage yes: "<<
-              ALIGN_W(m_GapTypeCnt[CAgpRow::eGapCount+CAgpRow::eGapFragment]);
+        xprint.line("  fragment, linkage yes: ", m_GapTypeCnt[CAgpRow::eGapCount+CAgpRow::eGapFragment]);
+
       if(m_GapTypeCnt[                   CAgpRow::eGapFragment])
-        cout<< "\n  fragment, linkage no : "<<
-              ALIGN_W(m_GapTypeCnt[                   CAgpRow::eGapFragment]);
+        xprint.line("  fragment, linkage no : ", m_GapTypeCnt[                   CAgpRow::eGapFragment],
+        "GapsWithinScaf_byType linkage=\"no\" type=\"fragment\""
+        );
+
       if(m_GapTypeCnt[CAgpRow::eGapCount+CAgpRow::eGapRepeat  ])
-        cout<< "\n  repeat  , linkage yes: "<<
-              ALIGN_W(m_GapTypeCnt[CAgpRow::eGapCount+CAgpRow::eGapRepeat  ]);
+        xprint.line("  repeat  , linkage yes: ", m_GapTypeCnt[CAgpRow::eGapCount+CAgpRow::eGapRepeat  ]);
       if(m_GapTypeCnt[CAgpRow::eGapCount+CAgpRow::eGapScaffold  ])
-        cout<< "\n  repeat  , linkage yes: "<<
-              ALIGN_W(m_GapTypeCnt[CAgpRow::eGapCount+CAgpRow::eGapRepeat  ]);
+        xprint.line("  scaffold, linkage yes: ", m_GapTypeCnt[CAgpRow::eGapCount+CAgpRow::eGapScaffold]);
     }
 
-    cout<< "\n- break it, linkage no : "<<ALIGN_W(breakCnt);
+    xprint.line("- break it, linkage no : ", breakCnt, "GapsBreakScaf");
     if(breakCnt) {
+      xprint.last_tag="GapsBreakScaf_byType linkage=\"no\" type=";
       for(int i=0; i<CAgpRow::eGapCount; i++) {
         if(i==CAgpRow::eGapFragment) continue;
         if(m_GapTypeCnt[i])
-          cout<< "\n\t"
+          /*
+          out<< "\n\t"
               << setw(15) << setiosflags(IOS_BASE::left) << CAgpRow::GapTypeToString(i)
               << ": " << ALIGN_W( m_GapTypeCnt[i] );
+          */
+          xprint.line(
+            string("\t") + CAgpRow::GapTypeToString(i) +
+            string("               ").substr(0, 15-strlen(CAgpRow::GapTypeToString(i)) ) + ": ",
+            m_GapTypeCnt[i]
+          );
       }
     }
-
   }
-  cout << "\n";
+
+  if( m_agp_version == eAgpVersion_2_0 && m_ln_ev_flags2count.size() ) {
+    xprint.line();
+    xprint.line("Linkage evidence:");
+    xprint.last_tag="LinkageEvidence value=";
+
+    // sort by count
+    typedef multimap<int,int> TMultiMapIntInt;
+    TMultiMapIntInt cnt2ln_ev;
+    int label_width=0;
+    for(TMapIntInt::iterator it = m_ln_ev_flags2count.begin();  it != m_ln_ev_flags2count.end(); ++it) {
+      cnt2ln_ev.insert(TMultiMapIntInt::value_type(it->second, it->first));
+      string label = CAgpRow::LinkageEvidenceFlagsToString(it->first);
+      if(label.size() > label_width) label_width = label.size();
+    }
+    if(label_width>40) label_width=40;
+    for(TMultiMapIntInt::reverse_iterator it = cnt2ln_ev.rbegin();  it != cnt2ln_ev.rend(); ++it) {
+      string label = CAgpRow::LinkageEvidenceFlagsToString(it->second);
+      if(label.size()<label_width) label +=
+        string("                                        ").substr(0, label_width-label.size());
+
+      xprint.m_strip_attrs=false;
+      xprint.line(string("  ") + label +": ", it->first );
+    }
+  }
+
 
   if(m_ObjCount) {
     x_PrintPatterns(m_objNamePatterns, "Object names",
-      m_CheckObjLen ? m_comp2len->m_count : 0
+      m_CheckObjLen ? m_comp2len->m_count : 0, NULL,
+      out, use_xml
     );
   }
 
@@ -734,10 +916,11 @@ void CAgpValidateReader::x_PrintTotals() // without comment counts
     }
     bool hasSuspicious = x_PrintPatterns(compNamePatterns, "Component names",
       m_CheckObjLen ? 0 : m_comp2len->m_count,
-      m_comp2len == &m_scaf2len ? " Scaffold from component AGP" : NULL
+      m_comp2len == &m_scaf2len ? " Scaffold from component AGP" : NULL,
+      out, use_xml
     );
     if(!m_CheckCompNames && hasSuspicious ) {
-      cout<< "Use -g or -a to print lines with suspicious accessions.\n";
+      xprint.line("Use -g or -a to print lines with suspicious accessions.");
     }
 
     const int MAX_objname_eq_comp=3;
@@ -752,31 +935,40 @@ void CAgpValidateReader::x_PrintTotals() // without comment counts
         }
       }
     }
-    if(cnt_objname_eq_comp) {
-      cout<< "\n" << cnt_objname_eq_comp << " name"
+
+    // to do: output to XML
+    if(cnt_objname_eq_comp && !use_xml) {
+      out<< "\n" << cnt_objname_eq_comp << " name"
           << (cnt_objname_eq_comp==1?" is":"s are")
           << " used both as object and as component_id:\n";
-      cout<< "  " << str_objname_eq_comp;
-      if(cnt_objname_eq_comp>MAX_objname_eq_comp) cout << ", ...";
-      cout << "\n";
+      out<< "  " << str_objname_eq_comp;
+      if(cnt_objname_eq_comp>MAX_objname_eq_comp) out << ", ...";
+      out << "\n";
     }
   }
+  if(use_xml) out << "</stats>\n";
 }
 
-void CAgpValidateReader::PrintTotals()
+void CAgpValidateReader::PrintTotals(CNcbiOstream& out, bool use_xml)
 {
-  x_PrintTotals();
+  x_PrintTotals(out, use_xml);
 
   if(m_comp2len->size()) {
-    x_PrintIdsNotInAgp();
+    x_PrintIdsNotInAgp(out, use_xml);
   }
 
-  if(m_CommentLineCount || m_EolComments) cout << "\n";
-  if(m_CommentLineCount) {
-    cout << "#Comment line count    : " << m_CommentLineCount << "\n";
+  if(use_xml) {
+    if(m_CommentLineCount) out << " <CommentLineCount>" << m_CommentLineCount << "</CommentLineCount>\n";
+    if(m_EolComments)      out << " <EolComments>"      << m_EolComments      << "</EolComments>\n";
   }
-  if(m_EolComments) {
-    cout << "End of line #comments  : " << m_EolComments << "\n";
+  else {
+    if(m_CommentLineCount || m_EolComments) out << "\n";
+    if(m_CommentLineCount) {
+      out << "#Comment line count    : " << m_CommentLineCount << "\n";
+    }
+    if(m_EolComments) {
+      out << "End of line #comments  : " << m_EolComments << "\n";
+    }
   }
 }
 
@@ -914,21 +1106,26 @@ static int GetNameCategory(const string& s)
 
 // Sort by accession count, print not more than MaxPatterns or 2*MaxPatterns
 bool CAgpValidateReader::x_PrintPatterns(
-  CAccPatternCounter& namePatterns, const string& strHeader, int fasta_count, const char* count_label)
+  CAccPatternCounter& namePatterns, const string& strHeader, int fasta_count, const char* count_label,
+  CNcbiOstream& out, bool use_xml)
 {
   const int MaxPatterns=10;
+  //const int MaxPatterns=2;
+  static const string SPACES="                                        ";
 
   // Sorted by count to print most frequent first
   CAccPatternCounter::TMapCountToString cnt_pat; // multimap<int,string>
   namePatterns.GetSortedPatterns(cnt_pat);
   SIZE_TYPE cnt_pat_size=cnt_pat.size();
 
-  cout << "\n";
+  //out << "\n";
 
-  // Calculate width of columns 1 (wPattern) and 2 (w, count)
+  // Calculate widths of columns 1 (wPattern) and 2 (w)
   int w = NStr::IntToString(
       cnt_pat.rbegin()->first // the biggest count
     ).size()+1;
+  XPrintTotalsItem xprint(out, use_xml, w);
+  xprint.line();
 
   int wPattern=strHeader.size()-2;
   int totalCount=0;
@@ -958,13 +1155,27 @@ bool CAgpValidateReader::x_PrintPatterns(
   bool mixedCategories=(nucCount && otherCount);
   if(mixedCategories && wPattern<20) wPattern=20;
   // Print the total
+  //string xml_tag_for_pattern_and_count = "";
+  string xml_outer_tag;
   if(strHeader.size()) {
-    cout<< setw(wPattern+2) << setiosflags(IOS_BASE::left)
-        << strHeader << ": " << ALIGN_W(totalCount);
-    if(fasta_count && fasta_count!=totalCount) {
-      cout << " != " << fasta_count << " in the " << (count_label ? count_label : "FASTA");
+    if(use_xml) {
+      xml_outer_tag = strHeader.substr(0, strHeader.find(' ')) + "Names";
+      out << "<" << xml_outer_tag << ">\n";
     }
-    cout<< "\n";
+    if(fasta_count && fasta_count!=totalCount) {
+      // this is not needed in XML since there is no FASTA to compare to yet...
+      xprint.m_eol_text = string(" != ") +
+        NStr::NumericToString(fasta_count) + " in the " +
+        (count_label ? count_label : "FASTA")+"\n";
+    }
+    xprint.line(
+        strHeader+SPACES.substr(
+          0, wPattern+2>(int)strHeader.size() ? wPattern+2-strHeader.size() : 0
+        ) + ": ",
+        totalCount,
+        "count"
+      );
+    //xml_tag_for_pattern_and_count = xprint.last_tag;
   }
 
   bool printNuc=(nucCount>0);
@@ -972,9 +1183,11 @@ bool CAgpValidateReader::x_PrintPatterns(
   for(;;) {
     if(mixedCategories) {
       // Could be an error - print extra sub-headings to get attention
-      cout<< string("------------------------").substr(0, wPattern-20);
-      if     (printNuc) cout << " Nucleotide accessions: " << ALIGN_W(nucCount  ) << "\n";
-      else              cout << " OTHER identifiers    : " << ALIGN_W(otherCount) << "\n";
+      xprint.m_eol_text=NcbiEmptyString; // no "\n" for the next line()
+      xprint.line(string("------------------------").substr(0, wPattern-20)+" ");
+
+      if     (printNuc) xprint.line("Nucleotide accessions: ", nucCount);
+      else              xprint.line("OTHER identifiers    : ", otherCount);
     }
 
     // Print the patterns
@@ -992,26 +1205,43 @@ bool CAgpValidateReader::x_PrintPatterns(
       if( ++patternsPrinted<=MaxPatterns ||
           cnt_pat_size<=2*MaxPatterns
       ) {
-        cout<< "  " << setw(wPattern) << setiosflags(IOS_BASE::left)
-            << it->second                    // pattern
-            << ": " << ALIGN_W( it->first ); // : count
+
+        string acc_warning;
         if(!printNuc) {
-          //if(code & fSome) cout << " - some";
-          //switch(code & f_category_mask)
           switch(code)
           {
             case fUnknownFormat|fSome:
             case fOneAccManyVer|fSome:
-              cout << " (some local or misspelled)"; break;
+              acc_warning ="some local or misspelled"; break;
             case fProtein|fSome:
-              cout << " (some look like protein accessions)"; break;
+              acc_warning ="some look like protein accessions"; break;
 
             case fUnknownFormat: if(!(mixedCategories || mixedPattern)) break;
-            case fOneAccManyVer: cout << " (local or misspelled)"; break;
-            case fProtein      : cout << " (looks like protein accession)"; break;
+            case fOneAccManyVer: acc_warning ="local or misspelled"; break;
+            case fProtein      : acc_warning ="looks like protein accession"; break;
           }
         }
-        cout << "\n";
+        string xml_tag = "";
+        if( acc_warning.size() ) {
+          xprint.m_eol_text = string(" (") + acc_warning + ")\n";
+        }
+
+        // output pattern as an attribute
+        xprint.last_tag =
+          //xml_tag_for_pattern_and_count +
+          "names"+
+          ( acc_warning.size() ? string(" warn=\"")+acc_warning+"\"" : "" )+
+          " pattern=";
+        xprint.m_strip_attrs=false;
+
+        xprint.line(
+          // pattern
+          string("  ") + it->second +
+          SPACES.substr(0, wPattern - it->second.size()) + ": ",
+          // count
+          it->first
+        );
+        xprint.last_tag = NcbiEmptyString;
       }
       else {
         // accessionsSkipped += CAccPatternCounter::GetCount(*it);
@@ -1024,20 +1254,29 @@ bool CAgpValidateReader::x_PrintPatterns(
       string s = "other ";
       s+=NStr::IntToString(patternsSkipped);
       s+=" patterns";
-      cout<< "  " << setw(wPattern) << setiosflags(IOS_BASE::left) << s
-          << ": " << ALIGN_W( accessionsSkipped )
-          << "\n";
+      xprint.line( "  " + s + SPACES.substr(0, wPattern - s.size()) + ": ",
+          accessionsSkipped,
+          //xml_tag_for_pattern_and_count +
+          "names"
+          " patterns=\"other\""
+      );
     }
 
     if(!mixedCategories || !printNuc) break;
     printNuc=false;
   }
+
+  if(use_xml && xml_outer_tag.size()) {
+    out << "</" << xml_outer_tag << ">\n";
+  }
+
   return mixedCategories||mixedPattern;
 }
 
 // label = "component(s) from FASTA not found in AGP"
 // label = "scaffold(s) not found in Chromosome from scaffold AGP"
-void CAgpValidateReader::x_PrintIdsNotInAgp()
+// 2012/02/21: xml mode not tested since the current CGI has no info to provide this kind of validation
+void CAgpValidateReader::x_PrintIdsNotInAgp(CNcbiOstream& out, bool use_xml)
 {
   CAccPatternCounter patterns;
   set<string> ids;
@@ -1073,21 +1312,35 @@ void CAgpValidateReader::x_PrintIdsNotInAgp()
       m_CheckObjLen ? "object name(s) in FASTA not found in AGP" :
       m_comp2len == &m_scaf2len ? "scaffold(s) not found in Chromosome from scaffold AGP":
       "component name(s) in FASTA not found in AGP";
-    string tmp;
-    NStr::Replace(label, "(s)", cnt==1 ? "" : "s", tmp);
-    cout << "\n" << cnt << " " << tmp << ": ";
 
-    if(cnt==1) {
-      cout << *(ids.begin()) << "\n";
-    }
-    else if(cnt<m_AgpErr->m_MaxRepeat||m_AgpErr->m_MaxRepeat==0) {
-      cout << "\n";
+    if(use_xml) {
+      // print both patterns and ALL missing names
+      label = label.substr(0, label.find(' '));
+      out << "<MissingSeqNames level=\""+label+"\">\n";
       for(set<string>::iterator it = ids.begin();  it != ids.end(); ++it) {
-        cout << "  " << *it << "\n";
+        out << " <name>" << NStr::XmlEncode(*it) << "</name>\n";
       }
     }
     else {
-      x_PrintPatterns(patterns, NcbiEmptyString, 0);
+      string tmp;
+      NStr::Replace(label, "(s)", cnt==1 ? "" : "s", tmp);
+      out << "\n" << cnt << " " << tmp << ": ";
+    }
+
+    if(!use_xml && cnt==1) {
+      out << *(ids.begin()) << "\n";
+    }
+    else if(!use_xml && (cnt<m_AgpErr->m_MaxRepeat||m_AgpErr->m_MaxRepeat==0)) {
+      out << "\n";
+      for(set<string>::iterator it = ids.begin();  it != ids.end(); ++it) {
+        out << "  " << *it << "\n";
+      }
+    }
+    else {
+      x_PrintPatterns(patterns, NcbiEmptyString, 0, NULL, out, use_xml);
+    }
+    if(use_xml) {
+      out << "</MissingSeqNames>\n";
     }
   }
 }
