@@ -1,4 +1,4 @@
-/*  $Id: bed_reader.cpp 348911 2012-01-05 16:30:20Z ludwigf $
+/*  $Id: bed_reader.cpp 372346 2012-08-16 17:40:01Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -80,6 +80,7 @@
 #include <objects/seqfeat/Gb_qual.hpp>
 #include <objects/seqfeat/Feat_id.hpp>
 
+#include <objtools/readers/read_util.hpp>
 #include <objtools/readers/reader_exception.hpp>
 #include <objtools/readers/line_error.hpp>
 #include <objtools/readers/error_container.hpp>
@@ -92,16 +93,16 @@
 #define NCBI_USE_ERRCODE_X   Objtools_Rd_RepMask
 
 BEGIN_NCBI_SCOPE
-
 BEGIN_objects_SCOPE // namespace ncbi::objects::
 
 //  ----------------------------------------------------------------------------
 CBedReader::CBedReader(
-    int flags )
+    unsigned int flags ) :
 //  ----------------------------------------------------------------------------
-    : CReaderBase(flags)
-    , m_columncount( 0 )
-    , m_usescore( false )
+    CReaderBase(flags),
+    m_currentId(""),
+    m_columncount(0),
+    m_usescore(false)
 {
 }
 
@@ -119,39 +120,65 @@ CBedReader::ReadSeqAnnot(
     IErrorContainer* pErrorContainer ) 
 //  ----------------------------------------------------------------------------                
 {
-    CRef< CSeq_annot > annot( new CSeq_annot );
-    CRef< CAnnot_descr > desc( new CAnnot_descr );
-    annot->SetDesc( *desc );
+    CRef<CSeq_annot> annot;
+    CRef<CAnnot_descr> desc;
+
+    annot.Reset(new CSeq_annot);
+    desc.Reset(new CAnnot_descr);
+    annot->SetDesc(*desc);
     annot->SetData().SetFtable();
 
     string line;
-    int linecount = 0;
+    int featureCount = 0;
 
-    while ( ! lr.AtEOF() ) {
-        ++linecount;
+    while (!lr.AtEOF()) {
+        ++m_uLineNumber;
         line = *++lr;
-        if ( NStr::TruncateSpaces( line ).empty() ) {
+        if (NStr::TruncateSpaces(line).empty()) {
             continue;
         }
         try {
-            if ( x_ParseBrowserLine( line, annot ) ) {
+            if (xParseComment(line, annot)) {
                 continue;
             }
-            if ( CReaderBase::x_ParseTrackLine( line, annot ) ) {
+            if (x_ParseBrowserLine(line, annot)) {
                 continue;
             }
-            if (!x_ParseFeature( line, annot )) {
+            if (xParseTrackLine(line, annot)) {
+                if (featureCount > 0) {
+                    --m_uLineNumber;
+                    lr.UngetLine();
+                    break;
+                }
+                continue;
+            }
+            if (!xParseFeature( line, annot )) {
+                --m_uLineNumber;
                 lr.UngetLine();
-                return annot;
+                break;
             }
+            ++featureCount;
         }
         catch( CObjReaderLineException& err ) {
-            err.SetLineNumber( linecount );
-            x_ProcessError( err, pErrorContainer );
+            xProcessError( err, pErrorContainer );
         }
         continue;
     }
-    x_AddConversionInfo( annot, pErrorContainer );
+    //  Only return a valid object if there was at least one feature
+    if (0 == featureCount) {
+        return CRef<CSeq_annot>();
+    }
+    x_AddConversionInfo(annot, pErrorContainer);
+
+    if(m_columncount >= 3) {
+        CRef<CUser_object> columnCountUser( new CUser_object() );
+        columnCountUser->SetType().SetStr( "NCBI_BED_COLUMN_COUNT" );
+        columnCountUser->AddField("NCBI_BED_COLUMN_COUNT", int ( m_columncount ) );
+    
+        CRef<CAnnotdesc> userDesc( new CAnnotdesc() );
+        userDesc->SetUser().Assign( *columnCountUser );
+        annot->SetDesc().Set().push_back( userDesc );
+    }
     return annot;
 }
 
@@ -175,42 +202,11 @@ CBedReader::ReadSeqAnnots(
     IErrorContainer* pErrorContainer )
 //  ----------------------------------------------------------------------------
 {
-//    CRef< CSeq_annot > annot( new CSeq_annot );
-//    CRef< CAnnot_descr > desc( new CAnnot_descr );
-//    annot->SetDesc( *desc );
-//    annots.push_back( annot );
-
-    CRef< CSeq_annot > annot = x_AppendAnnot(annots);
-
-    string line;
-    int linecount = 0;
-
-    while ( ! lr.AtEOF() ) {
-        ++linecount;
-        line = *++lr;
-        if ( NStr::TruncateSpaces( line ).empty() ) {
-            continue;
-        }
-        try {
-            if ( x_ParseBrowserLine( line, annot ) ) {
-                continue;
-            }
-            if ( x_ParseTrackLine( line, annots, annot ) ) {
-                continue;
-            }
-            x_ParseFeature( line, annot );
-        }
-        catch( CObjReaderLineException& err ) {
-            err.SetLineNumber( linecount );
-            x_ProcessError( err, pErrorContainer );
-        }
-        continue;
+    CRef<CSeq_annot> annot = ReadSeqAnnot(lr, pErrorContainer);
+    while (annot) {
+        annots.push_back(annot);
+        annot = ReadSeqAnnot(lr, pErrorContainer);
     }
-    if ( m_iFlags & fDumpStats ) {
-        x_DumpStats( cerr );
-    }
-    x_AddConversionInfo( annot, &m_ErrorsPrivate );
-//    return annot;
 }
                         
 //  ----------------------------------------------------------------------------                
@@ -227,28 +223,40 @@ CBedReader::ReadObject(
 
 //  ----------------------------------------------------------------------------
 bool
-CBedReader::x_ParseTrackLine(
+CBedReader::xParseTrackLine(
     const string& strLine,
-    vector< CRef< CSeq_annot > >& annots,
     CRef< CSeq_annot >& current )
 //  ----------------------------------------------------------------------------
 {
-    static bool bFirstTimeThrough = true;
-
     if ( ! NStr::StartsWith( strLine, "track" ) ) {
         return false;
     }
-
-    if ( ! bFirstTimeThrough ) {
-        x_AddConversionInfo( current, &m_ErrorsPrivate );    
+    vector<string> parts;
+    CReadUtil::Tokenize( strLine, " \t", parts );
+    if (parts.size() >= 3) {
+        try {
+            NStr::StringToInt(parts[1]);
+            NStr::StringToInt(parts[2]);
+            return false;
+        }
+        catch(...) {
+        }
+    }
+    if ( !m_currentId.empty() ) {
+//        x_AddConversionInfo( current, &m_ErrorsPrivate );    
         m_columncount = 0;
         m_ErrorsPrivate.ClearAll();
-        current = x_AppendAnnot( annots );
     }
-    else {
-        bFirstTimeThrough = false;
+    m_currentId.clear();
+    if (!CReaderBase::x_ParseTrackLine( strLine, current )) {
+        CObjReaderLineException err(
+            eDiag_Warning,
+            0,
+            "Bad track line: Expected \"track key1=value1 key2=value2 ...\". Ignored.",
+            ILineError::eProblem_BadTrackLine);
+        throw( err );    
     }
-    return CReaderBase::x_ParseTrackLine( strLine, current );
+    return true;
 }
 
 //  ----------------------------------------------------------------------------
@@ -265,13 +273,12 @@ CBedReader::x_AppendAnnot(
 }    
     
 //  ----------------------------------------------------------------------------
-bool CBedReader::x_ParseFeature(
+bool CBedReader::xParseFeature(
     const string& record,
     CRef<CSeq_annot>& annot ) /* throws CObjReaderLineException */
 //  ----------------------------------------------------------------------------
 {
     const int MAX_RECORDS = 100000;
-    static string strc;
     static int count = 0;
     count++;
 
@@ -280,14 +287,15 @@ bool CBedReader::x_ParseFeature(
 	string record_copy = record;
 	NStr::TruncateSpacesInPlace(record_copy);
 
-	// 'chr 8' fixup.
-	if (record_copy.find("chr ") == 0 || 
-		record_copy.find("Chr ") == 0 || 
-		record_copy.find("CHR ") == 0)
-		record_copy.erase(3, 1);
-
     //  parse
     NStr::Tokenize( record_copy, " \t", fields, NStr::eMergeDelims );
+    // better 'chr 8' fixup
+    if( fields.size() >= 2 ) {
+        if( NStr::EqualNocase(fields[0], "chr") ) {
+            fields[1] = fields[0] + fields[1];
+            fields.erase( fields.begin() );
+        }
+    }
     if (fields.size() != m_columncount) {
         if ( 0 == m_columncount ) {
             m_columncount = fields.size();
@@ -301,14 +309,15 @@ bool CBedReader::x_ParseFeature(
         }
     }
 
+    
     //  if feature tables get too big we _will_ run out of memory. To guard against
     //  that, limit feature tables to a single id, and to MAX_RECORDS at the most.
-    if (strc != fields[0]  ||  count == MAX_RECORDS+1) {
-        if (strc == "") {
-            strc = fields[0];
+    if (m_currentId != fields[0]  ||  count == MAX_RECORDS+1) {
+        if (m_currentId.empty()) {
+            m_currentId = fields[0];
         }
         else {
-            strc = "";
+            m_currentId.clear();
             count = 0;
             return false; //indicate no data has been processed 
         }
@@ -323,17 +332,29 @@ bool CBedReader::x_ParseFeature(
         x_SetFeatureDisplayData( feature, fields );
     }
     catch( ... ) {
+        m_currentId.clear();
         CObjReaderLineException err(
             eDiag_Error,
             0,
             "Bad data line: General parsing error." );
         throw( err );    
     }
-    x_CountRecord( fields[0] );
     ftable.push_back( feature );
     return true;
 }
 
+//  ----------------------------------------------------------------------------
+bool CBedReader::xParseComment(
+    const string& record,
+    CRef<CSeq_annot>& annot ) /* throws CObjReaderLineException */
+//  ----------------------------------------------------------------------------
+{
+    if (NStr::StartsWith(record, "#")) {
+        return true;
+    }
+    return false;
+}
+ 
 //  ----------------------------------------------------------------------------
 void CBedReader::x_SetFeatureDisplayData(
     CRef<CSeq_feat>& feature,
@@ -397,17 +418,14 @@ void CBedReader::x_SetFeatureLocation(
     const vector<string>& fields )
 //  ----------------------------------------------------------------------------
 {
-    feature->ResetLocation();
-    
-    CRef<CSeq_id> id = x_ResolvedId( fields[0] );
-
-    CRef<CSeq_loc> location( new CSeq_loc );
+    CRef<CSeq_loc> location(new CSeq_loc);
     int from, to;
-    CSeq_interval& interval = location->SetInt();
     try {
 		string cleaned_from;
 		NStr::Replace(fields[1], ",", "", cleaned_from);
-        from = NStr::StringToInt( cleaned_from ) - 1;
+        // BED 0-based 1st-point-in to ASN 0-based 1st-point-in
+        //  no conversion necessary
+        from = NStr::StringToInt(cleaned_from);
     }
     catch ( ... ) {
         CObjReaderLineException err( 
@@ -419,13 +437,15 @@ void CBedReader::x_SetFeatureLocation(
     try {
     	string cleaned_to;
 		NStr::Replace(fields[2], ",", "", cleaned_to);
-        to = NStr::StringToInt( cleaned_to ) - 1;
+        // BED 0-based 1st-point-out to ASN 0-based last-point-in
+        //  need substract 1
+        to = NStr::StringToInt(cleaned_to) - 1;
     }
     catch ( ... ) {
         CObjReaderLineException err( 
             eDiag_Error,
             0,
-            "Invalid data line --- Bad \"SeqStop\" value." );
+            "Invalid data line --- Bad \"SeqStop\" value.");
         throw( err );
     }
     if (from == to) {
@@ -448,44 +468,14 @@ void CBedReader::x_SetFeatureLocation(
         strand_field = 4;
     }
     if (strand_field < fields.size()) {
-        interval.SetStrand(( fields[strand_field] == "+" ) ?
+        location->SetStrand(( fields[strand_field] == "+" ) ?
                            eNa_strand_plus : eNa_strand_minus );
     }
-    location->SetId( *id );
     
-    feature->SetLocation( *location );
+    CRef<CSeq_id> id = CReadUtil::AsSeqId(fields[0]);
+    location->SetId(*id);
+    feature->SetLocation(*location);
 }
-
-//  ----------------------------------------------------------------------------
-CRef<CSeq_id> CBedReader::x_ResolvedId(
-    const string& strRawId )
-//  ----------------------------------------------------------------------------
-{
-    if (m_iFlags & fAllIdsAsLocal) {
-        if (NStr::StartsWith(strRawId, "lcl|")) {
-            return CRef<CSeq_id>(new CSeq_id( strRawId ) );
-        } else {
-            return CRef<CSeq_id>(new CSeq_id(CSeq_id::e_Local, strRawId ));
-        }
-    }
-
-    if (m_iFlags & fNumericIdsAsLocal) {
-        if ( strRawId.find_first_not_of("0123456789") == string::npos ) {
-            return CRef<CSeq_id>(new CSeq_id(CSeq_id::e_Local, strRawId));
-        }
-    }
-    try {
-        CRef< CSeq_id > pId( new CSeq_id( strRawId ) );
-        if (!pId || (pId->IsGi() && pId->GetGi() < 500) ) {
-            pId = new CSeq_id(CSeq_id::e_Local, strRawId);
-        }
-        return pId;
-    }
-    catch (CSeqIdException&) {
-        return CRef<CSeq_id>( new CSeq_id( CSeq_id::e_Local, strRawId ) );
-    }
-}    
-            
 //  ----------------------------------------------------------------------------
 void CBedReader::x_SetTrackData(
     CRef<CSeq_annot>& annot,
@@ -522,57 +512,14 @@ void CBedReader::x_SetTrackData(
 
 //  ----------------------------------------------------------------------------
 void
-CBedReader::x_ProcessError(
+CBedReader::xProcessError(
     CObjReaderLineException& err,
-    IErrorContainer* pContainer )
+    IErrorContainer* pContainer)
 //  ----------------------------------------------------------------------------
 {
-    err.SetLineNumber( m_uLineNumber );
-    m_ErrorsPrivate.PutError( err );
-    
-    if ( 0 == pContainer ) {
-        throw( err );
-    }
-    if ( ! pContainer->PutError( err ) )
-    {
-        throw( err );
-    }
-}
-
-//  ----------------------------------------------------------------------------
-void
-CBedReader::x_CountRecord(
-    const string& strRawId )
-//  ----------------------------------------------------------------------------
-{
-    if ( 0 == (m_iFlags & fDumpStats) ) {
-        return;
-    }
-     
-    map< string, unsigned int >::iterator it = m_RecordCounts.find( strRawId );
-    if ( it != m_RecordCounts.end() ) {
-        m_RecordCounts[ strRawId ] += 1;
-    }
-    else {
-        m_RecordCounts[ strRawId ] = 1;
-    }
-}
-
-//  ----------------------------------------------------------------------------
-void
-CBedReader::x_DumpStats(
-    CNcbiOstream& out )
-//  ----------------------------------------------------------------------------
-{
-    out << "---------------------------------------------------------" << endl;
-    out << "Record Counts:" << endl;
-    out << "---------------------------------------------------------" << endl;
-    for ( map<string, unsigned int>::iterator it = m_RecordCounts.begin(); 
-        it != m_RecordCounts.end(); ++it ) 
-    {
-        out << it->first << " :    " << it->second << endl;
-    } 
-    out << endl;      
+    err.SetLineNumber(m_uLineNumber);
+    m_ErrorsPrivate.PutError(err);
+    ProcessError(err, pContainer);
 }
 
 END_objects_SCOPE
