@@ -1,4 +1,4 @@
-/*  $Id: fasta.cpp 349158 2012-01-09 13:12:01Z kornbluh $
+/*  $Id: fasta.cpp 372587 2012-08-20 18:38:55Z ucko $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -179,7 +179,7 @@ CRef<CSeq_entry> CFastaReader::ReadOneSeq(void)
         char c = GetLineReader().PeekChar();
         if (GetLineReader().AtEOF()) {
             NCBI_THROW2(CObjReaderParseException, eEOF,
-                        "CFastaReader: Input stream no longer valid around line " + NStr::NumericToString(LineNumber()),
+                        "CFastaReader: Unexpected end-of-file around line " + NStr::NumericToString(LineNumber()),
                         LineNumber());
         }
         if (c == '>') {
@@ -188,14 +188,12 @@ CRef<CSeq_entry> CFastaReader::ReadOneSeq(void)
                 need_defline = false;
                 continue;
             } else {
-                if (TestFlag(fParseGaps)) {
-                    CTempString next_line = *++GetLineReader();
-                    if (next_line.size() > 2  &&  next_line[1] == '?'
-                        &&  ParseGapLine(next_line) ) {
-                        continue;
-                    } else {
-                        GetLineReader().UngetLine();
-                    }
+                CTempString next_line = *++GetLineReader();
+                if (next_line.size() > 2  &&  next_line[1] == '?'
+                    &&  ParseGapLine(next_line) ) {
+                    continue;
+                } else {
+                    GetLineReader().UngetLine();
                 }
                 // start of the next sequence
                 break;
@@ -205,7 +203,7 @@ CRef<CSeq_entry> CFastaReader::ReadOneSeq(void)
         } else if (c == ']') {
             if (need_defline) {
                 NCBI_THROW2(CObjReaderParseException, eEOF,
-                            "CFastaReader: Reached end of segmented set around line " + NStr::NumericToString(LineNumber()),
+                            "CFastaReader: Reached unexpected end of segmented set around line " + NStr::NumericToString(LineNumber()),
                             LineNumber());
             } else {
                 break;
@@ -241,7 +239,7 @@ CRef<CSeq_entry> CFastaReader::ReadOneSeq(void)
 
     if (need_defline  &&  GetLineReader().AtEOF()) {
         NCBI_THROW2(CObjReaderParseException, eEOF,
-                    "CFastaReader: Reached end of input around line " + NStr::NumericToString(LineNumber()),
+                    "CFastaReader: Expected defline around line " + NStr::NumericToString(LineNumber()),
                     LineNumber());
     }
 
@@ -437,6 +435,7 @@ void CFastaReader::ParseDefLine(const TStr& s)
         sa->SetDim(2);
         CDense_seg& ds = sa->SetSegs().SetDenseg();
         ds.SetNumseg(1);
+        ds.SetDim(2); // redundant, but required by validator
         ds.SetIds().push_back(GetIDs().front());
         ds.SetIds().push_back(m_BestID);
         ds.SetStarts().push_back(0);
@@ -543,6 +542,8 @@ size_t CFastaReader::ParseRange(const TStr& s, TSeqPos& start, TSeqPos& end)
     if ((negative ? (end > start) : (start > end))  ||  s[pos] != ':') {
         return 0;
     }
+    --start;
+    --end;
     return s.length() - pos;
 }
 
@@ -594,9 +595,9 @@ void CFastaReader::CheckDataLine(const TStr& s)
     }
     if (bad >= good / 3  &&  (len > 3  ||  good == 0  ||  bad > good)) {
         NCBI_THROW2(CObjReaderParseException, eFormat,
-                    "CFastaReader: Input not marked as defline or comment, but "
-                    "contains too many special characters to be plausible data around line " + NStr::NumericToString(LineNumber()),
-                    LineNumber());
+            "CFastaReader: Near line " + NStr::NumericToString(LineNumber()) +
+            ", there's a line that doesn't look like plausible data, but it's not marked as defline or comment.",
+            LineNumber());
     }
 }
 
@@ -638,12 +639,12 @@ void CFastaReader::ParseDataLine(const TStr& s)
         } else if ( !isspace(c) ) {
             if (TestFlag(fValidate)) {
                 NCBI_THROW2(CBadResiduesException, eBadResidues,
-                            string("CFastaReader: Invalid residue ") + s[pos]
+                            string("CFastaReader: Invalid " + x_NucOrProt() + "residue ") + s[pos]
                             + " at position " + NStr::UInt8ToString(pos+1), // "+1" because 1-based for user
                                 CBadResiduesException::SBadResiduePositions( m_BestID, pos, LineNumber() ) );
             } else {
                 ERR_POST_X(1, Warning
-                           << "CFastaReader: Ignoring invalid residue " << c
+                           << "CFastaReader: Ignoring invalid " + x_NucOrProt() + "residue " << c
                            << " at line " << LineNumber()
                            << ", position " << pos);
             }
@@ -696,7 +697,13 @@ bool CFastaReader::ParseGapLine(const TStr& line)
 {
     SGap gap = { GetCurrentPos(eRawPos),
                  NStr::StringToUInt(line.substr(2), NStr::fConvErr_NoThrow) };
-    if (gap.len > 0  ||  line == ">?unk100") {
+    if (gap.len > 0) {
+        m_Gaps.push_back(gap);
+        m_TotalGapLength += gap.len;
+        return true;
+    } else if (line == ">?unk100") {
+        gap.len = -100;
+        m_TotalGapLength += 100;
         m_Gaps.push_back(gap);
         return true;
     } else {
@@ -722,11 +729,32 @@ void CFastaReader::AssembleSeq(void)
         CSeqportUtil::Validate(tmp_data, &badIndexes);
         if ( ! badIndexes.empty() ) {
             NCBI_THROW2(CBadResiduesException, eBadResidues,
-                "CFastaReader: Invalid residue(s) in input sequence",
+                "CFastaReader: Invalid " + x_NucOrProt() + "residue(s) in input sequence",
                 CBadResiduesException::SBadResiduePositions( m_BestID, badIndexes, LineNumber() ) );
         }
     }
 
+    if ( !TestFlag(fParseGaps)  &&  m_TotalGapLength > 0 ) {
+        // Encountered >? lines; substitute runs of Ns or Xs as appropriate.
+        string    new_data;
+        char      gap_char(inst.IsAa() ? 'X' : 'N');
+        SIZE_TYPE pos = 0;
+        new_data.reserve(GetCurrentPos(ePosWithGaps));
+        ITERATE (TGaps, it, m_Gaps) {
+            if (it->pos > pos) {
+                new_data.append(m_SeqData, pos, it->pos - pos);
+                pos = it->pos;
+            }
+            new_data.append((it->len >= 0) ? it->len : -it->len, gap_char);
+        }
+        if (m_CurrentPos > pos) {
+            new_data.append(m_SeqData, pos, m_CurrentPos - pos);
+        }
+        swap(m_SeqData, new_data);
+        m_Gaps.clear();
+        m_CurrentPos += m_TotalGapLength;
+        m_TotalGapLength = 0;
+    }
 
     if (m_Gaps.empty()) {
         _ASSERT(m_TotalGapLength == 0);
@@ -769,12 +797,17 @@ void CFastaReader::AssembleSeq(void)
                 }
             }
 
-            if (m_Gaps[i].len == 0) { // unknown length
-                CRef<CDelta_seq> gap_ds(new CDelta_seq);
-                gap_ds->SetLoc().SetNull();
-                delta_ext.Set().push_back(gap_ds);
-            } else {
+            if (m_Gaps[i].len > 0) {
                 delta_ext.AddLiteral(m_Gaps[i].len);
+            } else {
+                CRef<CDelta_seq> gap_ds(new CDelta_seq);
+                if (m_Gaps[i].len == 0) { // totally unknown
+                    gap_ds->SetLoc().SetNull();
+                } else { // has a nominal length (normally 100)
+                    gap_ds->SetLiteral().SetLength(-m_Gaps[i].len);
+                    gap_ds->SetLiteral().SetFuzz().SetLim(CInt_fuzz::eLim_unk);
+                }
+                delta_ext.Set().push_back(gap_ds);
             }
 
             TSeqPos next_start = (i == n-1) ? m_CurrentPos : m_Gaps[i+1].pos;
@@ -833,7 +866,7 @@ void CFastaReader::AssignMolType(void)
     default:
         if (default_mol == CSeq_inst::eMol_not_set) {
             NCBI_THROW2(CObjReaderParseException, eAmbiguous,
-                        "CFastaReader: Unable to determine sequence type around line " + NStr::NumericToString(LineNumber()),
+                        "CFastaReader: Unable to determine sequence type (is it nucleotide? protein?) around line " + NStr::NumericToString(LineNumber()),
                         LineNumber());
         } else {
             inst.SetMol(default_mol);
@@ -1373,7 +1406,7 @@ static CSeq_inst::EMol s_ParseFastaDefline(CBioseq::TId& ids, string& title,
                 if (end == NPOS) {
                     if (pos > 0) {
                         NCBI_THROW2(CObjReaderParseException, eFormat,
-                                    "s_ParseFastaDefline: Bad ID "
+                                    "s_ParseFastaDefline: Bad defline ID "
                                     + name.substr(pos),
                                     pos);
                     } else if (s_IsValidLocalID(name)) {
@@ -1414,7 +1447,7 @@ static CSeq_inst::EMol s_ParseFastaDefline(CBioseq::TId& ids, string& title,
     if (ids.empty()) {
         if (flags & fReadFasta_RequireID) {
             NCBI_THROW2(CObjReaderParseException, eFormat,
-                        "s_ParseFastaDefline: no ID present", 0);
+                        "s_ParseFastaDefline: no defline ID present", 0);
         }
         CRef<CSeq_id> id(new CSeq_id);
         id->SetLocal().SetId((*counter)++);
@@ -1463,7 +1496,7 @@ CRef<CSeq_entry> s_ReadFasta_OLD(CNcbiIstream& in, TReadFastaFlags flags,
 {
     if ( !in ) {
         NCBI_THROW2(CObjReaderParseException, eFormat,
-                    "ReadFasta: Input stream no longer valid",
+                    "ReadFasta: Unexpected end of input",
                     in.tellg() - CT_POS_TYPE(0));
     } else {
         CT_INT_TYPE c = in.peek();
@@ -1674,7 +1707,7 @@ void CFastaReader::x_RecursiveApplyAllMods( CSeq_entry& entry )
             = seq.GetClosestDescriptor(CSeqdesc::e_Title);
         if (title_desc) {
             string& title(const_cast<string&>(title_desc->GetTitle()));
-            title = smp.ParseTitle(title);
+            title = smp.ParseTitle(title, CConstRef<CSeq_id>(seq.GetFirstId()) );
             smp.ApplyAllMods(seq);
             if( TestFlag(fUnknModThrow) ) {
                 CSourceModParser::TMods unused_mods = smp.GetMods(CSourceModParser::fUnusedMods);
@@ -1683,7 +1716,7 @@ void CFastaReader::x_RecursiveApplyAllMods( CSeq_entry& entry )
                     // there are unused mods and user specified to throw if any
                     // unused 
                     CNcbiOstrstream err;
-                    err << "CFastaReader: Unused mods on ";
+                    err << "CFastaReader: Unrecognized modifiers on ";
 
                     // get sequence ID
                     const CSeq_id* seq_id = seq.GetFirstId();
@@ -1711,5 +1744,16 @@ void CFastaReader::x_RecursiveApplyAllMods( CSeq_entry& entry )
     }
 }
 
+std::string CFastaReader::x_NucOrProt(void) const
+{
+    if( m_CurrentSeq && m_CurrentSeq->IsSetInst() && 
+        m_CurrentSeq->GetInst().IsSetMol() )
+    {
+        return ( m_CurrentSeq->GetInst().IsAa() ? "protein " : "nucleotide " );
+    } else {
+        return kEmptyStr;
+    }
+}
+
 END_SCOPE(objects)
 END_NCBI_SCOPE
