@@ -1,4 +1,4 @@
-/*  $Id: gff_reader.cpp 183247 2010-02-16 14:19:56Z ludwigf $
+/*  $Id: gff_reader.cpp 332617 2011-08-29 16:18:30Z ludwigf $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -658,7 +658,7 @@ CRef<CSeq_feat> CGFFReader::x_ParseFeatRecord(const SRecord& record)
             }
 
             CFeature_table_reader::AddFeatQual
-                (feat, tag, value, CFeature_table_reader::fKeepBadKey);
+                (feat, kEmptyStr, tag, value, CFeature_table_reader::fKeepBadKey);
         } else { // don't attempt to parse, just treat as imported
             CRef<CGb_qual> qual(new CGb_qual);
             qual->SetQual(tag);
@@ -703,11 +703,11 @@ CRef<CSeq_feat> CGFFReader::x_ParseFeatRecord(const SRecord& record)
 
         // translate
         CFeature_table_reader::AddFeatQual
-            (feat, "gene_id", gene_id,
+            (feat, kEmptyStr, "gene_id", gene_id,
              CFeature_table_reader::fKeepBadKey);
         if (x_GetFlags() & fGBQuals) {
             CFeature_table_reader::AddFeatQual
-                (feat, "gene", gene_id,
+                (feat, kEmptyStr, "gene", gene_id,
                  CFeature_table_reader::fKeepBadKey);
         }
     }
@@ -1282,7 +1282,11 @@ CRef<CSeq_id> CGFFReader::x_ResolveNewSeqName(const string& name)
         }
     }
     try {
-        return CRef<CSeq_id>(new CSeq_id(name));
+        CRef<CSeq_id> pId(new CSeq_id(name));
+        if (!pId || (pId->IsGi() && pId->GetGi() < 500) ) {
+            pId = new CSeq_id(CSeq_id::e_Local, name);
+        }
+        return pId;
     }
     catch (CSeqIdException&) {
         return CRef<CSeq_id>(new CSeq_id(CSeq_id::e_Local, name));
