@@ -1,4 +1,4 @@
-/*  $Id: source_mod_parser.cpp 348240 2011-12-27 12:00:48Z kornbluh $
+/*  $Id: source_mod_parser.cpp 366475 2012-06-14 16:49:11Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -32,6 +32,9 @@
 */
 
 #include <ncbi_pch.hpp>
+
+#include <sstream>
+
 #include <objtools/readers/source_mod_parser.hpp>
 
 #include <corelib/ncbiutil.hpp>
@@ -92,7 +95,7 @@ T* LeaveAsIs(void)
 }
 
 
-string CSourceModParser::ParseTitle(const CTempString& title)
+string CSourceModParser::ParseTitle(const CTempString& title, CConstRef<CSeq_id> seqid )
 {
     SMod   mod;
     string stripped_title;
@@ -100,6 +103,8 @@ string CSourceModParser::ParseTitle(const CTempString& title)
 
     m_Mods.clear();
 
+    mod.seqid = seqid;
+
     while (pos < title.size()) {
         size_t lb_pos = title.find('[', pos), eq_pos = title.find('=', lb_pos),
                end_pos = CTempString::npos;
@@ -273,7 +278,7 @@ void CSourceModParser::ApplyMods(CBioseq& seq)
         } else if (NStr::EqualNocase(mod->value, "circular")) {
             seq.SetInst().SetTopology(CSeq_inst::eTopology_circular);
         } else {
-            x_HandleBadModValue(*mod);
+            x_HandleBadModValue(*mod, "'linear', 'circular'", (TDummyModMap*)NULL);
         }
     }
 
@@ -284,7 +289,7 @@ void CSourceModParser::ApplyMods(CBioseq& seq)
         } else if (NStr::EqualNocase(mod->value, "rna")) {
             seq.SetInst().SetMol( CSeq_inst::eMol_rna );
         } else {
-            x_HandleBadModValue(*mod);
+            x_HandleBadModValue(*mod, "'dna', 'rna'", (TDummyModMap*)NULL);
         }
     }
 
@@ -297,7 +302,7 @@ void CSourceModParser::ApplyMods(CBioseq& seq)
         } else if (NStr::EqualNocase(mod->value, "mixed")) {
             seq.SetInst().SetStrand( CSeq_inst::eStrand_mixed );
         } else {
-            x_HandleBadModValue(*mod);
+            x_HandleBadModValue(*mod, "'single', 'double', 'mixed'", (TDummyModMap*)NULL);
         }
     }
 
@@ -341,7 +346,9 @@ void CSourceModParser::x_ApplyMods(CAutoInitRef<CBioSource>& bsrc,
                 bsrc->SetGenome(CBioSource::GetTypeInfo_enum_EGenome()
                                 ->FindValue(mod->value));
             } catch (CSerialException&) {
-                x_HandleBadModValue(*mod);
+                x_HandleBadModValue(
+                    *mod, "'mitochondrial', 'provirus', 'extrachromosomal', 'insertion sequence'", 
+                    (TDummyModMap*)NULL);
             }
         }
     }
@@ -359,7 +366,8 @@ void CSourceModParser::x_ApplyMods(CAutoInitRef<CBioSource>& bsrc,
                             ->FindValue(mod->value));
             }
         } catch (CSerialException&) {
-            x_HandleBadModValue(*mod);
+            x_HandleBadModValue(*mod, "'natural mutant', 'mutant'", (TDummyModMap*)NULL, 
+                CBioSource::GetTypeInfo_enum_EOrigin() );
         }
     }
 
@@ -572,71 +580,71 @@ void CSourceModParser::x_ApplyMods(CAutoInitRef<CBioSource>& bsrc,
 }
 
 
-typedef pair<const char*, CMolInfo::TBiomol> TBiomolMapEntry;
+typedef SStaticPair<const char*, CMolInfo::TBiomol> TBiomolMapEntry;
 static const TBiomolMapEntry sc_BiomolArray[] = {
-    TBiomolMapEntry("?",               CMolInfo::eBiomol_unknown),
-    TBiomolMapEntry("cRNA",            CMolInfo::eBiomol_cRNA),
-    TBiomolMapEntry("genomic",         CMolInfo::eBiomol_genomic),
-    TBiomolMapEntry("genomic-mRNA",    CMolInfo::eBiomol_genomic_mRNA),
-    TBiomolMapEntry("mRNA",            CMolInfo::eBiomol_mRNA),
-    TBiomolMapEntry("non-coding RNA",  CMolInfo::eBiomol_ncRNA),
-    TBiomolMapEntry("other-genetic",   CMolInfo::eBiomol_other_genetic),
-    TBiomolMapEntry("peptide",         CMolInfo::eBiomol_peptide),
-    TBiomolMapEntry("precursor RNA",   CMolInfo::eBiomol_pre_RNA),
-    TBiomolMapEntry("rRNA",            CMolInfo::eBiomol_rRNA),
-    TBiomolMapEntry("scRNA",           CMolInfo::eBiomol_scRNA),
-    TBiomolMapEntry("snoRNA",          CMolInfo::eBiomol_snoRNA),
-    TBiomolMapEntry("snRNA",           CMolInfo::eBiomol_snRNA),
-    TBiomolMapEntry("transcribed-RNA", CMolInfo::eBiomol_transcribed_RNA),
-    TBiomolMapEntry("transfer-messenger RNA", CMolInfo::eBiomol_tmRNA),
-    TBiomolMapEntry("tRNA",            CMolInfo::eBiomol_tRNA)
+    { "?",               CMolInfo::eBiomol_unknown },
+    { "cRNA",            CMolInfo::eBiomol_cRNA },
+    { "genomic",         CMolInfo::eBiomol_genomic },
+    { "genomic-mRNA",    CMolInfo::eBiomol_genomic_mRNA },
+    { "mRNA",            CMolInfo::eBiomol_mRNA },
+    { "non-coding RNA",  CMolInfo::eBiomol_ncRNA },
+    { "other-genetic",   CMolInfo::eBiomol_other_genetic },
+    { "peptide",         CMolInfo::eBiomol_peptide },
+    { "precursor RNA",   CMolInfo::eBiomol_pre_RNA },
+    { "rRNA",            CMolInfo::eBiomol_rRNA },
+    { "scRNA",           CMolInfo::eBiomol_scRNA },
+    { "snoRNA",          CMolInfo::eBiomol_snoRNA },
+    { "snRNA",           CMolInfo::eBiomol_snRNA },
+    { "transcribed-RNA", CMolInfo::eBiomol_transcribed_RNA },
+    { "transfer-messenger RNA", CMolInfo::eBiomol_tmRNA },
+    { "tRNA",            CMolInfo::eBiomol_tRNA }
 };
-typedef CStaticArrayMap<const char*, CMolInfo::TBiomol,
+typedef CStaticPairArrayMap<const char*, CMolInfo::TBiomol,
                         CSourceModParser::PKeyCompare>  TBiomolMap;
 DEFINE_STATIC_ARRAY_MAP(TBiomolMap, sc_BiomolMap, sc_BiomolArray);
 
-typedef pair<const char*, CMolInfo::TTech> TTechMapEntry;
+typedef SStaticPair<const char*, CMolInfo::TTech> TTechMapEntry;
 static const TTechMapEntry sc_TechArray[] = {
-    TTechMapEntry("?",                  CMolInfo::eTech_unknown),
-    TTechMapEntry("barcode",            CMolInfo::eTech_barcode),
-    TTechMapEntry("both",               CMolInfo::eTech_both),
-    TTechMapEntry("composite-wgs-htgs", CMolInfo::eTech_composite_wgs_htgs),
-    TTechMapEntry("concept-trans",      CMolInfo::eTech_concept_trans),
-    TTechMapEntry("concept-trans-a",    CMolInfo::eTech_concept_trans_a),
-    TTechMapEntry("derived",            CMolInfo::eTech_derived),
-    TTechMapEntry("EST",                CMolInfo::eTech_est),
-    TTechMapEntry("fli cDNA",           CMolInfo::eTech_fli_cdna),
-    TTechMapEntry("genetic map",        CMolInfo::eTech_genemap),
-    TTechMapEntry("htc",                CMolInfo::eTech_htc),
-    TTechMapEntry("htgs 0",             CMolInfo::eTech_htgs_0),
-    TTechMapEntry("htgs 1",             CMolInfo::eTech_htgs_1),
-    TTechMapEntry("htgs 2",             CMolInfo::eTech_htgs_2),
-    TTechMapEntry("htgs 3",             CMolInfo::eTech_htgs_3),
-    TTechMapEntry("physical map",       CMolInfo::eTech_physmap),
-    TTechMapEntry("seq-pept",           CMolInfo::eTech_seq_pept),
-    TTechMapEntry("seq-pept-homol",     CMolInfo::eTech_seq_pept_homol),
-    TTechMapEntry("seq-pept-overlap",   CMolInfo::eTech_seq_pept_overlap),
-    TTechMapEntry("standard",           CMolInfo::eTech_standard),
-    TTechMapEntry("STS",                CMolInfo::eTech_sts),
-    TTechMapEntry("survey",             CMolInfo::eTech_survey),
-    TTechMapEntry("tsa",                CMolInfo::eTech_tsa),
-    TTechMapEntry("wgs",                CMolInfo::eTech_wgs)
+    { "?",                  CMolInfo::eTech_unknown },
+    { "barcode",            CMolInfo::eTech_barcode },
+    { "both",               CMolInfo::eTech_both },
+    { "composite-wgs-htgs", CMolInfo::eTech_composite_wgs_htgs },
+    { "concept-trans",      CMolInfo::eTech_concept_trans },
+    { "concept-trans-a",    CMolInfo::eTech_concept_trans_a },
+    { "derived",            CMolInfo::eTech_derived },
+    { "EST",                CMolInfo::eTech_est },
+    { "fli cDNA",           CMolInfo::eTech_fli_cdna },
+    { "genetic map",        CMolInfo::eTech_genemap },
+    { "htc",                CMolInfo::eTech_htc },
+    { "htgs 0",             CMolInfo::eTech_htgs_0 },
+    { "htgs 1",             CMolInfo::eTech_htgs_1 },
+    { "htgs 2",             CMolInfo::eTech_htgs_2 },
+    { "htgs 3",             CMolInfo::eTech_htgs_3 },
+    { "physical map",       CMolInfo::eTech_physmap },
+    { "seq-pept",           CMolInfo::eTech_seq_pept },
+    { "seq-pept-homol",     CMolInfo::eTech_seq_pept_homol },
+    { "seq-pept-overlap",   CMolInfo::eTech_seq_pept_overlap },
+    { "standard",           CMolInfo::eTech_standard },
+    { "STS",                CMolInfo::eTech_sts },
+    { "survey",             CMolInfo::eTech_survey },
+    { "tsa",                CMolInfo::eTech_tsa },
+    { "wgs",                CMolInfo::eTech_wgs }
 };
-typedef CStaticArrayMap<const char*, CMolInfo::TTech,
+typedef CStaticPairArrayMap<const char*, CMolInfo::TTech,
 CSourceModParser::PKeyCompare>  TTechMap;
 DEFINE_STATIC_ARRAY_MAP(TTechMap, sc_TechMap, sc_TechArray);
 
-typedef pair<const char*, CMolInfo::TCompleteness> TCompletenessMapEntry;
+typedef SStaticPair<const char*, CMolInfo::TCompleteness> TCompletenessMapEntry;
 static const TCompletenessMapEntry sc_CompletenessArray[] = {
-    TCompletenessMapEntry("complete",  CMolInfo::eCompleteness_complete ),
-    TCompletenessMapEntry("has-left",  CMolInfo::eCompleteness_has_left ),
-    TCompletenessMapEntry("has-right", CMolInfo::eCompleteness_has_right ),
-    TCompletenessMapEntry("no-ends",   CMolInfo::eCompleteness_no_ends ),
-    TCompletenessMapEntry("no-left",   CMolInfo::eCompleteness_no_left ),
-    TCompletenessMapEntry("no-right",  CMolInfo::eCompleteness_no_right ),
-    TCompletenessMapEntry("partial",   CMolInfo::eCompleteness_partial )
+    { "complete",  CMolInfo::eCompleteness_complete  },
+    { "has-left",  CMolInfo::eCompleteness_has_left  },
+    { "has-right", CMolInfo::eCompleteness_has_right  },
+    { "no-ends",   CMolInfo::eCompleteness_no_ends  },
+    { "no-left",   CMolInfo::eCompleteness_no_left  },
+    { "no-right",  CMolInfo::eCompleteness_no_right  },
+    { "partial",   CMolInfo::eCompleteness_partial  }
 };
-typedef CStaticArrayMap<const char*, CMolInfo::TCompleteness,
+typedef CStaticPairArrayMap<const char*, CMolInfo::TCompleteness,
 CSourceModParser::PKeyCompare>  TCompletenessMap;
 DEFINE_STATIC_ARRAY_MAP(TCompletenessMap, sc_CompletenessMap, sc_CompletenessArray);
 
@@ -648,7 +656,7 @@ void CSourceModParser::x_ApplyMods(CAutoInitRef<CMolInfo>& mi)
     if ((mod = FindMod("moltype", "mol-type")) != NULL) {
         TBiomolMap::const_iterator it = sc_BiomolMap.find(mod->value.c_str());
         if (it == sc_BiomolMap.end()) {
-            x_HandleBadModValue(*mod);
+            x_HandleBadModValue(*mod, kEmptyStr, &sc_BiomolMap);
         } else {
             mi->SetBiomol(it->second);
         }
@@ -658,7 +666,7 @@ void CSourceModParser::x_ApplyMods(CAutoInitRef<CMolInfo>& mi)
     if ((mod = FindMod("tech")) != NULL) {
         TTechMap::const_iterator it = sc_TechMap.find(mod->value.c_str());
         if (it == sc_TechMap.end()) {
-            x_HandleBadModValue(*mod);
+            x_HandleBadModValue(*mod, kEmptyStr, &sc_TechMap);
         } else {
             mi->SetTech(it->second);
         }
@@ -668,7 +676,7 @@ void CSourceModParser::x_ApplyMods(CAutoInitRef<CMolInfo>& mi)
     if ((mod = FindMod("completeness", "completedness")) != NULL) {
         TTechMap::const_iterator it = sc_CompletenessMap.find(mod->value.c_str());
         if (it == sc_CompletenessMap.end()) {
-            x_HandleBadModValue(*mod);
+            x_HandleBadModValue(*mod, kEmptyStr, &sc_CompletenessMap);
         } else {
             mi->SetCompleteness(it->second);
         }
@@ -922,6 +930,26 @@ void CSourceModParser::ApplyPubMods(CSeq_descr& sd)
     s_ApplyPubMods(sd, FindAllMods("PMID"));
 }
 
+CSourceModParser::CBadModError::CBadModError( 
+    const SMod & badMod, 
+    const string & sAllowedValues )
+    : runtime_error(x_CalculateErrorString(badMod, sAllowedValues)),
+            m_BadMod(badMod), m_sAllowedValues(sAllowedValues) 
+{ 
+    // no further work required
+}
+
+string CSourceModParser::CBadModError::x_CalculateErrorString(
+            const SMod & badMod, 
+            const string & sAllowedValues )
+{
+    stringstream str_strm;
+    str_strm << "Bad modifier value at seqid '" 
+        << ( badMod.seqid ? badMod.seqid->AsFastaString() : "UNKNOWN")
+        << "'. '" << badMod.key << "' cannot have value '" << badMod.value
+        << "'.  Accepted values are [" << sAllowedValues << "]";
+    return str_strm.str();
+}
 
 CSourceModParser::TMods CSourceModParser::GetMods(TWhichMods which) const
 {
@@ -997,19 +1025,47 @@ void CSourceModParser::GetLabel(string* s, TWhichMods which) const
     }
 }
 
-void CSourceModParser::x_HandleBadModValue(const SMod& mod)
+template <class TModMap>
+void CSourceModParser::x_HandleBadModValue(
+    const SMod& mod, const string & sAllowedValues, 
+    const TModMap * modMap,
+    const CEnumeratedTypeValues* enum_values)
 {
     m_BadMods.insert(mod);
 
+    if( eHandleBadMod_Ignore == m_HandleBadMod ) {
+        return;
+    }
+
+    string sAllAllowedValues = sAllowedValues;
+    if( NULL != enum_values ) {
+        ITERATE( CEnumeratedTypeValues::TValues, enum_iter, enum_values->GetValues() ) {
+            if( ! sAllAllowedValues.empty() ) {
+                sAllAllowedValues += ", ";
+            }
+            sAllAllowedValues += '\'' + enum_iter->first + '\'';
+        }
+    }
+
+    if( NULL != modMap ) {
+        ITERATE( typename TModMap, modmap_iter, *modMap ) {
+            if( ! sAllAllowedValues.empty() ) {
+                sAllAllowedValues += ", ";
+            }
+            sAllAllowedValues += string("'") + modmap_iter->first + "'";
+        }
+    }
+
+    CBadModError badModError(mod, sAllAllowedValues);
+
     switch( m_HandleBadMod ) {
-    case eHandleBadMod_Ignore:
-        // nothing to do
-        break;
     case eHandleBadMod_Throw:
-        throw CBadModError(mod);
+        throw badModError;
     case eHandleBadMod_PrintToCerr:
-        cerr << "Warning: Bad modifier: " << mod.ToString() << endl;
+        cerr << badModError.what() << endl;
         break;
+    default:
+        _TROUBLE;
     }
 }
 
