@@ -1,4 +1,4 @@
-/*  $Id: vcf_reader.cpp 339628 2011-09-30 13:33:56Z ludwigf $
+/*  $Id: vcf_reader.cpp 372349 2012-08-16 17:41:09Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -83,6 +83,7 @@
 #include <objects/seqfeat/Variation_inst.hpp>
 #include <objects/seqfeat/VariantProperties.hpp>
 
+#include <objtools/readers/read_util.hpp>
 #include <objtools/readers/reader_exception.hpp>
 #include <objtools/readers/line_error.hpp>
 #include <objtools/readers/error_container.hpp>
@@ -509,7 +510,7 @@ CVcfReader::x_ParseData(
 //  ----------------------------------------------------------------------------
 {
     vector<string> columns;
-    NStr::Tokenize( line, " \t", columns, NStr::eMergeDelims );
+    NStr::Tokenize( line, "\t", columns, NStr::eMergeDelims );
     if ( columns.size() < 8 ) {
         return false;
     }
@@ -562,7 +563,7 @@ CVcfReader::x_AssignFeatureLocation(
     CRef<CSeq_feat> pFeature )
 //  ---------------------------------------------------------------------------
 {
-    CRef< CSeq_id > pId( new CSeq_id( CSeq_id::e_Local, data.m_strChrom ) );
+    CRef<CSeq_id> pId(CReadUtil::AsSeqId(data.m_strChrom, m_iFlags));
 
     pFeature->SetLocation().SetInt().SetId( *pId );
     pFeature->SetLocation().SetInt().SetFrom( data.m_iPos - 1 );
