@@ -1,4 +1,4 @@
-/*  $Id: wiggle_data.cpp 346995 2011-12-13 14:34:50Z ludwigf $
+/*  $Id: wiggle_data.cpp 372343 2012-08-16 17:38:49Z rafanovi $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -47,6 +47,7 @@
 #include <objects/seqtable/seqtable__.hpp>
 #include <objtools/readers/reader_exception.hpp>
 
+#include <objtools/readers/read_util.hpp>
 #include <objtools/readers/reader_exception.hpp>
 #include <objtools/readers/line_error.hpp>
 #include <objtools/readers/error_container.hpp>
@@ -129,11 +130,11 @@ void CWiggleRecord::ParseTrackDefinition(
 {
     Reset();
     if ( data.size() < 2 || data[0] != "track" ) {
-        CLineError err( 
-            ILineError::eProblem_GeneralParsingError,
-            eDiag_Error, 
-            "", 
-            0);
+        CObjReaderLineException err(
+            eDiag_Error,
+            0,
+            "",
+            ILineError::eProblem_GeneralParsingError);
         throw(err);
     }
     vector<string>::const_iterator it = data.begin();
@@ -141,11 +142,11 @@ void CWiggleRecord::ParseTrackDefinition(
         string strKey;
         string strValue;
         if ( ! NStr::SplitInTwo( *it, "=", strKey, strValue ) ) {
-            CLineError err( 
-                ILineError::eProblem_GeneralParsingError,
-                eDiag_Error, 
-                "", 
-                0);
+            CObjReaderLineException err(
+                eDiag_Error,
+                0,
+                "",
+                ILineError::eProblem_GeneralParsingError);
             throw(err);
         }
         NStr::ReplaceInPlace( strValue, "\"", "" );
@@ -161,11 +162,11 @@ void CWiggleRecord::ParseDataBed(
 //  ----------------------------------------------------------------------------
 {
     if ( data.size() != 4 ) {
-        CLineError err( 
-            ILineError::eProblem_GeneralParsingError,
-            eDiag_Error, 
-            "", 
-            0);
+        CObjReaderLineException err(
+            eDiag_Error,
+            0,
+            "",
+            ILineError::eProblem_GeneralParsingError);
         throw(err);
     }
     m_strChrom = data[0];
@@ -173,33 +174,33 @@ void CWiggleRecord::ParseDataBed(
         m_uSeqStart = NStr::StringToUInt( data[1] );
     }
     catch ( ... ) {
-        CLineError err( 
-            ILineError::eProblem_FeatureBadStartAndOrStop,
-            eDiag_Error, 
-            "", 
-            0);
+        CObjReaderLineException err(
+            eDiag_Error,
+            0,
+            "",
+            ILineError::eProblem_FeatureBadStartAndOrStop);
         throw(err);
     }
     try {
         m_uSeqSpan = NStr::StringToUInt( data[2] ) - m_uSeqStart;
     }
     catch ( ... ) {
-        CLineError err( 
-            ILineError::eProblem_FeatureBadStartAndOrStop,
-            eDiag_Error, 
-            "", 
-            0);
+        CObjReaderLineException err(
+            eDiag_Error,
+            0,
+            "",
+            ILineError::eProblem_FeatureBadStartAndOrStop);
         throw(err);
     }
     try {
         m_dValue = NStr::StringToDouble( data[3] );
     }
     catch ( ... ) {
-        CLineError err( 
-            ILineError::eProblem_BadScoreValue,
-            eDiag_Error, 
-            "", 
-            0);
+        CObjReaderLineException err(
+            eDiag_Error,
+            0,
+            "",
+            ILineError::eProblem_BadScoreValue);
         throw(err);
     }
 }
@@ -213,22 +214,22 @@ void CWiggleRecord::ParseDataVarstep(
         m_uSeqStart = NStr::StringToInt( data[0] ) - 1; // varStep is 1- based
     }
     catch ( ... ) {
-        CLineError err( 
-            ILineError::eProblem_FeatureBadStartAndOrStop,
-            eDiag_Error, 
-            "", 
-            0);
+        CObjReaderLineException err(
+            eDiag_Error,
+            0,
+            "",
+            ILineError::eProblem_FeatureBadStartAndOrStop);
         throw(err);
     }
     try {
         m_dValue = NStr::StringToDouble( data[1] );
     }
     catch ( ... ) {
-        CLineError err( 
-            ILineError::eProblem_BadScoreValue,
-            eDiag_Error, 
-            "", 
-            0);
+        CObjReaderLineException err(
+            eDiag_Error,
+            0,
+            "",
+            ILineError::eProblem_BadScoreValue);
         throw(err);
     }
 }
@@ -243,11 +244,11 @@ void CWiggleRecord::ParseDataFixedstep(
         m_dValue = NStr::StringToDouble( data[0] );
     }
     catch ( ... ) {
-        CLineError err( 
-            ILineError::eProblem_BadScoreValue,
-            eDiag_Error, 
-            "", 
-            0);
+        CObjReaderLineException err(
+            eDiag_Error,
+            0,
+            "",
+            ILineError::eProblem_BadScoreValue);
         throw(err);
     }
 }
@@ -265,14 +266,14 @@ void CWiggleRecord::ParseDeclarationVarstep(
     vector<string>::const_iterator it = data.begin();
     for ( ++it; it != data.end(); ++it ) {
         vector<string> key_value_pair;
-        CWiggleReader::Tokenize( *it, "=", key_value_pair );
+        CReadUtil::Tokenize( *it, "=", key_value_pair );
 
         if ( key_value_pair.size() != 2 ) {
-            CLineError err( 
-                ILineError::eProblem_GeneralParsingError,
-                eDiag_Error, 
-                "", 
-                0);
+            CObjReaderLineException err(
+                eDiag_Error,
+                0,
+                "",
+                ILineError::eProblem_GeneralParsingError);
             throw(err);
         }
         if ( key_value_pair[0] == "chrom" ) {
@@ -283,11 +284,11 @@ void CWiggleRecord::ParseDeclarationVarstep(
             m_uSeqSpan = NStr::StringToUInt( key_value_pair[1] );
             continue;
         }
-        CLineError err( 
-            ILineError::eProblem_UnrecognizedQualifierName,
-            eDiag_Error, 
-            "", 
-            0);
+        CObjReaderLineException err(
+            eDiag_Error,
+            0,
+            "",
+            ILineError::eProblem_UnrecognizedQualifierName);
         throw(err);
     }
 }
@@ -302,14 +303,14 @@ void CWiggleRecord::ParseDeclarationFixedstep(
     vector<string>::const_iterator it = data.begin();
     for ( ++it; it != data.end(); ++it ) {
         vector<string> key_value_pair;
-        CWiggleReader::Tokenize( *it, "=", key_value_pair );
+        CReadUtil::Tokenize( *it, "=", key_value_pair );
 
         if ( key_value_pair.size() != 2 ) {
-            CLineError err( 
-                ILineError::eProblem_GeneralParsingError,
-                eDiag_Error, 
-                "", 
-                0);
+            CObjReaderLineException err(
+                eDiag_Error,
+                0,
+                "",
+                ILineError::eProblem_GeneralParsingError);
             throw(err);
         }
         if ( key_value_pair[0] == "chrom" ) {
@@ -328,11 +329,11 @@ void CWiggleRecord::ParseDeclarationFixedstep(
             m_uSeqStep = NStr::StringToUInt( key_value_pair[1] );
             continue;
         }
-        CLineError err( 
-            ILineError::eProblem_UnrecognizedQualifierName,
-            eDiag_Error, 
-            "", 
-            0);
+        CObjReaderLineException err(
+            eDiag_Error,
+            0,
+            "",
+            ILineError::eProblem_UnrecognizedQualifierName);
         throw(err);
     }
     m_uSeqStart -= m_uSeqStep;
@@ -434,7 +435,7 @@ void CWiggleTrack::MakeTable(
         col_loc->SetHeader().SetField_name( "Seq-table location" );
         col_loc->SetDefault().SetLoc().SetInt().SetId(*s_ResolveId(m_strChrom));
         col_loc->SetDefault().SetLoc().SetInt().SetFrom( m_uSeqStart );
-        col_loc->SetDefault().SetLoc().SetInt().SetTo( m_uSeqStop );
+        col_loc->SetDefault().SetLoc().SetInt().SetTo( m_uSeqStop-1 );
     }
 
     { // Seq-id
