@@ -1,4 +1,4 @@
-/*  $Id: classstr.cpp 208612 2010-10-19 18:22:26Z gouriano $
+/*  $Id: classstr.cpp 338794 2011-09-22 15:43:54Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -42,6 +42,7 @@
 #include "statictype.hpp"
 #include "stlstr.hpp"
 #include "ptrstr.hpp"
+#include "reftype.hpp"
 
 BEGIN_NCBI_SCOPE
 
@@ -51,12 +52,12 @@ BEGIN_NCBI_SCOPE
 CClassTypeStrings::CClassTypeStrings(const string& externalName,
                                      const string& className,
                                      const string& namespaceName,
+                                     const CDataType* dataType,
                                      const CComments& comments)
-    : CParent(comments),
+    : CParent(namespaceName,dataType,comments),
       m_IsObject(true), m_HaveUserClass(true), m_HaveTypeInfo(true),
       m_ExternalName(externalName), m_ClassName(className)
 {
-    SetNamespaceName(namespaceName);
 }
 
 CClassTypeStrings::~CClassTypeStrings(void)
@@ -359,12 +360,12 @@ void CClassTypeStrings::GenerateClassCode(CClassCode& code,
                 if (mem != 0) {
                     const CDataMemberContainerType* elem =
                         dynamic_cast<const CDataMemberContainerType*>(mem->GetElementType());
-                    if (elem != 0 && NStr::StartsWith(elem->GetMemberName(),"E_")) {
+                    if (elem && elem->GetMemberName() == "E") {
                         string name;
                         if (elem->GetTypeStr()) {
                             name = elem->GetTypeStr()->GetClassNameDT();
                         } else {
-                            name = "C_" + elem->GetMemberName();
+                            name = elem->ClassName();
                         }
                         code.ClassPublic() <<
                             "    typedef "<< name <<" "<< "C_E;\n";
@@ -710,7 +711,11 @@ void CClassTypeStrings::GenerateClassCode(CClassCode& code,
                          !isNullWithAtt) {
                     code.Methods(inl) <<
                         "    if (!CanGet"<< i->cName<<"()) {\n"
-                        "        ThrowUnassigned("<<member_index<<");\n"
+                        "        ThrowUnassigned("<<member_index;
+#if 0
+                    code.Methods(inl) << ", __FILE__, __LINE__";
+#endif
+                    code.Methods(inl) << ");\n"
                         "    }\n";
                 }
                 code.Methods(inl) <<
@@ -1176,13 +1181,46 @@ void CClassTypeStrings::GenerateClassCode(CClassCode& code,
     methods <<
         "CLASS_INFO(\""<<GetExternalName()<<"\", "<<classPrefix<<GetClassNameDT()<<")\n"
         "{\n";
-    if ( !GetModuleName().empty() ) {
+    
+    SInternalNames names;
+    string module_name = GetModuleName(&names);
+    if ( GetExternalName().empty() && !names.m_OwnerName.empty() ) {
         methods <<
-            "    SET_CLASS_MODULE(\""<<GetModuleName()<<"\");\n";
+            "    SET_INTERNAL_NAME(\""<<names.m_OwnerName<<"\", ";
+        if ( !names.m_MemberName.empty() )
+            methods << "\""<<names.m_MemberName<<"\"";
+        else
+            methods << "0";
+        methods << ");\n";
+    }
+    if ( !module_name.empty() ) {
+        methods <<
+            "    SET_CLASS_MODULE(\""<<module_name<<"\");\n";
+    }
+
+    ENsQualifiedMode defNsqMode = eNSQNotSet;
+    if (DataType()) {
+        defNsqMode = DataType()->IsNsQualified();
+        if (defNsqMode == eNSQNotSet) {
+            const CDataMember *dm = DataType()->GetDataMember();
+            if (dm && dm->Attlist()) {
+                defNsqMode = eNSUnqualified;
+            }
+        }
     }
     if ( !GetNamespaceName().empty() ) {
         methods <<
-            "    SET_NAMESPACE(\""<<GetNamespaceName()<<"\");\n";
+            "    SET_NAMESPACE(\""<<GetNamespaceName()<<"\")";
+        if (defNsqMode != eNSQNotSet) {
+            methods << "->SetNsQualified(";
+            if (defNsqMode == eNSQualified) {
+                methods << "true";
+            } else {
+                methods << "false";
+            }
+            methods << ")";
+        }
+        methods << ";\n";
     }
     if ( !m_ParentClassName.empty() ) {
         code.SetParentClass(m_ParentClassName, m_ParentClassNamespace);
@@ -1295,7 +1333,7 @@ void CClassTypeStrings::GenerateClassCode(CClassCode& code,
                         if (defref) {
                             const CRefTypeStrings* refStr = dynamic_cast<const CRefTypeStrings*>(argStr);
                             if (refStr) {
-                                defTName = refStr->GetDataType()->GetCType(code.GetNamespace());
+                                defTName = refStr->GetDataTypeStr()->GetCType(code.GetNamespace());
                                 defTName += "::Tdata";
                             }
                             defref = false;
@@ -1352,6 +1390,16 @@ void CClassTypeStrings::GenerateClassCode(CClassCode& code,
                         methods << "->SetCompressed()";
                     }
                 }
+                ENsQualifiedMode memNsqMode = i->dataType->IsNsQualified();
+                if (memNsqMode != eNSQNotSet && memNsqMode != defNsqMode) {
+                    methods << "->SetNsQualified(";
+                    if (memNsqMode == eNSQualified) {
+                        methods << "true";
+                    } else {
+                        methods << "false";
+                    }
+                    methods << ")";
+                }
             }
             if (i->nonEmpty) {
                 methods << "->SetNonEmpty()";
@@ -1567,6 +1615,11 @@ void CClassRefTypeStrings::GenerateTypeCode(CClassContext& ctx) const
 
 void CClassRefTypeStrings::GeneratePointerTypeCode(CClassContext& ctx) const
 {
+    const CReferenceDataType* ref =
+        dynamic_cast<const CReferenceDataType*>(DataType());
+    if (ref && ref->IsRefToParent()) {
+        return;
+    }
     ctx.AddForwardDeclaration(m_ClassName, m_Namespace);
     ctx.CPPIncludes().insert(m_FileName);
 }
