@@ -1,4 +1,4 @@
-/*  $Id: datatool.cpp 352193 2012-02-03 20:27:54Z rafanovi $
+/*  $Id: datatool.cpp 366538 2012-06-14 20:27:59Z rafanovi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -81,7 +81,7 @@ int CDataTool::Run(void)
 
 CDataTool::CDataTool(void)
 {
-    SetVersion( CVersionInfo(2,4,4) );
+    SetVersion( CVersionInfo(2,5,0) );
 }
 
 void CDataTool::Init(void)
@@ -642,6 +642,7 @@ bool CDataTool::GenerateCode(void)
     if ( generator.GetOpt("pch", &opt) )
         CFileCode::SetPchHeader(opt);
     
+    cout << GetFullVersion().Print(GetProgramDisplayName(), CVersion::fVersionInfo);
     // generate code
     generator.GenerateCode();
 
@@ -758,9 +759,9 @@ SourceFile::EType CDataTool::LoadDefinitions(
 
 END_NCBI_SCOPE
 
-int main(int argc, const char* argv[])
+int NcbiSys_main(int argc, ncbi::TXChar* argv[])
 {
     USING_NCBI_SCOPE;
     CException::EnableBackgroundReporting(false);
-    return CDataTool().AppMain(argc, argv, 0, eDS_Default, 0, "datatool");
+    return CDataTool().AppMain(argc, argv, 0, eDS_Default, 0, _T_XSTRING("datatool"));
 }
