@@ -1,4 +1,4 @@
-/*  $Id: dtdparser.cpp 208442 2010-10-18 14:00:44Z gouriano $
+/*  $Id: dtdparser.cpp 294599 2011-05-25 15:14:49Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -377,6 +377,12 @@ void DTDParser::ConsumeElementContent(DTDElement& node)
         case K_PCDATA:
             id_name = s_SpecialName;
             break;
+        case K_EMPTY:
+            node.SetType(DTDElement::eEmpty);
+            ConsumeToken();
+            GetNextToken();
+            EndElementContent( node);
+            return;
         case T_SYMBOL:
             switch (symbol = NextToken().GetSymbol()) {
             case '(':
@@ -545,7 +551,9 @@ string DTDParser::CreateEmbeddedName(const DTDElement& node, int depth) const
                 refname = *i;
             }
             if (!refname.empty()) {
-                new_var += toupper((unsigned char) refname[0]);;
+                string::size_type name = refname.find(':');
+                name = (name != string::npos && (name+1) < refname.size()) ? (name+1) : 0;
+                new_var += toupper((unsigned char) refname[name]);;
 // try to avoid very long names
                 if (new_var.size() > 8) {
                     break;
@@ -656,7 +664,7 @@ void DTDParser::ParseEntityContent(const string& name)
     ConsumeSymbol('>');
 }
 
-void DTDParser::PushEntityLexer(const string& name)
+DTDEntity* DTDParser::PushEntityLexer(const string& name)
 {
     map<string,DTDEntity>::iterator i = m_MapEntity.find(name);
     if (i == m_MapEntity.end()) {
@@ -694,6 +702,7 @@ void DTDParser::PushEntityLexer(const string& name)
     if (m_MapEntity[name].IsExternal()) {
         Lexer().BeginFile();
     }
+    return &(m_MapEntity[name]);
 }
 
 bool DTDParser::PopEntityLexer(void)
@@ -865,6 +874,7 @@ void DTDParser::ParseEnumeratedList(DTDAttribute& attrib)
 void DTDParser::GenerateDataTree(CDataTypeModule& module)
 {
     m_GeneratedTypes.clear();
+    m_ElementEmbTypes.clear();
     map<string,DTDElement>::iterator i;
     for (i = m_MapElement.begin(); i != m_MapElement.end(); ++i) {
 
@@ -889,7 +899,9 @@ void DTDParser::GenerateDataTree(CDataTypeModule& module)
             string qname = i->second.GetName() + i->second.GetNamespaceName();
             if (m_GeneratedTypes.find(qname) == m_GeneratedTypes.end()) {
                 m_GeneratedTypes.insert(qname);
+                m_ElementEmbTypes.push_back(qname);
                 ModuleType(module, i->second);
+                m_ElementEmbTypes.clear();
             }
         }
     }
@@ -938,6 +950,7 @@ CDataType* DTDParser::x_Type(
                     emb.SetName(refNode.GetName());
                 }
                 type->SetNamespaceName( node.GetNamespaceName());
+                type->SetNsQualified(node.IsQualified());
                 return type;
             }
         }
@@ -950,8 +963,17 @@ CDataType* DTDParser::x_Type(
                  node.GetType() == DTDElement::eSet);
     bool attrib = !ignoreAttrib && node.HasAttributes();
     bool ref = fromInside && !node.IsEmbedded();
-    bool keep_global;
+    bool ref_to_parent = false;
 
+    string embtype;
+    if (fromInside && node.IsEmbedded() && !node.GetTypeName().empty()) {
+        embtype = node.GetName() + node.GetTypeName() + "###";
+        ref_to_parent = ref =
+            find(m_ElementEmbTypes.begin(), m_ElementEmbTypes.end(), embtype) != m_ElementEmbTypes.end();
+        m_ElementEmbTypes.push_back(embtype);
+    }
+
+    bool keep_global;
     keep_global = (cont && uniseq && (attrib ||
         (node.IsEmbedded() && !node.IsNamed() && fromInside))) ||
         (!cont && attrib);
@@ -961,7 +983,7 @@ CDataType* DTDParser::x_Type(
 
     if (keep_global) {
         if (ref) {
-            type = new CReferenceDataType(node.GetName());
+            type = new CReferenceDataType(node.GetName(), ref_to_parent);
         } else {
             type = CompositeNode(node, occ);
             uniseq = false;
@@ -1047,6 +1069,8 @@ CDataType* DTDParser::x_Type(
         }
     }
     type->SetSourceLine(node.GetSourceLine());
+    type->SetNamespaceName( node.GetNamespaceName());
+    type->SetNsQualified( node.IsQualified());
     if (uniseq) {
         CUniSequenceDataType* uniType = new CUniSequenceDataType(type);
         uniType->SetSourceLine(type->GetSourceLine());
@@ -1066,7 +1090,10 @@ CDataType* DTDParser::x_Type(
         container->AddMember(member);
         type = container.release();
     }
-    type->SetNamespaceName( node.GetNamespaceName());
+
+    if (!embtype.empty()) {
+        m_ElementEmbTypes.pop_back();
+    }
     return type;
 }
 
@@ -1319,6 +1346,7 @@ CDataType* DTDParser::x_AttribType(const DTDAttribute& att)
         break;
     }
     type->SetSourceLine(att.GetSourceLine());
+    type->SetNsQualified(att.IsQualified());
     return type;
 }
 
@@ -1511,6 +1539,11 @@ void DTDParser::PrintDocumentNode(const string& name, const DTDElement& node)
         cout << ", default=";
         cout << "\"" << node.GetDefault() << "\"";
     }
+    if (!node.GetNamespaceName().empty()) {
+        cout << endl;
+        cout << "Namespace: " << node.GetNamespaceName() << endl;
+        cout << "form: " << (node.IsQualified() ? "qualified" : "unqualified") << endl;
+    }
     cout << endl;
     if (!node.GetComments().Empty()) {
         cout << "        === Comments ===" << endl;
@@ -1520,9 +1553,6 @@ void DTDParser::PrintDocumentNode(const string& name, const DTDElement& node)
         cout << "        === AttribComments ===" << endl;
         node.GetAttribComments().PrintDTD(cout, CComments::eDoNotWriteBlankLine);
     }
-    if (!node.GetNamespaceName().empty()) {
-        cout << "Namespace: " << node.GetNamespaceName() << endl;
-    }
     if (node.HasAttributes()) {
         PrintNodeAttributes(node);
     }
@@ -1608,6 +1638,12 @@ void DTDParser::PrintAttribute(const DTDAttribute& attrib, bool indent/*=true*/)
     }
     cout << ", ";
     cout << "\"" << attrib.GetValue() << "\"";
+
+    cout << endl;
+    cout << "form:" << (attrib.IsQualified() ? "qualified" : "unqualified") << endl;
+    if (!attrib.GetNamespaceName().empty()) {
+        cout << "Namespace: " << attrib.GetNamespaceName() << endl;
+    }
     cout << endl;
     if (!attrib.GetComments().Empty()) {
         cout << "        === Comments ===" << endl;
